@@ -1,23 +1,24 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/baseframework/app/basecontroller',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/dialogs',
-     'io.ox/office/tk/object/triggerobject',
-     'io.ox/office/tk/object/baseobject'
-    ], function (Utils, KeyCodes, Dialogs, TriggerObject, BaseObject) {
+define('io.ox/office/baseframework/app/basecontroller', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/dialogs',
+    'io.ox/office/tk/object/baseobject',
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/tk/object/timermixin'
+], function (Utils, KeyCodes, Dialogs, BaseObject, TriggerObject, TimerMixin) {
 
     'use strict';
 
@@ -40,6 +41,7 @@ define('io.ox/office/baseframework/app/basecontroller',
      * @constructor
      *
      * @extends TriggerObject
+     * @extends TimerMixin
      *
      * @param {BaseApplication} app
      *  The application that has created this controller instance.
@@ -55,38 +57,14 @@ define('io.ox/office/baseframework/app/basecontroller',
         var // self reference
             self = this,
 
-            // all the little controller items
-            items = {
+            // the application view
+            view = null,
 
-                'app/quit': {
-                    // quit in a timeout (otherwise this controller becomes invalid while still running)
-                    set: function () { _.defer(function () { app.quit(); }); }
-                },
+            // all registered controller items, mapped by item key
+            items = {},
 
-                // disabled during import, enabled afterwards
-                'app/imported': {
-                    enable: function () { return app.isImportFinished(); }
-                },
-
-                'document/download': {
-                    // enabled in read-only mode, disabled during import
-                    parent: 'app/imported',
-                    set: function () { return app.downloadDocument(); }
-                },
-
-                'document/print': {
-                    // enabled in read-only mode, disabled during import
-                    parent: 'app/imported',
-                    set: function () { return app.print(); },
-                    shortcut: { keyCode: 'P', ctrlOrMeta: true }
-                },
-
-                'document/sendmail': {
-                    // enabled in read-only mode, disabled during import
-                    parent: 'app/imported',
-                    set: function () { return app.sendMail(); }
-                }
-            },
+            // dummy item, used e.g. for undefined parent items
+            dummyItem = null,
 
             // the delay time for the debounced BaseController.update() method
             // Bug 30479: must NOT be 0, to be able give some space for deferred
@@ -106,8 +84,44 @@ define('io.ox/office/baseframework/app/basecontroller',
             // shortcut definitions for 'keypress', mapped by char code (for performance)
             charShortcuts = {},
 
-            // dummy item singleton, used e.g. for undefined parent items
-            dummyItem = null;
+            // definitions for items supported by all application types
+            BASE_ITEM_DEFINITIONS = {
+
+                // quit/destroy the application (do not return the promise,
+                // otherwise this controller would wait for its own destruction)
+                'app/quit': {
+                    set: function () { app.quit(); }
+                },
+
+                // disabled during import, enabled afterwards
+                'app/imported': {
+                    enable: function () { return app.isImportFinished(); }
+                },
+
+                // disabled in stand-alone mode
+                'app/bundled': {
+                    enable: function () { return !app.isStandalone(); }
+                },
+
+                // download the document: enabled in read-only mode, disabled during import
+                'document/download': {
+                    parent: 'app/imported',
+                    set: function () { return app.download(); }
+                },
+
+                // print the document (download as PDF): enabled in read-only mode, disabled during import
+                'document/print': {
+                    parent: 'app/imported',
+                    set: function () { return app.print(); },
+                    shortcut: { keyCode: 'P', ctrlOrMeta: true }
+                },
+
+                // send document as mail: enabled in read-only mode, disabled during import, disabled in stand-alone-mode
+                'document/sendmail': {
+                    parent: ['app/imported', 'app/bundled'],
+                    set: function () { return app.sendMail(); }
+                }
+            };
 
         // class Item ---------------------------------------------------------
 
@@ -301,7 +315,7 @@ define('io.ox/office/baseframework/app/basecontroller',
                     // (but not for modal dialogs which have their own busy blocker)
                     pending = promise.state() === 'pending';
                     if (pending && !Dialogs.isModalDialogOpen()) {
-                        app.getView().enterBusy({ delay: 500 });
+                        view.enterBusy();
                     }
                 } else {
                     // item is disabled
@@ -313,7 +327,7 @@ define('io.ox/office/baseframework/app/basecontroller',
                     promise.always(function () {
                         if (pending) {
                             // execute in a timeout, needed for dialogs which are closed *after* resolve/reject
-                            app.executeDelayed(_.bind(self.grabFocus, self, focusOptions));
+                            self.executeDelayed(function () { self.grabFocus(focusOptions); });
                         } else {
                             self.grabFocus(focusOptions);
                         }
@@ -324,7 +338,7 @@ define('io.ox/office/baseframework/app/basecontroller',
                 Utils.logSelenium('time=' + _.now(), 'key=' + key, 'state=' + promise.state());
                 if (pending) {
                     promise.always(function () {
-                        app.getView().leaveBusy();
+                        view.leaveBusy();
                         self.update();
                         Utils.logSelenium('time=' + _.now(), 'key=' + key, 'state=' + promise.state());
                     });
@@ -354,6 +368,7 @@ define('io.ox/office/baseframework/app/basecontroller',
         // base constructor ---------------------------------------------------
 
         TriggerObject.call(this);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
@@ -495,9 +510,11 @@ define('io.ox/office/baseframework/app/basecontroller',
          *      false otherwise. If one or more parent items have been
          *      specified (see above) and at least one of them returned false,
          *      the item is in disabled state already, and this function will
-         *      not be called anymore. If the item does not specify an enable
-         *      handler, it defaults to the value true, so the enabled state is
-         *      dependent on the parent items only.
+         *      not be called anymore. Otherwise, the cached return values of
+         *      the parent getters (see option 'get' below) will be passed to
+         *      this function, in the same order as specified. If the item does
+         *      not specify an enable handler, it defaults to the value true,
+         *      so the enabled state is dependent on the parent items only.
          *  @param {Function} [definition.get]
          *      Getter function returning the current value of the item. Can be
          *      omitted for one-way action items (actions without a return
@@ -660,7 +677,7 @@ define('io.ox/office/baseframework/app/basecontroller',
          * @returns {BaseController}
          *  A reference to this controller.
          */
-        this.update = app.createDebouncedMethod(function () { return self; }, function () {
+        this.update = this.createDebouncedMethod(Utils.NOOP, function () {
 
             // collect states of all matching items in resultCache
             clearStateCache();
@@ -673,7 +690,7 @@ define('io.ox/office/baseframework/app/basecontroller',
                 // (more than 3sec could also mean 5sec. or more, because we have to check later)
 
                 var startTime = _.now();
-                _.find(items, function(item) {
+                _.any(items, function (item) {
                     item.getState();
                     if ((_.now() - startTime) > 3000) { return true; }
                 });
@@ -683,7 +700,7 @@ define('io.ox/office/baseframework/app/basecontroller',
 
             // notify all listeners (with locked GUI refresh for performance,
             // view elements are listening to controller items to change their visibility)
-            app.getView().lockPaneLayout(function () {
+            view.lockPaneLayout(function () {
                 self.trigger('change:items', stateCache);
             });
 
@@ -823,7 +840,7 @@ define('io.ox/office/baseframework/app/basecontroller',
             } else if (((focusTarget instanceof $) || (focusTarget instanceof HTMLElement)) && $(focusTarget).is(':visible')) {
                 $(focusTarget).focus();
             } else {
-                app.getView().grabFocus();
+                view.grabFocus();
             }
             return this;
         };
@@ -834,19 +851,26 @@ define('io.ox/office/baseframework/app/basecontroller',
         dummyItem = new Item('\x00');
 
         // register item definitions
-        this.registerDefinitions(items);
+        this.registerDefinitions(BASE_ITEM_DEFINITIONS);
+
+        // initialize class members when the application is alive
+        app.onInit(function () {
+            view = app.getView();
+        });
 
         // update once after import (successful and failed)
         app.onImport(function () {
             initializeKeyHandler();
-            self.listenTo(app.getView(), 'refresh:layout', function () { self.update(); });
+            self.listenTo(view, 'refresh:layout', function () { self.update(); });
             self.update();
         });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
             _.invoke(items, 'destroy');
-            self = items = null;
+            dummyItem.destroy();
+            self = items = dummyItem = view = stateCache = keyShortcuts = charShortcuts = null;
+            BASE_ITEM_DEFINITIONS = null;
         });
 
     } // class BaseController
