@@ -5,7 +5,7 @@
  * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
- * © 2012 Open-Xchange Inc., Tarrytown, NY, USA. info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -61,8 +61,11 @@ define('io.ox/office/baseframework/app/extensionregistry',
                     xlt:  { format: 'ooxml', template: true, templateLocked: true, macros: true, convert: true },
                     xla:  {                  macros: true }, // add-in
                     xlsb: { format: 'ooxml', macros: true, convert: true },
-                    ods:  { format: 'ooxml', macros: true, convert: true },
-                    ots:  { format: 'ooxml', template: true, templateLocked: true, macros: true, convert: true },
+                    csv:  {}, //empty only for viewing
+
+                    //disable ods for release!
+                    ods:  { format: 'odf', macros: true, convert: !Config.DEBUG }, //spreadsheet
+                    ots:  { format: 'odf', template: true, macros: true, convert: !Config.DEBUG }, //spreadsheet
                 }
             },
 
@@ -105,15 +108,18 @@ define('io.ox/office/baseframework/app/extensionregistry',
 
         // extension to template extension map
         EXT_TO_TEMPLATE_EXT_MAP = {
-            'docx' : 'dotx',
-            'docm' : 'dotx',
-            'xlsx' : 'xltx',
-            'xlsm' : 'xltx',
-            'pptx' : 'potx',
-            'pptm' : 'potx',
-            'odt' : 'ott',
-            'ods' : 'ots',
-            'odp' : 'otp'
+            docx: 'dotx',
+            docm: 'dotx',
+            dotm: 'dotx',
+            xlsx: 'xltx',
+            xlsm: 'xltx',
+            xltm: 'xltx',
+            pptx: 'potx',
+            pptm: 'potx',
+            potm: 'potx',
+            odt: 'ott',
+            ods: 'ots',
+            odp: 'otp'
         },
 
         // all application configurations, mapped by application identifier
@@ -206,6 +212,26 @@ define('io.ox/office/baseframework/app/extensionregistry',
         return (_.isString(editModule) && (editModule !== module)) ? null : extensionSettings;
     };
 
+    /**
+     * Returns the type category of the file with the passed file name.
+     *
+     * @param {String} fileName
+     *  The file name (case-insensitive).
+     *
+     * @returns {String}
+     *  The type category of the file with the passed file name; or an empty
+     *  string, if the file extension is not registered.
+     *  - 'text': A text document.
+     *  - 'spreadsheet': A spreadsheet document.
+     *  - 'presentation': A presentation document.
+     *  - 'preview': Another viewable document that is not part of the
+     *      preceding file categories.
+     */
+    ExtensionRegistry.getFileCategory = function (fileName) {
+        var extensionSettings = ExtensionRegistry.getExtensionSettings(fileName);
+        return Utils.getStringOption(extensionSettings, 'type', '');
+    };
+
     /**
      * Returns all extensions that can be viewed with OX Preview as array.
      *
@@ -237,6 +263,23 @@ define('io.ox/office/baseframework/app/extensionregistry',
         return Utils.getBooleanOption(extensionSettings, 'viewable', false);
     };
 
+    /**
+     * Returns the module name of the edit application that is able to modify
+     * the file with the passed file name.
+     *
+     * @param {String} fileName
+     *  The file name (case-insensitive).
+     *
+     * @returns {String}
+     *  The module name of the edit application that is able to modify the file
+     *  with the passed file name; or an empty string, if the file is not
+     *  editable.
+     */
+    ExtensionRegistry.getEditModule = function (fileName) {
+        var extensionSettings = ExtensionRegistry.getExtensionSettings(fileName);
+        return Utils.getStringOption(extensionSettings, 'module', '');
+    };
+
     /**
      * Returns whether the file with the passed name is editable by any of the
      * OX Documents edit applications.
@@ -301,6 +344,30 @@ define('io.ox/office/baseframework/app/extensionregistry',
         return Utils.getBooleanOption(extensionSettings, 'editable', false) && !Utils.getBooleanOption(extensionSettings, 'native', false);
     };
 
+    /**
+     * Returns the file format identifier of the file with the passed name.
+     *
+     * @param {String} fileName
+     *  The file name (case-insensitive).
+     *
+     * @param {String} [editModule]
+     *  If specified, must match the module name of the edit application
+     *  registered for the extension of the passed file name. If omitted, the
+     *  edit application module name for the extension will be ignored.
+     *
+     * @returns {String}
+     *  The file format identifier of the file with the passed name:
+     *  - 'ooxml' for a file that will be edited in the OfficeOpenXML file
+     *      format (either natively, or by conversion from a file format not
+     *      supported natively).
+     *  - 'odf' for a file that will be edited in the OpenDocument file format.
+     *  - An empty string for all other file names.
+     */
+    ExtensionRegistry.getFileFormat = function (fileName, editModule) {
+        var extensionSettings = ExtensionRegistry.getExtensionSettings(fileName, editModule);
+        return Utils.getStringOption(extensionSettings, 'format', '');
+    };
+
     /**
      * Returns whether the file with the passed name is a file that will be
      * edited in the OfficeOpenXML file format (either natively, or by
@@ -319,8 +386,7 @@ define('io.ox/office/baseframework/app/extensionregistry',
      *  format.
      */
     ExtensionRegistry.isOfficeOpenXML = function (fileName, editModule) {
-        var extensionSettings = ExtensionRegistry.getExtensionSettings(fileName, editModule);
-        return Utils.getStringOption(extensionSettings, 'format') === 'ooxml';
+        return ExtensionRegistry.getFileFormat(fileName, editModule) === 'ooxml';
     };
 
     /**
@@ -341,8 +407,7 @@ define('io.ox/office/baseframework/app/extensionregistry',
      *  format.
      */
     ExtensionRegistry.isOpenDocument = function (fileName, editModule) {
-        var extensionSettings = ExtensionRegistry.getExtensionSettings(fileName, editModule);
-        return Utils.getStringOption(extensionSettings, 'format') === 'odf';
+        return ExtensionRegistry.getFileFormat(fileName, editModule) === 'odf';
     };
 
     /**
@@ -440,13 +505,14 @@ define('io.ox/office/baseframework/app/extensionregistry',
      * @param {String} fileName
      *  The file name (case-insensitive).
      *
-     * @returns {String}
-     *  The template extension for the file name or null if the format cannot
+     * @returns {String|Null}
+     *  The template extension for the file name; or null if the format cannot
      *  be written to a template format.
      */
     ExtensionRegistry.getTemplateExtensionForFileName = function (fileName) {
         var extension = ExtensionRegistry.getExtension(fileName);
-        return EXT_TO_TEMPLATE_EXT_MAP.hasOwnProperty(extension) ? EXT_TO_TEMPLATE_EXT_MAP[extension] : null;
+        extension = EXT_TO_TEMPLATE_EXT_MAP[extension];
+        return _.isString(extension) ? extension : null;
     };
 
     // static initialization ==================================================
