@@ -1,19 +1,20 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/baseframework/model/basemodel',
-     ['io.ox/office/tk/object/triggerobject'
-     ], function (TriggerObject) {
+define('io.ox/office/baseframework/model/basemodel', [
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/tk/object/timermixin'
+], function (TriggerObject, TimerMixin) {
 
     'use strict';
 
@@ -25,15 +26,17 @@ define('io.ox/office/baseframework/model/basemodel',
      * @constructor
      *
      * @extends TriggerObject
+     * @extends TimerMixin
      *
      * @param {BaseApplication} app
      *  The application that has created this model instance.
      */
     function BaseModel(/*app*/) {
 
-        // base constructor ---------------------------------------------------
+        // base constructors --------------------------------------------------
 
         TriggerObject.call(this);
+        TimerMixin.call(this);
 
     } // class BaseModel
 
