@@ -1,31 +1,36 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/baseframework/view/baseview',
-    ['io.ox/core/notifications',
-     'io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/object/triggerobject',
-     'io.ox/office/baseframework/view/pane',
-     'gettext!io.ox/office/main',
-     'io.ox/office/baseframework/view/nodetracking',
-     'less!io.ox/office/baseframework/view/basestyle',
-     'less!io.ox/office/baseframework/view/docs-icons'
-    ], function (Notifications, Utils, KeyCodes, Forms, TriggerObject, Pane, gt) {
+define('io.ox/office/baseframework/view/baseview', [
+    'io.ox/core/notifications',
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/baseframework/view/pane',
+    'gettext!io.ox/office/main',
+    'io.ox/office/tk/nodetracking', // import only
+    'less!io.ox/office/baseframework/view/basestyle',
+    'less!io.ox/office/baseframework/view/docs-icons'
+], function (Notifications, Utils, KeyCodes, Forms, TriggerObject, TimerMixin, Pane, gt) {
 
     'use strict';
 
+    var // a special value for an element's 'tabindex' attribute to remove it from the
+        // focus tab chain, but to be able to find it later with an CSS selector
+        HIDDEN_TAB_INDEX = -42;
+
     // global functions =======================================================
 
     /**
@@ -70,6 +75,7 @@ define('io.ox/office/baseframework/view/baseview',
      * @constructor
      *
      * @extends TriggerObject
+     * @extends TimerMixin
      *
      * @param {BaseApplication} app
      *  The application containing this view instance.
@@ -82,11 +88,13 @@ define('io.ox/office/baseframework/view/baseview',
      *      application window.
      *  @param {Function} [initOptions.initHandler]
      *      A callback handler function called to initialize this view instance
-     *      after construction.
+     *      after construction. May return a promise to be able to run
+     *      asynchronous code during initialization.
      *  @param {Function} [initOptions.initGuiHandler]
      *      A callback handler function called to initialize the graphical
      *      elements of this view instance, after the document has been
-     *      imported.
+     *      imported. May return a promise to be able to run asynchronous code
+     *      during initialization.
      *  @param {Function} [initOptions.grabFocusHandler]
      *      A function that has to implement moving the browser focus somewhere
      *      into the application pane. Used in the method BaseView.grabFocus()
@@ -177,11 +185,15 @@ define('io.ox/office/baseframework/view/baseview',
             pendingYellOptions = null,
 
             // the timer waiting to fade in the blocker element in busy mode
-            blockerFadeTimer = null;
+            blockerFadeTimer = null,
+
+            // the timer waiting to show a warning message during busy mode
+            blockerWarningTimer = null;
 
         // base constructor ---------------------------------------------------
 
         TriggerObject.call(this);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
@@ -288,21 +300,9 @@ define('io.ox/office/baseframework/view/baseview',
                 }
             }
 
-            return app.createDebouncedMethod(registerRefresh, executeRefresh);
+            return self.createDebouncedMethod(registerRefresh, executeRefresh);
         }());
 
-        /**
-         * Handles closed notification alert boxes and returns the browser
-         * focus to the application.
-         */
-        function notificationRemovedHandler(event) {
-            // return focus to application after the notification closes
-            // bug 33735: but only if focus is inside the alert box (e.g. after mouse click)
-            if (Utils.containsFocus(event.currentTarget)) {
-                app.executeDelayed(function () { self.grabFocus(); });
-            }
-        }
-
         /**
          * Shows the specified notification message. If the message is of type
          * 'error', the message will be stored internally and automatically
@@ -330,7 +330,7 @@ define('io.ox/office/baseframework/view/baseview',
             yellNode = Notifications.yell(yellOptions);
 
             // wait for deferred initialization of the DOM node
-            yellNode.on('notification:appear', function () {
+            self.listenTo(yellNode, 'notification:appear', function () {
 
                 var // the message node as target for additional contents
                     messageNode = yellNode.find('.message'),
@@ -375,8 +375,13 @@ define('io.ox/office/baseframework/view/baseview',
                 messageNode.append($('<div>').append(anchorNode));
             });
 
-            // set focus back to the document container after the notification node is closed
-            yellNode.on('notification:removed', notificationRemovedHandler);
+            // return focus to application after the notification closes
+            self.listenTo(yellNode, 'notification:removed', function (event) {
+                // bug 33735: only if focus is inside the alert box (e.g. after mouse click)
+                if (Forms.containsFocus(event.currentTarget)) {
+                    self.executeDelayed(function () { self.grabFocus(); });
+                }
+            });
         }
 
         /**
@@ -385,7 +390,7 @@ define('io.ox/office/baseframework/view/baseview',
         function windowShowHandler() {
 
             // do not show the window contents if view is still hidden explicitly
-            if (viewHidden) { return; }
+            if (!self.isVisible()) { return; }
 
             // bug 29348: show pending notification also, if tempNode is empty
             if (pendingYellOptions && app.isImportFinished() && self.isVisible()) {
@@ -394,14 +399,14 @@ define('io.ox/office/baseframework/view/baseview',
 
             // move all application nodes from temporary storage into view
             if (tempNode.children().length > 0) {
-            	// change tabindex-attribute to re-activate tabbing through the application
-				Utils.changeTabState(tempNode, {tabable: true, childrenOfNode: true});
                 winBodyNode.append(tempNode.children());
+                // restore invalidated tabindex attribute for browser focus tab chain
+                tempNode.find('[tabindex="' + HIDDEN_TAB_INDEX + '"]').attr('tabindex', 1);
             }
 
             // do not update GUI and grab focus while document is still being imported
             if (app.isImportFinished()) {
-                _.defer(function () {
+                self.executeDelayed(function () {
                     refreshPaneLayout();
                     app.getController().update();
                     self.grabFocus();
@@ -415,9 +420,9 @@ define('io.ox/office/baseframework/view/baseview',
         function windowHideHandler() {
             // move all application nodes from view to temporary storage
             if (tempNode.children().length === 0) {
-            	// change tabindex-attribute to prevent tabbing through the hidden application
                 tempNode.append(winBodyNode.children());
-                Utils.changeTabState(tempNode, {tabable: false, childrenOfNode: true});
+                // remove all nodes from browser focus tab chain as long as they are hidden
+                tempNode.find('[tabindex="1"]').attr('tabindex', HIDDEN_TAB_INDEX);
             }
         }
 
@@ -432,20 +437,22 @@ define('io.ox/office/baseframework/view/baseview',
          *  Invoked from the import process of the application. DO NOT call
          *  from external code.
          *
-         * @returns {BaseView}
-         *  A reference to this instance.
+         * @returns {jQuery.Promise}
+         *  The promise of a Deferred object that will be resolved when the
+         *  initialization handler has finished.
          */
-        this.hideBeforeImport = _.once(_.bind(function () {
+        this.hideBeforeImport = _.once(function () {
 
             // set window to busy state
             app.getWindow().busy();
-            this.hide();
+            self.hide();
 
             // invoke the initialization handler passed to the constructor
-            Utils.getFunctionOption(initOptions, 'initHandler', $.noop).call(self);
+            var result = Utils.getFunctionOption(initOptions, 'initHandler', $.noop).call(self);
 
-            return this;
-        }, this));
+            // return a promise
+            return $.when(result);
+        });
 
         /**
          * Initializes the view after importing the document. Invokes the GUI
@@ -456,26 +463,37 @@ define('io.ox/office/baseframework/view/baseview',
          *  Invoked from the import process of the application. DO NOT call
          *  from external code.
          *
-         * @returns {BaseView}
-         *  A reference to this instance.
+         * @returns {jQuery.Promise}
+         *  The promise of a Deferred object that will be resolved when the GUI
+         *  initialization handler has finished.
          */
-        this.showAfterImport = function () {
+        this.showAfterImport = _.once(function () {
 
             // show the application window
-            this.show();
+            self.show();
 
             // invoke the GUI initialization handler passed to the constructor
-            Utils.getFunctionOption(initOptions, 'initGuiHandler', $.noop).call(self);
+            var result = Utils.getFunctionOption(initOptions, 'initGuiHandler', $.noop).call(self);
 
-            // immediately refresh all view panes to reduce flicker effects
-            silentRefreshPaneLayout();
-
-            this.leaveBusy();
-            return this;
-        };
+            // wait for the callback, immediately refresh all view panes to reduce flicker effects
+            return $.when(result).always(function () {
+                silentRefreshPaneLayout();
+                self.leaveBusy();
+            });
+        });
 
         // public methods -----------------------------------------------------
 
+        /**
+         * Returns whether the browser is located inside the application pane.
+         *
+         * @returns {Boolean}
+         *  Whether the browser is located inside the application pane.
+         */
+        this.hasAppFocus = function () {
+            return Forms.hasOrContainsFocus(this.getAppPaneNode());
+        };
+
         /**
          * Moves the browser focus into the application pane. Calls the
          * handler function passed to the constructor of this instance.
@@ -622,10 +640,10 @@ define('io.ox/office/baseframework/view/baseview',
          */
         this.getContentMargin = function () {
             return {
-                left: Utils.convertCssLength(appContentNode.css('margin-left'), 'px', 1),
-                right: Utils.convertCssLength(appContentNode.css('margin-right'), 'px', 1),
-                top: Utils.convertCssLength(appContentNode.css('margin-top'), 'px', 1),
-                bottom: Utils.convertCssLength(appContentNode.css('margin-bottom'), 'px', 1)
+                left: Utils.getElementCssLength(appContentNode, 'margin-left'),
+                right: Utils.getElementCssLength(appContentNode, 'margin-right'),
+                top: Utils.getElementCssLength(appContentNode, 'margin-top'),
+                bottom: Utils.getElementCssLength(appContentNode, 'margin-bottom')
             };
         };
 
@@ -777,18 +795,15 @@ define('io.ox/office/baseframework/view/baseview',
          *
          * @param {Object} [options]
          *  Optional parameters:
-         *  @param {Boolean} [options.showFileName=false]
-         *      If set to true, the file name will be shown in the top-left
-         *      corner of the blocker element.
          *  @param {Function} [options.initHandler]
          *      A function that can fill custom contents into the header and
          *      footer of the window blocker element. Receives the following
          *      parameters:
-         *      (1) {jQuery} header
+         *      (1) {jQuery} headerNode
          *          The header element above the centered progress bar.
-         *      (2) {jQuery} footer
+         *      (2) {jQuery} footerNode
          *          The footer element below the centered progress bar.
-         *      (3) {jQuery} blocker
+         *      (3) {jQuery} blockerNode
          *          The entire window blocker element (containing the header,
          *          footer, and progress bar elements).
          *      Will be called in the context of this view instance.
@@ -798,11 +813,28 @@ define('io.ox/office/baseframework/view/baseview',
          *      execute this callback function, and will leave the busy mode
          *      afterwards. Will be called in the context of this view
          *      instance.
-         *  @param {Number} [options.delay]
-         *      If set to a non-negative integer value, the busy blocker
-         *      element will be transparent initially, and will fade in after
-         *      the specified delay time in milliseconds. If omitted, the busy
-         *      blocker element will be shown immediately without fading in.
+         *  @param {Number} [options.delay=500]
+         *      The delay time before the busy blocker element becomes visible
+         *      (by fading to a half-transparent fill color). Note that the
+         *      busy blocker element exists and blocks everything right after
+         *      calling this method, the delay time just specifies when it
+         *      becomes visible.
+         *  @param {Boolean} [options.immediate=false]
+         *      If set to true, the half-transparent busy blocker element will
+         *      be shown immediately. Otherwise, the element will be set to
+         *      transparent initially, and will fade in after the specified
+         *      delay (see option 'delay').
+         *  @param {Boolean} [options.showFileName=false]
+         *      If set to true, the file name will be shown in the top-left
+         *      corner of the blocker element.
+         *  @param {String} [options.warningLabel]
+         *      If specified, an alert box containing the passed text will be
+         *      shown after a specific delay (see option 'warningDelay').
+         *      Intended to be used to show a message for long-running actions.
+         *  @param {Number} [options.warningDelay=3000]
+         *      The delay time before the warning alert becomes visible. If the
+         *      blocker element will be faded in delayed by itself, the warning
+         *      delay will start after that delay time.
          *
          * @returns {BaseView}
          *  A reference to this instance.
@@ -819,16 +851,22 @@ define('io.ox/office/baseframework/view/baseview',
                     initHandler = Utils.getFunctionOption(options, 'initHandler'),
                     // the cancel handler
                     cancelHandler = Utils.getFunctionOption(options, 'cancelHandler'),
-                    // the cancel handler
-                    delay = Utils.getIntegerOption(options, 'delay'),
+                    // the delay time before the blocker element becomes visible
+                    delay = Utils.getIntegerOption(options, 'delay', 500, 0),
+                    // the label text for the warning alert for long-running actions
+                    warningLabel = Utils.getStringOption(options, 'warningLabel', ''),
+                    // the delay time before the warning alert box becomes visible
+                    warningDelay = Utils.getIntegerOption(options, 'warningDelay', 3000, 0),
+                    // the DOM element containing the warning alert box, or the single cancel button
+                    warningNode = $('<div>').hide(),
                     // the window blocker element (bound to 'this')
                     blockerNode = this,
                     // the header container node
                     headerNode = blockerNode.find('.header').empty(),
                     // the footer container node
                     footerNode = blockerNode.find('.footer').empty(),
-                    // the container element with the button to cancel the busy mode
-                    cancelNode = null;
+                    // the cancel button element
+                    cancelButton = null;
 
                 // keyboard event handler for busy mode (ESCAPE key)
                 function busyKeydownHandler(event) {
@@ -846,17 +884,38 @@ define('io.ox/office/baseframework/view/baseview',
                     headerNode.append($('<div>').addClass('filename clear-title').text(_.noI18n(app.getFullFileName())));
                 }
 
-                // initialize 'Cancel' button (hide initially, show after a delay)
+                // on IE, remove stripes (they are displayed too nervous and flickering)
+                if (_.browser.IE) {
+                    blockerNode.find('.progress-striped').removeClass('progress-striped');
+                }
+
+                // execute initialization handler
+                if (_.isFunction(initHandler)) {
+                    initHandler.call(self, headerNode, footerNode, blockerNode);
+                }
+
+                // append the container node for the warning alert box and/or cancel button
+                footerNode.append(warningNode);
+
+                // show blocker immediately, or fade it in after a delay
+                if (Utils.getBooleanOption(options, 'immediate', false)) {
+                    blockerNode.css('opacity', '');
+                } else {
+                    blockerNode.css('opacity', 0);
+                    blockerFadeTimer = self.repeatDelayed(function (index) {
+                        blockerNode.css('opacity', (index + 1) / 10);
+                    }, { delay: delay, repeatDelay: 50, cycles: 10 });
+                    // defer showing the warning box until blocker is visible
+                    warningDelay += delay + 500;
+                }
+
+                // initialize a cancel button, if a callback handler has been passed
                 if (_.isFunction(cancelHandler)) {
 
-                    // create and insert the container node for the Cancel button
-                    cancelNode = $('<div>').addClass('cancel-node').hide().appendTo(footerNode);
-
                     // create the Cancel button
-                    $.button({ label: gt('Cancel') })
+                    cancelButton = $.button({ label: gt('Cancel') })
                         .addClass('btn-warning')
-                        .on('click', function () { cancelHandler.call(self); })
-                        .appendTo(cancelNode);
+                        .on('click', function () { cancelHandler.call(self); });
 
                     // register a keyboard handler for the ESCAPE key
                     winBodyNode.on('keydown', busyKeydownHandler);
@@ -866,44 +925,56 @@ define('io.ox/office/baseframework/view/baseview',
 
                     // make the blocker focusable for keyboard input
                     blockerNode.attr('tabindex', 1).focus();
-
-                    // show the Cancel button after a delay
-                    _.delay(function () { cancelNode.show().find('.btn').focus(); }, 5000);
                 }
 
-                // execute initialization handler
-                if (_.isFunction(initHandler)) {
-                    initHandler.call(self, headerNode, footerNode, blockerNode);
+                // create a warning alert box if specified, insert cancel button into the alert box
+                if (warningLabel.length > 0) {
+                    var alertNode = $('<div class="alert alert-warning"><div>' + Utils.escapeHTML(warningLabel) + '</div></div>');
+                    if (cancelButton) { alertNode.append($('<div>').append(cancelButton)); }
+                    warningNode.append(alertNode);
+                } else if (cancelButton) {
+                    warningNode.append(cancelButton);
                 }
 
-                // fade in the blocker if specified
-                if (_.isNumber(delay) && (delay >= 0)) {
-                    blockerNode.css('opacity', 0);
-                    blockerFadeTimer = app.repeatDelayed(function (index) {
-                        blockerNode.css('opacity', (index + 1) / 10);
-                    }, { delay: delay, repeatDelay: 50, cycles: 10 });
-                } else {
-                    blockerNode.css('opacity', '');
+                // show warning alert box after a delay
+                if (warningNode.children().length > 0) {
+                    blockerWarningTimer = self.executeDelayed(function () {
+                        warningNode.show();
+                        if (cancelButton) { cancelButton.focus(); }
+                    }, warningDelay);
                 }
-
             });
 
             return this;
         };
 
         /**
-         * updates the progress-bar of the window blocker.
-         * when invoked repeatedly, it will only update the progress
-         * every 500 milliseconds.
+         * Updates the progress bar of the busy blocker element.
          *
          * @param {Number} progress
-         *  value between 0 and 1
+         *  The progress value (floating-point number between 0 and 1).
+         *
+         * @returns {BaseView}
+         *  A reference to this instance.
          */
-        this.updateBusyProgress = _.throttle(function (progess) {
-            if (win.nodes.blocker.is(':visible')) {
-                win.busy(progess);
+        this.updateBusyProgress = (function () {
+
+            var // the current progress value to be shown at the progress bar
+                currProgress = 0;
+
+            function registerProgress(progress) {
+                currProgress = progress;
+                return self;
             }
-        }, 500);
+
+            function showProgress() {
+                if (win.nodes.blocker.is(':visible')) {
+                    win.busy(currProgress);
+                }
+            }
+
+            return self.createDebouncedMethod(registerProgress, showProgress, { delay: 500, maxDelay: 500 });
+        }());
 
         /**
          * Leaves the busy state of the application. Hides the window blocker
@@ -920,6 +991,12 @@ define('io.ox/office/baseframework/view/baseview',
                 blockerFadeTimer = null;
             }
 
+            // stop timer that wants to show a warning alert in the blocker element
+            if (blockerWarningTimer) {
+                blockerWarningTimer.abort();
+                blockerWarningTimer = null;
+            }
+
             win.idle();
             return this;
         };
@@ -1057,16 +1134,13 @@ define('io.ox/office/baseframework/view/baseview',
             }
         });
 
-        // safely destroy all image nodes, and prevent focusing destroyed application after yelling
-        win.on('quit', function () {
-            app.destroyImageNodes(winBodyNode);
-            app.destroyImageNodes(tempNode);
-            $('.io-ox-alert').off('notification:removed', notificationRemovedHandler);
-        });
-
         // destroy all class members on destruction
         this.registerDestructor(function () {
 
+            // destroy all image nodes (release the browser-internal bitmap data)
+            app.destroyImageNodes(winBodyNode);
+            app.destroyImageNodes(tempNode);
+
             _.invoke(fixedPanes, 'destroy');
             _.invoke(overlayPanes, 'destroy');
 
@@ -1075,7 +1149,7 @@ define('io.ox/office/baseframework/view/baseview',
 
             app = initOptions = self = grabFocusHandler = win = winBodyNode = appPaneNode = null;
             hiddenRootNode = contentRootNode = appContentNode = appBusyNode = tempNode = null;
-            fixedPanes = overlayPanes = blockerFadeTimer = null;
+            fixedPanes = overlayPanes = blockerFadeTimer = blockerWarningTimer = null;
         });
 
     } // class BaseView
