@@ -1,21 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/baseframework/view/component',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/baseframework/model/modelobject'
-    ], function (Utils, Forms, ModelObject) {
+define('io.ox/office/baseframework/view/component', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/baseframework/model/modelobject'
+], function (Utils, Forms, TimerMixin, ModelObject) {
 
     'use strict';
 
@@ -50,6 +51,7 @@ define('io.ox/office/baseframework/view/component',
      * @constructor
      *
      * @extends ModelObject
+     * @extends TimerMixin
      *
      * @param {BaseApplication} app
      *  The application containing this view component instance.
@@ -102,12 +104,37 @@ define('io.ox/office/baseframework/view/component',
             // whether a 'component:layout' event is currently triggering (prevent recursion)
             isLayoutTriggering = false;
 
-        // base constructor ---------------------------------------------------
+        // base constructors --------------------------------------------------
 
         ModelObject.call(this, app);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
+        /**
+         * Removes the passed object from the array in-place.
+         */
+        function removeFromArray(array, object) {
+            var index = _.indexOf(array, object);
+            if (index >= 0) { array.splice(index, 1); }
+        }
+
+        /**
+         * Inserts the passed group into the map under a specific group key.
+         */
+        function addGroupToMap(map, key, group) {
+            (map[key] || (map[key] = [])).push(group);
+        }
+
+        /**
+         * Removes the passed group from the map.
+         */
+        function removeGroupFromMap(map, group) {
+            _.each(map, function (array) {
+                removeFromArray(array, group);
+            });
+        }
+
         /**
          * Updates the effective visibility of the root node, according to the
          * own visible state, and the visibility of all control groups.
@@ -202,7 +229,7 @@ define('io.ox/office/baseframework/view/component',
          */
         var groupResizeHandler = (function () {
             var triggerLayout = false;
-            return app.createDebouncedMethod(function (event) {
+            return self.createDebouncedMethod(function (event) {
                 if (!isLayoutTriggering) { triggerLayout = true; }
                 if (event.type !== 'group:resize') { updateNodeVisibility(); }
             }, function () {
@@ -257,17 +284,16 @@ define('io.ox/office/baseframework/view/component',
          * group will be appended to the root node of this component.
          *
          * @attention
-         *  Intended to be used by the internal implementations of sub class
-         *  constructors. DO NOT CALL from external code!
+         *  Intended to be used by the internal implementations of sub classes.
+         *  DO NOT CALL from external code!
          *
          * @param {Function} inserter
          *  The callback function called for every control group inserted into
          *  this view component by calling the method Component.addGroup().
          *  Receives the reference to the new control group instance as first
-         *  parameter, and the options passed to the methods mentioned above
-         *  second parameter. Will be called in the context of this instance.
-         *  May return false to indicate that the group has not been inserted
-         *  yet.
+         *  parameter, and the options passed to that method as second
+         *  parameter. Will be called in the context of this instance. May
+         *  return false to indicate that the group has not been inserted yet.
          *
          * @returns {Component}
          *  A reference to this instance.
@@ -277,6 +303,27 @@ define('io.ox/office/baseframework/view/component',
             return this;
         };
 
+        /**
+         * Removes a callback function that has been registered with the method
+         * Component.registerGroupInserter(), to be able to register a custom
+         * node inserter temporarily.
+         *
+         * @attention
+         *  Intended to be used by the internal implementations of sub classes.
+         *  DO NOT CALL from external code!
+         *
+         * @param {Function} inserter
+         *  The callback function that has been registered by calling the
+         *  method Component.registerGroupInserter().
+         *
+         * @returns {Component}
+         *  A reference to this instance.
+         */
+        this.unregisterGroupInserter = function (inserter) {
+            removeFromArray(groupInserters, inserter);
+            return this;
+        };
+
         // methods ------------------------------------------------------------
 
         /**
@@ -366,11 +413,11 @@ define('io.ox/office/baseframework/view/component',
          *
          * @param {String|Null} key
          *  The key of a controller item associated to the control group. The
-         *  visible state, the enabled state, and the current value of the
-         *  group will be bound to the states and value of the controller item.
-         *  If the group triggers a 'group:change' event, the controller item
-         *  will be executed. The key can be set to the value null to insert an
-         *  unbound group into this component.
+         *  enabled state and the current value of the group will be bound to
+         *  the state and value of the controller item. If the group triggers a
+         *  'group:change' event, the controller item will be executed. The key
+         *  can be set to the value null to insert an unbound group into this
+         *  component.
          *
          * @param {Group} group
          *  The control group object to be inserted.
@@ -388,7 +435,7 @@ define('io.ox/office/baseframework/view/component',
          *      A DOM node that will receive the browser focus after the
          *      inserted group has been activated, an object that provides a
          *      grabFocus() method, or a callback function that returns a focus
-         *      target dynamically. Default focus target is the view.
+         *      target dynamically. Default focus target is the document view.
          *
          * @returns {Component}
          *  A reference to this view component.
@@ -431,7 +478,7 @@ define('io.ox/office/baseframework/view/component',
             if (_.isString(key)) {
 
                 // store group reference in internal map
-                (groupsByKey[key] || (groupsByKey[key] = [])).push(group);
+                addGroupToMap(groupsByKey, key, group);
 
                 // execute controller item for 'group:change' events
                 this.listenTo(group, 'group:change', function (event, value) {
@@ -453,7 +500,7 @@ define('io.ox/office/baseframework/view/component',
             if (_.isString(visibleKey)) {
 
                 // store group reference in internal map
-                (visibleGroupsByKey[visibleKey] || (visibleGroupsByKey[visibleKey] = [])).push(group);
+                addGroupToMap(visibleGroupsByKey, visibleKey, group);
             }
 
             // update internal layout (component visibility and size)
@@ -461,6 +508,36 @@ define('io.ox/office/baseframework/view/component',
             return this;
         };
 
+        /**
+         * Removes the specified group from this view component, if it has been
+         * registered before with the method Component.addGroup(). The group
+         * will be unregistered and removed, but it will NOT be destroyed.
+         * Caller MUST take ownership of the passed group.
+         *
+         * @param {Group} group
+         *  The control group object to be removed.
+         *
+         * @returns {Component}
+         *  A reference to this instance.
+         */
+        this.removeGroup = function (group) {
+
+            // unregister all event handlers for the group
+            this.stopListeningTo(group);
+
+            // remove group from internal containers
+            removeFromArray(groups, group);
+            removeGroupFromMap(groupsByKey, group);
+            removeGroupFromMap(visibleGroupsByKey, group);
+
+            // remove group root node from DOM
+            group.getNode().detach();
+
+            // update internal layout (component visibility and size)
+            updateNode();
+            return this;
+        };
+
         /**
          * Visits all control groups registered at this view component.
          *
@@ -486,15 +563,13 @@ define('io.ox/office/baseframework/view/component',
         };
 
         /**
-         * Returns whether this view component has groups or not
+         * Returns whether this view component contains visible groups.
+         *
+         * @returns {Boolean}
+         *  Whether this view component contains visible groups.
          */
         this.hasVisibleGroups = function () {
-            var visible = 0;
-
-            this.iterateGroups(function(group){
-                if (group.isVisible()) { visible++; }
-            });
-            return (visible > 0)?true:false;
+            return _.any(groups, function (group) { return group.isVisible(); });
         };
 
         /**
@@ -527,6 +602,11 @@ define('io.ox/office/baseframework/view/component',
 
         // initialization -----------------------------------------------------
 
+        // update all registered groups after change events of the application controller
+        app.onInit(function () {
+            self.listenTo(app.getController(), 'change:items', controllerChangeHandler);
+        });
+
         // initially hidden until the first visible group will be inserted
         Forms.showNodes(rootNode, false);
 
@@ -539,9 +619,6 @@ define('io.ox/office/baseframework/view/component',
         // hover effect for groups embedded in the view component
         rootNode.toggleClass('hover-effect', Utils.getBooleanOption(initOptions, 'hoverEffect', false));
 
-        // update all registered groups after change events of the application controller
-        this.listenTo(app.getController(), 'change:items', controllerChangeHandler);
-
         // log focus and layout events
         Utils.FocusLogger.logEvent(this, 'component:focus component:blur');
         Utils.LayoutLogger.logEvent(this, 'component:beforeshow component:show component:beforehide component:hide component:layout');
