@@ -1,23 +1,24 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/baseframework/view/pane',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/config',
-     'io.ox/office/tk/object/triggerobject'
-    ], function (Utils, KeyCodes, Forms, Config, TriggerObject) {
+define('io.ox/office/baseframework/view/pane', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/config',
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/tk/object/timermixin'
+], function (Utils, KeyCodes, Forms, Config, TriggerObject, TimerMixin) {
 
     'use strict';
 
@@ -53,6 +54,7 @@ define('io.ox/office/baseframework/view/pane',
      * @constructor
      *
      * @extends TriggerObject
+     * @extends TimerMixin
      *
      * @param {BaseApplication} app
      *  The application containing this pane element.
@@ -143,9 +145,10 @@ define('io.ox/office/baseframework/view/pane',
             // whether a 'pane:layout' event is currently triggering (prevent recursion)
             isLayoutTriggering = false;
 
-        // base constructor ---------------------------------------------------
+        // base constructors --------------------------------------------------
 
-        TriggerObject.call(this, app);
+        TriggerObject.call(this);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
@@ -240,7 +243,7 @@ define('io.ox/office/baseframework/view/pane',
          */
         var componentResizeHandler = (function () {
             var triggerLayout = false;
-            return app.createDebouncedMethod(function () {
+            return self.createDebouncedMethod(function () {
                 if (!isLayoutTriggering) { triggerLayout = true; }
             }, function () {
                 var options = { triggerLayout: triggerLayout };
@@ -573,8 +576,18 @@ define('io.ox/office/baseframework/view/pane',
             return this;
         };
 
+        /**
+         * Checks whether the component already exists
+         *
+         * @param {Component} component
+         *  The view component to be removed from this pane. Nothing will
+         *  happen, if the component has not been added to this view pane
+         *  before using the method Pane.addViewComponent().
+         *
+         * @return {Boolean}
+         *  Whether the component exists.
+         */
         this.hasViewComponent = function (component) {
-            // check wether the component exists already
             return _.contains(components, component);
         };
 
