@@ -1,21 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/drawinglayer/model/drawingcollection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/object/triggerobject',
-     'io.ox/office/drawinglayer/model/drawingmodel'
-    ], function (Utils, TriggerObject, DrawingModel) {
+define('io.ox/office/drawinglayer/model/drawingcollection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/drawinglayer/model/drawingmodel'
+], function (Utils, TriggerObject, TimerMixin, DrawingModel) {
 
     'use strict';
 
@@ -27,12 +28,12 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
      * Triggers the following events:
      * - 'insert:drawing'
      *      After a new drawing model has been inserted into this collection.
-     *      Event handlers receive the new drawing model, and the logical
-     *      position of the drawing object.
+     *      Event handlers receive the new drawing model, and the position of
+     *      the drawing object.
      * - 'delete:drawing'
      *      After a drawing model has been removed from this collection. Event
-     *      handlers receive the removed drawing model, and the last logical
-     *      position of the drawing object.
+     *      handlers receive the removed drawing model, and the last position
+     *      of the drawing object.
      * - 'change:drawing'
      *      After some contents of a drawing model in this collection have been
      *      changed (dependent on the type of the drawing object), including
@@ -42,7 +43,8 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
      *          The JQuery event object.
      *      (2) {DrawingModel} drawingModel
      *          The drawing model that caused the change event.
-     *      (3) {Number[]} The current logical position of the drawing object.
+     *      (3) {Number[]} position
+     *          The current position of the drawing object.
      *      (4) {String} type
      *          The exact type of the change event. The possible values are
      *          dependent on the type of the drawing object. Will be set to
@@ -57,6 +59,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
      * @constructor
      *
      * @extends TriggerObject
+     * @extends TimerMixin
      *
      * @param {EditApplication} app
      *  The application instance containing this drawing collection.
@@ -71,19 +74,19 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
      *      drawing model according to the passed type. May return null for
      *      unsupported drawing types.
      *  @param {Function} initOptions.clientResolveModelHandler
-     *      A function that receives an arbitrary logical document position,
-     *      and must return the model of a drawing object at that position, if
+     *      A function that receives an arbitrary document position, and must
+     *      return the model of a drawing object at that position, if
      *      available; otherwise null.
      *  @param {Function} initOptions.clientResolvePositionHandler
      *      A function that receives an arbitrary drawing model, and must
-     *      return its logical document position, if available; otherwise null.
+     *      return its document position, if available; otherwise null.
      *  @param {Function} initOptions.clientRegisterModelHandler
      *      A function that receives a new drawing model that has just been
-     *      inserted into this collection, and the logical document position of
-     *      the new drawing model. The function has to perform client-side
-     *      registration of the drawing model according to the document
-     *      position, and returns whether registration of the drawing model at
-     *      the passed logical position was successful.
+     *      inserted into this collection, and the document position of the new
+     *      drawing model. The function has to perform client-side registration
+     *      of the drawing model according to the document position, and
+     *      returns whether registration of the drawing model at the passed
+     *      position was successful.
      *  @param {Function} [initOptions.clientSetAttributesHandler]
      *      A function that will be called every time the formatting attributes
      *      of a drawing model contained in this collection will be changed by
@@ -98,10 +101,10 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
             // the client factory function for drawing model instances
             clientCreateModelHandler = Utils.getFunctionOption(initOptions, 'clientCreateModelHandler', $.noop),
 
-            // the client resolves logical document positions to models
+            // the client resolves document positions to models
             clientResolveModelHandler = Utils.getFunctionOption(initOptions, 'clientResolveModelHandler', $.noop),
 
-            // the client resolves models to logical document positions
+            // the client resolves models to document positions
             clientResolvePositionHandler = Utils.getFunctionOption(initOptions, 'clientResolvePositionHandler', $.noop),
 
             // the client has to register new drawing models
@@ -116,15 +119,19 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
             // cached number of drawing models in this collection (faster than counting map properties)
             modelCount = 0;
 
-        // base constructor ---------------------------------------------------
+        // base constructors --------------------------------------------------
 
         TriggerObject.call(this);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
         /**
-         * Returns the drawing model at the specified logical document position
+         * Returns the drawing model at the specified document position
          * contained in this drawing collection.
+         *
+         * @param {Number[]} position
+         *  The document position of the drawing model.
          */
         function getModel(position) {
 
@@ -145,7 +152,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
          * inside this collection.
          *
          * @param {Number[]} position
-         *  The logical document position of the target drawing model.
+         *  The document position of the drawing model.
          *
          * @returns {Object|Null}
          *  A result object containing the following properties:
@@ -153,7 +160,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
          *      The drawing collection of a drawing object inside this
          *      collection.
          *  - {Number[]} position
-         *      The remaining logical document position relative to the drawing
+         *      The remaining document position relative to the drawing
          *      collection.
          * If no drawing object with an own embedded collection has been found,
          * the value null will be returned.
@@ -190,12 +197,11 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
         };
 
         /**
-         * Tries to find the drawing model at the passed logical document
-         * position.
+         * Tries to find the drawing model at the passed document position.
          *
          * @param {Number[]} position
-         *  The logical document position of the drawing model. May specify a
-         *  position inside another drawing object.
+         *  The document position of the drawing model. May specify a position
+         *  inside another drawing object.
          *
          * @param {Object} [options]
          *  Optional parameters:
@@ -208,8 +214,8 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
          *      document position, this method returns null.
          *
          * @returns {DrawingModel|Null}
-         *  The drawing model at the specified logical document position; or
-         *  null, if no drawing model has been found.
+         *  The drawing model at the specified document position; or null, if
+         *  no drawing model has been found.
          */
         this.findModel = function (position, options) {
 
@@ -235,8 +241,7 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
         };
 
         /**
-         * Returns the logical document position of the specified drawing
-         * model.
+         * Returns the document position of the specified drawing model.
          *
          * @param {DrawingModel} model
          *  The drawing model whose position will be obtained.
@@ -245,19 +250,19 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
          *  Optional parameters:
          *  @param {Boolean} [options.deep=false]
          *      If set to true, also supports drawing objects embedded in other
-         *      drawing objects, and resolves the complete logical position of
-         *      the embedded drawing object. Otherwise, the passed drawing
-         *      object must be contained directly in this collection.
+         *      drawing objects, and resolves the complete position of the
+         *      embedded drawing object. Otherwise, the passed drawing object
+         *      must be contained directly in this collection.
          *
          * @returns {Number[]|Null}
-         *  The logical position of the drawing object, if it is contained in
-         *  this collection; otherwise null.
+         *  The position of the drawing object, if it is contained in this
+         *  collection; otherwise null.
          */
         this.getModelPosition = function (model/*, options*/) {
 
             var // whether to support embedded drawing models
                 /*deep = Utils.getBooleanOption(options, 'deep', false),*/
-                // logical position of the drawing model
+                // position of the drawing model
                 position = null;
 
             // TODO: support for deep option
@@ -301,8 +306,8 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
          * collection.
          *
          * @param {Number[]} position
-         *  The logical document position of the new drawing model. May specify
-         *  a position inside another drawing object (if the array without the
+         *  The document position of the new drawing model. May specify a
+         *  position inside another drawing object (if the array without the
          *  last element points to an existing drawing object). If that drawing
          *  model supports embedding drawing objects, the new drawing model
          *  will be inserted into the drawing collection of that drawing object
@@ -359,8 +364,8 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
          * passed settings.
          *
          * @param {Number[]} position
-         *  The logical document position of the new drawing model. May specify
-         *  a position inside another drawing object (if the array without the
+         *  The document position of the new drawing model. May specify a
+         *  position inside another drawing object (if the array without the
          *  last element points to an existing drawing object). If that drawing
          *  model supports embedding drawing objects, the new drawing model
          *  will be inserted into the drawing collection of that drawing object
@@ -394,13 +399,13 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
         };
 
         /**
-         * Removes the model of a drawing object located at the passed logical
-         * document position.
+         * Removes the model of a drawing object located at the passed document
+         * position.
          *
          * @param {Number[]} position
-         *  The logical document position of the drawing model. May specify a
-         *  position inside another drawing object (if the array without the
-         *  last element points to an existing drawing object).
+         *  The document position of the drawing model. May specify a position
+         *  inside another drawing object (if the array without the last
+         *  element points to an existing drawing object).
          *
          * @returns {Boolean}
          *  Whether the drawing model has been removed successfully.
@@ -408,16 +413,16 @@ define('io.ox/office/drawinglayer/model/drawingcollection',
         this.deleteModel = function (position) {
 
             var // embedded collection addressed by the passed position
-                embedInfo = null,
+                embedInfo = findEmbeddedCollection(position),
                 // the drawing model
                 model = null;
 
             // existing drawing object with embedded collection found: create new drawing model there
-            if ((embedInfo = findEmbeddedCollection(position))) {
+            if (embedInfo) {
                 return embedInfo.collection.deleteModel(embedInfo.position);
             }
 
-            // find the drawing model at the specified logical position
+            // find the drawing model at the specified position
             if (!(model = this.findModel(position))) { return false; }
 
             // notify all listeners
