@@ -1,20 +1,21 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/drawinglayer/model/drawingmodel',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/model/attributedmodel'
-    ], function (Utils, AttributedModel) {
+define('io.ox/office/drawinglayer/model/drawingmodel', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/editframework/model/attributedmodel'
+], function (Utils, TimerMixin, AttributedModel) {
 
     'use strict';
 
@@ -26,6 +27,7 @@ define('io.ox/office/drawinglayer/model/drawingmodel',
      * @constructor
      *
      * @extends AttributedModel
+     * @extends TimerMixin
      *
      * @param {EditApplication} app
      *  The application instance containing this drawing object.
@@ -40,20 +42,24 @@ define('io.ox/office/drawinglayer/model/drawingmodel',
      * @param {Object} [initOptions]
      *  A map with additional options for the drawing object. The following
      *  options are supported:
-     *  @param {String|Array} [options.additionalFamilies]
+     *  @param {String} [options.families]
      *      Additional explicit attribute families supported by this drawing
-     *      model object.
+     *      model object (space-separated). The main attribute family 'drawing'
+     *      supported by all drawing objects will be registered implicitly, and
+     *      does not have to be added here.
      */
     function DrawingModel(app, type, initAttributes, initOptions) {
 
-        // base constructor ---------------------------------------------------
+        // base constructors --------------------------------------------------
 
         AttributedModel.call(this, app, initAttributes, {
             styleFamily: 'drawing',
-            additionalFamilies: Utils.getOption(initOptions, 'additionalFamilies')
+            families: Utils.getStringOption(initOptions, 'families')
         });
 
-        // methods ------------------------------------------------------------
+        TimerMixin.call(this);
+
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the type of this drawing model, as specified by the
@@ -72,9 +78,9 @@ define('io.ox/office/drawinglayer/model/drawingmodel',
 
         // initialization -----------------------------------------------------
 
-        // constructor called from BaseObject.clone()
-        this.registerCloneConstructor(function () {
-            return new DrawingModel(app, type, this.getExplicitAttributes(), initOptions);
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            app = initAttributes = initOptions = null;
         });
 
     } // class DrawingModel
