@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
@@ -20,10 +20,9 @@ define('io.ox/office/drawinglayer/model/drawingstylecollection',
 
     'use strict';
 
-    var // definitions for common drawing attributes
-        DRAWING_ATTRIBUTE_DEFINITIONS = {
+    // definitions for common drawing attributes ------------------------------
 
-            // generic attributes ---------------------------------------------
+    var DRAWING_ATTRIBUTE_DEFINITIONS = {
 
             /**
              * Absolute horizontal position of the drawing, as number in 1/100
@@ -75,11 +74,67 @@ define('io.ox/office/drawinglayer/model/drawingstylecollection',
 
             /**
              * Replacement image for unsupported drawing objects. The string
-             * contains either Base64 encoded bitmap data, or SVG mark-up.
+             * contains either BASE64 encoded bitmap data, or SVG mark-up.
              */
-            replacementData: { def: '', scope: 'element' },
+            replacementData: { def: '', scope: 'element' }
+        };
 
-            // image specific attributes --------------------------------------
+    // definitions for drawing fill attributes --------------------------------
+
+    var FILL_ATTRIBUTE_DEFINITIONS = {
+
+            /**
+             *  Type of the fill. Supported values are:
+             *  - 'none': Not filled (transparent).
+             *  - 'auto': specific default fill, dependent on context.
+             *  - 'solid': single fill color ('color' attribute).
+             */
+            type: { def: 'none' },
+
+            /**
+             * Fill color (used for fill type 'solid').
+             */
+            color: { def: Color.AUTO }
+        };
+
+    // definitions for drawing outline attributes -----------------------------
+
+    var // supports ALL fill attributes
+        LINE_ATTRIBUTE_DEFINITIONS = _.extend({}, FILL_ATTRIBUTE_DEFINITIONS, {
+
+            /**
+             * The line dash style. Supported values are:
+             * - 'solid': No line dash pattern.
+             * - 'dashed': Repeating dashes.
+             * - 'dotted': Repeating dots.
+             * - 'dashDot': Repeating dash-dot pattern.
+             * - 'dashDotDot': Repeating dash-dot-dot pattern.
+             */
+            style: { def: 'solid' },
+
+            /**
+             * Width of the outline, in 1/100 of millimeters.
+             */
+            width: { def: 1 }
+        });
+
+    // definitions for linked text attributes ---------------------------------
+
+    var TEXT_ATTRIBUTE_DEFINITIONS = {
+
+            /**
+             * A specification for the source of the text contents of a drawing
+             * object, or of a sub object in a complex drawing (e.g. a chart
+             * title). Can be a plain array with literal strings, or a string
+             * with an application-dependent descriptor of the text source
+             * (e.g. a spreadsheet formula with cell addresses).
+             */
+            link: { def: [] }
+        };
+
+    // definitions for common image attributes --------------------------------
+
+    var IMAGE_ATTRIBUTE_DEFINITIONS = {
 
             /**
              * URL pointing to the image data. If the image is embedded in the
@@ -115,49 +170,42 @@ define('io.ox/office/drawinglayer/model/drawingstylecollection',
              * border, in percent.
              */
             cropBottom: { def: 0 }
-        },
+        };
 
-        // definitions for drawing fill attributes
-        FILL_ATTRIBUTE_DEFINITIONS = {
+    // definitions for common image attributes --------------------------------
+
+    var SHAPE_ATTRIBUTE_DEFINITIONS = {
 
             /**
-             *  Type of the fill. Supported values are:
-             *  - 'none': Not filled (transparent).
-             *  - 'auto': specific default fill, dependent on context.
-             *  - 'solid': single fill color ('color' attribute).
+             * Distance between left border and text, as number in 1/100 of millimeters.
              */
-            type: { def: 'none' },
+            paddingLeft: { def: 250 },
 
             /**
-             * Fill color (used for fill type 'solid').
+             * Distance between right border and text, as number in 1/100 of millimeters.
              */
-            color: { def: Color.AUTO }
-        },
-
-        // definitions for drawing outline attributes (supports ALL fill attributes)
-        LINE_ATTRIBUTE_DEFINITIONS = _.extend(_.clone(FILL_ATTRIBUTE_DEFINITIONS), {
+            paddingRight: { def: 250 },
 
             /**
-             * Width of the outline, in 1/100 of millimeters.
+             * Distance between top border and text, as number in 1/100 of millimeters.
              */
-            width: { def: 0 }
-        }),
-
-        // definitions for linked text attributes
-        TEXT_ATTRIBUTE_DEFINITIONS = {
+            paddingTop: { def: 125 },
 
             /**
-             * A specification for the source of the text contents of a drawing
-             * object, or of a sub object in a complex drawing (e.g. a chart
-             * title). Can be a plain array with literal strings, or a string
-             * with an application-dependent descriptor of the text source
-             * (e.g. a spreadsheet formula with cell addresses).
+             * Distance between bottom border and text, as number in 1/100 of millimeters.
              */
-            link: { def: [] }
-        },
+            paddingBottom: { def: 125 },
 
-        // definitions for generic chart attributes
-        CHART_ATTRIBUTE_DEFINITIONS = {
+            /**
+             * Whether the size of the shape shall be adapted to the text content.
+             */
+            autoResizeHeight: { def: false }
+
+        };
+
+    // definitions for common chart attributes --------------------------------
+
+    var CHART_ATTRIBUTE_DEFINITIONS = {
 
             /**
              * The type of the chart. Specifies how the data series will be
@@ -188,10 +236,11 @@ define('io.ox/office/drawinglayer/model/drawingstylecollection',
             varyColors: { def: false, scope: 'element' },
 
             dataLabel: { def: false, scope: 'element' }
-        },
+        };
 
-        // definitions for chart data series attributes
-        CHART_SERIES_ATTRIBUTE_DEFINITIONS = {
+    // definitions for chart data series attributes ---------------------------
+
+    var CHART_SERIES_ATTRIBUTE_DEFINITIONS = {
 
             /**
              * Data source for the series title. May be an array of constant
@@ -221,18 +270,20 @@ define('io.ox/office/drawinglayer/model/drawingstylecollection',
              * application-dependent dynamic data source definition.
              */
             bubbles: { def: [] }
-        },
+        };
 
-        // definitions for chart axis attributes
-        CHART_AXIS_ATTRIBUTE_DEFINITIONS = {
+    // definitions for chart axis attributes ----------------------------------
+
+    var CHART_AXIS_ATTRIBUTE_DEFINITIONS = {
             type: { def: 'value' },
             min: { def: 'auto' },
             max: { def: 'auto' },
             label: { def: false }
-        },
+        };
 
-        // definitions for chart legend attributes
-        CHART_LEGEND_ATTRIBUTE_DEFINITIONS = {
+    // definitions for chart legend attributes --------------------------------
+
+    var CHART_LEGEND_ATTRIBUTE_DEFINITIONS = {
 
             /**
              * Position of the legend object. The value 'off' hides the legend,
@@ -271,6 +322,8 @@ define('io.ox/office/drawinglayer/model/drawingstylecollection',
             .registerAttributeDefinitions('fill', FILL_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('line', LINE_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('text', TEXT_ATTRIBUTE_DEFINITIONS)
+            .registerAttributeDefinitions('image', IMAGE_ATTRIBUTE_DEFINITIONS)
+            .registerAttributeDefinitions('shape', SHAPE_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('chart', CHART_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('series', CHART_SERIES_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('axis', CHART_AXIS_ATTRIBUTE_DEFINITIONS)
