@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
@@ -154,7 +154,7 @@ define('io.ox/office/drawinglayer/view/chartstyleutil',
 
         target[usePrefix + 'Color'] = styles.getCssColor(character.color, getCorrectLineColor(chart, styles));
         target[usePrefix + 'Family'] = styles.getCssFontFamily(character.fontName);
-        target[usePrefix + 'Size'] = Utils.convertLength(view.getEffectiveZoom() * character.fontSize, 'pt', 'px', 0.01);
+        target[usePrefix + 'Size'] = view.getEffectiveZoom() * Utils.convertLength(character.fontSize, 'pt', 'px');
         target[usePrefix + 'Weight'] = character.bold ? 'bold' : 'normal';
         target[usePrefix + 'Style'] = character.italic ? 'italic' : 'normal';
     };
@@ -171,16 +171,15 @@ define('io.ox/office/drawinglayer/view/chartstyleutil',
         var app = chart.getApp();
         var view = app.getView();
         var styles = app.getModel().getDocumentStyles();
-        var w = line.width || 1;
+        var w = line.width ? line.width*(72/2540) : 1;
         var usePrefix = prefix;
 
         if (line.type === 'none') {
             target[usePrefix + 'Color'] = 'transparent';
         } else {
             target[usePrefix + 'Color'] = styles.getCssColor(line.color, getCorrectLineColor(chart, styles));
-            target[usePrefix + 'Thickness'] = view.getEffectiveZoom() * w;
+            target[usePrefix + 'Thickness'] = Math.max(1, Math.round(view.getEffectiveZoom() * w));
         }
-
     };
 
     ChartStyleUtil.getNumberFormat = function (app, value) {
