@@ -1,30 +1,30 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
-define('io.ox/office/drawinglayer/view/imageutil',
-    ['io.ox/core/config',
-     'io.ox/core/tk/folderviews',
-     'io.ox/core/folder/api',
-     'io.ox/core/cache',
-     'io.ox/core/tk/selection',
-     'io.ox/files/api',
-     'io.ox/office/tk/utils',
-     'io.ox/office/tk/io',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/editframework/view/editdialogs',
-     'settings!io.ox/office',
-     'gettext!io.ox/office/main'
-    ], function (Config, FolderViews, FolderAPI, CoreCache, CoreSelection, FilesAPI, Utils, IO, KeyCodes, EditDialogs, Settings, gt) {
+define('io.ox/office/drawinglayer/view/imageutil', [
+    'io.ox/core/config',
+    'io.ox/core/tk/folderviews',
+    'io.ox/core/folder/api',
+    'io.ox/core/cache',
+    'io.ox/core/tk/selection',
+    'io.ox/files/api',
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/io',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/editframework/view/editdialogs',
+    'settings!io.ox/office',
+    'gettext!io.ox/office/main'
+], function (Config, FolderViews, FolderAPI, CoreCache, CoreSelection, FilesAPI, Utils, IO, KeyCodes, EditDialogs, Settings, gt) {
 
     'use strict';
 
@@ -72,15 +72,15 @@ define('io.ox/office/drawinglayer/view/imageutil',
             png:  'image/png',
             gif:  'image/gif',
             bmp:  'image/bmp'
-        };
+        },
+
+        RE_IMAGE_URL = /^(?:([a-z]+):)?(\/{0,3})([0-9.\-a-z]+)(?::(\d+))?(?:\/([^?#]*))?(?:\?([^#]*))?(?:#(.*))?$/i;
 
     // private static functions ===============================================
 
     function toHex32(n) {
-        if (n < 0) {
-            n = 0xFFFFFFFF + n + 1;
-        }
-        return ('00000000' + n.toString(16).toUpperCase()).substr(-8);
+        if (n < 0) { n += 0x100000000; }
+        return ('0000000' + n.toString(16).toUpperCase()).substr(-8);
     }
 
     function crc32(n, iCRC) {
@@ -169,19 +169,16 @@ define('io.ox/office/drawinglayer/view/imageutil',
             crc32 = toHex32(crc32),
             extension = file.name.substring(file.name.lastIndexOf('.') + 1);
 
-//            Utils.error('sendFilterRequest', crc32, file.name, extension, dataUrl);
-
-            return app.sendFilterRequest({
-                method: 'POST',
-                params: {
+            var params = {
                     action: 'addfile',
                     add_crc32: crc32,
                     add_ext: extension,
                     alt_filename: file.name,
                     add_filedata: dataUrl,
                     requestdata: 'hallo'
-                }
-            });
+                };
+
+            return app.sendFileRequest(IO.FILTER_MODULE_NAME, params, { method: 'POST' });
         });
     }
 
@@ -230,8 +227,7 @@ define('io.ox/office/drawinglayer/view/imageutil',
         };
 
 
-        var action = app.getFilterModuleUrl({ action: 'addfile', add_ext: extension, alt_filename: file.name });
-        Utils.info('ImageUtils.getFilterModuleUrl():', extension, file.name, action);
+        var action = app.getServerModuleUrl(IO.FILTER_MODULE_NAME, { action: 'addfile', add_ext: extension, alt_filename: file.name });
 
         form.append(
                 $('<input type="hidden" name="' + options.field + '">').val(data)
@@ -264,16 +260,15 @@ define('io.ox/office/drawinglayer/view/imageutil',
 
         if (!fileDescriptor) { return $.Deferred().reject().promise(); }
 
-        return app.sendFilterRequest({
-            method: 'POST',
-            params: {
+        var params = {
                 action: 'addfile',
                 add_ext: fileDescriptor.filename.substring(fileDescriptor.filename.lastIndexOf('.') + 1),
                 add_fileid: fileDescriptor.id,
                 add_folderid: fileDescriptor.folder_id,
                 alt_filename: fileDescriptor.filename
-            }
-        });
+            };
+
+        return app.sendFileRequest(IO.FILTER_MODULE_NAME, params, { method: 'POST' });
     }
 
     // static class Image =====================================================
@@ -356,7 +351,7 @@ define('io.ox/office/drawinglayer/view/imageutil',
         if (url.substring(0, 10) === 'data:image' || (/:\/\//.test(url))) { return url; }
 
         // document URL
-        return app.getFilterModuleUrl({ action: 'getfile', get_filename: url });
+        return app.getServerModuleUrl(IO.FILTER_MODULE_NAME, { action: 'getfile', get_filename: url });
     };
 
     /**
@@ -375,49 +370,47 @@ define('io.ox/office/drawinglayer/view/imageutil',
     };
 
     /**
-     * Post processes the attributes of insert drawing operations.
-     * Based on the drawing attributes and the editor instance the operation is applied to:
-     *      - an image URL to an external location is always used.
-     *      - the image media URL is used if the operation is applied within the same editor instance.
-     *      - the base64 image data is used if the operation is applied in another editor instance.
-     * Finally the attributes will contain either the imageUrl or the imageData attribute.
+     * Post-processes the attributes of insert drawing operations. Based on the
+     * drawing attributes and the editor instance the operation is applied to:
+     *  - An image URL to an external location is always used.
+     *  - The image media URL is used if the operation is applied within the
+     *      same editor instance.
+     *  - The BASE64 image data is used if the operation is applied in another
+     *      editor instance.
+     * Finally, the attributes will contain either the imageUrl or the
+     * imageData attribute.
      *
-     * @param drawingAttrs
-     *  The drawing attrs of the operation
+     * @param {Object} drawingAttrs
+     *  The (incomplete) drawing attributes of the operation.
      *
-     * @param fileId
-     *  The id of the file where the operation is applied to
+     * @param {String} fileId
+     *  The identifier of the file where the operation is applied to.
      */
-    Image.postProcessOperationAttributes = function (drawingAttrs, fileId) {
+    Image.postProcessOperationAttributes = function (imageAttrs, fileId) {
 
-        if (drawingAttrs.fileId === fileId &&
-            drawingAttrs.sessionId === ox.session &&
-            _.isString(drawingAttrs.imageUrl)) {
+        if ((imageAttrs.fileId === fileId) && (imageAttrs.sessionId === ox.session) && _.isString(imageAttrs.imageUrl)) {
 
-            // we are targeting the same editor instance
-            // so we use the image URL and remove the image base64 data.
-            delete drawingAttrs.imageData;
+            // target is the same editor instance: use the image URL and remove the image BASE64 data
+            delete imageAttrs.imageData;
 
-        } else if (_.isString(drawingAttrs.imageUrl) && /:\/\//.test(drawingAttrs.imageUrl)) {
+        } else if (_.isString(imageAttrs.imageUrl) && /:\/\//.test(imageAttrs.imageUrl)) {
 
-            // we aren't targeting the same editor instance but have an external image URL
-            // that we can use and remove the image base64 data.
-            delete drawingAttrs.imageData;
+            // target is another editor instance but an external image URL is used: remove the image BASE64 data
+            delete imageAttrs.imageData;
 
-        } else if (_.isString(drawingAttrs.imageData) && drawingAttrs.imageData.substring(0, 10) === 'data:image') {
+        } else if (_.isString(imageAttrs.imageData) && /^data:image/.test(imageAttrs.imageData)) {
 
-            // we aren't targeting the same editor instance and have base64 data
-            // so we remove the image URL.
-            delete drawingAttrs.imageUrl;
+            // target is another editor instance, BASE64 data is available: remove the image URL
+            delete imageAttrs.imageUrl;
 
         } else {
 
-            // remove invalid image data attribute.
-            delete drawingAttrs.imageData;
+            // remove invalid image data attribute
+            delete imageAttrs.imageData;
         }
 
-        delete drawingAttrs.fileId;
-        delete drawingAttrs.sessionId;
+        delete imageAttrs.fileId;
+        delete imageAttrs.sessionId;
     };
 
     /**
@@ -442,7 +435,7 @@ define('io.ox/office/drawinglayer/view/imageutil',
         .then(function (data) {
             return {
                 url: Utils.getStringOption(data, 'added_filename', ''),
-                name : Image.getFileNameWithoutExt(file.name)
+                name: Image.getFileNameWithoutExt(file.name)
             };
         }, function () {
             app.getView().yell({ type: 'warning', message: gt('Image could not be loaded.') });
@@ -466,7 +459,7 @@ define('io.ox/office/drawinglayer/view/imageutil',
         return insertDriveImage(app, file).then(function (data) {
             return {
                 url: Utils.getStringOption(data, 'added_filename', ''),
-                name : Image.getFileNameWithoutExt(file.filename)
+                name: Image.getFileNameWithoutExt(file.filename)
             };
         });
     };
@@ -475,41 +468,17 @@ define('io.ox/office/drawinglayer/view/imageutil',
     /**
      * Inserts the image specified by a URL into a document.
      *
-     * @param {ox.ui.App} app
-     *  The application object representing the edited document.
-     *
      * @param {String} url
      *  The full URL of the image to be inserted.
      */
-    Image.insertURL = function (app, url) {
-        return Image.checkURL(url).then(function () {
-            return {
-                url: url,
-                name : Image.getFileNameWithoutExt(url)
-            };
-        });
-    };
-
-    /**
-     * Checks if the URL is valid
-     *
-     * @param {String} url
-     *  The full URL of the image to be checked.
-     *
-     * @returns {jQuery.Promise}
-     *  The Promise of a Deferred object that will be resolved if the URL is valid
-     *  and otherwise will be rejected.
-     */
-    Image.checkURL = function (url) {
-
-        var def = $.Deferred(),
-            urlRegEx = /^(?:([A-Za-z]+):)?(\/{0,3})([0-9.\-A-Za-z]+)(?::(\d+))?(?:\/([^?#]*))?(?:\?([^#]*))?(?:#(.*))?$/;
-
-        if (_.isString(url) && urlRegEx.test(url)) {
-            return def.resolve();
+    Image.insertURL = function (url) {
+        var def = $.Deferred();
+        if (_.isString(url) && RE_IMAGE_URL.test(url)) {
+            def.resolve({ url: url, name: Image.getFileNameWithoutExt(url) });
+        } else {
+            def.reject();
         }
-
-        return def.reject();
+        return def;
     };
 
     /**
@@ -805,10 +774,13 @@ define('io.ox/office/drawinglayer/view/imageutil',
         dialog.on('tab:show', function (event, tabId, tabPanel, tabButton) {
             switch (tabId) {
             case 'file':
-                // in most browsers, <input>s of type 'file' are not focusable,
-                // therefore fall-back to the tab button
-                tabButton.focus();
-                filePicker.focus();
+                // on Firefox fall-back to the tab button until bug is fixed:
+                // bug #505355 - input type=file cannot accept focus using javascript - https://bugzilla.mozilla.org/show_bug.cgi?id=505355
+                if (_.browser.Firefox) {
+                    tabButton.focus();
+                } else {
+                    fileInput.focus();
+                }
                 break;
             case 'url':
                 urlInput.focus();
@@ -890,7 +862,7 @@ define('io.ox/office/drawinglayer/view/imageutil',
 
         promise = promise.then(function (file, isDriveFile, isFileUpload, form) {
             if (isDriveFile) { return Image.insertDriveImage(app, file); }
-            return isFileUpload ? Image.insertFile(app, file, form) : Image.insertURL(app, file.trim());
+            return isFileUpload ? Image.insertFile(app, file, form) : Image.insertURL(file.trim());
         });
 
         return promise.always(function () {
