@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -104,13 +104,18 @@ define('io.ox/office/editframework/app/editcontroller',
                     parent: 'app/valid',
                     enable: function () { return model.getEditMode(); }
                 },
+                
+                'document/renameenabled': {
+                    parent: 'document/editable',
+                    enable: function () { return !Config.RENAME_DISABLED; }
+                },
 
                 'document/fullname': {
                     get: function () { return app.getFullFileName(); }
                 },
 
                 'document/rename': {
-                    parent: 'document/editable',
+                    parent: ['document/editable', 'app/bundled', 'document/renameenabled'],
                     get: function () { return app.getShortFileName(); },
                     set: function (fileName) { return app.rename(fileName); }
                 },
@@ -157,12 +162,7 @@ define('io.ox/office/editframework/app/editcontroller',
 
                 'document/users': {
                     parent: 'app/valid',
-                    enable : function () {
-                        var activeUsers = _.reject(app.getActiveUsers(), function (user) {
-                            return user.durationOfInactivity >= 60;
-                        });
-                        return activeUsers.length > 1;
-                    },
+                    enable : function () { return app.getActiveClients().length > 1; },
                     get : function () { return view.getCollaboratorPopup().isVisible(); },
                     set : function (state) { view.toggleCollaboratorPopup(state); }
                 },
@@ -192,9 +192,9 @@ define('io.ox/office/editframework/app/editcontroller',
 
         // update controller after operations, or changed state of edit mode
         app.onImport(function () {
-            app.on('docs:state', function () { self.update(); });
-            model.on('operations:after change:editmode', function () { self.update(); });
-            model.getUndoManager().on('change:count', function () { self.update(); });
+            self.listenTo(app, 'docs:state docs:users', function () { self.update(); });
+            self.listenTo(model, 'operations:after change:editmode', function () { self.update(); });
+            self.listenTo(model.getUndoManager(), 'change:count', function () { self.update(); });
         });
 
         // destroy all class members on destruction
