@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2011
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Kai Ahrens <kai.ahrens@open-xchange.com>
  * @author Carsten Driesner <carsten.driesner@open-xchange.com>
@@ -19,8 +19,10 @@ define('io.ox/office/editframework/app/rtconnection',
      'io.ox/office/tk/utils',
      'io.ox/office/tk/logger',
      'io.ox/office/tk/object/triggerobject',
-     'io.ox/office/editframework/utils/editconfig'
-    ], function (Events, RTGroups, Utils, Logger, TriggerObject, Config) {
+     'io.ox/office/tk/object/timermixin',
+     'io.ox/office/editframework/utils/editconfig',
+     'io.ox/office/tk/filedescriptor'
+    ], function (Events, RTGroups, Utils, Logger, TriggerObject, TimerMixin, Config, FileDescriptor) {
 
     'use strict';
 
@@ -51,6 +53,7 @@ define('io.ox/office/editframework/app/rtconnection',
      *  identifier of the current editor, etc.).
      *
      * @extends TriggerObject
+     * @extends TimerMixin
      *
      * @param {EditApplication} app
      *  The application containing this operations queue.
@@ -70,7 +73,7 @@ define('io.ox/office/editframework/app/rtconnection',
             file = app.getFileDescriptor(),
 
             // create the core real-time group instance for the document
-            rtGroup = RTGroups.getGroup('synthetic.office://operations/' + file.folder_id + '.' + file.id),
+            rtGroup = RTGroups.getGroup('synthetic.office://operations/' + FileDescriptor.getAsResourceId(file)),
 
             // private event hub for internal event handling
             eventHub = new Events(),
@@ -87,11 +90,18 @@ define('io.ox/office/editframework/app/rtconnection',
             applyActionsSent = 0,
 
             // in prepare losing edit rights, for debug purposes only!
-            inPrepareLosingEditRights = false;
+            inPrepareLosingEditRights = false,
+
+            // joining the group
+            joining = false,
+
+            // rejoining is possible
+            rejoinPossible = false;
 
         // base constructor ---------------------------------------------------
 
         TriggerObject.call(this);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
@@ -318,38 +328,19 @@ define('io.ox/office/editframework/app/rtconnection',
                 def = $.Deferred();
 
             orgDeferred.then(function (response) {
-                var payloads = Utils.getArrayOption(response, 'payloads', []);
-                if (self.destroyed) {
-                    // reject deferred after destroy
-                    def.reject({cause: 'closing'});
-                }
-                if (payloads.length > 0) {
-                    // Extract the data and provide it
-                    def.resolve(payloads[0].data);
-                } else {
-                    // Response from server is not in the expected format, therefore
-                    // we reject the deferred here.
-                    def.reject(response);
-                }
+                handleResolvedResponse(def, response);
             }, function (response) {
-                var payloads = Utils.getArrayOption(response, 'payloads', []);
-                if (payloads.length > 0) {
-                    // Extract data even for the rejected case
-                    def.reject(payloads[0].data);
-                } else {
-                    // If we don't find a payload just provide the original response
-                    def.reject(response);
-                }
+                handleRejectedResponse(def, response);
             });
 
             // prevent endless waiting
             if (_.isNumber(timeout) && (timeout > 0)) {
-                var timer = app.executeDelayed(function () {
+                var timer = self.executeDelayed(function () {
                     if (def.state() === 'pending') {
                         RTConnection.log('Timeout reached for RT request, timeout=' + timeout + 'ms.');
                         def.reject({ cause: 'timeout' });
                     }
-                }, { delay: timeout });
+                }, timeout);
                 def.always(function () { timer.abort(); timer = null; });
             }
 
@@ -380,24 +371,135 @@ define('io.ox/office/editframework/app/rtconnection',
 
             // prevent endless waiting
             if (_.isNumber(timeout) && (timeout > 0)) {
-                app.executeDelayed(function () { def.reject({ cause: 'timeout' }); }, { delay: timeout });
+                self.executeDelayed(function () { def.reject({ cause: 'timeout' }); }, timeout);
             }
 
             return def.promise();
         }
 
+        /**
+         * Handles the resolved response of the RT low-level code and
+         * tries to extract the data port from it.
+         *
+         * @param {jQuery.Deferred} def
+         *  The deferred to be resolved with the extracted data.
+         *
+         * @param {Object} response
+         *  The original response provided by the RT low-level code.
+         */
+        function handleResolvedResponse(def, response) {
+            var payloads = Utils.getArrayOption(response, 'payloads', []);
+
+            if (self.destroyed) {
+                // reject deferred after destroy
+                def.reject({cause: 'closing'});
+            }
+            if (payloads.length > 0) {
+                // Extract the data and provide it
+                def.resolve(payloads[0].data);
+            } else {
+                // Response from server is not in the expected format, therefore
+                // we reject the deferred here.
+                def.reject(response);
+            }
+        }
+
+        /**
+         * Handles the rejected response of the RT low-level code and
+         * tries to extract the data port from it.
+         *
+         * @param {jQuery.Deferred} def
+         *  The deferred to be rejected with the extracted data.
+         *
+         * @param {Object} response
+         *  The original response provided by the RT low-level code.
+         */
+        function handleRejectedResponse(def, response) {
+            var payloads = Utils.getArrayOption(response, 'payloads', []);
+
+            if (payloads.length > 0) {
+                // Extract data even for the rejected case
+                def.reject(payloads[0].data);
+            } else {
+                // If we don't find a payload just provide the original response
+                def.reject(response);
+            }
+        }
+
+        /**
+         * Join a group using the provided options and set the asynchronous
+         * state. An optional retryHandler is called, if there is a chance
+         * to retry the joining.
+         *
+         * @param {Object} connectOptions
+         *  Contains properties to be sent while joining to the
+         *  document real-time connection.
+         *
+         * @param {jQuery.Deferred} def
+         *  The deferred to be resolved/rejected, if joining completed/failed.
+         *
+         * @param {Function} retryHandler
+         *  A function that is called, if a retry is possible.
+         */
+        function joinGroup(connectOptions, def, retryHandler) {
+
+            joining = true;
+            rtGroup.join(connectOptions).then(function (response) {
+                var payloads = Utils.getArrayOption(response, 'payloads', []);
+
+                if (self.destroyed) {
+                    // reject deferred after destroy
+                    def.reject({cause: 'closing'});
+                    joining = false;
+                } else if (payloads.length > 0) {
+                    handleResolvedResponse(def, response);
+                    joining = false;
+                } else {
+                    // We encountered an illegal state for our join request.
+                    if (_.isFunction(retryHandler)) {
+                        // We have to wait for possible notifications from the
+                        // low-level RT part to determine, if a retry is possible.
+                        self.executeDelayed(function () {
+                            // now check the states and decide, if we can try to
+                            // start a reconnect using a possible retry handler
+                            if (rejoinPossible) {
+                                rejoinPossible = false;
+                                retryHandler.call(this, def);
+                                joining = false;
+                            } else {
+                                handleResolvedResponse(def, response);
+                                joining = false;
+                            }
+                        }, 1000);
+                    } else {
+                        // without retry handler just process the original
+                        // response
+                        handleResolvedResponse(def, response);
+                        joining = false;
+                    }
+                }
+            }, function (response) {
+                handleRejectedResponse(def, response);
+                joining = false;
+            });
+        }
+
         // methods ------------------------------------------------------------
 
         /**
          * Connects the real-time group and sends the initial request for the
          * document import operations.
          *
+         * @param {Object} connectData
+         *  Contains properties to be sent while joining to the
+         *  document real-time connection.
+         *
          * @returns {jQuery.Promise}
          *  The Promise of a Deferred object that will be resolved with the
          *  getActions results.
          */
         this.connect = function (connectData) {
-            var connectDeferred = null,
+            var connectDeferred = $.Deferred(),
                 // optional connect data
                 connectOptions = {expectWelcomeMessage: true};
 
@@ -413,10 +515,19 @@ define('io.ox/office/editframework/app/rtconnection',
 
             RTConnection.log('RTConnection.connect called');
 
-            connectDeferred = handleResponse(rtGroup.join(connectOptions));
+            // Handler to be called, if we detected a rejoin scenario - possible
+            // if a re-enroll is needed by the low-level RT framework.
+            function rejoinHandler(def) {
+                // call join again, but this time without retry handler
+                joinGroup(connectOptions, def, null);
+            }
+
+            // joins the group and provides the retry handler
+            joinGroup(connectOptions, connectDeferred, rejoinHandler);
+
             // Remember *connect* deferred to be able to reject them if user
             // closes document while we are still connecting to the realtime group!
-            return app.createAbortablePromise(connectDeferred);
+            return this.createAbortablePromise(connectDeferred);
         };
 
         /**
@@ -457,10 +568,6 @@ define('io.ox/office/editframework/app/rtconnection',
          *
          * @param {Object} userData
          *  An object containing relevant additional user data on the document.
-         *  @param {Object} userData.selection
-         *      Contains a 'recent' user selection on a document. This will be
-         *      used for showing user cursors of multiple collaborators while
-         *      editing on documents.
          *
          * @returns {jQuery.Promise}
          */
@@ -564,7 +671,7 @@ define('io.ox/office/editframework/app/rtconnection',
 
         RTConnection.log('RTConnection initialization');
         if (RTConnection.debug) {
-            app.repeatDelayed(checkPendingAcks, {delay: CHECK_PENDING_INTERVAL});
+            this.repeatDelayed(checkPendingAcks, CHECK_PENDING_INTERVAL);
         }
 
         // read traceable actions from page URL
@@ -578,8 +685,31 @@ define('io.ox/office/editframework/app/rtconnection',
         registerRTEvents(REQUEST_EVENTS + ' ' + PUSH_EVENTS);
 
         // forward runtime realtime events to own listeners
-        rtGroup.on(INTERNAL_EVENTS, function (event) {
-            self.trigger(event.type);
+        rtGroup.on(INTERNAL_EVENTS, function (event, data) {
+            // special handling for reset event
+            if (event.type === 'reset') {
+                if (joining) {
+                    // If we are joining, reset is an indicator that a re-enroll
+                    // has been done. In that case we have to try to join again.
+                    // This must be limited to error code 1007 which means that
+                    // we have to re-enroll. A reset after we received a reset with
+                    // data.code 1007 is also normal, ignore that, too.
+                    if ((!rejoinPossible && (data && data.code === 1007)) || rejoinPossible) {
+                        if (!rejoinPossible) {
+                            rejoinPossible = true;
+                        }
+                    }
+                    else {
+                        // trigger reset otherwise
+                        self.trigger(event.type);
+                    }
+                } else {
+                    // trigger reset outside of join
+                    self.trigger(event.type);
+                }
+            } else {
+                self.trigger(event.type);
+            }
         });
 
         // forward OX Documents runtime events to own listeners
