@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -23,16 +23,6 @@ define('io.ox/office/editframework/model/documentstyles',
 
     'use strict';
 
-    var // change track attribute definitions.
-        CHANGES_ATTRIBUTE_DEFINITIONS = {
-
-            inserted: { def: '', scope: 'element' },
-
-            removed: { def: '', scope: 'element' },
-
-            modified: { def: '', scope: 'element' }
-        };
-
     // class DocumentStyles ===================================================
 
     /**
@@ -85,13 +75,13 @@ define('io.ox/office/editframework/model/documentstyles',
             definitionsRegistry = {},
 
             // default values for all supported attribute families
-            defaultAttributes = {};
+            defaultAttributeSet = {};
 
         // base constructor ---------------------------------------------------
 
         TriggerObject.call(this);
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the global attributes of the document.
@@ -178,17 +168,30 @@ define('io.ox/office/editframework/model/documentstyles',
         this.registerAttributeDefinitions = function (family, definitions) {
 
             var // get existing or create a new entry in the global registry
-                registryEntry = definitionsRegistry[family] || (definitionsRegistry[family] = {}),
+                definitionsMap = definitionsRegistry[family] || (definitionsRegistry[family] = {}),
                 // the container for the default attribute values
-                attributeMap = defaultAttributes[family] || (defaultAttributes[family] = {});
+                defaultAttributes = defaultAttributeSet[family] || (defaultAttributeSet[family] = {});
 
             // store definitions, and the default attribute values
             _.each(definitions, function (definition, name) {
                 if (_.isNull(definition.def) || _.isUndefined(definition.def) || _.isFunction(definition.def)) {
-                    Utils.error('DocumentStyles.registerAttributeDefinitions(): invalid attribute default value, family=' + family + ', name=' + name + ', value=' + definition.def);
+                    Utils.error('DocumentStyles.registerAttributeDefinitions(): invalid attribute default value, family=' + family + ', name=' + name + ', value=', definition.def);
                 } else {
-                    registryEntry[name] = definition;
-                    attributeMap[name] = definition.def;
+                    definitionsMap[name] = _.clone(definition);
+                    defaultAttributes[name] = definition.def;
+                }
+            });
+
+            // reset all old settings for child attributes
+            _.each(definitionsMap, function (definition) { delete definition.childAttrs; });
+
+            // collect settings for child attributes
+            _.each(definitionsMap, function (definition, name) {
+                var parentName = Utils.getStringOption(definition.parent, 'name');
+                if (_.isString(parentName) && (parentName in definitionsMap)) {
+                    var parentDef = definitionsMap[parentName],
+                        childAttrs = parentDef.childAttrs || (parentDef.childAttrs = []);
+                    childAttrs.push({ name: name, value: definition.parent.value });
                 }
             });
 
@@ -224,15 +227,15 @@ define('io.ox/office/editframework/model/documentstyles',
         this.getDefaultAttributes = function (families) {
 
             var // the attribute set containing all specified families
-                attributes = {};
+                attributeSet = {};
 
             _.chain(families).getArray().each(function (family) {
-                if (family in defaultAttributes) {
-                    attributes[family] = _.copy(defaultAttributes[family], true);
+                if (family in defaultAttributeSet) {
+                    attributeSet[family] = _.copy(defaultAttributeSet[family], true);
                 }
             });
 
-            return attributes;
+            return attributeSet;
         };
 
         /**
@@ -242,21 +245,21 @@ define('io.ox/office/editframework/model/documentstyles',
          * be used before the values of any style sheet attributes and explicit
          * element attributes will be resolved.
          *
-         * @param {Object} attributes
+         * @param {Object} attributeSet
          *  An (incomplete) attribute set with new attribute default values.
          *
          * @returns {DocumentStyles}
          *  A reference to this instance.
          */
-        this.setDefaultAttributes = function (attributes) {
+        this.setDefaultAttributes = function (attributeSet) {
 
             // insert passed values into the default attribute set
-            _.each(attributes, function (attributeMap, family) {
-                var defaultMap = defaultAttributes[family];
-                if (_.isObject(attributeMap) && _.isObject(defaultMap)) {
-                    _.each(attributeMap, function (value, name) {
-                        if ((name in defaultMap) && !_.isNull(value) && !_.isUndefined(value)) {
-                            defaultMap[name] = _.copy(value, true);
+            _.each(attributeSet, function (attributes, family) {
+                var defaultAttributes = defaultAttributeSet[family];
+                if (_.isObject(attributes) && _.isObject(defaultAttributes)) {
+                    _.each(attributes, function (value, name) {
+                        if ((name in defaultAttributes) && !_.isNull(value) && !_.isUndefined(value)) {
+                            defaultAttributes[name] = _.copy(value, true);
                         }
                     });
                 }
@@ -282,24 +285,24 @@ define('io.ox/office/editframework/model/documentstyles',
         this.buildNullAttributes = function (families) {
 
             var // the resulting attribute set
-                attributes = {};
+                attributeSet = {};
 
-            _.chain(families).getArray().each(function (family) {
+            _.each(_.getArray(families), function (family) {
 
                 var // the attribute definitions of the current family
                     definitions = self.getAttributeDefinitions(family),
                     // attribute values of the current family
-                    attributeMap = attributes[family] = {};
+                    attributes = attributeSet[family] = {};
 
                 // add null values for all registered attributes
                 _.each(definitions, function (definition, name) {
                     if (!definition.special) {
-                        attributeMap[name] = null;
+                        attributes[name] = null;
                     }
                 });
             });
 
-            return attributes;
+            return attributeSet;
         };
 
         /**
@@ -310,12 +313,12 @@ define('io.ox/office/editframework/model/documentstyles',
          * both attribute sets, otherwise the value of the second attribute set
          * will be copied to the first attribute set.
          *
-         * @param {Object} attributes1
+         * @param {Object} attributeSet1
          *  (in/out) The first attribute set that will be extended in-place, as
          *  map of attribute value maps (name/value pairs), keyed by attribute
          *  family.
          *
-         * @param {Object} attributes2
+         * @param {Object} attributeSet2
          *  The second attribute set, as map of attribute value maps
          *  (name/value pairs), keyed by attribute family, whose attribute
          *  values will be inserted into the first attribute set.
@@ -330,44 +333,42 @@ define('io.ox/office/editframework/model/documentstyles',
          * @returns {Object}
          *  A reference to the first passed and extended attribute set.
          */
-        this.extendAttributes = function (attributes1, attributes2, options) {
+        this.extendAttributes = function (attributeSet1, attributeSet2, options) {
 
-            // add attributes existing in attributes2 to attributes1
-            _.each(attributes2, function (attributeValues, family) {
+            // add attributes existing in attributeSet2 to attributeSet1
+            _.each(attributeSet2, function (attributes2, family) {
 
                 var // the attribute definitions of the current family
                     definitions = this.getAttributeDefinitions(family);
 
                 // copy style identifier directly (may be string or null)
                 if (family === 'styleId') {
-                    attributes1.styleId = attributeValues;
+                    attributeSet1.styleId = attributes2;
                     return;
                 }
 
                 // restrict to valid attribute families
-                if (!definitions || !_.isObject(attributeValues)) { return; }
+                if (!definitions || !_.isObject(attributes2)) { return; }
 
                 // insert attribute values of attributes2
-                attributes1[family] = attributes1[family] || {};
-                _.each(attributeValues, function (value, name) {
-
-                    var // the merger function from the attribute definition
-                        merger = null;
-
+                var attributes1 = attributeSet1[family] || (attributeSet1[family] = {});
+                _.each(attributes2, function (value, name) {
                     if (AttributeUtils.isRegisteredAttribute(definitions, name, options)) {
-                        // try to find merger function from attribute definition
-                        merger = definitions[name].merge;
+
+                        var // try to find merger function from attribute definition
+                            merger = definitions[name].merge;
+
                         // either set return value from merger, or copy the attribute directly
-                        if ((name in attributes1[family]) && !_.isNull(attributes1[family][name]) && _.isFunction(merger)) {
-                            attributes1[family][name] = merger.call(this, attributes1[family][name], value);
+                        if ((name in attributes1) && !_.isNull(attributes1[name]) && _.isFunction(merger)) {
+                            attributes1[name] = merger.call(this, attributes1[name], value);
                         } else {
-                            attributes1[family][name] = value;
+                            attributes1[name] = value;
                         }
                     }
                 });
             }, this);
 
-            return attributes1;
+            return attributeSet1;
         };
 
         /**
@@ -649,9 +650,6 @@ define('io.ox/office/editframework/model/documentstyles',
 
         // initialization -----------------------------------------------------
 
-        // register change track attributes
-        this.registerAttributeDefinitions('changes', CHANGES_ATTRIBUTE_DEFINITIONS);
-
         // register own collections for public access
         this.addCustomCollection('fonts', fontCollection)
             .addCustomCollection('themes', themeCollection);
