@@ -1,24 +1,28 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/editframework/model/editmodel',
     ['io.ox/office/tk/utils',
+     'io.ox/office/tk/logger',
      'io.ox/office/baseframework/model/basemodel',
-     'io.ox/office/editframework/model/operationsgenerator'
-    ], function (Utils, BaseModel, OperationsGenerator) {
+     'io.ox/office/editframework/utils/operations'
+    ], function (Utils, Logger, BaseModel, Operations) {
 
     'use strict';
 
+    var // operations logger
+        logger = new Logger({ enable: 'office:log-ops', prefix: 'OP' });
+
     // class EditModel ========================================================
 
     /**
@@ -77,13 +81,13 @@ define('io.ox/office/editframework/model/editmodel',
         var // self reference for local functions
             self = this,
 
+            // Deferred object for current asynchronous actions, will remain in
+            // resolved/rejected state as long as no new actions will be applied
+            actionsDef = $.when(),
+
             // maps all operation names to operation handler functions
             operationHandlers = {},
 
-            // Deferred object for current asynchronous operations, will remain
-            // in resolved state as long as no operations are applied
-            operationsDef = $.when(),
-
             // whether operations are currently applied (prevent recursive calls)
             processingOperations = false,
 
@@ -107,12 +111,68 @@ define('io.ox/office/editframework/model/editmodel',
 
         // private methods ----------------------------------------------------
 
+        /**
+         * Switches this document model into 'action processing mode', and
+         * invokes the passed callback function. As long as processing mode is
+         * active, the method EditModel.getActionsPromise() returns a pending
+         * promise. The callback function may return a promise to control the
+         * duration of the action processing mode. This method MUST NOT be
+         * called recursively! The methods EditModel.applyOperations() and
+         * EditModel.applyActions() must not be used (these methods will enter
+         * the action processing mode by themselves).
+         *
+         * @param {Function} callback
+         *  The callback function that will be invoked with activated action
+         *  processing mode. Must return either a Boolean value that represents
+         *  the result of the callback function, or the promise of a Deferred
+         *  object to extend the processing mode until the promise has been
+         *  resolved or rejected. Will be called in the context of this
+         *  instance.
+         *
+         * @returns {Any}
+         *  The return value of the callback function.
+         */
+        function enterProcessActionsMode(callback) {
+
+            var // the result of the callback function
+                result = null;
+
+            // prevent recursive invocations
+            if (actionsDef.state() === 'pending') {
+                Utils.error('EditModel.enterProcessActionsMode(): recursive call');
+                return false;
+            }
+
+            // create an new Deferred object which remains in pending state as long
+            // as the callback function runs (and the promise it returns is pending)
+            actionsDef = $.Deferred().always(function () {
+                logger.log('EditModel.enterProcessActionsMode(): ' + actionsDef.state().replace(/ed$/, 'ing') + ' promise...');
+            });
+
+            // invoke the callback function
+            result = callback.call(self);
+
+            // resolve the actions promise
+            if (_.isObject(result) && _.isFunction(result.promise)) {
+                result.progress(_.bind(actionsDef.notify, actionsDef))
+                    .done(_.bind(actionsDef.resolve, actionsDef))
+                    .fail(_.bind(actionsDef.reject, actionsDef));
+            } else if (result === true) {
+                actionsDef.resolve();
+            } else {
+                actionsDef.reject();
+            }
+
+            return result;
+        }
+
         /**
          * Extracts and returns the operation filter callback function from the
          * passed options.
          */
         function getFilterCallback(options) {
-            return Utils.getFunctionOption(options, 'filter', function () { return true; });
+            var filterFunc = Utils.getFunctionOption(options, 'filter');
+            return _.isFunction(filterFunc) ? filterFunc : _.constant(Utils.getBooleanOption(options, 'filter', true));
         }
 
         /**
@@ -138,6 +198,11 @@ define('io.ox/office/editframework/model/editmodel',
                 return false;
             }
 
+            if (app.getState() === 'error') {
+                Utils.error('EditModel.beginProcessOperations(): illegal call - application is in error state.');
+                return false;
+            }
+
             self.trigger('operations:before', operations, external);
             processingOperations = true;
             return true;
@@ -197,7 +262,6 @@ define('io.ox/office/editframework/model/editmodel',
                 // check for the correct operation state number of an external operation
                 if (_.isNumber(operation.osn) && (operationStateNumber !== operation.osn)) {
                     // If this client is already synchronized -> there is a conflict in the operation state number
-                    Utils.error('EditModel.handleOperationStateNumber(): Wrong document state. Operation expected state ' + operation.osn + '. But local state is: ' + operationStateNumber);
                     return false;
                 }
             } else {
@@ -239,52 +303,54 @@ define('io.ox/office/editframework/model/editmodel',
                 // the function that executes the operation
                 operationHandler = null,
                 // success value of operation
-                success = false,
-                // a possible log message
-                message = null;
+                success = false;
 
-            // check operation
-            if (!_.isObject(operation) || !_.isString(operation.name)) {
-                Utils.error('EditModel.applyOperation(): expecting operation object.');
-                return false;
-            }
-
-            // handle an operation state number. This number needs to be synchronized
-            // between all clients and has then to be identical for all clients.
-            if (!handleOperationStateNumber(operation, external)) {
-                return false;
-            }
-
-            // get and check operation handler
-            operationHandler = operationHandlers[operation.name];
-            if (!_.isFunction(operationHandler)) {
-                message = 'EditModel.applyOperation(): invalid operation name "' + operation.name + '".';
+            function failAndReturn(message) {
+                message = 'EditModel.applyOperation(): ' + message;
                 Utils.error(message);
                 app.sendLogMessage(message);
                 return false;
             }
 
-            // Caching the last two operations
+            // log operation
+            logger.log('i=' + totalOperations + ', op=', operation);
+
+            // check operation
+            if (!_.isObject(operation) || !_.isString(operation.name)) {
+                return failAndReturn('Expecting operation object.');
+            }
+
+            // Handle an operation state number. This number needs to be synchronized
+            // between all clients and has then to be identical for all clients.
+            if (!handleOperationStateNumber(operation, external)) {
+                return failAndReturn('Wrong operation state number: ' + operationStateNumber + ' (expected: ' + operation.osn + ').');
+            }
+
+            // get and check operation handler
+            operationHandler = operationHandlers[operation.name];
+            if (!_.isFunction(operationHandler)) {
+                return failAndReturn('Invalid operation name "' + operation.name + '".');
+            }
+
+            // cache the last two operations
             operationsCache.previous = operationsCache.current;
             operationsCache.current = operation;
 
-            // execute the operation handler (unless the operation scope is
-            // 'filter' for operations intended to be executed in the filter
-            // component only, or the filter callback function returns false)
+            // execute the operation handler (unless the operation scope is 'filter' for operations intended
+            // to be executed in the filter component only, or the filter callback function returns false)
             try {
                 success = (scope === 'filter') || (filter.call(self, operation) !== true) || (operationHandler.call(self, operation, external) !== false);
             } catch (ex) {
                 Utils.exception(ex);
             }
 
-            if (success) {
-                totalOperations += 1;
-            } else {
-                message = 'EditModel.applyOperation(): ' + (totalOperations + 1) + (external ? ' E ' : ' I ') + JSON.stringify(operation) + ' failed';
-                Utils.error(message);
-                app.sendLogMessage(message);
+            // log error and return with failure
+            if (!success) {
+                return failAndReturn((totalOperations + 1) + (external ? ' E ' : ' I ') + JSON.stringify(operation) + ' failed.');
             }
-            return success;
+
+            totalOperations += 1;
+            return true;
         }
 
         /**
@@ -370,7 +436,7 @@ define('io.ox/office/editframework/model/editmodel',
                 promise = undoManager.enterUndoGroup(function () {
 
                     // return the Deferred to enterUndoGroup() to defer the open undo group
-                    return app.iterateArraySliced(operations, function (operation) {
+                    return self.iterateArraySliced(operations, function (operation) {
 
                         // if the operation's origin is not an external client and
                         // we don't have edit rights then we skip the operations silently
@@ -381,14 +447,14 @@ define('io.ox/office/editframework/model/editmodel',
 
                         // break the loop and reject the promise if an operation has failed
                         if (!applyOperation(operation, external, filter)) {
-                            return $.Deferred().reject();
+                            return $.Deferred().reject({ cause: 'operation' });
                         }
-                    });
+                    }, { delay: 'immediate' });
                 });
 
             } else {
                 // currently applying operations (called recursively)
-                promise = $.Deferred().reject();
+                promise = $.Deferred().reject({ cause: 'recursive' });
             }
 
             // post-processing (trigger end events, leave operations mode)
@@ -406,7 +472,7 @@ define('io.ox/office/editframework/model/editmodel',
             });
         }
 
-        // methods ------------------------------------------------------------
+        // protected methods --------------------------------------------------
 
         /**
          * Registers a function that will be executed when an operation with
@@ -435,16 +501,53 @@ define('io.ox/office/editframework/model/editmodel',
             return this;
         };
 
+        // public methods -----------------------------------------------------
+
         /**
-         * Creates and returns an initialized operations generator instance (an
-         * instance of the class specified in the constructor parameter
-         * OperationsGeneratorClass).
+         * Returns whether the 'action processing mode' of this document model
+         * is currently active (whether this model currently generates,
+         * applies, or sends operation actions, either synchronously or
+         * asynchronously).
          *
-         * @returns {OperationsGenerator}
-         *  A new operations generator instance.
+         * @returns {Boolean}
+         *  Whether the 'action processing mode' is currently active.
          */
-        this.createOperationsGenerator = function () {
-            return new OperationsGeneratorClass(documentStyles);
+        this.isProcessingActions = function () {
+            return actionsDef.state() === 'pending';
+        };
+
+        /**
+         * Returns the promise of a Deferred object representing the state of
+         * the 'action processing mode'. If in pending state, this model
+         * currently generates, applies, or sends operation actions (either
+         * synchronously or asynchronously). If in resolved state, the last
+         * operations have been applied successfully. If in rejected state, the
+         * last operations have not been applied successfully.
+         *
+         * @attention
+         *  The promise MUST NOT be cached externally. Its solely purpose is to
+         *  register done/fail callbacks from operation handlers or other code
+         *  depending on operations. Whenever new operations will be applied, a
+         *  new promise will be created representing the new operations (the
+         *  old promises returned by this method before will remain in
+         *  resolved/rejected state forever).
+         *
+         * @returns {jQuery.Promise}
+         *  The promise of a Deferred object representing the state of the
+         *  action processing mode.
+         */
+        this.getActionsPromise = function () {
+            return actionsDef.promise();
+        };
+
+        /**
+         * Returns whether operations are currently processed.
+         *
+         * @returns {Boolean}
+         *  Whether the model processes operations.
+         */
+        this.isProcessingOperations = function () {
+            return processingOperations;
         };
 
         /**
@@ -460,9 +563,9 @@ define('io.ox/office/editframework/model/editmodel',
          * @returns {Boolean}
          *  Whether registering all operations for sending was successful.
          */
-        this.sendOperations = function (operations) {
-            return applyOperationsSync(_.getArray(operations), false, $.noop);
-        };
+        this.sendOperations = logger.profileMethod('EditModel.sendOperations()', function (operations) {
+            return applyOperationsSync(_.getArray(operations), false, _.constant(false));
+        });
 
         /**
          * Executes the handler functions for all passed operations. If the
@@ -475,33 +578,25 @@ define('io.ox/office/editframework/model/editmodel',
          *
          * @param {Object} [options]
          *  Optional parameters:
-         *  @param {Function} [options.filter]
+         *  @param {Function|Boolean} [options.filter]
          *      A predicate function that will be invoked for each operation
          *      contained in the passed actions. Receives the operation as
          *      first parameter, and must return a Boolean specifying whether
-         *      to invoke the operation handler.
+         *      to invoke the operation handler. Can be set to the Boolean
+         *      value false to prevent applying any operation locally (only
+         *      send them to the server).
          *
          * @returns {Boolean}
          *  Whether applying all operations was successful.
          */
-        this.applyOperations = function (operations, options) {
+        this.applyOperations = logger.profileMethod('EditModel.applyOperations()', function (operations, options) {
 
-            var // whether the operations have been applied successfully
-                success = false;
-
-            // create an new Deferred object which remains in pending state as
-            // long as the operations will be applied
-            operationsDef = $.Deferred();
-
-            // apply the operations synchronously
-            success = applyOperationsSync(_.getArray(operations), false, getFilterCallback(options));
-
-            // resolve or reject the Deferred object, this will invoke all
-            // callbacks that have been registered inside the operation handlers
-            operationsDef[success ? 'resolve' : 'reject']();
-
-            return success;
-        };
+            // apply the operations synchronously, but enter action processing mode anyway,
+            // operation handler may register themselves at the actions promise
+            return enterProcessActionsMode(function () {
+                return applyOperationsSync(_.getArray(operations), false, getFilterCallback(options));
+            });
+        });
 
         /**
          * Executes the handler functions for all operations contained in the
@@ -527,39 +622,31 @@ define('io.ox/office/editframework/model/editmodel',
          *      A predicate function that will be invoked for each operation
          *      contained in the passed actions. Receives the operation as
          *      first parameter, and must return a Boolean specifying whether
-         *      to invoke the operation handler.
+         *      to invoke the operation handler. Can be set to the Boolean
+         *      value false to prevent applying any operation locally (only
+         *      send them to the server).
          *
          * @returns {jQuery.Promise}
          *  The promise of a Deferred object that will be resolved after all
          *  actions have been applied successfully, or rejected immediately
          *  after applying an action has failed. The caller has to wait for the
-         *  Promise before new actions can be applied. The Promise regularly
+         *  promise before new actions can be applied. The promise regularly
          *  sends progress updates while applying the actions. In synchronous
          *  mode (option 'async' not set), the returned promise is already
          *  resolved or rejected when this method finishes. In asynchronous
-         *  mode, the Promise will contain an additional method 'abort()' that
-         *  can be used to stop applying the operations at any time.
+         *  mode, the promise will contain an additional method 'abort()' that
+         *  can be used to stop applying the operations at any time. When
+         *  rejected, the promise will pass a result object with the following
+         *  properties:
+         *  - {String} result.cause
+         *      The error code. Will be set to 'operation', if an operation
+         *      callback handler has failed; or to 'recursive', if this method
+         *      has been invoked recursively.
          */
-        this.applyActions = function (actions, options) {
+        this.applyActions = logger.profileAsyncMethod('EditModel.applyActions()', function (actions, options) {
 
-            var // the timer processing the array asynchronously
-                timer = null,
-                // whether the actions have been received from the server
-                external = Utils.getBooleanOption(options, 'external', false),
-                // whether to execute asynchronously
-                async = Utils.getBooleanOption(options, 'async', false),
-                // invocation filter callback
-                filter = getFilterCallback(options),
-                // total number of operations
-                operationCount = 0,
-                // number of operations already applied
-                operationIndex = 0,
-                // the promise of the inner loop processing operations of an action
-                innerPromise = null,
-                // resulting promise returned by this method
-                resultPromise = null,
-                // whether the operations have been applied successfully in synchronous mode
-                success = false;
+            var // total number of operations
+                operationCount = 0;
 
             // convert parameter to an array, filter for actions containing
             // at least one operation, get total number of operations
@@ -572,88 +659,142 @@ define('io.ox/office/editframework/model/editmodel',
             // no operations: early exit
             if (operationCount === 0) { return $.when(); }
 
-            // create an new Deferred object which remains in pending state as
-            // long as the operations will be applied
-            operationsDef = $.Deferred();
+            // apply in action processing mode
+            return enterProcessActionsMode(function () {
 
-            // apply actions, get resulting promise
-            if (async) {
+                var // whether the actions have been received from the server
+                    external = Utils.getBooleanOption(options, 'external', false),
+                    // whether to execute asynchronously
+                    async = Utils.getBooleanOption(options, 'async', false),
+                    // invocation filter callback
+                    filter = getFilterCallback(options),
+                    // resulting Deferred object returned by this method
+                    resultDef = $.Deferred();
+
+                // apply actions synchronously if specified
+                if (!async) {
+
+                    var // synchronous mode: apply all actions at once, but bail out on error
+                        success = _.all(actions, function (action) {
+                            // applyOperationsSync() creates undo groups by itself
+                            return applyOperationsSync(action.operations, external, filter);
+                        });
+
+                    // resolve or reject the resulting Deferred object immediately
+                    return success ? resultDef.resolve() : resultDef.reject({ cause: 'operation' });
+                }
+
+                var // the timer processing the array asynchronously
+                    timer = null,
+                    // start index of first operations in the current action (for progress)
+                    startIndex = 0,
+                    // the promise of the inner loop processing operations of an action
+                    innerPromise = null;
 
                 // apply the actions in a browser timeout loop
-                timer = app.iterateArraySliced(actions, function (action) {
+                timer = this.iterateArraySliced(actions, function (action) {
 
-                    // apply the operations and calculate the progress
-                    innerPromise = applyOperationsAsync(action.operations, external, filter)
-                        .progress(function (progress) {
-                            operationsDef.notify((operationIndex + action.operations.length * progress) / operationCount);
-                        })
-                        .done(function () {
-                            operationIndex += action.operations.length;
-                        });
+                    // apply the operations of the current action
+                    innerPromise = applyOperationsAsync(action.operations, external, filter);
+
+                    // calculate and notify the total progress of all actions
+                    innerPromise.progress(function (progress) {
+                        var currIndex = startIndex + action.operations.length * progress;
+                        resultDef.notify(currIndex / operationCount);
+                    });
+
+                    // update the start index for the next action
+                    innerPromise.done(function () { startIndex += action.operations.length; });
 
                     // clean up after finishing the loop
                     return innerPromise.always(function () { innerPromise = null; });
-                });
+
+                }, { delay: 'immediate' });
 
                 // forward the result of the timer to the own Deferred object
-                timer.done(function () { operationsDef.notify(1).resolve(); }).fail(_.bind(operationsDef.reject, operationsDef));
+                timer.done(function () { resultDef.notify(1).resolve(); }).fail(_.bind(resultDef.reject, resultDef));
 
-                // create an abortable promise from the Deferred object
-                resultPromise = app.createAbortablePromise(operationsDef, function () {
+                // create an abortable promise for the Deferred object
+                return this.createAbortablePromise(resultDef, function () {
                     // abort the outer loop processing the actions
                     timer.abort();
                     // abort the inner loop processing the operations of an action
                     if (innerPromise) { innerPromise.abort(); }
                 });
+            }, this);
+        });
 
-            } else {
-
-                // synchronous mode: apply all actions at once, but bail out on error
-                success = _.all(actions, function (action) {
-                    // applyOperationsSync() creates undo groups by itself
-                    return applyOperationsSync(action.operations, external, filter);
-                });
-
-                // resolve or reject the Deferred object, this will invoke all
-                // callbacks that have been registered inside the operation handlers
-                resultPromise = operationsDef[success ? 'resolve' : 'reject']().promise();
-            }
-
-            return resultPromise;
+        /**
+         * Creates and returns an initialized operations generator instance (an
+         * instance of the class specified in the constructor parameter
+         * OperationsGeneratorClass).
+         *
+         * @returns {OperationsGenerator}
+         *  A new operations generator instance.
+         */
+        this.createOperationsGenerator = function () {
+            return new OperationsGeneratorClass(app);
         };
 
         /**
-         * Returns the promise of a Deferred object representing the state of
-         * operations currently applied. If in pending state, this model
-         * currently applies operations (synchronously or asynchronously). If
-         * in resolved state, the last operations have been applied
-         * successfully. If in rejected state, the last operations have not
-         * been applied successfully.
+         * Creates a new operations generator, invokes the callback function,
+         * and then applies all operations contained in the generator, and
+         * sends them to the server.
          *
-         * @attention
-         *  The promise MUST NOT be cached externally. Its solely purpose is to
-         *  register done/fail callbacks from operation handlers or other code
-         *  depending on operations. Whenever new operations will be applied, a
-         *  new promise will be created representing the new operations (the
-         *  old promises will remain in resolved/rejected state forever).
+         * @param {Function} callback
+         *  The callback function. Receives a new operations generator as first
+         *  parameter. May return the promise of a Deferred object to defer
+         *  applying and sending the operations until the promise has been
+         *  resolved. Will be called in the context of this instance.
+         *
+         * @param {Object} [options]
+         *  Optional parameters:
+         *  @param {Function} [options.filter]
+         *      A predicate function that will be invoked for each operation
+         *      contained in the passed actions. Receives the operation as
+         *      first parameter, and must return a Boolean specifying whether
+         *      to invoke the operation handler. Can be set to the Boolean
+         *      value false to prevent applying any operation locally (only
+         *      send them to the server).
+         *  @param {OperationsGeneratoe} [options.generator]
+         *      A custom operations generator instance that will be passed to
+         *      the callback function. If omitted, a new default operations
+         *      generator will be created (see method
+         *      EditModel.createOperationsGenerator() for details).
          *
          * @returns {jQuery.Promise}
-         *  The promise of a Deferred object representing the currently or last
-         *  applied operations.
+         *  The promise of a Deferred object that will be resolved after the
+         *  operations have been applied and sent successfully (with the result
+         *  of the callback function); or rejected, if the callback has
+         *  returned a rejected promise (with the result of that promise), or
+         *  if applying the operations has failed (with an object containing a
+         *  property 'cause' set to the value 'operation').
          */
-        this.getOperationsPromise = function () {
-            return operationsDef.promise();
-        };
+        this.createAndApplyOperations = logger.profileAsyncMethod('EditModel.createAndApplyOperations()', function (callback, options) {
 
-        /**
-         * Returns whether operations are currently processed.
-         *
-         * @returns {Boolean}
-         *  Whether the model processes operations.
-         */
-        this.isProcessingOperations = function () {
-            return processingOperations;
-        };
+            // simulate actions processing mode for deferred callbacks
+            return enterProcessActionsMode(function () {
+
+                var // the generator passed to the callback function
+                    generator = Utils.getObjectOption(options, 'generator'),
+                    // the promise returned by this method
+                    promise = null;
+
+                // create default operations generator if none has been passed to this method
+                if (!_.isObject(generator)) { generator = self.createOperationsGenerator(); }
+
+                // invoke the callback function
+                promise = $.when(callback.call(self, generator));
+
+                // apply and send the operations after the callback has finished
+                promise = promise.then(function (result) {
+                    var success = applyOperationsSync(generator.getOperations(), false, getFilterCallback(options));
+                    return success ? result : $.Deferred().reject({ cause: 'operation' });
+                });
+
+                return promise;
+            });
+        });
 
         /**
          * Returns whether the document can be modified by the current user.
@@ -789,7 +930,7 @@ define('io.ox/office/editframework/model/editmodel',
         // operation handlers -------------------------------------------------
 
         // setting default attributes is not undoable
-        this.registerOperationHandler(OperationsGenerator.SET_DOCUMENT_ATTRIBUTES, function (operation) {
+        this.registerOperationHandler(Operations.SET_DOCUMENT_ATTRIBUTES, function (operation) {
 
             var // the attribute set from the passed operation
                 attributes = _.clone(Utils.getObjectOption(operation, 'attrs', {}));
@@ -804,15 +945,15 @@ define('io.ox/office/editframework/model/editmodel',
             documentStyles.setDefaultAttributes(attributes);
         });
 
-        this.registerOperationHandler(OperationsGenerator.INSERT_THEME, function (operation) {
+        this.registerOperationHandler(Operations.INSERT_THEME, function (operation) {
             this.getThemeCollection().insertTheme(operation.themeName, operation.attrs);
         });
 
-        this.registerOperationHandler(OperationsGenerator.INSERT_FONT_DESCRIPTION, function (operation) {
+        this.registerOperationHandler(Operations.INSERT_FONT_DESCRIPTION, function (operation) {
             this.getFontCollection().insertFont(operation.fontName, operation.attrs);
         });
 
-        this.registerOperationHandler(OperationsGenerator.INSERT_STYLESHEET, function (operation) {
+        this.registerOperationHandler(Operations.INSERT_STYLESHEET, function (operation) {
 
             var // target style sheet container
                 styleSheets = documentStyles.getStyleCollection(operation.type),
@@ -834,7 +975,7 @@ define('io.ox/office/editframework/model/editmodel',
             });
         });
 
-        this.registerOperationHandler(OperationsGenerator.DELETE_STYLESHEET, function (operation) {
+        this.registerOperationHandler(Operations.DELETE_STYLESHEET, function (operation) {
 
             var // target style sheet container
                 styleSheets = documentStyles.getStyleCollection(operation.type);
@@ -850,7 +991,7 @@ define('io.ox/office/editframework/model/editmodel',
             return styleSheets.deleteStyleSheet(operation.styleId);
         });
 
-        this.registerOperationHandler(OperationsGenerator.NOOP, $.noop);
+        this.registerOperationHandler(Operations.NOOP, _.constant(true));
 
         // initialization -----------------------------------------------------
 
