@@ -1,21 +1,21 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/editframework/model/fontcollection',
     ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/editframework/utils/canvaswrapper'
-    ], function (Utils, ModelObject, CanvasWrapper) {
+     'io.ox/office/tk/canvaswrapper',
+     'io.ox/office/baseframework/model/modelobject'
+    ], function (Utils, CanvasWrapper, ModelObject) {
 
     'use strict';
 
@@ -60,7 +60,7 @@ define('io.ox/office/editframework/model/fontcollection',
         fontMetricsCache = {},
 
         // the dummy element used to calculate font metrics
-        helperNode = $('<div>').appendTo($('<div>').css({ position: 'absolute', left: 0, top: 0, width: 1, height: 1, overflow: 'hidden' }).prependTo('body'));
+        helperNode = $('<div>').appendTo($('<div class="noI18n">').css({ position: 'absolute', left: 0, top: 0, width: 1, height: 1, overflow: 'hidden' }).prependTo('body'));
 
     // class FontCollection ===================================================
 
@@ -190,7 +190,7 @@ define('io.ox/office/editframework/model/fontcollection',
 
             var // capitalized font name (used as cache key)
                 fontName = Utils.capitalizeWords(attributes.fontName),
-                // font size, rounded to tenths of points
+                // font size, rounded to tenth of points
                 fontSize = Utils.round(attributes.fontSize, 0.1),
                 // font weight
                 bold = attributes.bold === true,
@@ -271,36 +271,37 @@ define('io.ox/office/editframework/model/fontcollection',
                 charWidthCache = {},
 
                 // dummy canvas element (not required to be inserted into the DOM)
-                canvasWrapper = new CanvasWrapper(),
-
-                // the rendering context
-                context = canvasWrapper.getContext();
+                canvasWrapper = new CanvasWrapper();
 
             // the actual calculateTextWidth() method returned from the local scope
             function calculateTextWidth(attributes, callback, cacheKey) {
 
-                var // capitalized font name (used as cache key)
+                var // cache entry for the passed key
+                    cacheEntry = cacheKey ? (charWidthCache[cacheKey] || (charWidthCache[cacheKey] = {})) : null,
+                    // capitalized font name (used as cache key)
                     fontName = Utils.capitalizeWords(attributes.fontName),
-                    // font size, rounded to tenths of points
+                    // font size, rounded to tenth of points
                     fontSize = Utils.round(attributes.fontSize, 0.1),
                     // the complete font style for the canvas
-                    fontStyle = (attributes.bold ? 'bold ' : '') + (attributes.italic ? 'italic ' : '') + fontSize + 'pt ' + self.getCssFontFamily(fontName),
-                    // cache entry for the passed key
-                    cacheEntry = cacheKey ? (charWidthCache[cacheKey] || (charWidthCache[cacheKey] = {})) : null,
+                    fontStyle = { family: self.getCssFontFamily(fontName), size: fontSize, bold: attributes.bold, italic: attributes.italic },
+                    // the font style, as textual map key
+                    fontStyleKey = JSON.stringify(fontStyle),
                     // the result of the callback function
                     result = 0;
 
                 // look for an existing value in the cache
-                if (cacheEntry && (fontStyle in cacheEntry)) {
-                    return cacheEntry[fontStyle];
+                if (cacheEntry && (fontStyleKey in cacheEntry)) {
+                    return cacheEntry[fontStyleKey];
                 }
 
                 // initialize font in the rendering context, fetch callback result
-                context.font = fontStyle;
-                result = callback.call(self, context);
+                canvasWrapper.render(function (context) {
+                    context.setFontStyle(fontStyle);
+                    result = callback.call(self, context);
+                });
 
                 // insert the result into the cache entry
-                if (cacheEntry) { cacheEntry[fontStyle] = result; }
+                if (cacheEntry) { cacheEntry[fontStyleKey] = result; }
                 return result;
             }
 
