@@ -1,17 +1,19 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/model/operationsgenerator', function () {
+define('io.ox/office/editframework/model/operationsgenerator', [
+    'io.ox/office/editframework/utils/operations'
+], function (Operations) {
 
     'use strict';
 
@@ -23,13 +25,15 @@ define('io.ox/office/editframework/model/operationsgenerator', function () {
      *
      * @constructor
      *
-     * @param {DocumentStyles} documentStyles
-     *  Global collection with the style sheet containers and custom formatting
-     *  containers of a document.
+     * @param {EditApplication} app
+     *  The application instance containing this generator.
      */
-    function OperationsGenerator(documentStyles) {
+    function OperationsGenerator(app) {
 
-        var // the operations buffer
+        var // the document styles (style sheet containers etc.)
+            documentStyles = app.getModel().getDocumentStyles(),
+
+            // the operations buffer
             operations = [];
 
         // methods ------------------------------------------------------------
@@ -74,6 +78,20 @@ define('io.ox/office/editframework/model/operationsgenerator', function () {
             return operation;
         };
 
+        /**
+         * Adding operations to this operations generator.
+         *
+         * @param {String[]} ops
+         *  The additional operations.
+         *
+         * @returns {Number}
+         *  The number of operations in the operations generator.
+         */
+        this.addOperations = function (ops) {
+            operations = operations.concat(ops);
+            return operations.length;
+        };
+
         /**
          * Generates an 'insertStyleSheet' operation, if the specified style
          * sheet exists in the style sheet collection, and is marked dirty.
@@ -111,7 +129,7 @@ define('io.ox/office/editframework/model/operationsgenerator', function () {
                 if (!_.isNumber(operationOptions.uiPriority)) { delete operationOptions.uiPriority; }
 
                 // generate and store the 'insertStyleSheet' operation
-                this.generateOperation(OperationsGenerator.INSERT_STYLESHEET, operationOptions);
+                this.generateOperation(Operations.INSERT_STYLESHEET, operationOptions);
 
                 // remove the dirty flag from the style sheet
                 styleSheets.setDirty(styleId, false);
@@ -122,49 +140,6 @@ define('io.ox/office/editframework/model/operationsgenerator', function () {
 
     } // class OperationsGenerator
 
-    // constants --------------------------------------------------------------
-
-    // define names of generic operations
-    _.extend(OperationsGenerator, {
-
-        /**
-         * Name of the initial operation while importing a document, sets
-         * global attributes of the model, including default formatting
-         * attributes.
-         */
-        SET_DOCUMENT_ATTRIBUTES: 'setDocumentAttributes',
-
-        /**
-         * Name of the operation that inserts a new theme into the theme
-         * collection.
-         */
-        INSERT_THEME: 'insertTheme',
-
-        /**
-         * Name of the operation that inserts a new font description into the
-         * font collection.
-         */
-        INSERT_FONT_DESCRIPTION: 'insertFontDescription',
-
-        /**
-         * Name of the operation that inserts a new style sheet into the style
-         * sheet collection of a specific style family.
-         */
-        INSERT_STYLESHEET: 'insertStyleSheet',
-
-        /**
-         * Name of the operation that removes a style sheet from the style
-         * sheet collection of a specific style family.
-         */
-        DELETE_STYLESHEET: 'deleteStyleSheet',
-
-        /**
-         * Name of the operation that does nothing (internal use).
-         */
-        NOOP: 'noOp'
-
-    });
-
     // exports ================================================================
 
     return _.makeExtendable(OperationsGenerator);
