@@ -1,21 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/model/stylecollection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/editframework/utils/attributeutils'
-    ], function (Utils, ModelObject, AttributeUtils) {
+define('io.ox/office/editframework/model/stylecollection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/baseframework/model/modelobject',
+    'io.ox/office/editframework/utils/attributeutils'
+], function (Utils, TimerMixin, ModelObject, AttributeUtils) {
 
     'use strict';
 
@@ -62,6 +63,7 @@ define('io.ox/office/editframework/model/stylecollection',
      * @constructor
      *
      * @extends ModelObject
+     * @extends TimerMixin
      *
      * @param {EditApplication} app
      *  The root application instance.
@@ -84,10 +86,11 @@ define('io.ox/office/editframework/model/stylecollection',
      *      If set to false, this container is not allowed to contain style
      *      sheets. It can only be used to format DOM elements with explicit
      *      formatting attributes. DEPRECATED!
-     *  @param {String|Array} [initOptions.additionalFamilies]
-     *      The name of an additional attribute family, or an array with names
-     *      of additional attribute families supported by the style sheets of
-     *      this container instance.
+     *  @param {String} [initOptions.families]
+     *      The names of additional attribute families supported by the style
+     *      sheets of this container instance (space-separated). It is not
+     *      needed to repeat the name of the style family (as passed to the
+     *      'styleFamily' parameter) here.
      *  @param {Object} [initOptions.parentResolvers]
      *      The parent style families whose associated style sheets can contain
      *      attributes of the family supported by this style sheet container
@@ -195,9 +198,10 @@ define('io.ox/office/editframework/model/stylecollection',
             // custom resolver for explicit element attributes depending on a context element
             elementAttributesResolver = Utils.getFunctionOption(initOptions, 'elementAttributesResolver');
 
-        // base constructor ---------------------------------------------------
+        // base constructors --------------------------------------------------
 
         ModelObject.call(this, app);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
@@ -301,22 +305,6 @@ define('io.ox/office/editframework/model/stylecollection',
             return mergedAttributes;
         }
 
-        /**
-         * Returns the identifier of the style sheet that will be effectively
-         * used for the specified style sheet identifier.
-         *
-         * @param {String} styleId
-         *  The unique identifier of a style sheet.
-         *
-         * @returns {String}
-         *  The passed style sheet identifier, if such a style sheet exists in
-         *  this collection, otherwise the current default style sheet
-         *  identifier if it is valid.
-         */
-        function getEffectiveStyleId(styleId) {
-            return (!(styleId in styleSheets) && (defaultStyleId in styleSheets)) ? defaultStyleId : styleId;
-        }
-
         /**
          * Returns the attributes from the specified style sheet and its parent
          * style sheets. Does not add default attribute values, or attributes
@@ -372,7 +360,7 @@ define('io.ox/office/editframework/model/stylecollection',
             }
 
             // fall-back to default style sheet if passed identifier is invalid
-            styleId = getEffectiveStyleId(styleId);
+            styleId = self.getEffectiveStyleId(styleId);
 
             // collect attributes from the style sheet and its parents
             collectStyleAttributes(styleSheets[styleId]);
@@ -552,6 +540,25 @@ define('io.ox/office/editframework/model/stylecollection',
             return defaultStyleId;
         };
 
+        /**
+         * Returns the identifier of the style sheet that will be effectively
+         * used for the specified style sheet identifier. If the specified
+         * style sheet does not exist, but this collection contains a default
+         * style sheet, its unique identifier will be returned instead.
+         * Otherwise, the passed style sheet identifier will be returned.
+         *
+         * @param {String} styleId
+         *  The unique identifier of a style sheet.
+         *
+         * @returns {String}
+         *  The passed style sheet identifier, if such a style sheet exists in
+         *  this collection, otherwise the identifier of the default style
+         *  sheet.
+         */
+        this.getEffectiveStyleId = function (styleId) {
+            return (!(styleId in styleSheets) && (defaultStyleId in styleSheets)) ? defaultStyleId : styleId;
+        };
+
         /**
          * Inserts a new style sheet into this container. An existing style
          * sheet with the specified identifier will be replaced.
@@ -791,7 +798,7 @@ define('io.ox/office/editframework/model/stylecollection',
         this.getStyleSheetAttributes = function (styleId) {
 
             var // the style sheet entry
-                styleSheet = styleSheets[getEffectiveStyleId(styleId)],
+                styleSheet = styleSheets[this.getEffectiveStyleId(styleId)],
                 // the cached style sheet attributes
                 mergedAttributes = _.isObject(styleSheet) ? styleSheet.mergedAttributes : null;
 
@@ -1220,8 +1227,7 @@ define('io.ox/office/editframework/model/stylecollection',
         // initialization -----------------------------------------------------
 
         // add additional attribute families passed in the options
-        supportedFamilies = Utils.getStringOption(initOptions, 'additionalFamilies') || Utils.getArrayOption(initOptions, 'additionalFamilies', []);
-        supportedFamilies = _.getArray(supportedFamilies);
+        supportedFamilies = Utils.getTokenListOption(initOptions, 'families', []);
         supportedFamilies.push(styleFamily);
         supportedFamilies = _.unique(supportedFamilies);
 
