@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Malte Timmermann <malte.timmermann@open-xchange.com>
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
@@ -91,8 +91,9 @@ define('io.ox/office/editframework/model/undomanager',
      *      UndoManager.enterUndoGroup().
      * - 'undogroup:close': After an undo group has been closed by the method
      *      UndoManager.enterUndoGroup().
-     * - 'changed:count': After the number of available undo/redo actions has
-     *      changed remotely.
+     * - 'change:count': After the number of available undo/redo actions has
+     *      changed. Event listeners receive the current number of available
+     *      undo and redo actions.
      *
      * @constructor
      *
@@ -162,6 +163,14 @@ define('io.ox/office/editframework/model/undomanager',
 
         // private methods ----------------------------------------------------
 
+        /**
+         * Triggers a 'change:count' event with the current number of available
+         * undo actions, and redo actions.
+         */
+        function triggerChangeCountEvent() {
+            self.trigger('change:count', self.getUndoCount(), self.getRedoCount());
+        }
+
         /**
          * Opens a new undo group. If no undo group is open yet, triggers an
          * 'undogroup:open' event.
@@ -227,6 +236,7 @@ define('io.ox/office/editframework/model/undomanager',
 
             actions.push(newAction);
             currentAction = actions.length;
+            triggerChangeCountEvent();
         }
 
         /**
@@ -235,9 +245,15 @@ define('io.ox/office/editframework/model/undomanager',
          * state according to the passed state of the remote undo manager.
          */
         function remoteUpdateHandler(data) {
-            remoteUndoCount = Utils.getIntegerOption(data, 'undoCount', remoteUndoCount);
-            remoteRedoCount = Utils.getIntegerOption(data, 'redoCount', remoteRedoCount);
-            self.trigger('change:count');
+
+            var newUndoCount = Utils.getIntegerOption(data, 'undoCount', remoteUndoCount),
+                newRedoCount = Utils.getIntegerOption(data, 'redoCount', remoteRedoCount);
+
+            if ((remoteUndoCount !== newUndoCount) || (remoteRedoCount !== newRedoCount)) {
+                remoteUndoCount = newUndoCount;
+                remoteRedoCount = newRedoCount;
+                triggerChangeCountEvent();
+            }
         }
 
         /**
@@ -279,8 +295,11 @@ define('io.ox/office/editframework/model/undomanager',
          *  A reference to this instance.
          */
         this.clearUndoActions = function () {
-            actions = [];
-            currentAction = 0;
+            if (actions.length > 0) {
+                actions = [];
+                currentAction = 0;
+                triggerChangeCountEvent();
+            }
             return this;
         };
 
@@ -418,6 +437,7 @@ define('io.ox/office/editframework/model/undomanager',
             return promise.always(function () {
                 processing = false;
                 self.trigger('undo:after');
+                triggerChangeCountEvent();
             });
         };
 
@@ -467,6 +487,7 @@ define('io.ox/office/editframework/model/undomanager',
             return promise.always(function () {
                 processing = false;
                 self.trigger('redo:after');
+                triggerChangeCountEvent();
             });
         };
 
@@ -474,9 +495,12 @@ define('io.ox/office/editframework/model/undomanager',
 
         // remote undo stack: listen to update push messages from application
         if (remoteUndo) {
-            app.on('docs:update', remoteUpdateHandler);
+            this.listenTo(app, 'docs:update', remoteUpdateHandler);
         }
 
+        // initial notification about undo stack size
+        app.onImportSuccess(triggerChangeCountEvent);
+
     } // class UndoManager
 
    // exports =================================================================
