@@ -1,17 +1,19 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/editframework/utils/attributeutils', ['io.ox/office/tk/utils'], function (Utils) {
+define('io.ox/office/editframework/utils/attributeutils', [
+    'io.ox/office/tk/utils'
+], function (Utils) {
 
     'use strict';
 
@@ -19,7 +21,7 @@ define('io.ox/office/editframework/utils/attributeutils', ['io.ox/office/tk/util
 
     var AttributeUtils = {};
 
-    // methods ----------------------------------------------------------------
+    // public methods ---------------------------------------------------------
 
     /**
      * Returns whether the passed string is the name of a attribute that is
@@ -51,6 +53,71 @@ define('io.ox/office/editframework/utils/attributeutils', ['io.ox/office/tk/util
         return (name in definitions) && (special || (definitions[name].special !== true));
     };
 
+    /**
+     * Returns whether the passed (incomplete) attribute set contains the
+     * specified attribute, regardless of its value.
+     *
+     * @param {Object} attributeSet
+     *  An (incomplete) attribute set to be checked for an attribute.
+     *
+     * @param {String} family
+     *  The family of the attribute to be checked.
+     *
+     * @param {String} name
+     *  The name of the attribute to be checked.
+     *
+     * @returns {Boolean}
+     *  Whether the passed attribute set contains the specified attribute.
+     */
+    AttributeUtils.hasAttribute = function (attributeSet, family, name) {
+        return _.isObject(attributeSet[family]) && (name in attributeSet[family]);
+    };
+
+    /**
+     * Inserts a new attribute, or updates an existing attribute in the passed
+     * (incomplete) attribute set.
+     *
+     * @param {Object} attributeSet
+     *  An (incomplete) attribute set to be changed.
+     *
+     * @param {String} family
+     *  The family of the attribute to be changed.
+     *
+     * @param {String} name
+     *  The name of the attribute to be changed.
+     *
+     * @param {Any} value
+     *  The new value of the attribute. The value null will be inserted too,
+     *  see method AttributeUtils.deleteAttribute() to delete an attribute from
+     *  the attribute set.
+     *
+     * @param {Boolean} [missing=false]
+     *  If set to true, only missing attributes will be inserted; existing
+     *  attributes will not be changed.
+     */
+    AttributeUtils.insertAttribute = function (attributeSet, family, name, value, missing) {
+        var attributes = _.isObject(attributeSet[family]) ? attributeSet[family] : (attributeSet[family] = {});
+        if (!missing || !(name in attributes)) { attributes[name] = value; }
+    };
+
+    /**
+     * Deletes an attribute from the passed (incomplete) attribute set.
+     *
+     * @param {Object} attributeSet
+     *  An (incomplete) attribute set to be changed.
+     *
+     * @param {String} family
+     *  The family of the attribute to be deleted.
+     *
+     * @param {String} name
+     *  The name of the attribute to be deleted.
+     */
+    AttributeUtils.deleteAttribute = function (attributeSet, family, name) {
+        if (_.isObject(attributeSet[family])) {
+            delete attributeSet[family][name];
+        }
+    };
+
     /**
      * Returns the attribute set containing all explicit attributes stored in
      * the passed DOM element.
@@ -67,9 +134,9 @@ define('io.ox/office/editframework/utils/attributeutils', ['io.ox/office/tk/util
      *      attribute set object with all attributes mapped by their family.
      *  @param {Boolean} [options.direct=false]
      *      If set to true, the returned attribute set will be a reference to
-     *      the original map stored in the attribute, which MUST NOT be
-     *      modified. By default, a deep clone of the attribute set that can be
-     *      modified freely will be returned.
+     *      the original map stored in the passed DOM node, which MUST NOT be
+     *      modified! By default, a deep clone of the attribute set will be
+     *      returned that can be freely modified.
      *
      * @returns {Object}
      *  The (incomplete) attribute set if existing, otherwise an empty object.
