@@ -1,17 +1,17 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Carsten Driesner <carsten.driesner@open-xchange.com>
  */
 
-define('io.ox/office/editframework/utils/color', ['io.ox/office/tk/utils'], function (Utils) {
+define('io.ox/office/editframework/utils/color', ['io.ox/office/tk/utils', 'io.ox/office/tk/canvaswrapper'], function (Utils, CanvasWrapper) {
 
     'use strict';
 
@@ -219,7 +219,29 @@ define('io.ox/office/editframework/utils/color', ['io.ox/office/tk/utils'], func
                 crgb.a = Utils.mixMax(crgb.a * alphaMod / 100000, 0, 1);
                 return crgb;
             }
-        };
+        },
+
+        /**
+         * draws system color string as fillstyle in a canvas, to get its rgb values
+         *
+         * @param {String} System color name
+         *
+         * @return {String} hex string of rgb
+         */
+        getSystemRGBColor = (function() {
+            var cvs = new CanvasWrapper();
+            cvs.initialize({ width: 1, height: 1 });
+
+            return function(name) {
+                cvs.render(function (ctx) {
+                    ctx.setGlobalAlpha(1);
+                    ctx.setFillStyle({style: name});
+                    ctx.drawRect(0, 0, 1, 1);
+                });
+                var imgData = cvs.getImageData();
+                return ('00000' + (imgData[0] * 0x10000 + imgData[1] * 0x100 + imgData[2]).toString(16)).substr(-6);
+            };
+        }());
 
     // private static functions ===============================================
 
@@ -403,9 +425,7 @@ define('io.ox/office/editframework/utils/color', ['io.ox/office/tk/utils'], func
         var entry = SYSTEM_COLORS[lower];
         if (entry) {
             if (!entry.rgb) {
-                var div = $('div');
-                div.css('color', lower);
-                entry.rgb = div.css('color');
+                entry.rgb = getSystemRGBColor(entry.name);
             }
             return entry.rgb;
         }
