@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -50,6 +50,13 @@ define('io.ox/office/editframework/utils/editconfig', ['io.ox/office/tk/config']
      * @constant
      */
     EditConfig.LOG_ERROR_DATA = EditConfig.getFlag('logErrorData', false);
+    
+    /**
+     * A Boolean flag specifying whether the Document Name edit field should enabled or
+     * disabled. This ensures that in environments, which don't support transparent
+     * changes of the file name, don't break an active RT connection.
+     */
+    EditConfig.RENAME_DISABLED = EditConfig.getFlag('renamedisabled', false);
 
     // exports ================================================================
 
