@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -16,8 +16,10 @@ define('io.ox/office/editframework/view/edittoolpane',
      'io.ox/office/tk/forms',
      'io.ox/office/baseframework/view/toolpane',
      'io.ox/office/baseframework/view/toolbar',
-     'io.ox/office/baseframework/view/basecontrols'
-    ], function (Utils, Forms, ToolPane, ToolBar, BaseControls) {
+     'io.ox/office/editframework/view/editlabels',
+     'io.ox/office/editframework/view/editcontrols',
+     'gettext!io.ox/office/editframework'
+    ], function (Utils, Forms, ToolPane, ToolBar, Labels, Controls, gt) {
 
     'use strict';
 
@@ -43,6 +45,9 @@ define('io.ox/office/editframework/view/edittoolpane',
         var // self reference
             self = this,
 
+            // view reference
+            view = app.getView(),
+
             // the tool bars, mapped by tab identifier
             toolBarsMap = {},
 
@@ -51,7 +56,25 @@ define('io.ox/office/editframework/view/edittoolpane',
             centerAreaNode = null,
 
             // state of "paneLayoutHandler" (to prevent multiple calls)
-            reLayout = false;
+            reLayout = false,
+            // store for last known pane width
+            paneWidthStore = 0,
+            // possibility to force invoking the paneLayoutHandler
+            forceLayoutHandler = false,
+
+            // the new menu including the tab buttons to activate the different edit tool bars, as drop-down list
+            flatTabView = null,
+            // new document close button for flat-pane
+            closeBtn = null,
+            // new search button for the new flat-pane-menu
+            searchBtn = null,
+
+            // reference to the topPane
+            topPane = view.getTopPane(),
+            // reference to the view-menu-group
+            viewMenuGroup = topPane.getViewMenuGroup(),
+            // reference to the view-menu
+            viewMenu = viewMenuGroup.getMenu();
 
         // base constructor ---------------------------------------------------
 
@@ -74,7 +97,7 @@ define('io.ox/office/editframework/view/edittoolpane',
                     if (_.isObject(toolBarInfo.shrinkedToolBar)) {
                         toolBarInfo.toolBar.toggle(state && toolBarInfo.visible);
 
-                        if (toolBarInfo.shrinked && toolBarInfo.shrinkedMenu.getComponent() && toolBarInfo.shrinkedMenu.getComponent().hasVisibleGroups()) {
+                        if (toolBarInfo.shrinked && toolBarInfo.shrinkedMenu.hasVisibleGroups()) {
                             toolBarInfo.shrinkedToolBar.toggle(state && toolBarInfo.visible);
                         }else {
                             toolBarInfo.shrinkedToolBar.hide();
@@ -87,13 +110,11 @@ define('io.ox/office/editframework/view/edittoolpane',
             }
         }
 
-
-
         /**
          * Updates the layout of tool bars according to available space in the
          * tool pane.
          */
-        function paneLayoutHandler() {
+        var paneLayoutHandler = this.createDebouncedMethod(Utils.NOOP, function () {
             // check if another process/trigger is running this already.
             // cancel in this case, to prevent flashing controls
             if (reLayout) { return; }
@@ -101,32 +122,40 @@ define('io.ox/office/editframework/view/edittoolpane',
 
             var // pane width
                 paneWidth = centerWrapperNode.width(),
-                // toolbars width
+                // tool bars width
                 toolbarsWidth = centerAreaNode.outerWidth(),
                 // overflow size of the center tool bar area
                 overflowWidth = toolbarsWidth - paneWidth,
-                // all toolbars of the active tab
+                // all tool bars of the active tab
                 toolbars = _.sortBy(toolBarsMap[toolBarTabs.getActiveTabId()], _.property('priority')),
-                // shrinkable toolbars
-                shrinkableToolbars = _.filter(toolbars, function(bar){ return _.has(bar, 'shrinkedToolBar'); }),
-                // get the number of shrinked toolbars
-                countShrinked = _.filter(toolbars, function(bar){ return bar.shrinked; }).length,
-                // hideable toolbars
-                hideableToolbars = _.filter(toolbars, function(bar){ return (bar.hideable === true); }),
-                // get the number of hidden toolbars
-                countHidden = _.filter(toolbars, function(bar){ return bar.hidden; }).length,
+                // shrinkable tool bars
+                shrinkableToolbars = _.filter(toolbars, function (bar) { return _.has(bar, 'shrinkedToolBar'); }),
+                // get the number of shrunken tool bars
+                countShrinked = _.filter(toolbars, function (bar) { return bar.shrinked; }).length,
+                // hideable tool bars
+                hideableToolbars = _.filter(toolbars, function (bar) { return (bar.hideable === true); }),
+                // get the number of hidden tool bars
+                countHidden = _.filter(toolbars, function (bar) { return bar.hidden; }).length,
                 // stop acting
                 forceStop = false;
 
+            // if the width hasn't changed and no control is out of the visible range
+            // and no force is requested - get out here. Do not repaint the panes
+            if (paneWidthStore === paneWidth && overflowWidth < 0 && !forceLayoutHandler) {
+                reLayout = false;
+                return;
+            } else {
+                paneWidthStore = paneWidth;
+            }
 
             // catch new widths
-            function resetDimensions(){
+            function resetDimensions() {
                 // go on, if the node isn't visible
                 if (centerAreaNode.outerWidth() === 0) { return false; }
 
                 // pane width
                 paneWidth = centerWrapperNode.width();
-                // toolbars width
+                // tool bars width
                 toolbarsWidth = centerAreaNode.outerWidth();
                 // overflow size of the center tool bar area
                 return toolbarsWidth - paneWidth;
@@ -141,21 +170,21 @@ define('io.ox/office/editframework/view/edittoolpane',
              * @param {Boolean} visible
              *  whether add or remove the classes
              */
-            function toolbarClass(addClass, visible){
-                var node = app.getView().getToolPane().getNode();
+            function toolbarClass(addClass, visible) {
+                var node = view.getToolPane().getNode();
                     node.toggleClass(addClass, visible);
             }
 
             /**
-             * add/remove class "vertical-list" to/from popup
+             * add/remove class "vertical-list" to/from pop-up
              *
              * @param {Integer} i
-             *  which of the shrinkable toolbars
+             *  which of the shrinkable tool bars
              *
              * @param {Boolean} state
              *  whether add or remove the classes
              */
-            function makeVerticalList(i, state){
+            function makeVerticalList(i, state) {
                 var popupWrapper = shrinkableToolbars[i].shrinkedMenu.getNode();
                     popupWrapper.toggleClass('vertical-list drop-down', state);
             }
@@ -164,31 +193,32 @@ define('io.ox/office/editframework/view/edittoolpane',
              * moves the toolbar from pane to drop-down or reversed
              *
              * @param {Integer} i
-             *  which of the shrinkable toolbars
+             *  which of the shrinkable tool bars
              *
              * @param {String} type
-             *  whether shrink or unshrink = move to default pane or to small drop-down
+             *  whether shrink or expand = move to default pane or to small drop-down
              */
-            function moveToolBar(i, type){
+            function moveToolBar(i, type) {
+
                 if (type === 'shrink' && self.hasViewComponent(shrinkableToolbars[i].toolBar)) {
                     // remove component from pane
                     self.removeViewComponent(shrinkableToolbars[i].toolBar);
-                    // add component to shrinked (drop-down) menu
-                    shrinkableToolbars[i].shrinkedMenu.addComponent(shrinkableToolbars[i].toolBar);
+                    // add component to shrunken (drop-down) menu
+                    shrinkableToolbars[i].shrinkedMenu.replaceComponent(shrinkableToolbars[i].toolBar);
 
                     // go through all groups
-                    shrinkableToolbars[i].toolBar.iterateGroups(function(group){
+                    shrinkableToolbars[i].toolBar.iterateGroups(function (group) {
                         // and, if exists a menu
-                        if (_.has(group, 'getMenu')) {
-                            // add the components to the focusable nodes (to prevent closing the popup)
+                        if (_.isFunction(group.getMenu)) {
+                            // add the components to the focusable nodes (to prevent closing the pop-up)
                             shrinkableToolbars[i].shrinkedMenu.registerFocusableNodes(group.getMenu().getContentNode());
                         }
-                        // show Controls, which should only be shown in dropdown-version
-                        group.dropDownVisible(true);
+                        // show controls which should only be shown in drop-down version
+                        group.toggleDropDownMode(true);
                     });
 
                     // maximize all groups
-                    shrinkableToolbars[i].toolBar.iterateGroups(function(group){
+                    shrinkableToolbars[i].toolBar.iterateGroups(function (group) {
                         group.deactivateSmallVersion();
                     });
 
@@ -198,22 +228,21 @@ define('io.ox/office/editframework/view/edittoolpane',
 
 
 
-                } else if(type === 'unshrink' && !self.hasViewComponent(shrinkableToolbars[i].toolBar)) {
-                    // add Component to default pane
-                    self.addViewComponent(shrinkableToolbars[i].shrinkedMenu.getComponent(), { insertBefore: shrinkableToolbars[i].shrinkedToolBar });
-                    // remove component from shrinked drop-down menu
-                    shrinkableToolbars[i].shrinkedMenu.removeComponent(false);
+                } else if (type === 'unshrink' && !self.hasViewComponent(shrinkableToolbars[i].toolBar)) {
 
-                    // minify all groups
-                    Utils.iterateArray(shrinkableToolbars, function(toolbar){
-                        if (toolbar.shrinked === false) {
-                            toolbar.toolBar.iterateGroups(function(group){
-                                group.activateSmallVersion();
-                                // hide Controls, which should only be shown in dropdown-version
-                                group.dropDownVisible(false);
-                            });
-                        }
-                    });
+                    // remove component from shrunken drop-down menu
+                    var component = shrinkableToolbars[i].shrinkedMenu.releaseComponent();
+                    // add view component to default pane
+                    self.addViewComponent(component, { insertBefore: shrinkableToolbars[i].shrinkedToolBar });
+
+                    // minimize all groups
+                    if (shrinkableToolbars[i].shrinked === true) {
+                        shrinkableToolbars[i].toolBar.iterateGroups(function (group) {
+                            group.activateSmallVersion();
+                            // hide controls which should only be shown in drop-down version
+                            group.toggleDropDownMode(false);
+                        });
+                    }
 
                     // remove vertical-design
                     makeVerticalList(i, false);
@@ -225,25 +254,31 @@ define('io.ox/office/editframework/view/edittoolpane',
             }
 
 
-            app.getView().lockPaneLayout(function(){
+            view.lockPaneLayout(function() {
                 // returns whether the toolbar(s) are too big for the pane
-                function tooBig(){
+                function tooBig() {
                     overflowWidth = resetDimensions();
-                    return (overflowWidth > 0);
+                    if (overflowWidth > 0) {
+                        // reset force-option to stop invoking paneLayoutHandler here
+                        forceLayoutHandler = false;
+                        return true;
+                    } else {
+                        return false;
+                    }
                 }
                 // returns whether the toolbar(s) fit in the pane
-                function fitIn(){
+                function fitIn() {
                     overflowWidth = resetDimensions();
                     return (overflowWidth < 0);
                 }
                 // returns the sum of all shrinked toolbars of this pane
-                function getCountShrinked(){
-                    countShrinked = _.filter(toolBarsMap[toolBarTabs.getActiveTabId()], function(bar){ return bar.shrinked; }).length;
+                function getCountShrinked() {
+                    countShrinked = _.filter(toolBarsMap[toolBarTabs.getActiveTabId()], function(bar) { return bar.shrinked; }).length;
                     return countShrinked;
                 }
                 // returns the sum of all hidden toolbars of this pane
-                function getCountHidden(){
-                    countHidden = _.filter(toolBarsMap[toolBarTabs.getActiveTabId()], function(bar){ return bar.hidden; }).length;
+                function getCountHidden() {
+                    countHidden = _.filter(toolBarsMap[toolBarTabs.getActiveTabId()], function(bar) { return bar.hidden; }).length;
                     return countHidden;
                 }
 
@@ -254,17 +289,20 @@ define('io.ox/office/editframework/view/edittoolpane',
 
                     // activate small versions of groups
                     if ( tooBig() ) {
-                        Utils.iterateArray(toolbars, function(toolbar){
-                            toolbar.toolBar.iterateGroups(function(group){
-                                group.activateSmallVersion();
-                            });
-                            if (fitIn()) { return Utils.BREAK; }
+                        Utils.iterateArray(toolbars, function(toolbar) {
+                            // only activate the small version of the groups, if their were not shrinked
+                            if (!toolbar.shrinked) {
+                                toolbar.toolBar.iterateGroups(function(group) {
+                                    group.activateSmallVersion();
+                                });
+                                if (fitIn()) { return Utils.BREAK; }
+                            }
                         }, {reverse: true});
 
                         // shrink complete toolbar to dropdown-menu
                         if (tooBig()) {
                             // iterate over all toolbars (reversed)
-                            Utils.iterateArray(shrinkableToolbars, function(toolbar, i){
+                            Utils.iterateArray(shrinkableToolbars, function(toolbar, i) {
                                 if (toolbar.shrinked) { return; }
 
                                 if (i===(shrinkableToolbars.length-1) || (i < (shrinkableToolbars.length-1) && shrinkableToolbars[(i+1)].shrinked)) {
@@ -278,7 +316,7 @@ define('io.ox/office/editframework/view/edittoolpane',
                             // hide toolbars (last step to fit in the given area)
                             if (tooBig()) {
                                 // iterate backwards through all hideable toolbars
-                                Utils.iterateArray(hideableToolbars, function(toolbar, i){
+                                Utils.iterateArray(hideableToolbars, function(toolbar, i) {
                                     if (toolbar.hidden) { return; }
 
                                     if (i===(hideableToolbars.length-1) || (i < (hideableToolbars.length-1) && hideableToolbars[(i+1)].hidden)) {
@@ -298,7 +336,7 @@ define('io.ox/office/editframework/view/edittoolpane',
                 } else {
 
                     // show hidden toolbars
-                    Utils.iterateArray(hideableToolbars, function(toolbar, i){
+                    Utils.iterateArray(hideableToolbars, function(toolbar, i) {
                         if (!toolbar.hidden) { return; }
 
                         if (i===0 || (i > 0 && !hideableToolbars[(i-1)].hidden)) {
@@ -320,7 +358,7 @@ define('io.ox/office/editframework/view/edittoolpane',
                     if (fitIn() && forceStop === false && getCountHidden() === 0) {
 
                         // unshrink toolbars
-                        Utils.iterateArray(shrinkableToolbars, function(toolbar, i){
+                        Utils.iterateArray(shrinkableToolbars, function(toolbar, i) {
                             if (!toolbar.shrinked) { return; }
 
                             // toolbar have to be shrinked
@@ -347,33 +385,20 @@ define('io.ox/office/editframework/view/edittoolpane',
                         // maximize groups
                         if (fitIn() && forceStop === false && getCountShrinked() === 0) {
                             // iterate through all groups of all toolbars
-                            Utils.iterateArray(toolbars, function(toolbar){
-                                toolbar.toolBar.iterateGroups(function(group){
+                            Utils.iterateArray(toolbars, function(toolbar) {
+                                toolbar.toolBar.iterateGroups(function(group) {
                                     group.deactivateSmallVersion();
                                 });
 
                                 // if it was too much, revert to last state
                                 if (tooBig()) {
-                                    toolbar.toolBar.iterateGroups(function(group){
+                                    toolbar.toolBar.iterateGroups(function(group) {
                                         group.activateSmallVersion();
                                     });
                                     forceStop = true;
                                     return Utils.BREAK;
                                 }
                             });
-
-
-                            // if the overflow now is present
-                            if (fitIn() && forceStop === false) {
-                                // reset the spaces between the controls to default padding (13px)
-                                toolbarClass('small-distance', false);
-
-                                // if it was too much, revert to last state
-                                if (tooBig()) {
-                                    // shrink the space again
-                                    toolbarClass('small-distance', true);
-                                }
-                            }
                         }
                     }
                 }
@@ -384,17 +409,74 @@ define('io.ox/office/editframework/view/edittoolpane',
 
             // reset status (actually not in layout-progess)
             reLayout = false;
+        }, { delay: 500 });
+
+        /**
+         *  Checks the height of the window-node and (de-)activates the flat-pane
+         */
+        function setPaneHeight() {
+            var isStandAlone    = app.isStandalone();
+
+            self.getNode().toggleClass('flat-pane', true);  // add special flat-pane css-class
+
+            viewMenu.iterateGroups(function (group) {
+                // add sub-menus to focusable nodes, so that the parent-menu don't close automatically
+                if (_.isFunction(group.getMenu)) {
+                    flatTabView.getMenu().registerFocusableNodes(group.getMenu().getNode());
+                }
+
+                // check whether something should happen with this view-menu-group if its moved to the flatPane
+                var options = group.getOptions();
+                if (_.has(options, 'flatPane') && options.flatPane === 'hide') {
+                    group.hide();
+                }
+            });
+
+            topPane.hide();                                 // hide top pane (tab bar)
+            flatTabView.show();                             // show new tab menu (in leading container of tool pane)
+            closeBtn.toggle(!isStandAlone);                 // show new close button (in trailing container of tool pane) if standAloneMode is off
+            self.show();                                    // make tool pane visible (maybe the user has made the pane invisible)
         }
 
+        function tabCreateHandler(event, tabId, options) {
+
+            var tabButton = new Controls.Button({
+                    label: Utils.getStringOption(options, 'label', ''),
+                    value: tabId,
+                    highlight: function (value) { return value === tabId; }
+                });
+
+            flatTabView.addGroup('view/toolbars/tab', tabButton, {
+                visibleKey: options.visibleKey,
+                insertBefore: searchBtn.getNode(),
+                saveSeparator: true
+            });
+        }
+
+        /**
+         *  Sets the correct label to the dynamic filled "drawing-tab"
+         */
+        function tabLabelHandler(event, tabId, label) {
+            flatTabView.getMenu().iterateGroups(function (group) {
+                var options = group.getOptions();
+                if (options.value === tabId) {
+                    group.setLabel(label);
+                    return Utils.BREAK;
+                }
+            });
+        }
 
         /**
          * Updates the visibility of the tool bars according to the active tab.
          */
         function tabActivateHandler(event, newTabId, oldTabId) {
-            app.getView().lockPaneLayout(function () {
+            view.lockPaneLayout(function () {
                 toggleToolBars(oldTabId, false);
                 if (_.isString(newTabId)) { self.show(); }
                 toggleToolBars(newTabId, true);
+                // the changing of the tab invokes paneLayoutHandler by force
+                forceLayoutHandler = true;
+                paneLayoutHandler();
             });
         }
 
@@ -415,6 +497,50 @@ define('io.ox/office/editframework/view/edittoolpane',
             toggleToolBars(toolBarTabs.getActiveTabId(), true);
         }
 
+        /**
+          * Prepare shrunken version of the toolbar
+          */
+        function addShrinkedVersion(options) {
+
+            var // use SplitButton
+                splitBtn = Utils.getOption(options, 'splitBtn', false),
+                // controller key for split-button
+                controllerKey = null,
+                // shrunken menu
+                smenu = null,
+                // shrunken object (includes the toolbar and the menu)
+                shrinkObj = {};
+
+            // without split button in toolbar
+            if (splitBtn === false) {
+                smenu = new Controls.CompoundButton(app, {
+                    tooltip:    Utils.getStringOption(options, 'tooltip', ''),
+                    label:      Utils.getStringOption(options, 'label', ''),
+                    icon:       Utils.getStringOption(options, 'icon', ''),
+                    classes:    Utils.getStringOption(options, 'classes', '')
+                });
+
+            // with split button in toolbar
+            } else {
+                smenu = new Controls.CompoundSplitButton(app, Utils.extendOptions({
+                    tooltip:        Utils.getStringOption(options, 'tooltip', ''),
+                    caretTooltip:   Utils.getStringOption(options, 'caretTooltip', ''),
+                    label:          Utils.getStringOption(options, 'label', ''),
+                    icon:           Utils.getStringOption(options, 'icon', ''),
+                    classes:        Utils.getStringOption(options, 'classes', '')
+                }, Utils.getOption(options.splitBtn, 'options')));
+
+                controllerKey = Utils.getStringOption(options.splitBtn, 'key');
+            }
+
+            shrinkObj.shrinkedMenu = smenu.getMenu();
+            shrinkObj.shrinkedToolBar = new ToolBar(app).addGroup(controllerKey, smenu).hide();
+
+            self.addViewComponent(shrinkObj.shrinkedToolBar);
+
+            return shrinkObj;
+        }
+
         // methods ------------------------------------------------------------
 
         /**
@@ -435,10 +561,9 @@ define('io.ox/office/editframework/view/edittoolpane',
          *      of the respective controller item. If omitted, the tool bar
          *      will always be visible.
          *  @param {Boolean} [options.withShrink=false]
-         *      Prepare a (smaller) dropdown menu where all the controls will
+         *      Prepare a (smaller) drop-down menu where all the controls will
          *      be moved to, if there isn't enough free space for the normal
-         *      toolbar.
-         *      Special logic for mobile-ui.
+         *      toolbar. Special logic for mobile UI.
          *
          * @returns {ToolBar}
          *  The new tool bar instance.
@@ -447,46 +572,8 @@ define('io.ox/office/editframework/view/edittoolpane',
 
             var // the new tool bar instance
                 toolBar = new ToolBar(app, options),
-                // create shrinked toolbar
-                withShrink = Utils.getBooleanOption(options, 'prepareShrink', false),
-                // shrinked menu
-                smenu = null,
-                // shrinked object (includes the toolbar and the menu)
-                shrinkObj = {},
-                // use SplitButton
-                splitBtn = Utils.getOption(options, 'splitBtn', false),
-                // controller key for split-button
-                controllerKey = null;
-
-            // prepare shrinked toolbar
-            if (withShrink) {
-                // without split button in toolbar
-                if (splitBtn === false) {
-                    smenu = new BaseControls.ComponentMenuButton(app, {
-                        tooltip:    Utils.getStringOption(options, 'tooltip', ''),
-                        label:      Utils.getStringOption(options, 'label', ''),
-                        icon:       Utils.getStringOption(options, 'icon', ''),
-                        classes:    Utils.getStringOption(options, 'classes', '')
-                    });
-
-                // with split button in toolbar
-                } else {
-                    smenu = new BaseControls.ComponentMenuSplitButton(app, Utils.extendOptions({
-                        tooltip:        Utils.getStringOption(options, 'tooltip', ''),
-                        caretTooltip:   Utils.getStringOption(options, 'caretTooltip', ''),
-                        label:          Utils.getStringOption(options, 'label', ''),
-                        icon:           Utils.getStringOption(options, 'icon', ''),
-                        classes:        Utils.getStringOption(options, 'classes', '')
-                    }, Utils.getOption(options.splitBtn, 'options')));
-
-                    controllerKey = Utils.getStringOption(options.splitBtn, 'key');
-                }
-
-                shrinkObj.shrinkedMenu = smenu.getMenu();
-                shrinkObj.shrinkedToolBar = new ToolBar(app).addGroup(controllerKey, smenu).hide();
-
-                this.addViewComponent(shrinkObj.shrinkedToolBar);
-            }
+                // prepare shrunken toolbar
+                shrinkObj = (Utils.getBooleanOption(options, 'prepareShrink', false)) ? addShrinkedVersion(options) : {};
 
             // set to hidden state initially
             toolBar.hide();
@@ -496,10 +583,10 @@ define('io.ox/office/editframework/view/edittoolpane',
                 toolBar: toolBar,                                                   // normal toolbar
                 visibleKey: Utils.getStringOption(options, 'visibleKey'),           // toolbar visible at
                 visible: true,                                                      // toolbar visibility
-                priority: Utils.getIntegerOption(options, 'priority', null),        // toolbar priority (1 = high prio, 10 = low)
+                priority: Utils.getIntegerOption(options, 'priority', null),        // toolbar priority (1: high, 10: low)
                 hideable: Utils.getBooleanOption(options, 'hideable', null),        // is toolbar hideable
                 hidden: false,                                                      // is toolbar hidden
-                shrinked: false                                                     // is toolbar shrinked
+                shrinked: false                                                     // is toolbar shrunken
             }, shrinkObj));
             this.addViewComponent(toolBar);
 
@@ -508,10 +595,33 @@ define('io.ox/office/editframework/view/edittoolpane',
 
         // initialization -----------------------------------------------------
 
+        // new menu for flat-pane (in leading-container of toolbar)
+        flatTabView = new Controls.CompoundButton(app, { icon: 'fa-bars', caret: false, anchorPadding: -3 });
+
+        // add static controls to new flat-pane-menu
+        flatTabView
+            .addSeparator()
+            .addGroup('view/searchpane', searchBtn = new Controls.Button({ icon: 'fa-search', label: gt('Search'), tooltip: gt('Toggle search'), toggle: true }), { visibleKey: 'app/valid' })
+            .addSeparator()
+            .addGroup('document/undo', new Controls.Button({ icon: 'docs-undo', tooltip: gt('Revert last operation') }), { visibleKey: 'document/editable', inline: true })
+            .addGroup('document/redo', new Controls.Button({ icon: 'docs-redo', tooltip: gt('Restore last operation') }), { visibleKey: 'document/editable', inline: true });
+
+        // create the tool bar containing the tab buttons and add it to the leading-container of the toolBar
+        this.addViewComponent(new ToolBar(app)
+            .addGroup(null, flatTabView.hide()),
+        { targetArea: 'leading' });
+
+        // create the toolbar containing the close button and add it to the trailing-container of the toolBar
+        this.addViewComponent(new ToolBar(app)
+            .addGroup('app/quit', closeBtn = new Controls.Button(Labels.QUIT_BUTTON_OPTIONS).hide()),
+        { targetArea: 'trailing' });
+
         // update layout of tool bars according to available space
         this.on('pane:layout', paneLayoutHandler);
 
-        // update visibility of the tool bars
+        // listen to tab events, update visibility of the tool bars
+        this.listenTo(toolBarTabs, 'tab:create', tabCreateHandler);
+        this.listenTo(toolBarTabs, 'tab:label', tabLabelHandler);
         this.listenTo(toolBarTabs, 'tab:activate', tabActivateHandler);
         this.listenTo(app.getController(), 'change:items', controllerChangeHandler);
 
@@ -526,6 +636,17 @@ define('io.ox/office/editframework/view/edittoolpane',
         centerWrapperNode = this.getAreaWrapperNode('center');
         centerAreaNode = centerWrapperNode.find('>.area-container');
 
+        // decide whether to show the combined tool/tab bar
+        if (view.panesCombined()){
+            setPaneHeight();
+        }
+        app.registerWindowResizeHandler(function(){
+            // only remove the 'small-distance' marker, if the width has changed
+            if (centerWrapperNode.width() !== paneWidthStore) {
+                app.getView().getToolPane().getNode().removeClass('small-distance');
+            }
+        });
+
         // destroy all class members on destruction
         this.registerDestructor(function () {
             app = toolBarTabs = self = toolBarsMap = centerWrapperNode = centerAreaNode = null;
