@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -17,24 +17,23 @@ define('io.ox/office/editframework/view/editview',
      'io.ox/office/baseframework/app/extensionregistry',
      'io.ox/office/baseframework/view/baseview',
      'io.ox/office/editframework/utils/editconfig',
+     'io.ox/office/editframework/view/editlabels',
      'io.ox/office/editframework/view/toolbartabcollection',
      'io.ox/office/editframework/view/toppane',
      'io.ox/office/editframework/view/edittoolpane',
      'io.ox/office/editframework/view/searchpane',
      'io.ox/office/editframework/view/operationspane',
      'io.ox/office/editframework/view/clipboardpane',
+     'io.ox/office/editframework/view/editlabels',
      'io.ox/office/editframework/view/editcontrols',
      'io.ox/office/editframework/view/editdialogs',
+     'io.ox/office/editframework/view/popup/userslayermenu',
      'gettext!io.ox/office/editframework',
      'less!io.ox/office/editframework/view/editstyle'
-    ], function (Utils, KeyCodes, ExtensionRegistry, BaseView, Config, ToolBarTabCollection, TopPane, EditToolPane, SearchPane, OperationsPane, ClipboardPane, EditControls, EditDialogs, gt) {
+    ], function (Utils, KeyCodes, ExtensionRegistry, BaseView, Config, EditLabels, ToolBarTabCollection, TopPane, EditToolPane, SearchPane, OperationsPane, ClipboardPane, Labels, Controls, Dialogs, UsersLayerMenu, gt) {
 
     'use strict';
 
-    var // class name shortcuts
-        Button = EditControls.Button,
-        CheckBox = EditControls.CheckBox;
-
     // class EditView =========================================================
 
     /**
@@ -59,30 +58,43 @@ define('io.ox/office/editframework/view/editview',
      *      (2) {ClipboardPane} clipboardPane
      *          The view pane with the HTML mark-up contained in the browser
      *          clipboard.
+     *      May return a promise to be able to run asynchronous code during
+     *      initialization.
      *  @param {Function} [initOptions.initGuiHandler]
      *      A callback handler function called to initialize the graphical
      *      elements of this view instance, after the document has been
      *      imported. Receives the following parameters:
-     *      (1) {ComponentMenuButton} viewMenuGroup
+     *      (1) {CompoundButton} viewMenuGroup
      *          The 'View' drop-down menu shown in the tool pane, containing
      *          all view settings for the document.
+     *      May return a promise to be able to run asynchronous code during
+     *      initialization.
      *  @param {Function} [initOptions.initDebugGuiHandler]
      *      A callback handler function called to add graphical elements for
      *      debugging purposes to this view instance. Will be called in debug
      *      mode ONLY. Will be called after the GUI initialization handler (see
      *      option 'initGuiHandler'). Receives the following parameters:
-     *      (1) {ComponentMenuButton} viewMenuGroup
+     *      (1) {CompoundButton} viewMenuGroup
      *          The 'View' drop-down menu shown in the tool pane, containing
      *          all view settings for the document.
-     *      (2) {ComponentMenuButton} actionMenuGroup
+     *      (2) {CompoundButton} actionMenuGroup
      *          The 'Actions' drop-down menu shown in the debug tool bar,
      *          containing action commands for debugging.
+     *      May return a promise to be able to run asynchronous code during
+     *      initialization.
      */
     function EditView(app, initOptions) {
 
         var // self reference
             self = this,
 
+            // beneath this min height, the tool-/tabbars should be combined
+            // Nexus 7 = min resolution 601
+            minHeight = 610,
+
+            // show combined Panes
+            panesCombined = false,
+
             // the tool bar tab collection
             toolBarTabs = new ToolBarTabCollection(app),
 
@@ -121,12 +133,17 @@ define('io.ox/office/editframework/view/editview',
                 // the clipboard pane (debug mode only)
                 clipboardPane = null;
 
+            // determine whether the panes are combined or not
+            if ((Utils.TOUCHDEVICE && Math.min(Utils.getScreenWidth(), Utils.getScreenHeight()) <= minHeight) || (Config.DEBUG && Config.getUrlFlag('office:panes-combined'))) {
+                panesCombined = true;
+            }
+
             // disable internal browser handles on tables and drawings after edit mode has changed
             self.listenTo(app.getModel(), 'change:editmode', function () { self.disableBrowserEditControls(); });
 
             // create the tool bars (initialize themselves internally)
             self.addPane(topPane = new TopPane(app, toolBarTabs));
-            self.addPane(toolPane = new EditToolPane(app, toolBarTabs, {landmark: false}));
+            self.addPane(toolPane = new EditToolPane(app, toolBarTabs, { landmark: false }));
             self.addPane(searchPane = new SearchPane(app).hide());
 
             // deactivate tool bar tabs while tool pane is hidden
@@ -137,6 +154,11 @@ define('io.ox/office/editframework/view/editview',
                 toolPane.hide();
             });
 
+            // update controller immediately when toggling the collaborator pop-up menu
+            this.listenTo(collaboratorPopup, 'popup:show popup:hide', function () {
+                app.getController().update();
+            });
+
             // initialize debug panes before other custom panes of the application
             if (Config.DEBUG) {
 
@@ -195,12 +217,14 @@ define('io.ox/office/editframework/view/editview',
             }
 
             // call initialization handler passed by sub class
-            Utils.getFunctionOption(initOptions, 'initHandler', $.noop).call(self);
+            var result = Utils.getFunctionOption(initOptions, 'initHandler', $.noop).call(self);
 
             // call debug initialization handler passed by sub class
             if (Config.DEBUG) {
-                Utils.getFunctionOption(initOptions, 'initDebugHandler', $.noop).call(self, operationsPane, clipboardPane);
+                result = $.when(result).then(_.bind(Utils.getFunctionOption(initOptions, 'initDebugHandler', $.noop), self, operationsPane, clipboardPane));
             }
+
+            return result;
         }
 
         /**
@@ -211,48 +235,69 @@ define('io.ox/office/editframework/view/editview',
          */
         function initGuiHandler() {
 
-            var // the text field for the file name
-                fileNameField = new EditControls.FileNameField(app),
-                // toolbar to rename the document
-                renameToolbar = null,
-                // toolbar to save the document
-                saveToolbar = null,
-                // toolbar to download, print or send the document
-                commonToolbar = null,
-                // toolbar for other actions
-                actionToolbar = null;
+            var
+                renameToolbar   = null,
+                saveToolbar     = null,
+                commonToolbar   = null,
+                actionToolbar   = null,
 
-            self.createToolBarTab('file', { label: EditControls.FILE_HEADER_LABEL, priority: 99 });
+                btnSaveAs           = new Controls.Button({ label: gt('Save as'), value: 'file' }),
+                btnSaveAsTemplate   = new Controls.Button({ label: gt('Save as template'), value: 'template' }),
+                btnDownload         = new Controls.Button({ icon: 'fa-cloud-download', tooltip: gt('Download'), classes: 'link-style', dropDownVersion: { label: gt('Download') } }),
+                btnPrint            = new Controls.Button({ icon: 'fa-print', tooltip: gt('Print as PDF'), classes: 'link-style', dropDownVersion: { label: gt('Print as PDF') } }),
+                btnSendAsMail       = new Controls.Button({ icon: 'fa-envelope-o', tooltip: gt('Send as mail'), classes: 'link-style', dropDownVersion: { label: gt('Send as mail') } }),
+                btnReload           = new Controls.Button({ icon: 'fa-repeat', label: gt('Reload'), tooltip: gt('Reload document'), classes: 'link-style' }),
+                btnAcquireEdit      = new Controls.AcquireEditButton(app, { classes: 'link-style' }),
 
-            renameToolbar   = self.createToolBar('file');
-            saveToolbar     = self.createToolBar('file', {priority: 1});
-            commonToolbar   = self.createToolBar('file', {priority: 3, prepareShrink: true, label: gt('Actions'), classes: 'link-style' });
-            actionToolbar   = self.createToolBar('file', {priority: 2, tooltip: gt('actions'), classes: 'link-style' });
+                cBoxAutosave        = new Controls.CheckBox({ label: gt('AutoSave') }),
 
-            renameToolbar
-                .addGroup('document/rename', fileNameField);
+                fileNameField       = new Controls.FileNameField(app);
 
-            saveToolbar
-                .addGroup(null, new EditControls.ComponentMenuButton(app, { label: gt('Save in Drive'), classes: 'link-style', smallerVersion:{ icon: 'fa-save', hideLabel: true } })
-                    .addGroup('document/saveas/dialog', new Button({ label: gt('Save as'), value: 'file' }))
-                    .addGroup('document/saveas/dialog', new Button({ label: gt('Save as template'), value: 'template' }))
-                    .addSeparator()
-                    .addGroup('document/autosave', new CheckBox({ label: gt('AutoSave') }))
-                );
 
+            // -----------------------------------------------------
+            // TABS
+            //      prepare all tabs (for normal or combined panes)
+            // -----------------------------------------------------
+            self.createToolBarTab('file', { label: Labels.FILE_HEADER_LABEL, priority: 99 });
+
+            // -----------------------------------------------------
+            // TOOLBARS
+            //      prepare all toolbars (for normal or combined panes)
+            // -----------------------------------------------------
+            if (!self.panesCombined()) {
+                renameToolbar   = self.createToolBar('file');
+                saveToolbar     = self.createToolBar('file', {priority: 2, visibleKey: 'app/bundled'});
+            }
+            commonToolbar       = self.createToolBar('file', {priority: 3, label: gt('Actions'), classes: 'link-style', prepareShrink: true });
+            actionToolbar       = self.createToolBar('file', {priority: 1, tooltip: gt('actions'), classes: 'link-style' });
+
+            // -----------------------------------------------------
+            // CONTROLS
+            //      add all controls
+            // -----------------------------------------------------
+            if (!self.panesCombined()) {
+                renameToolbar
+                    .addGroup('document/rename', fileNameField);
+
+                saveToolbar
+                    .addGroup(null, new Controls.CompoundButton(app, { label: gt('Save in Drive'), classes: 'link-style', smallerVersion:{ icon: 'fa-save', hideLabel: true } })
+                        .addGroup('document/saveas/dialog', btnSaveAs)
+                        .addGroup('document/saveas/dialog', btnSaveAsTemplate)
+                        .addSeparator()
+                        .addGroup('document/autosave', cBoxAutosave)
+                    );
+            }
             commonToolbar
-                .addGroup('document/download', new Button({ icon: 'fa-cloud-download', tooltip: gt('Download'), classes: 'link-style', dropDownVersion: { label: gt('Download') } }))
+                .addGroup('document/download', btnDownload)
                 .addGap()
-                .addGroup('document/print',    new Button({ icon: 'fa-print', tooltip: gt('Print as PDF'), classes: 'link-style', dropDownVersion: { label: gt('Print as PDF') } }))
+                .addGroup('document/print', btnPrint)
                 .addGap()
-                .addGroup('document/sendmail', new Button({ icon: 'fa-envelope-o', tooltip: gt('Send as mail'), classes: 'link-style', dropDownVersion: { label: gt('Send as mail') } }));
+                .addGroup('document/sendmail', btnSendAsMail, { visibleKey: 'app/bundled' });
 
             actionToolbar
-                .addGroup('document/acquireedit', new EditControls.AcquireEditButton(app, { classes: 'link-style' }), { visibleKey: 'document/acquireedit' })
-                .addGroup('document/reload', new Button({ icon: 'fa-repeat', label: gt('Reload'), tooltip: gt('Reload document'), classes: 'link-style' }), { visibleKey: 'document/reload' });
+                .addGroup('document/acquireedit', btnAcquireEdit, { visibleKey: 'document/acquireedit' })
+                .addGroup('document/reload', btnReload, { visibleKey: 'document/reload' });
 
-            // create collaborator popup
-            collaboratorPopup = new EditControls.UsersLayerMenu(app, gt('Collaborators'), { tooltip: gt('List of collaborators of this document') });
 
             // register double-click handler for the application launcher
             self.listenTo(app.getImportPromise(), 'done', function () {
@@ -266,14 +311,15 @@ define('io.ox/office/editframework/view/editview',
             });
 
             // call initialization handler passed by sub class
-            Utils.getFunctionOption(initOptions, 'initGuiHandler', $.noop).call(self, topPane.getViewMenuGroup());
+            var result = Utils.getFunctionOption(initOptions, 'initGuiHandler', $.noop).call(self, topPane.getViewMenuGroup());
 
             // debug initialization
-            if (Config.DEBUG) { initDebugGuiHandler(); }
+            if (Config.DEBUG) {
+                result = $.when(result).then(initDebugGuiHandler);
+            }
 
             // immediately activate the first available tool bar tab
-            toolBarTabs.activateFirstTab();
-
+            return $.when(result).done(function () { toolBarTabs.activateFirstTab(); });
         }
 
         /**
@@ -288,39 +334,39 @@ define('io.ox/office/editframework/view/editview',
 
             // add view options
             viewMenuGroup
-                .addHeaderLabel(_.noI18n('Debug'))
-                .addGroup('debug/operationspane', new CheckBox({ label: _.noI18n('Show operations panel') }))
-                .addGroup('debug/clipboardpane',  new CheckBox({ label: _.noI18n('Show clipboard panel') }))
-                .addGroup('debug/highlight',      new CheckBox({ label: _.noI18n('Highlight DOM elements') }))
-                .addGroup('debug/toolbars/white', new CheckBox({ label: _.noI18n('White toolbar panels') }));
+                .addSectionLabel(_.noI18n('Debug'))
+                .addGroup('debug/operationspane', new Controls.CheckBox({ label: _.noI18n('Show operations panel') }))
+                .addGroup('debug/clipboardpane',  new Controls.CheckBox({ label: _.noI18n('Show clipboard panel') }))
+                .addGroup('debug/highlight',      new Controls.CheckBox({ label: _.noI18n('Highlight DOM elements') }))
+                .addGroup('debug/toolbars/white', new Controls.CheckBox({ label: _.noI18n('White toolbar panels') }));
 
             // create the drop-down list for the debug actions
-            actionMenuGroup = new EditControls.ComponentMenuButton(app, { icon: 'fa-wrench', label: EditControls.ACTIONS_LABEL, tooltip: _.noI18n('Additional debug actions'), classes: 'link-style' })
+            actionMenuGroup = new Controls.CompoundButton(app, { icon: 'fa-wrench', label: EditLabels.ACTIONS_LABEL, tooltip: _.noI18n('Additional debug actions'), classes: 'link-style' })
                 // register realtime debug actions
-                .addHeaderLabel(_.noI18n('Offline mode'))
-                .addGroup('debug/realtime/trigger', new Button({ label: _.noI18n('Switch to offline mode'),      value: 'offline' }))
-                .addGroup('debug/realtime/trigger', new Button({ label: _.noI18n('Switch to online mode'),       value: 'online' }))
-                .addHeaderLabel(_.noI18n('Realtime connection'))
-                .addGroup('debug/realtime/trigger', new Button({ label: _.noI18n('Simulate connection reset'),   value: 'reset' }))
-                .addGroup('debug/realtime/trigger', new Button({ label: _.noI18n('Simulate connection timeout'), value: 'timeout' }))
-                .addGroup('debug/realtime/trigger', new Button({ label: _.noI18n('Simulate connection hangup'),  value: 'hangup' }))
+                .addSectionLabel(_.noI18n('Offline mode'))
+                .addGroup('debug/realtime/trigger', new Controls.Button({ label: _.noI18n('Switch to offline mode'),      value: 'offline' }))
+                .addGroup('debug/realtime/trigger', new Controls.Button({ label: _.noI18n('Switch to online mode'),       value: 'online' }))
+                .addSectionLabel(_.noI18n('Realtime connection'))
+                .addGroup('debug/realtime/trigger', new Controls.Button({ label: _.noI18n('Simulate connection reset'),   value: 'reset' }))
+                .addGroup('debug/realtime/trigger', new Controls.Button({ label: _.noI18n('Simulate connection timeout'), value: 'timeout' }))
+                .addGroup('debug/realtime/trigger', new Controls.Button({ label: _.noI18n('Simulate connection hangup'),  value: 'hangup' }))
                 // register debug actions for operations
-                .addHeaderLabel(_.noI18n('Operations'))
-                .addGroup('debug/operations/apply',              new Button({ label: _.noI18n('Apply operation with invalid name'), value: { name: '_invalid_operation_' } }))
-                .addGroup('debug/operations/apply',              new Button({ label: _.noI18n('Apply operation with invalid position'), value: { name: 'insertText', start: [987654321, 0], text: 'test' } }))
-                .addGroup('debug/realtime/sendinvalidoperation', new Button({ label: _.noI18n('Send operation with invalid OSN'), value: { operation: { name: 'delete', start: [987654321, 0]}, badOsn: true }}))
-                .addGroup('debug/realtime/sendinvalidoperation', new Button({ label: _.noI18n('Send invalid operation'), value: { operation: { name: 'createError', start: [987654321, 0]}}}))
+                .addSectionLabel(_.noI18n('Operations'))
+                .addGroup('debug/operations/apply',              new Controls.Button({ label: _.noI18n('Apply operation with invalid name'), value: { name: '_invalid_operation_' } }))
+                .addGroup('debug/operations/apply',              new Controls.Button({ label: _.noI18n('Apply operation with invalid position'), value: { name: 'insertText', start: [987654321, 0], text: 'test' } }))
+                .addGroup('debug/realtime/sendinvalidoperation', new Controls.Button({ label: _.noI18n('Send operation with invalid OSN'), value: { operation: { name: 'delete', start: [987654321, 0]}, badOsn: true }}))
+                .addGroup('debug/realtime/sendinvalidoperation', new Controls.Button({ label: _.noI18n('Send invalid operation'), value: { operation: { name: 'createError', start: [987654321, 0]}}}))
                 // register debug actions for application quit
-                .addHeaderLabel(_.noI18n('Quit'))
-                .addGroup('debug/quit', new Button({ label: _.noI18n('Close document with unsaved changes'), value: 'unsaved' }))
-                .addGroup('debug/quit', new Button({ label: _.noI18n('Close document with server error'),    value: 'error' }));
+                .addSectionLabel(_.noI18n('Quit'))
+                .addGroup('debug/quit', new Controls.Button({ label: _.noI18n('Close document with unsaved changes'), value: 'unsaved' }))
+                .addGroup('debug/quit', new Controls.Button({ label: _.noI18n('Close document with server error'),    value: 'error' }));
 
             // create a new tool bar tab for debugging
             self.createToolBarTab('debug', { label: _.noI18n('Debug'), visibleKey: 'debug/enabled', priority: 9999 });
             self.createToolBar('debug').addGroup(null, actionMenuGroup);
 
             // call initialization handler passed by sub class
-            Utils.getFunctionOption(initOptions, 'initDebugGuiHandler', $.noop).call(self, viewMenuGroup, actionMenuGroup);
+            return Utils.getFunctionOption(initOptions, 'initDebugGuiHandler', $.noop).call(self, viewMenuGroup, actionMenuGroup);
         }
 
         // methods ------------------------------------------------------------
@@ -442,7 +488,7 @@ define('io.ox/office/editframework/view/editview',
          */
         this.disableBrowserEditControls = function () {
             // execute delayed to prevent conflicts with ongoing event handling
-            app.executeDelayed(function () {
+            this.executeDelayed(function () {
                 try {
                     // disable FireFox table manipulation handlers
                     window.document.execCommand('enableObjectResizing', false, false);
@@ -511,7 +557,7 @@ define('io.ox/office/editframework/view/editview',
         this.showRenameDialog = function () {
 
             var // the dialog instance
-                dialog = new EditDialogs.ModalInputDialog({
+                dialog = new Dialogs.ModalInputDialog({
                     title: gt('Rename document'),
                     okLabel: gt('Rename'),
                     value: this.getShortFileName(),
@@ -573,7 +619,7 @@ define('io.ox/office/editframework/view/editview',
             }
 
             // create the dialog
-            dialog = new EditDialogs.ModalSaveAsDialog({
+            dialog = new Dialogs.ModalSaveAsDialog({
                 title: title,
                 value: newFileName,
                 folderPicker: type !== 'template'
@@ -585,6 +631,16 @@ define('io.ox/office/editframework/view/editview',
             });
         };
 
+        /**
+         * return whether tool-/tabbars should be combined or not
+         *
+         * @return {Boolean}
+         *  'true' for combining the bars
+         */
+        this.panesCombined = function(){
+            return panesCombined;
+        };
+
         // undo manager -------------------------------------------------------
 
         /**
@@ -633,10 +689,16 @@ define('io.ox/office/editframework/view/editview',
 
         // initialization -----------------------------------------------------
 
+        // early initialization after MVC construction
+        app.onInit(function () {
+            collaboratorPopup = new UsersLayerMenu(app);
+        });
+
         // destroy all class members on destruction
         this.registerDestructor(function () {
             toolBarTabs.destroy();
-            app = initOptions = self = toolBarTabs = topPane = toolPane = null;
+            collaboratorPopup.destroy();
+            app = initOptions = self = toolBarTabs = topPane = toolPane = collaboratorPopup = null;
         });
 
     } // class EditView
