@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -58,7 +58,7 @@ define('io.ox/office/editframework/view/searchpane',
         }());
 
         // when ESCAPE bubbles up to the window (not handled by application yet), hide the search pane
-        app.getWindowNode().on('keydown', function (event) {
+        this.listenTo(app.getWindowNode(), 'keydown', function (event) {
             if ((event.keyCode === KeyCodes.ESCAPE) && self.isVisible()) {
                 app.getView().grabFocus();
                 self.hide();
@@ -67,7 +67,7 @@ define('io.ox/office/editframework/view/searchpane',
         });
 
         // hide search pane when application is in internal error state
-        app.on('docs:state:error', function () { self.hide(); });
+        this.listenTo(app, 'docs:state:error', function () { self.hide(); });
 
         // fake the 'search:open' and 'search:close' events
         this.on({
