@@ -1,150 +1,27 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
 define('io.ox/office/editframework/view/toppane',
     ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
      'io.ox/office/baseframework/view/toolpane',
      'io.ox/office/baseframework/view/toolbar',
+     'io.ox/office/editframework/view/editlabels',
      'io.ox/office/editframework/view/editcontrols',
-     'io.ox/office/tk/keycodes',
      'gettext!io.ox/office/editframework'
-    ], function (Utils, Forms, ToolPane, ToolBar, EditControls, KeyCodes, gt) {
+    ], function (Utils, ToolPane, ToolBar, Labels, Controls, gt) {
 
     'use strict';
 
-    var // class name shortcuts
-        Button = EditControls.Button,
-        RadioGroup = EditControls.RadioGroup,
-        RadioList = EditControls.RadioList;
-
-    // class ToolBarTabGroup ==================================================
-
-    /**
-     * @constructor
-     *
-     * @extends RadioGroup
-     */
-    var ToolBarTabGroup = RadioGroup.extend({ constructor: function (app, toolBarTabs) {
-
-        var // self reference
-            self = this;
-
-        // base constructor ---------------------------------------------------
-
-        RadioGroup.call(this, { classes: 'tab-group', role: 'tablist' });
-
-        // private methods ----------------------------------------------------
-
-        function tabCreateHandler(event, tabId, options) {
-            self.createOptionButton(tabId, Utils.extendOptions(options, { attributes: { role: 'tab' } }));
-        }
-
-        function tabLabelHandler(event, tabId, label) {
-            Forms.setCaptionText(self.findOptionButtons(tabId), label);
-        }
-
-        function keyDownHandler(event) {
-
-            var startNode = $(event.target),
-                focusableControls = Forms.findFocusableNodes(self.getNode()),
-                newSelectedNode = null;
-
-            if (KeyCodes.matchKeyCode(event, 'LEFT_ARROW')) {
-                newSelectedNode = Utils.moveNodeFocus(startNode, focusableControls, 'left');
-            } else if (KeyCodes.matchKeyCode(event, 'RIGHT_ARROW')) {
-                newSelectedNode = Utils.moveNodeFocus(startNode, focusableControls, 'right');
-            }
-
-            if (newSelectedNode) {
-                self.triggerChange(Forms.getButtonValue(newSelectedNode), { preserveFocus: true });
-                event.preventDefault();
-            }
-        }
-
-        // initialization -----------------------------------------------------
-
-        // update control according to the tab collection
-        toolBarTabs.on({
-            // create button element for a new tab
-            'tab:create': tabCreateHandler,
-            'tab:label': tabLabelHandler,
-            // show/hide tab buttons
-            'tab:show': function (event, tabId) { Forms.showNodes(self.findOptionButtons(tabId), true); },
-            'tab:hide': function (event, tabId) { Forms.showNodes(self.findOptionButtons(tabId), false); },
-            // update selected button
-            'tab:activate': function (event, tabId) { self.setValue(tabId); }
-        });
-
-        // add own keyboard handling
-        this.getNode().on('keydown', keyDownHandler);
-
-    }}); // class ToolBarTabGroup
-
-    // class ToolBarTabGroup ==================================================
-
-    /**
-     * @constructor
-     *
-     * @extends RadioList
-     */
-    var ToolBarTabList = RadioList.extend({ constructor: function (app, toolBarTabs) {
-
-        var // self reference
-            self = this;
-
-        // base constructor ---------------------------------------------------
-
-        RadioList.call(this, {
-            label: /*#. default title for the drop-down selector for tool bars */ gt('Toolbars'),
-            tooltip: /*#. default tool tip for the drop-down selector for tool bars */ gt('Switch toolbars'),
-            width: 150,
-            highlight: true,
-            smallerVersion: {
-                css: {
-                    width: 20
-                },
-                icon: 'fa fa-bars',
-                hideLabel: true
-            }
-        });
-
-        // private methods ----------------------------------------------------
-
-        function tabCreateHandler(event, tabId, options) {
-            self.createOptionButton(tabId, options);
-        }
-
-        function tabLabelHandler(event, tabId, label) {
-            Forms.setCaptionText(self.getMenu().findItemNodes(tabId), label);
-        }
-
-        // initialization -----------------------------------------------------
-
-        // update control according to the tab collection
-        toolBarTabs.on({
-            // create button element for a new tab
-            'tab:create': tabCreateHandler,
-            'tab:label': tabLabelHandler,
-            // show/hide tab buttons
-            'tab:show': function (event, tabId) { Forms.showNodes(self.getMenu().findItemNodes(tabId), true); },
-            'tab:hide': function (event, tabId) { Forms.showNodes(self.getMenu().findItemNodes(tabId), false); },
-            // update selected button
-            'tab:activate': function (event, tabId) { self.setValue(tabId); }
-        });
-
-    }}); // class ToolBarTabGroup
-
     // class TopPane ==========================================================
 
     /**
@@ -291,7 +168,7 @@ define('io.ox/office/editframework/view/toppane',
 
                 // if still not enough place
                 if (getTotalNeededWidth() > paneWidth) {
-                    // beginn to shrink the controls
+                    // begin to shrink the controls
                     // as long as possible, or it fits in
                     shrink();
                 }
@@ -331,7 +208,7 @@ define('io.ox/office/editframework/view/toppane',
          * Returns the 'View' drop-down menu group containing controls for view
          * settings of the application.
          *
-         * @returns {ComponentMenuButton}
+         * @returns {CompoundButton}
          *  The 'View' drop-down menu group.
          */
         this.getViewMenuGroup = function () {
@@ -340,9 +217,17 @@ define('io.ox/office/editframework/view/toppane',
 
         // initialization -----------------------------------------------------
 
+        // initialize class members
+        leadWrapperNode = this.getAreaWrapperNode('leading');
+        leadAreaNode = leadWrapperNode.find('>.area-container');
+        centerWrapperNode = this.getAreaWrapperNode('center');
+        centerAreaNode = centerWrapperNode.find('>.area-container');
+        trailWrapperNode = this.getAreaWrapperNode('trailing');
+        trailAreaNode = trailWrapperNode.find('>.area-container');
+
         // create the tab button/list controls
-        tabGroup = new ToolBarTabGroup(app, toolBarTabs);
-        tabList = new ToolBarTabList(app, toolBarTabs);
+        tabGroup = new Controls.ToolBarTabGroup(app, toolBarTabs);
+        tabList = new Controls.ToolBarTabList(app, toolBarTabs);
 
         // create the tool bar containing the tab buttons for the edit tool pane
         this.addViewComponent(new ToolBar(app)
@@ -351,47 +236,39 @@ define('io.ox/office/editframework/view/toppane',
         { targetArea: 'leading' });
 
         // create the tool bar containing the application status label
-        statusLabel = new EditControls.ApplicationStatusLabel(app, {
-            smallerVersion: {
-                css: {
-                    width: 20
-                },
-                hideLabel: true
-            }
-        });
+        statusLabel = new Controls.ApplicationStatusLabel(app, { smallerVersion: { css: { width: 20 }, hideLabel: true } });
         this.addViewComponent(new ToolBar(app).addGroup(null, statusLabel));
 
+        // center status label in top bar when loading a document
+        this.listenTo(app, 'docs:state', function (state) {
+            centerWrapperNode.css('text-align', (state === 'preview' ? 'center' : ''));
+        });
+
         // create the 'View' drop-down menu
-        viewMenuGroup = new EditControls.ComponentMenuButton(app, { label: EditControls.VIEW_LABEL, anchorPadding: -3 });
+        viewMenuGroup = new Controls.CompoundButton(app, { label: Labels.VIEW_LABEL, anchorPadding: -3 });
 
         // create the tool bar with the standard controls
         this.addViewComponent(new ToolBar(app)
-            .addGroup('document/undo', new Button({ icon: 'docs-undo', tooltip: gt('Revert last operation') }), { visibleKey: 'document/editable' })
-            .addGroup('document/redo', new Button({ icon: 'docs-redo', tooltip: gt('Restore last operation') }), { visibleKey: 'document/editable' })
+            .addGroup('document/undo', new Controls.Button({ icon: 'docs-undo', tooltip: gt('Revert last operation') }), { visibleKey: 'document/editable' })
+            .addGroup('document/redo', new Controls.Button({ icon: 'docs-redo', tooltip: gt('Restore last operation') }), { visibleKey: 'document/editable' })
             .addGap()
-            .addGroup('view/searchpane', new Button({ icon: 'fa-search', tooltip: gt('Toggle search'), toggle: true }), { visibleKey: 'app/valid' })
+            .addGroup('view/searchpane', new Controls.Button({ icon: 'fa-search', tooltip: gt('Toggle search'), toggle: true }), { visibleKey: 'app/valid' })
             .addGap()
             .addGroup(null, viewMenuGroup)
             .addSeparator()
-            .addGroup('app/quit', new Button(EditControls.QUIT_BUTTON_OPTIONS)),
+            .addGroup('app/quit', new Controls.Button(Labels.QUIT_BUTTON_OPTIONS), { visibleKey: 'app/bundled' }),
         { targetArea: 'trailing' });
 
         // update view pane after it has been resized, or contents have been changed
         this.on('pane:layout', paneLayoutHandler);
+        this.listenTo(toolBarTabs, 'tab:show', function () { self.updateAreaSizes(); });
+        this.listenTo(toolBarTabs, 'tab:hide', function () { self.updateAreaSizes(); });
+
 
         // show tab buttons, hide drop-down list
         tabGroup.show();
         tabList.hide();
 
-        leadWrapperNode = this.getAreaWrapperNode('leading');
-        leadAreaNode = leadWrapperNode.find('>.area-container');
-
-        centerWrapperNode = this.getAreaWrapperNode('center');
-        centerAreaNode = centerWrapperNode.find('>.area-container');
-
-        trailWrapperNode = this.getAreaWrapperNode('trailing');
-        trailAreaNode = trailWrapperNode.find('>.area-container');
-
         // destroy all class members on destruction
         this.registerDestructor(function () {
             app = toolBarTabs = self = tabGroup = tabList = statusLabel = viewMenuGroup = null;
