@@ -1,11 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2014 Open-Xchange Inc., Tarrytown, NY, USA. info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Edy Haryono <edy.haryono@open-xchange.com>
  */
@@ -48,18 +49,18 @@ function (Ext, Links, CoreActions, PortalUtils) {
                     .attr({
                         'data-toggle': 'tooltip',
                         'data-placement': 'top',
-                        'title': baton.data.filename,
+                        'title': _.noI18n(baton.data.filename),
                         'tabindex': 0
                     }),
                     metadata = $('<div>').addClass('document-metadata'),
                     name = $('<div>').addClass('document-name col-xs-8'),
                     date = $('<div>').addClass('document-date col-xs-4')
-                        .text(PortalUtils.formatDate(baton.data.last_modified ? baton.data.last_modified : baton.data.last_opened));
+                        .text(_.noI18n(PortalUtils.formatDate(baton.data.last_modified ? baton.data.last_modified : baton.data.last_opened)));
 
                 // enable Bootstrap tooltip except for touch devices, since it is buggy (see https://github.com/twbs/bootstrap/issues/6232)
                 if (!Modernizr.touch) { link.tooltip(); }
 
-                name.append(PortalUtils.createDocumentIcon(baton.data.filename), $('<div>').text(baton.data.filename));
+                name.append(PortalUtils.createDocumentIcon(baton.data.filename), $('<div>').text(_.noI18n(baton.data.filename)));
                 link.append(metadata.append(name, date));
 
                 link.click(function () {
