@@ -1,11 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2014 Open-Xchange Inc., Tarrytown, NY, USA. info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Edy Haryono <edy.haryono@open-xchange.com>
  */
@@ -85,14 +86,14 @@ define('io.ox/office/portal/action/templateactions',
                         .attr({
                             'data-toggle': 'tooltip',
                             'data-placement': 'top',
-                            'title': baton.data.filename,
+                            'title': _.noI18n(baton.data.filename),
                             'tabindex' : 0
                         })
                         .tooltip(),
                         templatePreview = $('<div>').addClass('template-preview').css({
                             backgroundImage: 'url(' + createTemplateURL(baton.data, options) + ')'
                         }),
-                        templateName = $('<p>').addClass('template-name').text(PortalUtils.removeFileExtension(baton.data.filename));
+                        templateName = $('<p>').addClass('template-name').text(_.noI18n(PortalUtils.removeFileExtension(baton.data.filename)));
 
                     // add new blank template
                     templateItem.append(templatePreview, templateName);
@@ -134,13 +135,13 @@ define('io.ox/office/portal/action/templateactions',
                         .attr({
                             'data-toggle': 'tooltip',
                             'data-placement': 'top',
-                            'title': fileName,
+                            'title': _.noI18n(fileName),
                             'tabindex': 0
                         })
                         .tooltip(),
                         templatePreview = $('<div>').addClass('new-blank template-preview')
                             .append($('<i>').addClass('fa fa-plus')),
-                        templateName = $('<p>').addClass('template-name').text(fileName);
+                        templateName = $('<p>').addClass('template-name').text(_.noI18n(fileName));
 
                     // add new blank template
                     templateItem.append(templatePreview, templateName);
