@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Kai Ahrens <kai.ahrens@open-xchange.com>
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
@@ -16,13 +16,16 @@ define('io.ox/office/preview/app/application',
     ['io.ox/core/collection',
      'io.ox/core/api/folder',
      'io.ox/office/tk/utils',
+     'io.ox/office/tk/io',
      'io.ox/office/baseframework/app/baseapplication',
      'io.ox/office/baseframework/app/extensionregistry',
      'io.ox/office/preview/model/model',
      'io.ox/office/preview/view/view',
+     'io.ox/office/preview/view/printview',
      'io.ox/office/preview/app/controller',
+     'io.ox/office/baseframework/pdf/pdfdocument',
      'gettext!io.ox/office/preview'
-    ], function (Collection, FolderAPI, Utils, BaseApplication, ExtensionRegistry, PreviewModel, PreviewView, PreviewController, gt) {
+    ], function (Collection, FolderAPI, Utils, IO, BaseApplication, ExtensionRegistry, PreviewModel, PreviewView, PrintView, PreviewController, PDFDocument, gt) {
 
     'use strict';
 
@@ -76,8 +79,10 @@ define('io.ox/office/preview/app/application',
          */
         function importDocument() {
 
-            var // promise of the initial server request
-                convertPromise = null,
+            var // the PDF document
+                pdfDocument,
+                // promise of the initial image server request
+                documentImagesPromise = null,
                 // promise of a Deferred that returns the collection properties
                 collectionPromise = null,
                 // promise of a Deferred that returns the folder properties
@@ -85,19 +90,31 @@ define('io.ox/office/preview/app/application',
                 // the file descriptor
                 file = self.getFileDescriptor();
 
+            //disable user-select on body, that only the "user-select-text" classes can really select the text
+            self.registerVisibleStateHandler(function(state) {
+                $('body').toggleClass('user-select-none', state);
+            });
+
             // disable drop events
             self.getWindowNode().on('drop dragstart dragover', false);
 
-            // show the busy blocker element
-            self.getView().enterBusy({ showFileName: true, cancelHandler: function () { self.quit(); } });
+            // immediately show the busy blocker element
+            self.getView().enterBusy({ cancelHandler: _.bind(self.quit, self), showFileName: true, immediate: true });
 
-            // open the file on the server
-            convertPromise = self.sendConverterRequest({
-                params: {
-                    action: 'convertdocument',
-                    convert_format: 'html',
-                    convert_action: 'beginconvert'
-                }
+            pdfDocument = new PDFDocument(self.getServerModuleUrl(IO.CONVERTER_MODULE_NAME, {
+                action: 'getdocument',
+                documentformat: 'pdf',
+                priority: 'instant',
+                mimetype: file.file_mimetype ? encodeURIComponent(file.file_mimetype) : '',
+                nocache: _.uniqueId() // needed to trick the browser cache (is not evaluated by the backend)
+            }));
+
+            // begin stateful image conversion for the document on the server
+            documentImagesPromise = self.sendFileRequest(IO.CONVERTER_MODULE_NAME, {
+                action: 'convertdocument',
+                convert_action: 'beginconvert',
+                convert_format: 'image'
+                // convert_format: 'html'
             });
 
             // check edit rights of user, check whether file is located in the trash folder
@@ -111,29 +128,36 @@ define('io.ox/office/preview/app/application',
             }
 
             // wait for all promises and check the response objects
-            return $.when(convertPromise, collectionPromise, folderPromise).then(function (convertData, collectionData, folderData) {
-
+            return $.when(pdfDocument.getLoadPromise(), documentImagesPromise, collectionPromise, folderPromise).then(function (pageCount, imageConvertData, collectionData, folderData) {
                 // check that no error has occurred while opening the document
-                if (_.isString(convertData.JobID) && (convertData.JobID.length > 0) && _.isNumber(convertData.PageCount) && (convertData.PageCount > 0)) {
+                if (_.isString(imageConvertData.JobID) && (imageConvertData.JobID.length > 0) && _.isNumber(pageCount) && (pageCount > 0)) {
 
                     // store whether the document can really be edited
                     isEditable = _.isObject(collectionData) && collectionData.one && collectionData.read && collectionData.modify && _.isObject(folderData) && !FolderAPI.is('trash', folderData);
 
-                    // further initialization after import
-                    jobId = convertData.JobID;
-                    self.getModel().setPageCount(convertData.PageCount);
+                    // further initialization after all imports
+                    jobId = imageConvertData.JobID;
 
-                    // resolve the promise with the response data
-                    return convertData;
+                    // set the PDF.js document promise with page count,
+                    // if available, or just the page count at the model
+                    self.getModel().setPDFDocument(pdfDocument);
+
+                    // resolve the promise with the response data for the beginConvert request
+                    return imageConvertData;
                 }
 
                 // special text for password protected files
-                if (Utils.getStringOption(convertData, 'cause') === 'passwordProtected') {
-                    convertData.message = gt('The document is protected by a password.');
+                var errorCause = Utils.getStringOption(imageConvertData, 'cause');
+
+                if (errorCause === 'passwordProtected') {
+                    imageConvertData.message = gt('The document is protected by a password.');
+                } else if (errorCause === 'maxSourceSizeExceeded') {
+                    //#. %1$s is the maximum allowed source file size in MB
+                    imageConvertData.message = gt('The document exceeds the maximum permitted size of %1$s MB.', imageConvertData.MaxSourceSizeMB);
                 }
 
                 // convert result to a rejected Deferred object
-                return $.Deferred().reject(convertData);
+                return $.Deferred().reject(imageConvertData);
             });
         }
 
@@ -143,20 +167,25 @@ define('io.ox/office/preview/app/application',
          */
         function quitHandler() {
             if (!self.isImportFailed()) {
-                return self.sendPreviewRequest({ params: { convert_action: 'endconvert' } });
+                return self.sendPreviewRequest({ convert_action: 'endconvert' });
             }
         }
 
         // methods ------------------------------------------------------------
 
-        this.sendPreviewRequest = function (options) {
-            return _.isString(jobId) ? this.sendConverterRequest(Utils.extendOptions({
-                params: { action: 'convertdocument', job_id: jobId }
-            }, options)) : $.Deferred().reject();
+        this.sendPreviewRequest = function (params, options) {
+
+            if (!_.isString(jobId)) { return $.Deferred().reject(); }
+
+            // add correct action and job identifier to the request parameters
+            params = _.extend({}, params, { action: 'convertdocument', job_id: jobId });
+
+            // send the server request
+            return this.sendFileRequest(IO.CONVERTER_MODULE_NAME, params, options);
         };
 
         this.getPreviewModuleUrl = function (options) {
-            return _.isString(jobId) ? this.getConverterModuleUrl(Utils.extendOptions({
+            return _.isString(jobId) ? this.getServerModuleUrl(IO.CONVERTER_MODULE_NAME, Utils.extendOptions({
                 action: 'convertdocument',
                 job_id: jobId
             }, options)) : undefined;
@@ -178,10 +207,8 @@ define('io.ox/office/preview/app/application',
 
             var // the file descriptor of the current document
                 file = this.getFileDescriptor(),
-                // the configuration of the file extension of the current document
-                extensionSettings = this.isDocumentEditable() ? ExtensionRegistry.getExtensionSettings(this.getFullFileName()) : null,
                 // the edit application module identifier
-                editModule = Utils.getStringOption(extensionSettings, 'module', '');
+                editModule = this.isDocumentEditable() ? ExtensionRegistry.getEditModule(this.getFullFileName()) : '';
 
             // TODO: edit mail attachments and task attachments
             if (editModule.length > 0) {
@@ -197,16 +224,23 @@ define('io.ox/office/preview/app/application',
                 }
 
                 // close this application
-                _.defer(function () { self.quit(); });
+                this.quit();
             }
         };
 
+        /**
+         * Currently using old print as pdf functionality.
+         */
+        this.print = function () {
+            new PrintView(this).doPrintAsPDF();
+        };
+
         // initialization -----------------------------------------------------
 
         this.registerQuitHandler(quitHandler);
 
         // destroy all class members
-        this.on('docs:destroy', function () {
+        this.registerDestructor(function () {
             launchOptions = appOptions = null;
         });
 
