@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -45,9 +45,9 @@ define('io.ox/office/preview/app/controller',
                     enable: function () { return model.getPageCount() > 0; }
                 },
 
-                // start editing the document
+                // start editing the document (disabled in stand-alone mode)
                 'document/edit': {
-                    parent: 'document/valid',
+                    parent: ['document/valid', 'app/bundled'],
                     enable: function () { return app.isDocumentEditable(); },
                     set: function () { app.editDocument(); }
                 },
@@ -123,7 +123,7 @@ define('io.ox/office/preview/app/controller',
 
                 // zoom -------------------------------------------------------
 
-                
+
                 'zoom/dec': {
                     parent: 'document/valid',
                     enable: function () { return view.getZoomFactor() > view.getMinZoomFactor(); },
