@@ -1,31 +1,28 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/preview/view/pageloader',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/object/baseobject'
-     ], function (Utils, Forms, BaseObject) {
+define('io.ox/office/preview/view/pageloader', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/object/baseobject',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/baseframework/pdf/pdfview',
+    'io.ox/office/baseframework/lib/pdftextlayerbuilder',
+], function (Utils, Forms, BaseObject, TimerMixin, PDFView) {
 
     'use strict';
 
-    var // default page size used before a page has been loaded
-        DEFAULT_PAGE_SIZE = {
-            width: Utils.convertLength(210, 'mm', 'px', 1),
-            height: Utils.convertLength(297, 'mm', 'px', 1)
-        },
-
-        // the maximum number of simultaneous pending AJAX page requests
+    var // the maximum number of simultaneous pending AJAX page requests
         MAX_REQUESTS = Modernizr.touch ? 2 : 5,
 
         // the number of AJAX requests currently running
@@ -96,21 +93,27 @@ define('io.ox/office/preview/view/pageloader',
      * @constructor
      *
      * @extends BaseObject
+     * @extends TimerMixin
      */
     function PageLoader(app) {
 
-        var // list of pages waiting to load their page contents, keyed by priority
+        var // self reference
+            self = this,
+            // list of pages waiting to load their page contents, keyed by priority
             queue = { high: [], medium: [], low: [] },
 
             // waiting and running requests, keyed by page number
             map = {},
 
             // the background loop processing the pending pages
-            timer = null;
+            timer = null,
 
-        // base constructor ---------------------------------------------------
+            pdfView = null;
+
+        // base constructors --------------------------------------------------
 
         BaseObject.call(this);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
@@ -123,29 +126,52 @@ define('io.ox/office/preview/view/pageloader',
          *  be resolved with the original size of the page (as object with the
          *  properties 'width' and 'height', in pixels).
          */
-        function loadPageIntoNode(pageNode, page, options) {
+        function loadPageIntoNode(pageNode, pageNumber, options) {
 
             var // the Deferred object waiting for the image
                 def = null,
+                // the jquery node
+                jqPageNode = $(pageNode),
                 // the target image format
                 format = Utils.getStringOption(options, 'format', 'png');
+                // default options contain a page zoom of 1.0
+                options = $.extend({pageNumber: pageNumber, pageZoom: 1.0, }, options || {});
 
             function resolveSize() {
-                var pageSize = calculateImageSize(pageNode.children().first(), Utils.getNumberOption(options, 'zoom'));
-                if (pageSize) { pageNode.data('page-size', pageSize); }
+                var pageSize = calculateImageSize(jqPageNode.children().first(), Utils.getNumberOption(options, 'zoom'));
+
+                if (pageSize) {
+                    jqPageNode.data('page-size', pageSize);
+                }
+
                 return pageSize || $.Deferred().reject();
             }
 
-            if (_.browser.Chrome && (format === 'svg')) {
+            // -----------------------------------------------------------------
+
+            if (format === 'pdf') {
+                jqPageNode.data('data-rendertype', 'pdf');
+                jqPageNode.data('data-pagezoom', options.pageZoom);
+
+                var pageSize = pdfView.createPDFPageNode(jqPageNode, options);
+
+                if (pageSize) {
+                    (def = $.Deferred()).resolve(pageSize);
+                } else {
+                    (def = $.Deferred()).reject();
+                }
+            } else if (_.browser.Chrome && (format === 'svg')) {
+                jqPageNode.data('data-rendertype', 'svg');
 
                 // as SVG mark-up (Chrome does not show embedded images in <img> elements linked to an SVG file)
-                def = app.getModel().loadPageAsSvgMarkup(page, Utils.getStringOption(options, 'priority', 'medium')).then(function (svgMarkup) {
+                def = app.getModel().loadPageAsSvgMarkup(pageNumber, Utils.getStringOption(options, 'priority', 'medium')).then(function (svgMarkup) {
                     // do NOT use the jQuery.html() method for SVG mark-up!
-                    pageNode[0].innerHTML = svgMarkup;
+                    jqPageNode[0].innerHTML = svgMarkup;
                     // resolve with original image size
                     return resolveSize();
                 });
             } else {
+                jqPageNode.data('data-rendertype', 'img');
 
                 // Bug 25765: Safari cannot parse SVG mark-up, and cannot show images embedded in <img> elements linking to SVG
                 if (_.browser.Safari && (format === 'svg')) {
@@ -153,8 +179,8 @@ define('io.ox/office/preview/view/pageloader',
                 }
 
                 // preferred: as an image element linking to the image file
-                def = app.getModel().loadPageAsImage(page, options).then(function (imgNode) {
-                    pageNode.empty().append(imgNode);
+                def = app.getModel().loadPageAsImage(pageNumber, options).then(function (imgNode) {
+                    jqPageNode.empty().append(imgNode);
                     // resolve with original image size (naturalWidth/naturalHeight with SVG does not work in IE10)
                     return resolveSize();
                 });
@@ -166,23 +192,27 @@ define('io.ox/office/preview/view/pageloader',
         /**
          * Registers a page node for deferred loading.
          */
-        function registerPageNode(pageNode, page, options) {
+        function registerPageNode(pageNode, pageNumber, options) {
 
             var // the pending data to be inserted into the array
                 pageData = null,
+                // the jquery node
+                jqPageNode = $(pageNode),
                 // the request priority
                 priority = Utils.getStringOption(options, 'priority', 'medium');
 
             // check if the page has been registered already
-            if (page in map) { return map[page].def.promise(); }
+            if (pageNumber in map) {
+                return map[pageNumber].def.promise();
+            }
 
             // insert a busy node into the page node
-            pageNode.empty().append($('<div>').addClass('abs').busy());
+            jqPageNode.empty().append($('<div>').addClass('abs').busy());
 
             // insert the page information into the pending array
-            pageData = { node: pageNode, page: page, options: options, def: $.Deferred() };
+            pageData = { node: jqPageNode, page: pageNumber, options: options, def: $.Deferred() };
             queue[priority].push(pageData);
-            map[page] = pageData;
+            map[pageNumber] = pageData;
 
             return pageData.def.promise();
         }
@@ -192,24 +222,31 @@ define('io.ox/office/preview/view/pageloader',
          */
         function loadQueuedPages() {
 
+            if (!pdfView) {
+                pdfView = new PDFView(app.getModel().getPDFDocument());
+            }
+
             // check if the background loop is already running
-            if (timer) { return; }
+            if (timer) {
+                return;
+            }
 
             // create a new background loop that processes all waiting pages
-            timer = app.repeatDelayed(function () {
+            timer = self.repeatDelayed(function () {
 
                 var // data of next page to be loaded
                     pageData = null;
 
-                // restrict number of requests running simultaneously (but do not break the loop)
-                if (runningRequests > MAX_REQUESTS) { return; }
-
+                // restrict number of requests running simultaneously (but do not break the loop);
                 // load high-priority page even if limit has been reached but not exceeded
-                if ((runningRequests === MAX_REQUESTS) && (queue.high.length === 0)) { return; }
+                if ((runningRequests > MAX_REQUESTS) || ((runningRequests === MAX_REQUESTS) && (queue.high.length === 0))) {
+                    return;
+                }
 
                 // abort the background loop, if no more pages have to be loaded
-                pageData = queue.high.shift() || queue.medium.shift() || queue.low.shift();
-                if (!pageData) { return Utils.BREAK; }
+                if (!(pageData = queue.high.shift() || queue.medium.shift() || queue.low.shift())) {
+                    return Utils.BREAK;
+                }
 
                 // load the page contents
                 runningRequests += 1;
@@ -223,16 +260,37 @@ define('io.ox/office/preview/view/pageloader',
                     pageData.def.resolve(pageSize);
                 })
                 .fail(function (result) {
-                    pageData.node.addClass('page-error').html('<div class="error-icon">' + Forms.createIconMarkup('fa-times') + '</div>');
+                    pageData.node.addClass('page-error').css({ visibility: 'visible' }).html('<div class="error-icon">' + Forms.createIconMarkup('fa-times') + '</div>');
                     pageData.def.reject(result);
                 });
 
-            }, { delay: 20 });
+            }, 20);
 
             // forget reference to the timer, when all page requests are running
-            timer.always(function () { timer = null; });
+            timer.always(function () {
+                timer = null;
+            });
         }
 
+        /**
+         * Renders the PDF page with given page number and zoom
+         * into the given canvas node, that already has the
+         * appropriate size and style size attributes set
+         * @param {jQuery} canvasNode
+         *  The target canvas node, as jQuery object.
+         *
+         * @param {Number} page
+         *  The one-based index of the page to be rendered.
+         *
+         * @param {Number} pageZoom
+         *  The zoom of the page for rendering.
+         *
+         * @returns {jquery promise}
+         *  The jQuery promise that will be resolved when the
+         *  page and the page text have been rendered successfully
+         *  or rejected on error.
+         */
+
         // methods ------------------------------------------------------------
 
         /**
@@ -263,7 +321,7 @@ define('io.ox/office/preview/view/pageloader',
          * @param {jQuery} pageNode
          *  The target page node, as jQuery object.
          *
-         * @param {Number} page
+         * @param {Number} pageNumber
          *  The one-based index of the page to be loaded.
          *
          * @param {Object} [options]
@@ -278,30 +336,14 @@ define('io.ox/office/preview/view/pageloader',
          *      If specified, the requested height of the image, in pixels. If
          *      the option 'options.width' is specified too, the resulting
          *      height may be less than this value.
-         *  @param {String} [options.priority='medium']
-         *      Specifies with which priority the server will handle the image
-         *      request. Must be one of the strings 'low', 'medium' (default),
-         *      or 'high'.
+         *  @param {Boolean} [options.printing=false]
+         *      Specifies if the page is to be rendered for printing or just viewing
          *
          * @returns {jQuery.Promise}
          *  The Promise of a Deferred object that will be resolved when the
          *  page has been loaded successfully; or rejected on error.
          */
-        this.loadPage = app.createDebouncedMethod(registerPageNode, loadQueuedPages);
-
-        /**
-         * Returns the original size of the image in the passed page node. The
-         * size will be stored in the page node after the page has been loaded.
-         * If the passed page has not been loaded yet, returns a default page
-         * size.
-         *
-         * @returns {Object}
-         *  The original page size (in pixels), in the properties 'width' and
-         *  'height'.
-         */
-        this.getPageSize = function (pageNode) {
-            return $(pageNode).data('page-size') || DEFAULT_PAGE_SIZE;
-        };
+        this.loadPage = this.createDebouncedMethod(registerPageNode, loadQueuedPages);
 
         /**
          * Returns the current zoom factor of the passed page node. The zoom
@@ -331,26 +373,35 @@ define('io.ox/office/preview/view/pageloader',
          *  A reference to this instance.
          */
         this.setPageZoom = function (pageNode, pageZoom) {
-
-            var // the original size of the page
-                pageSize = this.getPageSize(pageNode),
-                // the child node in the page, containing the SVG
-                childNode = $(pageNode).children().first(),
+            var // the page number
+                pageNumber = Utils.getElementAttributeAsInteger(pageNode, 'data-page', 1),
+                // the jquery node
+                jqPageNode = $(pageNode),
+                // the original size of the page
+                pageSize = app.getModel().getOriginalPageSize(pageNumber),
+                // the type od node to be rendered ('pdf', 'svg' or 'img)
+                renderType = jqPageNode.data('data-rendertype'),
                 // the resulting width/height
                 width = Math.floor(pageSize.width * pageZoom),
                 height = Math.ceil(pageSize.height * pageZoom);
 
-            if (childNode.is('img')) {
+            // make node visible, it may be invisible after initial loading
+            jqPageNode.css({visibility: 'visible'});
+
+            if (renderType === 'pdf') {
+                // <canvas> element: render page into canvas and create textoverlay
+                pdfView.renderPDFPage(jqPageNode, Utils.getElementAttributeAsInteger(jqPageNode, 'data-page', 1), pageZoom);
+            } else if (renderType === 'img') {
                 // <img> element: resize with CSS width/height
-                childNode.width(width).height(height);
+                jqPageNode.children().first().width(width).height(height);
             } else {
                 // <svg> element (Chrome): scale with CSS zoom (supported in WebKit)
-                childNode.css('zoom', pageZoom);
+                jqPageNode.children().first().css('zoom', pageZoom);
             }
 
             // Chrome bug/problem: sometimes, the page node has width 0 (e.g., if browser zoom is
             // not 100%) regardless of existing SVG, must set its size explicitly to see anything...
-            $(pageNode).width(width).height(height).data('page-zoom', pageZoom);
+            jqPageNode.width(width).height(height).data('page-zoom', pageZoom);
 
             return this;
         };
@@ -359,7 +410,7 @@ define('io.ox/office/preview/view/pageloader',
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = queue = map = timer = null;
+            app = self = queue = map = timer = null;
         });
 
     } // class PageLoader
