@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -17,23 +17,18 @@ define('io.ox/office/preview/view/view',
      'io.ox/office/baseframework/view/baseview',
      'io.ox/office/baseframework/view/sidepane',
      'io.ox/office/baseframework/view/component',
-     'io.ox/office/baseframework/view/toolbox',
-     'io.ox/office/preview/view/toppane',
      'io.ox/office/preview/view/controls',
-     'io.ox/office/preview/view/pagegroup',
+     'io.ox/office/preview/view/toppane',
      'io.ox/office/preview/view/pageloader',
      'io.ox/office/preview/view/nodetouch',
      'gettext!io.ox/office/preview',
      'less!io.ox/office/preview/view/style'
-    ], function (Utils, ExtensionRegistry, BaseView, SidePane, Component, ToolBox, TopPane, Controls, PageGroup, PageLoader, NodeTouch, gt) {
+    ], function (Utils, ExtensionRegistry, BaseView, SidePane, Component, Controls, TopPane, PageLoader, NodeTouch, gt) {
 
     'use strict';
 
     var // predefined zoom factors
-        ZOOM_FACTORS = [25, 35, 50, 75, 100, 150, 200, 300, 400, 600, 800],
-
-        // the speed of scroll animations
-        ANIMATION_DELAY = 25;
+        ZOOM_FACTORS = [25, 35, 50, 75, 100, 150, 200, 300, 400, 600, 800];
 
     // class PreviewView ======================================================
 
@@ -129,46 +124,6 @@ define('io.ox/office/preview/view/view',
             }
         }
 
-        /**
-         * Scrolls the application pane node by the passed distance.
-         */
-        function scrollContentRootNode(diffX, diffY) {
-            var scrollLeft = contentRootNode.scrollLeft() + Math.round(diffX),
-                scrollTop = contentRootNode.scrollTop() + Math.round(diffY);
-            contentRootNode.scrollLeft(scrollLeft).scrollTop(scrollTop);
-        }
-
-        /**
-         * Aborts the current scroll animation.
-         */
-        function abortScrollAnimation() {
-            if (scrollAnimation) {
-                scrollAnimation.abort();
-                scrollAnimation = null;
-            }
-        }
-
-        /**
-         * Starts a new scroll animation.
-         *
-         * @param {Function} callback
-         *  A callback function that has to return the scroll distance for each
-         *  animation frame, as object in the properties 'x' and 'y'. The
-         *  function receives the zero-based index of the current animation
-         *  frame.
-         *
-         * @param {Numner} frames
-         *  The number of animation frames.
-         */
-        function startScrollAnimation(callback, frames) {
-            abortScrollAnimation();
-            scrollAnimation = app.repeatDelayed(function (index) {
-                var move = callback.call(self, index);
-                scrollContentRootNode(move.x, move.y);
-            }, { delay: ANIMATION_DELAY, cycles: frames })
-            .always(function () { scrollAnimation = null; });
-        }
-
         /**
          * Scrolls the view to the specified page.
          *
@@ -194,9 +149,12 @@ define('io.ox/office/preview/view/view',
             var // the page node of the specified page
                 pageNode = pageNodes.eq(page - 1);
 
-            // do not load initialized page again
+            // do not load correctly initialized page again
             if (pageNode.children().length === 0) {
-                pageLoader.loadPage(pageNode, page, { format: 'svg', priority: priority })
+                // choose either 'svg' or 'pdf' as rendering format;
+                // 'svg' is directly rendered via the browser, if possible;
+                // 'pdf' is rendered via the PDF.js library onto HTML5 canvas elements
+                pageLoader.loadPage(pageNode, page, { format: 'pdf', priority: priority })
                 .done(function () {
                     loadedPageNodes[page] = pageNode;
                     refreshLayout(page);
@@ -262,10 +220,10 @@ define('io.ox/office/preview/view/view',
 
             // load two pages above and below the visible area with medium priority after a short delay
             cancelMorePagesTimer();
-            loadMorePagesTimer = app.executeDelayed(function () {
+            loadMorePagesTimer = self.executeDelayed(function () {
                 Utils.iterateRange(endPage, endPageLowerPrio, loadPageMediumPriority);
                 Utils.iterateRange(beginPage - 1, beginPageLowerPrio - 1, loadPageMediumPriority, { step: -1 });
-            }, { delay: 500 });
+            }, 500);
 
             // clear all other pages
             _.each(loadedPageNodes, function (pageNode, page) {
@@ -286,8 +244,10 @@ define('io.ox/office/preview/view/view',
          */
         function calculatePageZoom(pageNode) {
 
-            var // the original size of the current page
-                pageSize = pageLoader.getPageSize(pageNode),
+            var // the current page number
+                pageNumber = Utils.getElementAttributeAsInteger(pageNode, 'data-page', 1),
+                // the original size of the current page
+                originalPageSize = model.getOriginalPageSize(pageNumber),
                 // the effective zoom factor for the page
                 pageZoomFactor = 100;
 
@@ -295,14 +255,13 @@ define('io.ox/office/preview/view/view',
             if (_.isNumber(zoomType)) {
                 pageZoomFactor = zoomType;
             } else if (zoomType === 'width') {
-                pageZoomFactor = availableSize.width / pageSize.width * 100;
+                pageZoomFactor = availableSize.width / originalPageSize.width * 100;
             } else if (zoomType === 'page') {
-                pageZoomFactor = Math.min(availableSize.width / pageSize.width * 100, availableSize.height / pageSize.height * 100);
+                pageZoomFactor = Math.min(availableSize.width / originalPageSize.width * 100, availableSize.height / originalPageSize.height * 100);
             }
-            pageZoomFactor = Utils.minMax(pageZoomFactor, self.getMinZoomFactor(), self.getMaxZoomFactor());
 
             // set the zoom factor at the page node
-            pageLoader.setPageZoom(pageNode, pageZoomFactor / 100);
+            pageLoader.setPageZoom(pageNode, Utils.minMax(pageZoomFactor, self.getMinZoomFactor(), self.getMaxZoomFactor()) / 100);
         }
 
         /**
@@ -315,6 +274,15 @@ define('io.ox/office/preview/view/view',
          *  be updated.
          */
         function refreshLayout(page) {
+            function emptyPageNode(pageNode) {
+                if (pageNode) {
+                    pageNode.empty();
+                }
+            }
+
+            if (!page && pageNodes) {
+                pageNodes.detach().each(function () { emptyPageNode($(this)); }).appendTo(pageContainerNode);
+            }
 
             var // half of the content root node height
                 contentCenter = contentRootNode.height() / 2,
@@ -438,75 +406,6 @@ define('io.ox/office/preview/view/view',
             }
         }
 
-        /**
-         * Handles tracking events originating from the mouse and simulates
-         * touch-like scrolling on the page nodes.
-         *
-         * @param {jQuery.Event} event
-         *  The jQuery tracking event.
-         */
-        var trackingHandler = (function () {
-
-            var // the duration to collect move events, in milliseconds
-                COLLECT_DURATION = 150,
-                // distances of last move events in COLLECT_DURATION
-                lastMoves = null;
-
-            // removes all outdated entries from the lastMoves array
-            function updateLastMoves(moveX, moveY) {
-
-                var now = _.now();
-
-                while ((lastMoves.length > 0) && (now - lastMoves[0].now > COLLECT_DURATION)) {
-                    lastMoves.shift();
-                }
-                if (_.isNumber(moveX) && _.isNumber(moveY)) {
-                    lastMoves.push({ now: now, x: moveX, y: moveY });
-                }
-                return now;
-            }
-
-            // starts the trailing animation
-            function trailingAnimation() {
-
-                var now = updateLastMoves(),
-                    move = _.reduce(lastMoves, function (memo, entry) {
-                        var factor = (now - entry.now) / COLLECT_DURATION;
-                        memo.x -= entry.x * factor;
-                        memo.y -= entry.y * factor;
-                        return memo;
-                    }, { x: 0, y: 0 });
-
-                // reduce distance for initial animation frame
-                move.x /= (COLLECT_DURATION / ANIMATION_DELAY / 2);
-                move.y /= (COLLECT_DURATION / ANIMATION_DELAY / 2);
-
-                // run the scroll animation
-                startScrollAnimation(function () { move.x *= 0.85; move.y *= 0.85; return move; }, 20);
-            }
-
-            function trackingHandler(event) {
-
-                switch (event.type) {
-                case 'tracking:start':
-                    lastMoves = [];
-                    abortScrollAnimation();
-                    break;
-                case 'tracking:move':
-                    scrollContentRootNode(-event.moveX, -event.moveY);
-                    updateLastMoves(event.moveX, event.moveY);
-                    break;
-                case 'tracking:end':
-                    trailingAnimation();
-                    break;
-                case 'tracking:cancel':
-                    break;
-                }
-            }
-
-            return trackingHandler;
-        }()); // end of trackingHandler() local scope
-
         /**
          * Handles pinch events.
          *
@@ -732,7 +631,7 @@ define('io.ox/office/preview/view/view',
          * @param {jQuery.Event} event
          *  The jQuery scroll event.
          */
-        var scrollHandler = app.createDebouncedMethod(cancelMorePagesTimer, loadVisiblePages, {
+        var scrollHandler = this.createDebouncedMethod(cancelMorePagesTimer, loadVisiblePages, {
             delay: Modernizr.touch ? 200 : 100,
             maxDelay: Modernizr.touch ? 1000 : 500
         });
@@ -754,11 +653,11 @@ define('io.ox/office/preview/view/view',
                 size: Modernizr.touch ? 152 : SidePane.DEFAULT_WIDTH,
                 resizable: !Modernizr.touch,
                 minSize: SidePane.DEFAULT_WIDTH,
-                maxSize: PageGroup.getRequiredWidth(4) + Utils.SCROLLBAR_WIDTH
+                maxSize: Controls.PageGroup.getRequiredWidth(4) + Utils.SCROLLBAR_WIDTH
             }));
 
             // create the page preview group
-            pageGroup = new PageGroup(app, sidePane);
+            pageGroup = new Controls.PageGroup(app, sidePane);
 
             // initialize the side pane
             sidePane
@@ -792,6 +691,11 @@ define('io.ox/office/preview/view/view',
 
             // search pane is a focus landmark (TODO: move to core code)
             searchPaneNode.addClass('f6-target');
+
+            // #36049: Disable context menu for content view
+            contentRootNode.on('contextmenu', function () {
+                return false;
+            });
         }
 
         function initGuiHandler() {
@@ -804,21 +708,20 @@ define('io.ox/office/preview/view/view',
             if (pageCount >= 1) {
 
                 // save the scroll position while view is hidden
-                app.getWindow().on({ beforehide: windowHideHandler, show: windowShowHandler });
+                self.listenTo(app.getWindow(), 'beforehide', windowHideHandler);
+                self.listenTo(app.getWindow(), 'show', windowShowHandler);
 
                 // no layout refreshes without any pages
                 self.on('refresh:layout', function () { refreshLayout(); });
 
-                // initialize touch-like tracking with mouse, attach the scroll event handler
+                // attach the scroll event handler
                 contentRootNode
-                    .enableTracking({selector: '.page', sourceEvents: 'mouse'})
                     .enableTouch({selector: '.page', pinchHandler: pinchHandler, tapHandler: tapHandler, horSwipeHandler: horSwipeHandler})
-                    .on('tracking:start tracking:move tracking:end tracking:cancel', trackingHandler)
                     .on('scroll', scrollHandler);
 
                 // create mark-up for all page nodes
                 _.times(pageCount, function (index) {
-                    pageMarkup += '<div class="page" data-page="' + (index + 1) + '"></div>';
+                    pageMarkup += '<div class="page" data-page="' + (index + 1) + '" style="visibility:hidden"></div>';
                 });
 
                 // insert the mark-up into the container node and initialize the
@@ -1007,21 +910,17 @@ define('io.ox/office/preview/view/view',
         // initialize the view after import
         app.onImportSuccess(function () {
 
-            var // file type configuration (by file extension)
-                extSettings = ExtensionRegistry.getExtensionSettings(app.getFullFileName());
-
-            // determine zoom type by application type (full width for text documents)
-            zoomType = 'page';
-            if (extSettings) {
-                switch (extSettings.type) {
-                case 'text':
-                    zoomType = 'width';
-                    break;
-                case 'spreadsheet':
-                case 'presentation':
-                    zoomType = 'page';
-                    break;
-                }
+            // determine zoom type by file category (full width for text documents)
+            switch (ExtensionRegistry.getFileCategory(app.getFullFileName())) {
+            case 'text':
+                zoomType = 'width';
+                break;
+            case 'spreadsheet':
+            case 'presentation':
+                zoomType = 'page';
+                break;
+            default:
+                zoomType = 'page';
             }
 
             // first set zoom, then the first page (to keep it at the top screen border)
