@@ -1,22 +1,23 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/colrowcollection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/model/cellattributesmodel'
-    ], function (Utils, ModelObject, SheetUtils, CellAttributesModel) {
+define('io.ox/office/spreadsheet/model/colrowcollection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/baseframework/model/modelobject',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/model/cellattributesmodel'
+], function (Utils, TimerMixin, ModelObject, SheetUtils, CellAttributesModel) {
 
     'use strict';
 
@@ -61,7 +62,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             self = this,
 
             // the document model
-            model = app.getModel(),
+            docModel = app.getModel(),
 
             // different property names for column or row properties
             FAMILY_NAME = columns ? 'column' : 'row',
@@ -72,7 +73,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             entries = [],
 
             // the largest valid column/row index
-            maxIndex = model.getMaxIndex(columns),
+            maxIndex = docModel.getMaxIndex(columns),
 
             // the current zoom factor of the document
             zoom = sheetModel.getEffectiveZoom();
@@ -80,6 +81,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
         // base constructor ---------------------------------------------------
 
         ModelObject.call(this, app);
+        TimerMixin.call(this);
 
         // private class EntryModel -------------------------------------------
 
@@ -90,7 +92,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *
          * @extends CellAttributesModel
          */
-        var EntryModel = CellAttributesModel.extend({ constructor: function (first, last, attributes) {
+        var EntryModel = CellAttributesModel.extend({ constructor: function (first, last, initAttributes, initOptions) {
 
             // column/row interval covered by this entry
             this.first = first;
@@ -106,11 +108,11 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
 
             // base constructor - - - - - - - - - - - - - - - - - - - - - - - -
 
-            // do not trigger any events (collection entries are potential mass objects)
-            CellAttributesModel.call(this, app, attributes, {
-                additionalFamilies: FAMILY_NAME,
-                parentModel: sheetModel
-            });
+            CellAttributesModel.call(this, app, initAttributes, _.extend({
+                families: FAMILY_NAME,
+                parentModel: sheetModel,
+                listenToParent: false
+            }, initOptions));
 
             // initialization - - - - - - - - - - - - - - - - - - - - - - - - -
 
@@ -126,7 +128,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          * be part of another collection instance (e.g. copy construction).
          */
         EntryModel.cloneEntry = function (sourceEntry) {
-            var newEntry = new EntryModel(sourceEntry.first, sourceEntry.last, sourceEntry.getExplicitAttributes());
+            var newEntry = new EntryModel(sourceEntry.first, sourceEntry.last, sourceEntry.getExplicitAttributes(true));
             newEntry.offsetHmm = sourceEntry.offsetHmm;
             newEntry.offset = sourceEntry.offset;
             return newEntry;
@@ -179,10 +181,29 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
         };
 
         var // default model for entries missing in the collection
-            defaultEntry = new EntryModel(0, maxIndex);
+            defaultEntry = new EntryModel(0, maxIndex, null, { listenToParent: true });
 
         // private methods ----------------------------------------------------
 
+        /**
+         * Dumps the entire collection to the debug console.
+         */
+        var debugDump = SheetUtils.isLoggingActive() ? (function () {
+
+            function dumpMsg(msg) {
+                SheetUtils.info(sheetModel.getName() + '.' + (columns ? 'Col' : 'Row') + 'Collection: ' + msg);
+            }
+
+            var dumpContents = app.createDebouncedActionsMethod(self, function () {
+                dumpMsg('new contents');
+                _.each(entries, function (entry) {
+                    SheetUtils.log('  interval=' + SheetUtils.getIntervalName(entry, columns) + ' offset=' + entry.offset + ' size=' + entry.size + ' attrs=' + Utils.stringifyForDebug(entry.getExplicitAttributes(true)));
+                });
+            });
+
+            return self.createDebouncedMethod(dumpMsg, dumpContents);
+        }()) : $.noop;
+
         /**
          * Converts the passed zoom-independent length in 1/100 mm to a length
          * in pixels according to the current sheet zoom factor.
@@ -219,23 +240,26 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
         }
 
         /**
-         * Updates the dirty entry offsets up to the end of the collection.
+         * Updates the dirty entry offsets and sizes up to the end of the
+         * collection, after changing the collection, or the sheet attributes.
          *
-         * @param {Number} index
+         * @param {Number} startIndex
          *  The zero-based column/row index where updating the offsets starts.
+         *
+         * @param {Boolean} [updateSize=false]
+         *  If set to true, the size of the entries will be updated too. By
+         *  default, only the offset positions of the entries will be updated.
          */
-        function updateEntryOffsets(index) {
+        function updateEntryGeometry(startIndex, updateSize) {
 
             var // the array index of the first dirty collection entry
-                arrayIndex = 0,
+                arrayIndex = findEntryIndex(startIndex),
                 // the last valid collection entry
-                prevEntry = null,
+                prevEntry = entries[arrayIndex - 1],
                 // the current collection entry to be updated
                 currEntry = null;
 
             // update start offsets of all dirty collection entries
-            arrayIndex = findEntryIndex(index);
-            prevEntry = entries[arrayIndex - 1];
             for (; arrayIndex < entries.length; arrayIndex += 1) {
 
                 // set offset of current entry to the end offset of the previous entry
@@ -249,10 +273,35 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                     currEntry.offset += defaultEntry.size * (currEntry.first - (prevEntry ? (prevEntry.last + 1) : 0));
                 }
 
+                // update size of the current entry (first update the merged
+                // attributes, parent sheet attributes may have been changed)
+                if (updateSize) { currEntry.refreshMergedAttributeSet().updateSize(); }
+
                 prevEntry = currEntry;
             }
         }
 
+        /**
+         * Updates the default column/row size according to the current
+         * formatting attributes of the sheet model.
+         */
+        function updateDefaultSize() {
+
+            var // the old default size, in 1/100 mm, and in pixels
+                oldDefSizeHmm = defaultEntry.sizeHmm,
+                oldDefSize = defaultEntry.size;
+
+            // calculate the new default size
+            defaultEntry.updateSize();
+
+            // do nothing, if the effective size has not changed (check both sizes to catch any rounding errors)
+            if ((defaultEntry.sizeHmm !== oldDefSizeHmm) || (defaultEntry.size !== oldDefSize)) {
+                updateEntryGeometry(0, true);
+                self.trigger('change:entries', self.getFullInterval());
+                debugDump('default size changed');
+            }
+        }
+
         /**
          * Creates a descriptor object for the specified column/row.
          *
@@ -445,7 +494,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             var // the result object
                 result = {};
 
-            // tries to merge the passed entries, removes the preceding entry
+            // tries to merge the entry with its predecessor, removes the preceding entry
             function tryMerge(currIndex) {
 
                 var // the preceding collection entry
@@ -454,7 +503,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                     thisEntry = entries[currIndex];
 
                 // check that the entries are equal and do not have a gap
-                if (prevEntry && thisEntry && (prevEntry.last + 1 === thisEntry.first) && _.isEqual(prevEntry.getExplicitAttributes(), thisEntry.getExplicitAttributes())) {
+                if (prevEntry && thisEntry && (prevEntry.last + 1 === thisEntry.first) && _.isEqual(prevEntry.getExplicitAttributes(true), thisEntry.getExplicitAttributes(true))) {
                     thisEntry.first = prevEntry.first;
                     thisEntry.offsetHmm = prevEntry.offsetHmm;
                     thisEntry.offset = prevEntry.offset;
@@ -484,13 +533,13 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *  (1) {EntryModel} entry
          *      The current collection entry.
          *  (2) {Number} first
-         *      The zero-based index of the first column/row. May be located
-         *      inside the passed collection entry, if it is the first entry
-         *      visited by this method.
+         *      The zero-based index of the first column/row. May be located in
+         *      the middle of the visited collection entry, if it is the first
+         *      entry of the interval passed to this method.
          *  (3) {Number} last
-         *      The zero-based index of the last column/row. May be located
-         *      inside the passed collection entry, if it is the last entry
-         *      visited by this method.
+         *      The zero-based index of the last column/row. May be located in
+         *      the middle of the visited collection entry, if it is the last
+         *      entry of the interval passed to this method.
          *  If the iterator returns the Utils.BREAK object, the iteration
          *  process will be stopped immediately.
          *
@@ -568,8 +617,15 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                 // invoke the iterator
                 invokeIterator(entry, other);
 
-                // Try to merge the new entry with its predecessor (arrayIndex
-                // points after the new entry, after merging, it has to be decreased).
+                // remove entry from the collection, if it does not contain any explicit attributes
+                if (modify && !entry.hasExplicitAttributes()) {
+                    arrayIndex -= 1;
+                    deleteEntries(arrayIndex, 1);
+                    return;
+                }
+
+                // try to merge the new entry with its predecessor
+                // (arrayIndex points behind the new entry, after merging, it has to be decreased)
                 if (modify && mergeEntry(arrayIndex - 1, { prev: true }).prev) {
                     arrayIndex -= 1;
                 }
@@ -581,12 +637,22 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                 // invoke the iterator
                 invokeIterator(entries[arrayIndex], other);
 
-                // Try to merge the entry with its predecessor. On success, do
-                // not change the array index (it already points to the next
-                // entry after merging the entries).
-                if (!(modify && mergeEntry(arrayIndex, { prev: true }).prev)) {
-                    arrayIndex += reverse ? -1 : 1;
+                // Remove entry from the collection, if it does not contain any explicit attributes
+                // anymore. On success, do not change the array index (it already points to the next
+                // entry after deleting the current entry).
+                if (modify && !entries[arrayIndex].hasExplicitAttributes()) {
+                    deleteEntries(arrayIndex, 1);
+                    return;
                 }
+
+                // Try to merge the entry with its predecessor. On success, do not change the
+                // array index (it already points to the next entry after merging the entries).
+                if (modify && mergeEntry(arrayIndex, { prev: true }).prev) {
+                    return;
+                }
+
+                // go to next (or previous) array element
+                arrayIndex += reverse ? -1 : 1;
             }
 
             // modify reversely not implemented
@@ -627,8 +693,8 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                 invokeIteratorForEntry(reverse ? Math.max(interval.first, entry.first) : Math.min(interval.last, entry.last));
             }
 
-            // Try to merge last visited entry with its successor (arrayIndex
-            // already points to the successor of the last visited entry).
+            // try to merge last visited entry with its successor
+            // (arrayIndex already points to the successor of the last visited entry)
             if (modify) {
                 mergeEntry(arrayIndex, { prev: true });
             }
@@ -694,6 +760,13 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                 // the resulting entry descriptor
                 entryData = null;
 
+            // special handling if all entries are hidden: return first entry
+            if (totalSize === 0) {
+                entryData = self.getEntry(0);
+                entryData.relOffset = entryData.relOffsetHmm = 0;
+                return entryData;
+            }
+
             // offset outside sheet limits: always return first/last collection entry if specified
             if (Utils.getBooleanOption(options, 'outerHidden', false)) {
                 if (offset < 0) {
@@ -739,33 +812,36 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
         function changeViewAttributesHandler(event, attributes) {
             if ('zoom' in attributes) {
                 zoom = sheetModel.getEffectiveZoom();
-                _.invoke(entries, 'updateSize');
                 defaultEntry.updateSize();
-                updateEntryOffsets(0);
+                updateEntryGeometry(0, true);
+                debugDump('zoom changed');
             }
         }
 
+        // protected methods --------------------------------------------------
+
         /**
-         * Updates the default column/row size according to the current
-         * formatting attributes of the sheet model.
+         * Creates and returns a cloned instance of this column/row collection
+         * for the specified sheet.
+         *
+         * @internal
+         *  Used by the class SheetModel during clone construction. DO NOT CALL
+         *  from external code!
+         *
+         * @param {SheetModel} targetModel
+         *  The model instance of the new cloned sheet that will own the clone
+         *  returned by this method.
+         *
+         * @returns {ColRowCollection}
+         *  A clone of this column/row collection, initialized for ownership by
+         *  the passed sheet model.
          */
-        function updateDefaultSize() {
+        this.clone = function (targetModel) {
+            // construct a new collection, pass all own entries as hidden parameter
+            return new ColRowCollection(app, targetModel, columns, entries);
+        };
 
-            var // the old default size, in 1/100 mm, and in pixels
-                oldDefSizeHmm = defaultEntry.sizeHmm,
-                oldDefSize = defaultEntry.size;
-
-            // calculate the new default size
-            defaultEntry.updateSize();
-
-            // do nothing, if the effective size has not changed (check both sizes to catch any rounding errors)
-            if ((defaultEntry.sizeHmm !== oldDefSizeHmm) || (defaultEntry.size !== oldDefSize)) {
-                updateEntryOffsets(0);
-                self.trigger('change:entries', { first: 0, last: maxIndex });
-            }
-        }
-
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the maximum valid column/row index available for an entry in
@@ -778,6 +854,16 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             return maxIndex;
         };
 
+        /**
+         * Returns a new column/row interval covering the entire collection.
+         *
+         * @returns {Object}
+         *  A new column/row interval covering the entire collection.
+         */
+        this.getFullInterval = function () {
+            return { first: 0, last: maxIndex };
+        };
+
         /**
          * Returns the total size of all columns/rows represented by this
          * collection, in 1/100 of millimeters, independent from the current
@@ -859,10 +945,20 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *  (1) {Object} entryData
          *      An entry descriptor object with all properties as described for
          *      the return value of the method ColRowCollection.getEntry().
-         *  (2) {Object} interval
-         *      The column/row interval from the array passed to the parameter
-         *      'intervals' of this method, containing the column/row currently
-         *      visited.
+         *  (2) {Object} uniqueInterval
+         *      The column/row interval containing all columns/rows with the
+         *      same formatting attributes as the column/row currently visited.
+         *      Useful in 'unique' mode (see option 'unique' below) where only
+         *      the first entry (or last entry in reverse mode) of such an
+         *      interval will be visited. Will be shrunken to the intervals
+         *      passed to this method, regardless how large the entire equally
+         *      formatted interval is.
+         *  (3) {Object} origInterval
+         *      The original interval from the interval array passed to this
+         *      method that contains the column/row currently visited.
+         *  (4) {Object} intervalIndex
+         *      The array index of the interval contained in the 'origInterval'
+         *      parameter.
          *  If the iterator returns the Utils.BREAK object, the iteration
          *  process will be stopped immediately.
          *
@@ -887,6 +983,12 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
          *  @param {Boolean} [options.reverse=false]
          *      If set to true, visits the passed intervals, and the single
          *      column/row entries in each interval from last to first.
+         *  @param {Boolean} [options.unique=false]
+         *      If set to true, the iterator callback function will be invoked
+         *      once only for an entire column/row interval with the same
+         *      formatting attributes. In forward mode, the first available
+         *      column/row of such an interval will be visited; in reverse
+         *      mode, the last available column/row will be visited.
          *
          * @returns {Utils.BREAK|Undefined}
          *  A reference to the Utils.BREAK object, if the iterator has returned
@@ -904,6 +1006,8 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                 customFormat = columns ? null : Utils.getBooleanOption(options, 'customFormat', null),
                 // whether to iterate in reversed order
                 reverse = Utils.getBooleanOption(options, 'reverse', false),
+                // whether to iterate once per formatting interval
+                unique = Utils.getBooleanOption(options, 'unique', false),
                 // the last processed collection entry
                 lastEntry = null;
 
@@ -936,9 +1040,15 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                         // do not invoke iterator for hidden columns/rows (unless hidden mode is 'all')
                         if ((hiddenMode !== 'all') && (entry.size === 0)) { return; }
 
+                        // invoke iterator once only in unique mode
+                        var uniqueInterval = { first: first, last: last };
+                        if (unique) {
+                            return iterator.call(context, makeDescriptorForEntry(entry, reverse ? last : first), uniqueInterval, interval);
+                        }
+
                         // invoke iterator for every column/row covered by the current collection entry
                         return Utils.iterateRange(reverse ? last : first, reverse ? (first - 1) : (last + 1), function (index) {
-                            return iterator.call(context, makeDescriptorForEntry(entry, index), interval);
+                            return iterator.call(context, makeDescriptorForEntry(entry, index), uniqueInterval, interval);
                         }, { step: reverse ? -1 : 1 });
 
                     // always update the variable 'lastEntry', regardless where this method returns
@@ -951,55 +1061,45 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
         };
 
         /**
-         * Returns the mixed formatting attributes of all columns/rows covered
-         * by the passed index intervals.
+         * Returns the mixed column/row attributes of all entries covered by
+         * the passed index intervals.
          *
          * @param {Object|Array} intervals
          *  A column/row interval, or an array of column/row intervals.
          *
          * @returns {Object}
-         *  A complete attribute set containing the mixed attributes of all
-         *  columns/rows covered by the passed intervals. All attributes that
-         *  could not be resolved unambiguously will be set to the value null.
+         *  The mixed attributes of all columns/rows covered by the passed
+         *  intervals, as simple object (NOT mapped as a 'column' or 'row'
+         *  sub-object). The result does not contain any cell formatting
+         *  attributes. All attributes that could not be resolved unambiguously
+         *  will be set to the value null.
          */
         this.getMixedAttributes = function (intervals) {
 
-            var // the resulting mixed attribute set
+            var // the resulting mixed attributes
                 mixedAttributes = null;
 
             // visit all equally formatted intervals, and build the mixed attributes
             Utils.iterateArray(SheetUtils.getUnifiedIntervals(intervals), function (interval) {
                 return iterateEntries(interval, function (entry) {
 
-                    var // the merged attribute set of the current interval
-                        attributes = entry.getMergedAttributes(),
+                    var // the merged attributes of the current interval (type column/row only)
+                        attributes = entry.getMergedAttributes()[FAMILY_NAME],
                         // whether any attribute is still unambiguous
                         hasNonNull = false;
 
-                    // mixes the passed attribute value into the object
-                    function mixAttribute(map, name, value) {
-                        if (_.isEqual(value, map[name])) {
-                            hasNonNull = true;
-                        } else {
-                            map[name] = null;
-                        }
-                    }
-
                     // first visited interval: store initial attributes
                     if (!mixedAttributes) {
                         mixedAttributes = _.copy(attributes, true);
                         return;
                     }
 
-                    // mix the style identifier of the visited interval
-                    _.each(attributes, function (values, family) {
-                        if (family === 'styleId') {
-                            mixAttribute(mixedAttributes, 'styleId', values);
+                    // mix the attributes of the visited interval into the result
+                    _.each(attributes, function (value, name) {
+                        if (_.isEqual(value, mixedAttributes[name])) {
+                            hasNonNull = true;
                         } else {
-                            var mixedValues = mixedAttributes[family];
-                            _.each(values, function (value, name) {
-                                mixAttribute(mixedValues, name, value);
-                            });
+                            mixedAttributes[name] = null;
                         }
                     });
 
@@ -1026,7 +1126,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
 
         /**
          * Returns information about the column/row with the passed index, if
-         * it is visible; otherwise about the first visible column/row that
+         * it is visible; otherwise about the nearest visible column/row that
          * follows the column/row with the passed index.
          *
          * @param {Number} index
@@ -1058,7 +1158,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
 
         /**
          * Returns information about the column/row with the passed index, if
-         * it is visible; otherwise about the first visible column/row that
+         * it is visible; otherwise about the nearest visible column/row that
          * precedes the column/row with the passed index.
          *
          * @param {Number} index
@@ -1080,7 +1180,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             var // the resulting entry data
                 entryData = null;
 
-            // use iterator to visit the first visible entry following the passed index
+            // use reverse iterator to visit the first visible entry preceding the passed index
             this.iterateEntries({ first: _.isNumber(firstIndex) ? firstIndex : 0, last: index }, function (entry) {
                 entryData = entry;
                 return Utils.BREAK;
@@ -1170,57 +1270,6 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             return this.getIntervalPosition(interval).size === 0;
         };
 
-        /**
-         * Returns a copy of the passed column/row interval. If the first entry
-         * of the passed interval is hidden, the returned interval will start
-         * at the first visible entry contained in the interval. Same applies
-         * to the last entry of the passed interval. If the entire interval is
-         * hidden, null will be returned.
-         *
-         * @param {Object} interval
-         *  The column/row interval to be processed, with the zero-based index
-         *  properties 'first' and 'last'.
-         *
-         * @returns {Object|Null}
-         *  The visible part of the passed interval (but may contain inner
-         *  hidden columns/rows); or null, if the entire interval is hidden.
-         */
-        this.getVisibleInterval = function (interval) {
-
-            var // first visible collection entry
-                firstEntry = this.getNextVisibleEntry(interval.first, interval.last),
-                // last visible collection entry
-                lastEntry = firstEntry ? this.getPrevVisibleEntry(interval.last, interval.first) : null;
-
-            // return the resulting interval
-            return (firstEntry && lastEntry) ? { first: firstEntry.index, last: lastEntry.index } : null;
-        };
-
-        /**
-         * Returns a copy of the passed column/row interval, which will be
-         * expanded to include the next available visible entry in both
-         * directions.
-         *
-         * @param {Object} interval
-         *  The column/row interval to be expanded, with the zero-based index
-         *  properties 'first' and 'last'.
-         *
-         * @returns {Object}
-         *  The expanded interval. If no visible entry has been found before or
-         *  after the passed interval, the respective index of the passed
-         *  original interval will be returned.
-         */
-        this.expandVisibleInterval = function (interval) {
-
-            var // preceding visible collection entry
-                prevEntry = this.getPrevVisibleEntry(interval.first - 1),
-                // following visible collection entry
-                nextEntry = this.getNextVisibleEntry(interval.last + 1);
-
-            // return the resulting interval
-            return { first: prevEntry ? prevEntry.index : interval.first, last: nextEntry ? nextEntry.index : interval.last };
-        };
-
         /**
          * Returns the visible column/row intervals contained in the passed
          * interval.
@@ -1288,6 +1337,32 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             return indexes;
         };
 
+        /**
+         * Returns a copy of the passed column/row interval. If the first entry
+         * of the passed interval is hidden, the returned interval will start
+         * at the first visible entry contained in the interval. Same applies
+         * to the last entry of the passed interval. If the entire interval is
+         * hidden, null will be returned.
+         *
+         * @param {Object} interval
+         *  The column/row interval to be processed, with the zero-based index
+         *  properties 'first' and 'last'.
+         *
+         * @returns {Object|Null}
+         *  The visible part of the passed interval (but may contain inner
+         *  hidden columns/rows); or null, if the entire interval is hidden.
+         */
+        this.shrinkToVisibleInterval = function (interval) {
+
+            var // first visible collection entry
+                firstEntry = this.getNextVisibleEntry(interval.first, interval.last),
+                // last visible collection entry
+                lastEntry = firstEntry ? this.getPrevVisibleEntry(interval.last, interval.first) : null;
+
+            // return the resulting interval
+            return (firstEntry && lastEntry) ? { first: firstEntry.index, last: lastEntry.index } : null;
+        };
+
         /**
          * Returns the absolute sheet offset of a specific position inside a
          * column/row in 1/100 mm, independent from the current sheet zoom
@@ -1574,15 +1649,18 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
                         oldSizeHmm = entry.sizeHmm,
                         oldSize = entry.size;
 
-                    // set the passed attributes; post-processing, if any attributes have changed
-                    if (entry.setCellAttributes(attributes, attributeOptions)) {
-                        changed = true;
-                        // update effective size of the collection entry
-                        entry.updateSize();
-                        // set index of entries with dirty pixel offsets, if size has been changed
-                        if (((oldSizeHmm !== entry.sizeHmm) || (oldSize !== entry.size)) && _.isNull(dirtyIndex)) {
-                            dirtyIndex = entry.last + 1;
-                        }
+                    // set the passed attributes; post-processing only if any attributes have changed
+                    if (!entry.setCellAttributes(attributes, attributeOptions)) { return; }
+
+                    // collection has changed
+                    changed = true;
+
+                    // update effective size of the collection entry
+                    entry.updateSize();
+
+                    // set index of entries with dirty pixel offsets, if size has been changed
+                    if (((oldSizeHmm !== entry.sizeHmm) || (oldSize !== entry.size)) && _.isNull(dirtyIndex)) {
+                        dirtyIndex = entry.last + 1;
                     }
                 }, { modify: true });
             }
@@ -1598,13 +1676,16 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
 
             // update pixel offsets of following entries, if the size has been changed
             if (_.isNumber(dirtyIndex)) {
-                updateEntryOffsets(dirtyIndex);
+                updateEntryGeometry(dirtyIndex);
             }
 
             // notify change listeners
             if (changed && !Utils.getBooleanOption(options, 'silent', false)) {
                 this.trigger('change:entries', interval, attributes, options);
             }
+
+            // log the entire collection
+            debugDump('interval ' + SheetUtils.getIntervalName(interval, columns) + ' changed');
             return this;
         };
 
@@ -1681,10 +1762,11 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             }
 
             // update pixel offsets of the entries
-            updateEntryOffsets(interval.first);
+            updateEntryGeometry(interval.first);
 
             // notify insert listeners
             this.trigger('insert:entries', interval);
+            debugDump('interval ' + SheetUtils.getIntervalName(interval, columns) + ' inserted');
             return this;
         };
 
@@ -1739,10 +1821,11 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             mergeEntry(arrayIndex, { prev: true });
 
             // update pixel offsets of the entries
-            updateEntryOffsets(interval.first);
+            updateEntryGeometry(interval.first);
 
             // notify delete listeners
             this.trigger('delete:entries', interval);
+            debugDump('interval ' + SheetUtils.getIntervalName(interval, columns) + ' deleted');
             return this;
         };
 
@@ -1760,14 +1843,9 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
             this.listenTo(app.getImportPromise(), 'done', updateDefaultSize);
         }
 
-        // constructor called from BaseObject.clone()
-        this.registerCloneConstructor(function (newSheetModel) {
-            // pass own entries as hidden argument to the constructor
-            return new ColRowCollection(app, newSheetModel, columns, entries);
-        });
-
-        // clone collection entries passed as hidden argument to the c'tor
+        // clone collection entries passed as hidden argument by the clone() method
         if (_.isArray(arguments[ColRowCollection.length])) {
+            // static method EntryModel.cloneEntry() creates clones in the context of this collection
             entries = _.map(arguments[ColRowCollection.length], EntryModel.cloneEntry);
         }
 
@@ -1775,7 +1853,7 @@ define('io.ox/office/spreadsheet/model/colrowcollection',
         this.registerDestructor(function () {
             _.invoke(entries, 'destroy');
             defaultEntry.destroy();
-            model = sheetModel = entries = defaultEntry = null;
+            app = sheetModel = self = docModel = entries = defaultEntry = null;
         });
 
     } // class ColRowCollection
