@@ -1,29 +1,31 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/documentstyles',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/editframework/utils/color',
-     'io.ox/office/editframework/utils/border',
-     'io.ox/office/editframework/model/documentstyles',
-     'io.ox/office/drawinglayer/model/drawingstylecollection',
-     'io.ox/office/spreadsheet/model/cellstylecollection'
-    ], function (Utils, Color, Border, DocumentStyles, DrawingStyleCollection, CellStyleCollection) {
+define('io.ox/office/spreadsheet/model/documentstyles', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/utils/color',
+    'io.ox/office/editframework/utils/border',
+    'io.ox/office/editframework/model/documentstyles',
+    'io.ox/office/drawinglayer/model/drawingstylecollection',
+    'io.ox/office/spreadsheet/model/cellstylecollection',
+    'io.ox/office/spreadsheet/model/tablestylecollection'
+], function (Utils, Color, Border, DocumentStyles, DrawingStyleCollection, CellStyleCollection, TableStyleCollection) {
 
     'use strict';
 
-    var // default values of spreadsheet document attributes
-        SPREADSHEET_DOCUMENT_ATTRIBUTES = {
+    // definitions for spreadsheet document attributes ------------------------
+
+    var SPREADSHEET_DOCUMENT_ATTRIBUTES = {
 
             /**
              * Number of columns in a sheet.
@@ -41,55 +43,71 @@ define('io.ox/office/spreadsheet/model/documentstyles',
              * Index of the active (displayed) sheet.
              */
             activeSheet: 0
-        },
+        };
 
-        // definitions for character attributes in cells
-        CHARACTER_ATTRIBUTE_DEFINITIONS = {
-            fontName: { def: 'Arial' },
-            fontSize: { def: 11 },
-            bold: { def: false },
-            italic: { def: false },
-            underline: { def: false },
-            strike: { def: 'none' },
-            color: { def: Color.AUTO },
+    // definitions for character attributes in cells --------------------------
+
+    var CHARACTER_ATTRIBUTE_DEFINITIONS = {
+            fontName: { def: 'Arial', apply: 'font' },
+            fontSize: { def: 11, apply: 'font' },
+            bold: { def: false, apply: 'font' },
+            italic: { def: false, apply: 'font' },
+            underline: { def: false, apply: 'font' },
+            strike: { def: 'none', apply: 'font' },
+            color: { def: Color.AUTO, apply: 'font' },
             url: { def: '', scope: 'element' }
-        },
+        };
 
-        // definitions for cell attributes
-        CELL_ATTRIBUTE_DEFINITIONS = {
-            alignHor: { def: 'auto' },
-            alignVert: { def: 'bottom' },
-            wrapText: { def: false },
-            numberFormat: { def: { id: 0, code: 'General' }, merge: mergeNumberFormat },
-            fillColor: { def: Color.AUTO },
-            borderLeft: { def: Border.NONE },
-            borderRight: { def: Border.NONE },
-            borderTop: { def: Border.NONE },
-            borderBottom: { def: Border.NONE },
+    // definitions for cell attributes ----------------------------------------
+
+    var CELL_ATTRIBUTE_DEFINITIONS = {
+            alignHor: { def: 'auto', apply: 'align' },
+            alignVert: { def: 'bottom', apply: 'align' },
+            wrapText: { def: false, apply: 'align' },
+            numberFormat: { def: { id: 0 }, apply: 'number' },
+            fillColor: { def: Color.AUTO, apply: 'fill' },
+            borderLeft: { def: Border.NONE, apply: 'border' },
+            borderRight: { def: Border.NONE, apply: 'border' },
+            borderTop: { def: Border.NONE, apply: 'border' },
+            borderBottom: { def: Border.NONE, apply: 'border' },
             borderInsideHor: { def: Border.NONE, scope: 'element' },
             borderInsideVert: { def: Border.NONE, scope: 'element' },
-            unlocked: { def: false },
-            hidden: { def: false }
-        },
+            unlocked: { def: false, apply: 'protect' },
+            hidden: { def: false, apply: 'protect' }
+        };
 
-        // definitions for column attributes
-        COLUMN_ATTRIBUTE_DEFINITIONS = {
+    // definitions for apply group flags --------------------------------------
+
+    var APPLY_ATTRIBUTE_DEFINITIONS = {
+            font: { def: true },
+            fill: { def: true },
+            border: { def: true },
+            align: { def: true },
+            number: { def: true },
+            protect: { def: true }
+        };
+
+    // definitions for column attributes --------------------------------------
+
+    var COLUMN_ATTRIBUTE_DEFINITIONS = {
             visible: { def: true },
             width: { def: 2000 },
             autoFit: { def: true },
             customWidth: { def: false }
-        },
+        };
 
-        // definitions for row attributes
-        ROW_ATTRIBUTE_DEFINITIONS = {
+    // definitions for row attributes -----------------------------------------
+
+    var ROW_ATTRIBUTE_DEFINITIONS = {
             visible: { def: true },
             height: { def: 500 },
             customHeight: { def: false },
             customFormat: { def: false }
-        },
+        };
 
-        // definitions for sheet attributes
-        SHEET_ATTRIBUTE_DEFINITIONS = {
+    // definitions for sheet attributes ---------------------------------------
+
+    var SHEET_ATTRIBUTE_DEFINITIONS = {
             visible: { def: true },
             locked: { def: false },
             baseColWidth: { def: 8 },
@@ -110,14 +128,75 @@ define('io.ox/office/spreadsheet/model/documentstyles',
             scrollRight: { def: 0 },
             scrollTop: { def: 0 },
             scrollBottom: { def: 0 }
-        },
+        };
 
-        // definitions for data validation attributes
-        VALIDATION_ATTRIBUTE_DEFINITIONS = {
+    // definitions for table attributes ---------------------------------------
+
+    var TABLE_ATTRIBUTE_DEFINITIONS = {
+
+            /**
+             * Whether the first row of the table is a header row. Header rows
+             * will not be filtered or sorted.
+             */
+            headerRow: { def: false, scope: 'element' },
+
+            /**
+             * Whether the last row of the table is a footer row. Footer rows
+             * will not be filtered or sorted.
+             */
+            footerRow: { def: false, scope: 'element' },
+
+            /**
+             * Whether the table can be filtered (this flag does not mean that
+             * the table has actual filter rules applied, but the GUI may
+             * provide visual feedback and control elements for filtering).
+             */
+            filtered: { def: false, scope: 'element' }
+        };
+
+    // definitions for filter attributes in table columns ---------------------
+
+    var TABLE_FILTER_ATTRIBUTE_DEFINITIONS = {
+
+            /**
+             * The type of the filter rule in this column. The following values
+             * are supported:
+             * - 'none': No filter rule is active.
+             * - 'discrete': Discrete filter. The attribute 'entries' contains
+             *      all cell entries to be shown for the table column.
+             */
+            type: { def: 'none' },
+
+            /**
+             * All entries (cell display strings) that will be shown for this
+             * table column. Only used, if the filter type is 'discrete'.
+             */
+            entries: { def: [], parent: { name: 'type', value: 'discrete' } },
+
+            /**
+             * Whether the drop-down button of the filter column is currently
+             * hidden by a merged cell range.
+             */
+            hideMergedButton: { def: false }
+        };
+
+    // definitions for sorting attributes in table columns --------------------
+
+    var TABLE_SORT_ATTRIBUTE_DEFINITIONS = {
+
+            /**
+             * Whether to sort the column entries in descending order.
+             */
+            descending: { def: false },
+        };
+
+    // definitions for data validation attributes -----------------------------
+
+    var VALIDATION_ATTRIBUTE_DEFINITIONS = {
             type: { def: 'all' },
-            compare: { def: '' },
-            value1: { def: '' },
-            value2: { def: '' },
+            compare: { def: '', parent: { name: 'type', value: function (value) { return (/^(integer|number|date|time|length)$/).test(value); } } },
+            value1: { def: '', parent: { name: 'type', value: function (value) { return value !== 'all'; } } },
+            value2: { def: '', parent: { name: 'type', value: function (value) { return value !== 'all'; } } },
             showInfo: { def: true },
             infoTitle: { def: '' },
             infoText: { def: '' },
@@ -127,10 +206,11 @@ define('io.ox/office/spreadsheet/model/documentstyles',
             errorType: { def: 'error' },
             showDropDown: { def: true },
             ignoreEmpty: { def: true }
-        },
+        };
 
-        // additional definitions for drawing attributes
-        DRAWING_ATTRIBUTE_DEFINITIONS = {
+    // additional definitions for drawing attributes --------------------------
+
+    var DRAWING_ATTRIBUTE_DEFINITIONS = {
 
             /**
              * Type of the drawing anchor (specifies which anchor attributes
@@ -190,19 +270,6 @@ define('io.ox/office/spreadsheet/model/documentstyles',
             endRowOffset: { def: 0, scope: 'element' }
         };
 
-    // global private functions ===============================================
-
-    /**
-     * Returns a number format attribute value that results from updating an
-     * existing number format with another number format. Keeps the format
-     * identifier of the existing number format, if the format codes of both
-     * formats are equal, and the new number format does not contain an own
-     * identifier.
-     */
-    function mergeNumberFormat(oldFormat, newFormat) {
-        return (!('id' in newFormat) && (oldFormat.code === newFormat.code)) ? oldFormat : newFormat;
-    }
-
     // class SpreadsheetDocumentStyles ========================================
 
     /**
@@ -222,10 +289,13 @@ define('io.ox/office/spreadsheet/model/documentstyles',
         var // the font collection of the document
             fontCollection = null,
 
-            // the cell style collection of the document
+            // the style sheet collection for cells, columns, and rows
             cellStyles = null,
 
-            // the drawing style collection of the document
+            // the style sheet collection for table ranges
+            tableStyles = null,
+
+            // the style sheet collection for drawing objects
             drawingStyles = null;
 
         // base constructor ---------------------------------------------------
@@ -306,21 +376,27 @@ define('io.ox/office/spreadsheet/model/documentstyles',
 
         fontCollection = this.getFontCollection();
         cellStyles = new CellStyleCollection(app, this);
+        tableStyles = new TableStyleCollection(app, this);
         drawingStyles = new DrawingStyleCollection(app, this);
 
         this.registerAttributeDefinitions('character', CHARACTER_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('cell', CELL_ATTRIBUTE_DEFINITIONS)
+            .registerAttributeDefinitions('apply', APPLY_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('column', COLUMN_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('row', ROW_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('sheet', SHEET_ATTRIBUTE_DEFINITIONS)
+            .registerAttributeDefinitions('table', TABLE_ATTRIBUTE_DEFINITIONS)
+            .registerAttributeDefinitions('filter', TABLE_FILTER_ATTRIBUTE_DEFINITIONS)
+            .registerAttributeDefinitions('sort', TABLE_SORT_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('validation', VALIDATION_ATTRIBUTE_DEFINITIONS)
             .registerAttributeDefinitions('drawing', DRAWING_ATTRIBUTE_DEFINITIONS);
 
         this.addStyleCollection(cellStyles)
+            .addStyleCollection(tableStyles)
             .addStyleCollection(drawingStyles);
 
         this.registerDestructor(function () {
-           app = fontCollection = cellStyles = drawingStyles = null;
+           app = fontCollection = cellStyles = tableStyles = drawingStyles = null;
         });
 
     } // class SpreadsheetDocumentStyles
