@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
@@ -26,7 +26,8 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
 
     var PERCENTFORMAT = {
         exp: -2,
-        format: '0%'
+        format: '0%',
+        labelLength: 4
     };
 
     ///////////////////////////////////////////////////////////////////////////
@@ -41,6 +42,7 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
         var zeroLine = null;
         var grid = null;
         var format = null;
+        var labelLen = 0;
 
         var dataAxis = chart.getModelData()['axis' + type.toUpperCase()];
         if(dataAxis){
@@ -59,22 +61,19 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
 
         // base constructor ---------------------------------------------------
 
-        AttributedModel.call(this, chart.getApp(), {}, { additionalFamilies: ['axis', 'line', 'character'] });
+        AttributedModel.call(this, chart.getApp(), null, { families: 'axis line character' });
 
         // methods ------------------------------------------------------------
 
         this.refreshInfo = function () {
-
+            labelLen = 0;
+            if (format) {
+                labelLen = format.labelLength;
+            }
             refreshMinMax();
 
-            var app = chart.getApp();
-            if (!dataAxis) {
-                return;
-            }
-            var view = app.getView();
-            if (!view) {
-                return;
-            }
+            if (!dataAxis) { return; }
+
             var attrs = this.getMergedAttributes();
 
             ChartStyleUtil.handleLineProps(chart, attrs.line, dataAxis, 'line');
@@ -87,6 +86,20 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
                 dataAxis.labelFontSize = 1;
             }
 
+            dataAxis.labelAutoFit = false;
+            dataAxis.labelAngle = 0;
+
+            var drawingType = chart.getAxisIdForDrawing(type);
+            if (drawingType === 'x') {
+                if (labelLen > 10 ) {
+                    dataAxis.labelAngle = -20;
+                }
+            } else {
+                if (labelLen > 10 ) {
+                    dataAxis.labelAutoFit = true;
+                }
+            }
+
             if (zeroLine) {
                 zeroLine.color = dataAxis.lineColor;
                 zeroLine.thickness = dataAxis.lineThickness;
@@ -95,17 +108,17 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
             var overwriteLabels = false;
             var stripLines = dataAxis.stripLines;
             _.each(stripLines, function(stripLine) {
-                if(!stripLine.zero){
+                if (!stripLine.zero) {
                     if (attrs.axis.label) {
                         ChartStyleUtil.handleCharacterProps(chart, attrs.character, stripLine, 'label');
-                    }else{
+                    } else {
                         stripLine.labelFontColor = 'transparent';
                         stripLine.labelFontSize = 1;
                     }
                     overwriteLabels = true;
                 }
             });
-            if(overwriteLabels){
+            if (overwriteLabels) {
                 dataAxis.labelFontColor = 'transparent';
             }
 
@@ -193,6 +206,7 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
             var formatter = chart.getApp().getModel().getNumberFormatter();
             var steps = (dataAxis.maximum - dataAxis.minimum) / dataAxis.interval;
             var lastLine = null;
+            labelLen = 0;
 
             for (var i = 0; i <= steps; i ++) {
                 var value = dataAxis.minimum + i * dataAxis.interval;
@@ -220,7 +234,7 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
                     if(format && format.format){
                         line.label = formatter.formatValue(realValue, format.format);
                     }else{
-                        line.label = formatter.formatStandardNumber(realValue, 8).text;
+                        line.label = formatter.formatStandardNumber(realValue, 8);
                     }
                     if(lastLine && lastLine.label === line.label){
                         line.label = '';
@@ -228,6 +242,7 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
                         lastLine = line;
                     }
                 }
+                labelLen = Math.max(labelLen, line.label.length);
                 index++;
             }
             for (; index < stripLines.length; index++) {
@@ -313,7 +328,9 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
                 _.times(first.dataPoints.length, function (j) {
                     var value = 0;
                     _.each(data.data, function (dd) {
-                        var useValue = dd.dataPoints[j][type];
+                        var jValue = dd.dataPoints[j];
+                        if (!jValue) { return; }
+                        var useValue = jValue[type];
                         value += useValue;
                     });
                     minValue = Math.min(minValue, value);
@@ -332,9 +349,9 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
             var intervalCount = 8;
 
             if (chart.isXYType()) {
-                var diff = maxValue - minValue;
-                maxValue += 0.005 + diff / intervalCount;
-                minValue -= 0.005 + diff / intervalCount;
+//                var diff = maxValue - minValue;
+//                maxValue += 0.005 + diff / intervalCount;
+//                minValue -= 0.005 + diff / intervalCount;
             } else if (stacking === 'percentStacked') {
                 format = PERCENTFORMAT;
                 //Trial & error...
@@ -391,7 +408,7 @@ define('io.ox/office/spreadsheet/model/drawing/axismodel',
 
             if (minValue < maxValue) {
                 var interval = (maxValue - minValue) / intervalCount;
-				interval = round(interval, precision - 1);
+                interval = round(interval, precision - 1);
 
                 if (interval === 0) {
                     interval = null;
