@@ -1,33 +1,33 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Carsten Driesner <carsten.driesner@open-xchange.com>
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/drawing/chartmodel',
-        ['io.ox/office/tk/utils',
-         'io.ox/office/editframework/utils/color',
-         'io.ox/office/drawinglayer/model/drawingmodel',
-         'io.ox/office/drawinglayer/view/chartstyleutil',
-         'io.ox/office/drawinglayer/view/chartformatter',
-         'io.ox/office/spreadsheet/utils/sheetutils',
-         'io.ox/office/spreadsheet/model/operations',
-         'io.ox/office/spreadsheet/model/formula/tokenarray',
-         'io.ox/office/spreadsheet/model/drawing/drawingmodelmixin',
-         'io.ox/office/spreadsheet/model/drawing/manualcolorchart',
-         'io.ox/office/spreadsheet/model/drawing/axismodel',
-         'io.ox/office/spreadsheet/model/drawing/dataseriesmodel',
-         'io.ox/office/spreadsheet/model/drawing/legendmodel',
-         'io.ox/office/spreadsheet/model/drawing/axistitlemodel'
-        ], function (Utils, Color, DrawingModel, ChartStyleUtil, ChartFormatter, SheetUtils, Operations, TokenArray, DrawingModelMixin, ManualColor, AxisModel, DataSeriesModel, LegendModel, AxisTitleModel) {
+define('io.ox/office/spreadsheet/model/drawing/chartmodel', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/editframework/utils/color',
+     'io.ox/office/drawinglayer/model/drawingmodel',
+     'io.ox/office/drawinglayer/view/chartstyleutil',
+     'io.ox/office/drawinglayer/view/chartformatter',
+     'io.ox/office/spreadsheet/utils/operations',
+     'io.ox/office/spreadsheet/utils/sheetutils',
+     'io.ox/office/spreadsheet/model/formula/tokenarray',
+     'io.ox/office/spreadsheet/model/drawing/drawingmodelmixin',
+     'io.ox/office/spreadsheet/model/drawing/manualcolorchart',
+     'io.ox/office/spreadsheet/model/drawing/axismodel',
+     'io.ox/office/spreadsheet/model/drawing/dataseriesmodel',
+     'io.ox/office/spreadsheet/model/drawing/legendmodel',
+     'io.ox/office/spreadsheet/model/drawing/axistitlemodel'
+], function (Utils, Color, DrawingModel, ChartStyleUtil, ChartFormatter, Operations, SheetUtils, TokenArray, DrawingModelMixin, ManualColor, AxisModel, DataSeriesModel, LegendModel, AxisTitleModel) {
 
     'use strict';
 
@@ -139,8 +139,8 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
 
         // base constructors --------------------------------------------------
 
-        DrawingModel.call(this, app, 'chart', initAttributes, { additionalFamilies: ['chart', 'fill'] });
-        DrawingModelMixin.call(this, app, sheetModel);
+        DrawingModel.call(this, app, 'chart', initAttributes, { families: 'chart fill line' });
+        DrawingModelMixin.call(this, app, sheetModel, cloneConstructor);
 
 
         this.getApp = function() {
@@ -168,7 +168,49 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
 
         // private methods ----------------------------------------------------
 
-        function colRowChangeHandler(event, sheet, interval) {
+        /**
+         * Returns a clone of this chart model for the specified sheet. Used by
+         * the implementation of the public clone() method defined by the class
+         * DrawingModelMixin.
+         *
+         * @param {SheetModel} targetModel
+         *  The sheet model that will contain the cloned chart model.
+         *
+         * @returns {SheetChartModel}
+         *  A clone of this chart model, initialized for ownership by the
+         *  passed target sheet model.
+         */
+        function cloneConstructor(targetModel) {
+
+            var // a data object passed as hidden parameter to the constructor of the clone
+                cloneData = { series: [], axes: axes, legend: legendModel, title: mainTitleModel },
+                // a temporary formula token array used to relocate sheet references of chart source links
+                tokenArray = new TokenArray(app, sheetModel),
+                // the index of the own sheet
+                oldSheet = sheetModel.getIndex(),
+                // the index of the target sheet
+                newSheet = targetModel.getIndex();
+
+            _.each(data.data, function (dataSeries) {
+                var series = {};
+
+                dataSeries.model.iterateTokens(function (token, name) {
+                    tokenArray.parseFormula(token.getFormula());
+                    tokenArray.relocateSheet(oldSheet, newSheet);
+                    series[name] = tokenArray.getFormula();
+                });
+                var nsp = dataSeries.model.getExplicitAttributes();
+                nsp.series = series;
+                cloneData.series.push(nsp);
+            });
+            tokenArray.destroy();
+            tokenArray = null;
+
+            // pass private data as hidden argument to the constructor
+            return new SheetChartModel(app, targetModel, this.getExplicitAttributes(), cloneData);
+        }
+
+        function colRowChangeHandler(event, sheet, interval, attrs) {
             switch (event.type) {
             case 'insert:columns':
                 checkDataInsideInterval(sheet, interval, true, true);
@@ -176,12 +218,24 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             case 'delete:columns':
                 checkDataInsideInterval(sheet, interval, false, true);
                 break;
+            case 'change:columns':
+                // attributes may be missing, e.g. after changing column default attributes at the sheet
+                if (!attrs || (attrs.column && ('visible' in attrs.column))) {
+                    rangeUpdateNotificationHandler(sheet, docModel.makeColRange(interval));
+                }
+                break;
             case 'insert:rows':
                 checkDataInsideInterval(sheet, interval, true, false);
                 break;
             case 'delete:rows':
                 checkDataInsideInterval(sheet, interval, false, false);
                 break;
+            case 'change:rows':
+                // attributes may be missing, e.g. after changing row default attributes at the sheet
+                if (!attrs || (attrs.row && ('visible' in attrs.row))) {
+                    rangeUpdateNotificationHandler(sheet, docModel.makeRowRange(interval));
+                }
+                break;
             }
         }
 
@@ -201,6 +255,11 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             return res;
         }
 
+        function rangeUpdateNotificationHandler(sheet, range) {
+            range.sheet = sheet;
+            updateNotificationHandler(null, [range]);
+        }
+
         /**
          * Handles 'docs:update:cells' notifications. If this chart refers to
          * changed cells, the respective data series will be updated.
@@ -254,7 +313,7 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             }
 
             function refreshChart() {
-                if (changed && self && !self.destroyed) {
+                if (changed) {
                     changed = false;
                     self.refresh();
                 }
@@ -262,7 +321,7 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
 
             // wait quite a long time before refreshing the chart, cell ranges
             // may be queried in a background loop
-            return app.createDebouncedMethod(eventHandler, refreshChart, { delay: 500 });
+            return self.createDebouncedMethod(eventHandler, refreshChart, { delay: 500 });
         }());
 
         function changeAttributes(evt, mergedAttributes, oldMergedAttributes) {
@@ -270,8 +329,10 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             var oc = oldMergedAttributes.chart;
             var mf = mergedAttributes.fill;
             var of = oldMergedAttributes.fill;
+            var ml = mergedAttributes.line;
+            var ol = oldMergedAttributes.line;
 
-            if (!_.isEqual(mc, oc) || !_.isEqual(mf, of)) {
+            if (!_.isEqual(mc, oc) || !_.isEqual(mf, of) || !_.isEqual(ml, ol)) {
 
                 updateChartType();
 
@@ -295,8 +356,8 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
                 if(dataSeries.needUpdate){
                     return;
                 }
-                if (subtype === 'series' || !index) {
-                    colorHandler.handleColor(dataSeries.model.getMergedAttributes(), index);
+                if (subtype === 'series' || !dataSeries.seriesIndex) {
+                    colorHandler.handleColor(dataSeries.model.getMergedAttributes(), dataSeries, dataSeries.seriesIndex);
                 }
                 if (cjst === 'pie' || cjst === 'doughnut') {
                     dataSeries.startAngle = attributes.chart.rotation - 90;
@@ -316,6 +377,10 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
                     }
                 }
 
+                if (dataSeries.showInLegend && (!dataSeries.dataPoints || !dataSeries.dataPoints.length)) {
+                    dataSeries.showInLegend = false;
+                }
+
                 ChartStyleUtil.handleCharacterProps(self, indexLabelAttrs, dataSeries, 'indexLabel');
 
                 if (attributes.chart.dataLabel) {
@@ -427,7 +492,7 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
          * refresh checks if something is changed in the series,
          * if true it calls the 'updateview'-request to the server
          */
-        this.refresh = app.createDebouncedMethod($.noop, function () {
+        this.refresh = this.createDebouncedMethod($.noop, function () {
 
             if (!app.isImportFinished()) {
                 return;
@@ -438,9 +503,6 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
                 // maps series indexes and source link types to array elements in 'sourceRanges'
                 collectInfo = null;
 
-            // debounced method: chart may have been deleted in the meantime
-            if (!self || self.destroyed) { return; }
-
             _(data.data).each(function (seriesData, series) {
                 if (!seriesData.needUpdate) { return; }
 
@@ -472,7 +534,7 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             if (sourceRanges.length > 0) {
 
                 var // query cell contents for the data series
-                    request = docModel.queryCellContents(sourceRanges, { attributes: true });
+                    request = docModel.queryCellContents(sourceRanges, { maxCount: 1000, attributes: true });
 
                 // do not call the handler function, if the chart has been deleted in the meantime
                 self.listenTo(request, 'done', function (allContents) {
@@ -486,9 +548,6 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
 
                         // series values must exist
                         var valuesSource = getCellContents('values');
-                        if (valuesSource.length === 0) { return; }
-
-                        // all other source data may be missing
                         var titleSource = getCellContents('title');
                         var namesSource = getCellContents('names');
                         var bubblesSource = getCellContents('bubbles');
@@ -539,6 +598,7 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             var model = new DataSeriesModel(app, sheetModel, operation.attrs);
 
             dataSeries.model = model;
+            dataSeries.seriesIndex = operation.series;
 
             dataSeries.needUpdate = true;
 
@@ -659,7 +719,7 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             _(data.data).each(function (dataSeries, index) {
                 var at = dataSeries.model.getMergedAttributes();
                 if (at.fill.color.type !== 'auto' || at.line.color.type !== 'auto') {
-                    generator.generateDrawingOperation(Operations.SET_CHART_DATASERIES_ATTRIBUTES, position[0], position[1], { series: index, attrs: { fill: { color: Color.AUTO }, line: { color: Color.AUTO } } });
+                    generator.generateDrawingOperation(Operations.SET_CHART_DATASERIES_ATTRIBUTES, position, { series: index, attrs: { fill: { color: Color.AUTO }, line: { color: Color.AUTO } } });
                 }
             });
         }
@@ -674,6 +734,20 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             return drAttrs;
         }
 
+        function sourceExistsComplete(name) {
+            if (!data.data.length) {
+                return false;
+            }
+            var exists = true;
+            _.find(data.data, function (dataSeries) {
+                if (_.isEmpty(dataSeries.model.getMergedAttributes().series[name])) {
+                    exists = false;
+                    return true;
+                }
+            });
+            return exists;
+        }
+
         this.isXYType = isXYType;
 
         /**
@@ -724,14 +798,8 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
          * @returns {String}
          */
         this.getStyleSet = function () {
-            var styleId = getStyleId() - 1;
-            var styleSet = (styleId / 8) | 0;
-
-            if (styleSet < 4) {
-                return 'ss0';
-            } else {
-                return 'ss' + styleSet;
-            }
+            var styleSet = Math.floor((getStyleId() - 1) / 8);
+            return 'ss' + ((styleSet < 4) ? 0 : styleSet);
         };
 
         /**
@@ -740,20 +808,20 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
          * @param colorSet
          */
         this.changeColorSet = function (colorSet) {
+
             var styleId = getStyleId() - 1;
             var csId = colorSet.replace('cs', '') | 0;
             var styleSet = (styleId / 8) | 0;
 
             styleId = styleSet * 8 + csId + 1;
 
-
-            var position = self.getPosition();
-            var generator = docModel.createOperationsGenerator();
+            var position = self.getSheetPosition(),
+                generator = sheetModel.createOperationsGenerator();
 
             clearSeriesColor(generator, position);
 
-            var drAttrs = createBackgroundColorOp(styleId);
-            generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, position[0], position[1], { attrs: drAttrs });
+            var attrs = createBackgroundColorOp(styleId);
+            generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, position, { attrs: attrs });
             docModel.applyOperations(generator.getOperations());
 
         };
@@ -770,12 +838,11 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
 
             styleId = ssId * 8 + colorSet + 1;
 
-            var position = self.getPosition();
+            var position = self.getSheetPosition(),
+                generator = sheetModel.createOperationsGenerator(),
+                attrs = createBackgroundColorOp(styleId);
 
-            var generator = docModel.createOperationsGenerator();
-
-            var drAttrs = createBackgroundColorOp(styleId);
-            generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, position[0], position[1], { attrs: drAttrs });
+            generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, position, { attrs: attrs });
             docModel.applyOperations(generator.getOperations());
 
         };
@@ -786,13 +853,12 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
          * @param colorSet
          */
         this.changeVaryColors = function (state) {
-            var position = self.getPosition();
-            var generator = docModel.createOperationsGenerator();
+
+            var position = self.getSheetPosition(),
+                generator = sheetModel.createOperationsGenerator();
 
             clearSeriesColor(generator, position);
-
-            generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, position[0], position[1], { attrs: { chart: {varyColors: state} } });
-
+            generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, position, { attrs: { chart: {varyColors: state} } });
             docModel.applyOperations(generator.getOperations());
         };
 
@@ -815,6 +881,31 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             data.axisY._oldOptions = null;
         };
 
+
+        /**
+         * normally the assigned axisId (x or y) will be returned
+         * except chart is a bar-type, then x & y are interchanged
+         *
+         * @param axisId
+         * @returns
+         */
+        this.getAxisIdForDrawing = function(axisId) {
+            var chartType = self.getMergedAttributes().chart.type;
+            if (chartType.indexOf('bar') === 0) {
+                if (axisId === 'x') { return 'y'; }
+                if (axisId === 'y') { return 'x'; }
+            }
+            return axisId;
+        };
+
+        /**
+         * transfers model-data to CanvasJs-data
+         */
+        this.updateRenderInfo = function() {
+            changeDrawing();
+            self.refresh();
+        };
+
         /**
          * iterates all sources and calculates min and max positions,
          * direction of the series-values and the sheetindex
@@ -884,57 +975,44 @@ define('io.ox/office/spreadsheet/model/drawing/chartmodel',
             }
         };
 
+        /**
+         * checks if all dataseries have a sourcelink 'title'
+         * which is the first row in a normal chart
+         */
+        this.isFirstRowLabel = function() {
+            return sourceExistsComplete('title');
+        };
+
+        /**
+         * checks if all dataseries have a sourcelink 'names'
+         * which is the first column in a normal chart
+         */
+        this.isFirstColLabel = function() {
+            return sourceExistsComplete('names');
+        };
+
         // initialization -----------------------------------------------------
 
         this.on('change:attributes', changeAttributes);
         this.on('change:drawing', changeDrawing);
         this.listenTo(app, 'docs:update:cells', updateNotificationHandler);
-        this.listenTo(docModel, 'insert:columns delete:columns insert:rows delete:rows', colRowChangeHandler);
+        this.listenTo(docModel, 'insert:columns delete:columns change:columns insert:rows delete:rows change:rows', colRowChangeHandler);
         this.listenTo(docModel, 'change:cells', changeCellsHandler);
 
-		this.listenTo(app.getImportPromise(), 'done', function() {
-		    app.executeDelayed(function() {
-		        //small hack for fastcalc-loading
-		        var position = self.getPosition();
-		        var view = app.getView();
-
-		        if (position[0] === view.getActiveSheet()) {
-		            self.refresh();
-		        }
+        this.listenTo(app.getImportPromise(), 'done', function () {
+            self.executeDelayed(function() {
+                //small hack for fastcalc-loading
+                if (sheetModel === app.getView().getSheetModel()) {
+                    self.refresh();
+                }
             });
         });
 
-        // constructor called from BaseObject.clone()
-        this.registerCloneConstructor(function (newSheetModel, newSheet) {
-
-            var cloneData = { series: [], axes: axes, oldSheet: sheetModel.getIndex(), newSheet: newSheet, legend: legendModel, title: mainTitleModel },
-                tokenArray = new TokenArray(app, sheetModel);
-
-            _.each(data.data, function (dataSeries) {
-                var series = {};
-
-                dataSeries.model.iterateTokens(function (token, name) {
-                    tokenArray.parseFormula(token.getFormula());
-                    tokenArray.relocateSheet(cloneData.oldSheet, newSheet);
-                    series[name] = tokenArray.getFormula();
-                });
-                var nsp = dataSeries.model.getExplicitAttributes();
-                nsp.series = series;
-                cloneData.series.push(nsp);
-            });
-            tokenArray.destroy();
-            tokenArray = null;
-
-            // pass private data as hidden arguments to the constructor
-            return new SheetChartModel(app, newSheetModel, this.getExplicitAttributes(), cloneData);
-        });
-
         // clone private data passed as hidden argument to the c'tor
         (function (args) {
             var cloneData = args[SheetChartModel.length];
             if (_.isObject(cloneData)) {
-                app.executeDelayed(function () {
-                    if (!self || self.destroyed) { return; }
+                self.executeDelayed(function () {
                     _.each(cloneData.axes, function (axisModel, axisType) {
                         self.setAxisAttributes({ axis: axisType, attrs: axisModel.getExplicitAttributes() });
                         var axGrid = axisModel.getGrid();
