@@ -1,23 +1,26 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/drawinglayer/model/drawingcollection',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/model/drawing/drawingmodel',
-     'io.ox/office/spreadsheet/model/drawing/chartmodel'
-    ], function (Utils, DrawingCollection, SheetUtils, SheetDrawingModel, SheetChartModel) {
+define('io.ox/office/spreadsheet/model/drawing/drawingcollection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/drawinglayer/model/drawingmodel',
+    'io.ox/office/drawinglayer/model/shapemodel',
+    'io.ox/office/drawinglayer/model/imagemodel',
+    'io.ox/office/drawinglayer/model/drawingcollection',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/model/drawing/drawingmodelmixin',
+    'io.ox/office/spreadsheet/model/drawing/chartmodel'
+], function (Utils, DrawingModel, ShapeModel, ImageModel, DrawingCollection, SheetUtils, DrawingModelMixin, SheetChartModel) {
 
     'use strict';
 
@@ -42,10 +45,136 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
         },
 
         // maps 'anchorType' attribute values to the internal anchor mode used for anchor attributes
-        INTERNAL_ANCHOR_MODES = { twoCell: 'twoCell', oneCell: 'oneCell', absolute: 'absolute', twoCellAsOneCell: 'twoCell', twoCellAsAbsolute: 'twoCell' },
+        INTERNAL_ANCHOR_MODES = {
+            twoCell: 'twoCell',
+            oneCell: 'oneCell',
+            absolute: 'absolute',
+            twoCellAsOneCell: 'twoCell',
+            twoCellAsAbsolute: 'twoCell'
+        },
 
         // maps 'anchorType' attribute values to display anchor mode defining runtime behavior
-        DISPLAY_ANCHOR_MODES = { twoCell: 'twoCell', oneCell: 'oneCell', absolute: 'absolute', twoCellAsOneCell: 'oneCell', twoCellAsAbsolute: 'absolute' };
+        DISPLAY_ANCHOR_MODES = {
+            twoCell: 'twoCell',
+            oneCell: 'oneCell',
+            absolute: 'absolute',
+            twoCellAsOneCell: 'oneCell',
+            twoCellAsAbsolute: 'absolute'
+        };
+
+    // class SheetDrawingModel ================================================
+
+    /**
+     * The model of a generic drawing object contained in a sheet.
+     *
+     * @constructor
+     *
+     * @extends DrawingModel
+     * @extends DrawingModelMixin
+     *
+     * @param {EditApplication} app
+     *  The application instance containing this drawing object.
+     *
+     * @param {SheetModel} sheetModel
+     *  The sheet model instance containing this drawing object.
+     *
+     * @param {String} type
+     *  The type of this drawing object.
+     *
+     * @param {Object} [initAttributes]
+     *  An attribute set with initial formatting attributes for the object.
+     */
+    var SheetDrawingModel = DrawingModel.extend({ constructor: function (app, sheetModel, type, initAttributes) {
+
+        // base constructors --------------------------------------------------
+
+        DrawingModel.call(this, app, type, initAttributes);
+        DrawingModelMixin.call(this, app, sheetModel, function cloneConstructor(targetModel) {
+            return new SheetDrawingModel(app, targetModel, this.getType(), this.getExplicitAttributes(true));
+        });
+
+        // initialization -----------------------------------------------------
+
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            app = sheetModel = initAttributes = null;
+        });
+
+    }}); // class SheetImageModel
+
+    // class SheetShapeModel ==================================================
+
+    /**
+     * The model of a shape object contained in a sheet.
+     *
+     * @constructor
+     *
+     * @extends ShapeModel
+     * @extends DrawingModelMixin
+     *
+     * @param {EditApplication} app
+     *  The application instance containing this shape object.
+     *
+     * @param {SheetModel} sheetModel
+     *  The sheet model instance containing this shape object.
+     *
+     * @param {Object} [initAttributes]
+     *  An attribute set with initial formatting attributes for the object.
+     */
+    var SheetShapeModel = ImageModel.extend({ constructor: function (app, sheetModel, initAttributes) {
+
+        // base constructors --------------------------------------------------
+
+        ShapeModel.call(this, app, initAttributes);
+        DrawingModelMixin.call(this, app, sheetModel, function cloneConstructor(targetModel) {
+            return new SheetShapeModel(app, targetModel, this.getExplicitAttributes(true));
+        });
+
+        // initialization -----------------------------------------------------
+
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            app = sheetModel = initAttributes = null;
+        });
+
+    }}); // class SheetShapeModel
+
+    // class SheetImageModel ==================================================
+
+    /**
+     * The model of a image/picture object contained in a sheet.
+     *
+     * @constructor
+     *
+     * @extends ImageModel
+     * @extends DrawingModelMixin
+     *
+     * @param {EditApplication} app
+     *  The application instance containing this image object.
+     *
+     * @param {SheetModel} sheetModel
+     *  The sheet model instance containing this image object.
+     *
+     * @param {Object} [initAttributes]
+     *  An attribute set with initial formatting attributes for the object.
+     */
+    var SheetImageModel = ImageModel.extend({ constructor: function (app, sheetModel, initAttributes) {
+
+        // base constructors --------------------------------------------------
+
+        ImageModel.call(this, app, initAttributes);
+        DrawingModelMixin.call(this, app, sheetModel, function cloneConstructor(targetModel) {
+            return new SheetImageModel(app, targetModel, this.getExplicitAttributes(true));
+        });
+
+        // initialization -----------------------------------------------------
+
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            app = sheetModel = initAttributes = null;
+        });
+
+    }}); // class SheetImageModel
 
     // class SheetDrawingCollection ===========================================
 
@@ -69,13 +198,13 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
             self = this,
 
             // the document model
-            model = app.getModel(),
+            docModel = app.getModel(),
 
             // the column and row collections (size and formatting of all columns/rows)
             colCollection = sheetModel.getColCollection(),
             rowCollection = sheetModel.getRowCollection(),
 
-            // maps drawing model indexes (used as logical positions) to drawing models
+            // maps drawing model indexes (used as positions) to drawing models
             drawingModels = [];
 
         // base constructor ---------------------------------------------------
@@ -104,29 +233,34 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
          */
         function createDrawingModel(type, attributes) {
 
-            // own implementations for specific drawing types
-            if (type === 'chart') {
+            switch (type) {
+            case 'shape':
+                return new SheetShapeModel(app, sheetModel, attributes);
+
+            case 'image':
+                return new SheetImageModel(app, sheetModel, attributes);
+
+            case 'chart':
                 return new SheetChartModel(app, sheetModel, attributes);
+
+            default:
+                if (/^(group|diagram|ole|undefined)$/.test(type)) {
+                    return new SheetDrawingModel(app, sheetModel, type, attributes);
+                }
             }
 
-            // return generic drawing model for remaining supported types
-            if (/^(shape|group|image|diagram|ole|undefined)$/.test(type)) {
-                return new SheetDrawingModel(app, sheetModel, type, attributes);
-            }
-
-            // unsupported type
             return null;
         }
 
         /**
          * Tries to return a drawing model contained in this sheet at the
-         * specified logical position.
+         * specified position.
          *
          * @param {Number[]} position
-         *  An arbitrary logical document position.
+         *  An arbitrary document position.
          *
          * @returns {DrawingModel|Null}
-         *  The drawing model at the specified logical position, if available;
+         *  The drawing model at the specified position, if available;
          *  otherwise null.
          */
         function resolveDrawingModel(position) {
@@ -134,14 +268,14 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
         }
 
         /**
-         * Tries to return the logical position of the specified drawing model.
+         * Tries to return the position of the specified drawing model.
          *
          * @param {DrawingModel} drawingModel
          *  An arbitrary drawing model.
          *
          * @param {Number[]|Null}
-         *  The logical position of the passed drawing model, if the model is
-         *  part of this drawing collection, otherwise null.
+         *  The position of the passed drawing model, if the model is part of
+         *  this drawing collection, otherwise null.
          */
         function resolveDrawingPosition(drawingModel) {
             var index = _.indexOf(drawingModels, drawingModel);
@@ -156,11 +290,11 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
          *  The new drawing model.
          *
          * @param {Number[]} position
-         *  The logical position of the new drawing object.
+         *  The position of the new drawing object.
          *
          * @returns {Boolean}
-         *  Whether the passed logical position is valid, and the drawing model
-         *  has been registered successfully.
+         *  Whether the passed position is valid, and the drawing model has
+         *  been registered successfully.
          */
         function registerDrawingModel(drawingModel, position) {
             if ((position.length === 1) && (0 <= position[0]) && (position[0] <= drawingModels.length)) {
@@ -453,7 +587,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
             var // the attribute names according to the passed direction
                 ATTR_NAMES = columns ? HORIZONTAL_ATTRIBUTE_NAMES : VERTICAL_ATTRIBUTE_NAMES,
                 // the maximum column/row
-                maxIndex = model.getMaxIndex(columns),
+                maxIndex = docModel.getMaxIndex(columns),
                 // the number of columns/rows to be inserted
                 move = SheetUtils.getIntervalSize(interval);
 
@@ -562,18 +696,42 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
             });
         }
 
-        // methods ------------------------------------------------------------
+        // protected methods --------------------------------------------------
+
+        /**
+         * Creates and returns a cloned instance of this drawing collection for
+         * the specified sheet.
+         *
+         * @internal
+         *  Used by the class SheetModel during clone construction. DO NOT CALL
+         *  from external code!
+         *
+         * @param {SheetModel} targetModel
+         *  The model instance of the new cloned sheet that will own the clone
+         *  returned by this method.
+         *
+         * @returns {SheetDrawingCollection}
+         *  A clone of this drawing collection, initialized for ownership by
+         *  the passed sheet model.
+         */
+        this.clone = function (targetModel) {
+            // construct a new collection, pass own drawing models as hidden parameter
+            return new SheetDrawingCollection(app, targetModel, drawingModels);
+        };
+
+        // public methods -----------------------------------------------------
 
         /**
          * Invokes the passed iterator function for all drawing models in this
-         * sheet. The drawing models will be visited in order of their logical
-         * document positions (Z order).
+         * sheet. The drawing models will be visited in order of their document
+         * positions (Z order).
          *
          * @param {Function} iterator
          *  The iterator function called for all drawing models. Receives the
-         *  current drawing model as first parameter, and its logical position
-         *  as second parameter. If the iterator returns the Utils.BREAK
-         *  object, the iteration process will be stopped immediately.
+         *  current drawing model as first parameter, and its document position
+         *  (single-element array with the index) as second parameter. If the
+         *  iterator returns the Utils.BREAK object, the iteration process will
+         *  be stopped immediately.
          *
          * @param {Object} [options]
          *  Optional parameters:
@@ -691,7 +849,7 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
         this.addGenericAnchorAttributes = function (attributes) {
 
             var // the merged attribute set (with default values for all missing attributes)
-                mergedAttributes = model.getStyleCollection('drawing').getMergedAttributes(attributes),
+                mergedAttributes = docModel.getStyleCollection('drawing').getMergedAttributes(attributes),
                 // the inactive anchor attributes
                 anchorAttributes = getInactiveAnchorAttributes(mergedAttributes.drawing);
 
@@ -725,25 +883,17 @@ define('io.ox/office/spreadsheet/model/drawing/drawingcollection',
             'change:entries': function (event, interval) { changeInterval(interval, false); }
         });
 
-        // constructor called from BaseObject.clone()
-        this.registerCloneConstructor(function (newSheetModel, newSheet) {
-            // pass drawing models as hidden argument to the constructor
-            return new SheetDrawingCollection(app, newSheetModel, { entries: drawingModels, newSheet: newSheet });
-        });
-
-        // clone drawing models passed as hidden argument to the c'tor
-        (function (args) {
-            var cloneData = args[SheetDrawingCollection.length];
-            if (_.isObject(cloneData)) {
-                _.each(cloneData.entries, function (drawingModel, index) {
-                    self.insertModelInstance([index], drawingModel.clone(sheetModel, cloneData.newSheet));
-                });
-            }
-        }(arguments));
+        // clone drawing models passed as hidden argument by the clone() method
+        if (_.isArray(arguments[SheetDrawingCollection.length])) {
+            _.each(arguments[SheetDrawingCollection.length], function (drawingModel, index) {
+                self.insertModelInstance([index], drawingModel.clone(sheetModel));
+            });
+        }
 
         // destroy all class members
         this.registerDestructor(function () {
-            model = colCollection = rowCollection = drawingModels = null;
+            // drawing models are owned by the base class DrawingCollection
+            docModel = colCollection = rowCollection = drawingModels = null;
         });
 
     } // class SheetDrawingCollection
