@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -23,8 +23,6 @@ define('io.ox/office/spreadsheet/model/drawing/drawingmodelmixin', function () {
      *
      * @constructor
      *
-     * @extends DrawingModel
-     *
      * @internal
      *  This is a mix-in class supposed to extend an existing instance of the
      *  class DrawingModel or any of its sub classes. Expects the symbol 'this'
@@ -36,31 +34,79 @@ define('io.ox/office/spreadsheet/model/drawing/drawingmodelmixin', function () {
      * @param {SheetModel} sheetModel
      *  The sheet model containing the drawing collection with this drawing
      *  model object.
+     *
+     * @param {Function} cloneConstructor
+     *  A callback function invoked from the public method clone() which has to
+     *  create and return a new instance cloned from this drawing model.
      */
-    function DrawingModelMixin(app, sheetModel) {
+    function DrawingModelMixin(app, sheetModel, cloneConstructor) {
 
-        // methods ------------------------------------------------------------
+        // protected methods --------------------------------------------------
 
         /**
-         * Returns the complete logical document position of this drawing
-         * model.
+         * Creates and returns a cloned instance of this drawing object model
+         * for the specified sheet.
+         *
+         * @internal
+         *  Used by the class SheetDrawingCollection during clone construction.
+         *  DO NOT CALL from external code!
+         *
+         * @param {SheetModel} targetModel
+         *  The model instance of the new cloned sheet that will own the clone
+         *  returned by this method.
+         *
+         * @returns {DrawingModelMixin}
+         *  A clone of this drawing model, initialized for ownership by the
+         *  passed sheet model.
+         */
+        this.clone = function (targetModel) {
+            return cloneConstructor.call(this, targetModel);
+        };
+
+        // public methods -----------------------------------------------------
+
+        /**
+         * Returns the sheet model instance that contains this drawing model.
+         *
+         * @returns {SheetModel}
+         *  The sheet model instance that contains this drawing model.
+         */
+        this.getSheetModel = function () {
+            return sheetModel;
+        };
+
+        /**
+         * Returns the position of this drawing model in the drawing collection
+         * of the sheet.
          *
          * @returns {Number[]|Null}
-         *  The logical position of the drawing object, if it is still
+         *  The sheet position of the drawing object, if it is still contained
+         *  in the drawing collection; otherwise null.
+         */
+        this.getSheetPosition = function () {
+            return sheetModel.getDrawingCollection().getModelPosition(this, { deep: true });
+        };
+
+        /**
+         * Returns the complete document position of this drawing model, as
+         * used in document operations, including the leading sheet index.
+         *
+         * @returns {Number[]|Null}
+         *  The complete position of the drawing object, if it is still
          *  contained in the drawing collection, including the zero-based sheet
          *  index as first array element; otherwise null.
          */
-        this.getPosition = function () {
-            var position = sheetModel.getDrawingCollection().getModelPosition(this, { deep: true });
+        this.getDocumentPosition = function () {
+            var position = this.getSheetPosition();
             return position ? [sheetModel.getIndex()].concat(position) : null;
         };
 
         /**
-         * Returns the logical address of the range covered by this drawing
-         * model object.
+         * Returns the address of the cell range covered by this drawing model
+         * object.
          *
          * @returns {Object}
-         *  The logical address of the range covered by this drawing model.
+         *  The address of the range covered by this drawing model.
          */
         this.getRange = function () {
 
@@ -87,6 +133,13 @@ define('io.ox/office/spreadsheet/model/drawing/drawingmodelmixin', function () {
             return sheetModel.getDrawingCollection().getRectangleForAttributes(this.getMergedAttributes());
         };
 
+        // initialization -----------------------------------------------------
+
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            app = sheetModel = cloneConstructor = null;
+        });
+
     } // class DrawingModelMixin
 
     // exports ================================================================
