@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
@@ -132,7 +132,7 @@ define('io.ox/office/spreadsheet/model/drawing/manualcolorchart',
             var cells = app.getModel().getStyleCollection('cell');
 
             var cssColor = cells.getCssColor(fill.color, 'fill');
-            if (fill.type !== 'solid') {
+            if (fill.type === 'none') {
                 cjsPoint.color = 'transparent';
             } else {
                 cjsPoint.color = cssColor;
@@ -153,10 +153,9 @@ define('io.ox/office/spreadsheet/model/drawing/manualcolorchart',
             return false;
         };
 
-        this.handleColor = function (attrs, index) {
+        this.handleColor = function (attrs, dataSeries, index) {
             var vary = getVaryColor();
             var fill = accessColor(attrs);
-            var dataSeries = data.data[index];
             var attributes = chartmodel.getMergedAttributes();
 
             _(dataSeries.dataPoints).each(function (dataPoint) {
