@@ -1,22 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/formula/compiler',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/spreadsheet/model/formula/tokenutils',
-     'io.ox/office/spreadsheet/model/formula/matrix'
-    ], function (Utils, ModelObject, TokenUtils, Matrix) {
+define('io.ox/office/spreadsheet/model/formula/compiler', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/model/modelobject',
+    'io.ox/office/spreadsheet/model/formula/tokenutils',
+    'io.ox/office/spreadsheet/model/formula/matrix'
+], function (Utils, ModelObject, TokenUtils, Matrix) {
 
     'use strict';
 
@@ -130,8 +130,8 @@ define('io.ox/office/spreadsheet/model/formula/compiler',
             /**
              * Pushes an operator descriptor to the compiled array.
              */
-            function pushOperator(token, index, params) {
-                compiled.splice(index, 0, new OperatorToken(token, params));
+            function pushOperator(token, insert, params) {
+                compiled.splice(insert, 0, new OperatorToken(token, params));
             }
 
             /**
@@ -188,7 +188,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler',
              * @param {Token} opToken
              *  The operator token to be inserted into the token array.
              *
-             * @param {Number} index
+             * @param {Number} insert
              *  The target array index in the token array for the passed
              *  operator token.
              *
@@ -198,9 +198,9 @@ define('io.ox/office/spreadsheet/model/formula/compiler',
              * @returns {Token|Null}
              *  The token returned by the passed callback function.
              */
-            function compileTermAndPush(compilerFunc, opToken, index, params) {
+            function compileTermAndPush(compilerFunc, opToken, insert, params) {
                 var token = compilerFunc(getNextToken());
-                pushOperator(opToken, index, params);
+                pushOperator(opToken, insert, params);
                 return token;
             }
 
@@ -313,7 +313,7 @@ define('io.ox/office/spreadsheet/model/formula/compiler',
             }
 
             /**
-             * Compiles a intersection operator subexpression.
+             * Compiles an intersection operator subexpression.
              */
             function compileIntersectTerm(token) {
                 var index = compiled.length;
