@@ -1,23 +1,23 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/formula/tokenarray',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/model/formula/tokenutils',
-     'io.ox/office/spreadsheet/model/formula/tokenizer'
-    ], function (Utils, ModelObject, SheetUtils, TokenUtils, Tokenizer) {
+define('io.ox/office/spreadsheet/model/formula/tokenarray', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/model/modelobject',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/model/formula/tokenutils',
+    'io.ox/office/spreadsheet/model/formula/tokenizer'
+], function (Utils, ModelObject, SheetUtils, TokenUtils, Tokenizer) {
 
     'use strict';
 
@@ -108,7 +108,7 @@ define('io.ox/office/spreadsheet/model/formula/tokenarray',
             self = this,
 
             // the spreadsheet document model
-            model = app.getModel(),
+            docModel = app.getModel(),
 
             // the formula grammar for the tokenizer
             grammar = Utils.getStringOption(initOptions, 'grammar', 'op'),
@@ -256,8 +256,8 @@ define('io.ox/office/spreadsheet/model/formula/tokenarray',
         this.appendRange = function (range, options) {
 
             var // whether the range spans entire columns or rows
-                colRange = model.isColRange(range),
-                rowRange = model.isRowRange(range),
+                colRange = docModel.isColRange(range),
+                rowRange = docModel.isRowRange(range),
                 // the cell references
                 abs = Utils.getBooleanOption(options, 'abs', false),
                 cell1Ref = { address: range.start, absCol: abs || rowRange, absRow: abs || (colRange && !rowRange) },
@@ -1019,18 +1019,36 @@ define('io.ox/office/spreadsheet/model/formula/tokenarray',
          * Calculates the result of the formula represented by this token
          * array.
          *
+         * @param {String} contextType
+         *  The formula context type influencing the type of the formula
+         *  result. See method Interpreter.interpretTokens() for details.
+         *
+         * @param {Number} refSheet
+         *  The index of the reference sheet the formula is related to. Will be
+         *  used if this token array does not have its own reference sheet
+         *  model (see constructor parameter 'sheetModel').
+         *
          * @param {Number[]} refAddress
          *  The address of the reference cell the formula is related to.
          *
+         * @param {Object} [options]
+         *  Optional parameters:
+         *  @param {Boolean} [options.relocate=false]
+         *      If set to true, all relative references contained in this token
+         *      array will be interpreted with relocated position. The address
+         *      contained in the parameter 'refAddress' will be used as
+         *      relocation distance. Used when interpreting defined names that
+         *      always contain cell references relative to cell position A1.
+         *
          * @returns {Object}
          *  The result descriptor for the formula. See method
          *  Interpreter.interpretTokens() for details.
          */
-        this.interpretFormula = function (refAddress) {
+        this.interpretFormula = function (contextType, refSheet, refAddress, options) {
 
             // compile the infix tokens to prefix notation
             if (!compiled) {
-                compiled = model.getFormulaCompiler().compileTokens(tokens, tokenizer);
+                compiled = docModel.getFormulaCompiler().compileTokens(tokens, tokenizer);
             }
 
             // check compiler result
@@ -1038,8 +1056,11 @@ define('io.ox/office/spreadsheet/model/formula/tokenarray',
                 return { type: 'error', value: compiled.error };
             }
 
+            // prefer own sheet model as reference sheet over passed sheet index
+            if (sheetModel) { refSheet = sheetModel.getIndex(); }
+
             // interpret the formula
-            return model.getFormulaInterpreter().interpretTokens(compiled.tokens, tokenizer, sheetModel, refAddress);
+            return docModel.getFormulaInterpreter().interpretTokens(contextType, compiled.tokens, tokenizer, refSheet, refAddress, options);
         };
 
         // initialization -----------------------------------------------------
@@ -1055,14 +1076,14 @@ define('io.ox/office/spreadsheet/model/formula/tokenarray',
         }
 
         // refresh all sheet references after the collection of sheets has been changed
-        this.listenTo(model, 'insert:sheet:after', function (event, sheet) { invokeForAllTokensAndTrigger('transformSheet', sheet, null); });
-        this.listenTo(model, 'delete:sheet:after', function (event, sheet) { invokeForAllTokensAndTrigger('transformSheet', null, sheet); });
-        this.listenTo(model, 'move:sheet:after', function (event, to, from) { invokeForAllTokensAndTrigger('transformSheet', to, from); });
-        this.listenTo(model, 'rename:sheet', function (event, sheet) { invokeForAllTokensAndTrigger('renameSheet', sheet); });
+        this.listenTo(docModel, 'insert:sheet:after', function (event, sheet) { invokeForAllTokensAndTrigger('transformSheet', sheet, null); });
+        this.listenTo(docModel, 'delete:sheet:after', function (event, sheet) { invokeForAllTokensAndTrigger('transformSheet', null, sheet); });
+        this.listenTo(docModel, 'move:sheet:after', function (event, to, from) { invokeForAllTokensAndTrigger('transformSheet', to, from); });
+        this.listenTo(docModel, 'rename:sheet', function (event, sheet) { invokeForAllTokensAndTrigger('renameSheet', sheet); });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            self = model = sheetModel = tokens = compiled = null;
+            self = docModel = sheetModel = tokens = compiled = null;
         });
 
     } // class TokenArray
