@@ -1,22 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/formula/tokenizer',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/model/formula/tokenutils'
-    ], function (Utils, ModelObject, SheetUtils, TokenUtils) {
+define('io.ox/office/spreadsheet/model/formula/tokenizer', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/model/modelobject',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/model/formula/tokenutils'
+], function (Utils, ModelObject, SheetUtils, TokenUtils) {
 
     'use strict';
 
@@ -66,6 +66,8 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
                 getErrorCodeText: _.property('code'),
                 // converts text representation to error code object
                 getErrorCodeValue: SheetUtils.makeErrorCode,
+                // returns whether a string names a supported function
+                isSheetFunction: _.bind(app.isNativeFunctionName, app),
                 // converts native function name to text representation
                 getFunctionText: toUpperCase,
                 // converts text representation to native function name
@@ -93,6 +95,8 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
                 getErrorCodeText: _.bind(app.convertErrorCodeToString, app),
                 // converts text representation to error code object
                 getErrorCodeValue: _.bind(app.convertStringToErrorCode, app),
+                // returns whether a string names a supported function
+                isSheetFunction: _.bind(app.isLocalizedFunctionName, app),
                 // converts native function name to text representation
                 getFunctionText: _.bind(app.getLocalizedFunctionName, app),
                 // converts text representation to native function name
@@ -129,19 +133,22 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
             config = createBaseConfig(app, grammar),
 
             // RE pattern snippet for the leading character of a simple sheet name (without apostrophes).
-            SHEET_LEADING_CHAR_TOKEN = '[a-z_\\xa1-\\u2027\\u202a-\\uffff]',
+            SHEET_LEADING_CHAR_TOKEN = '[a-z_\xa1-\u2027\u202a-\uffff]',
 
             // RE pattern snippet for other characters of a simple sheet name (without apostrophes).
-            SHEET_SIMPLE_CHAR_TOKEN = '[\\w.\\xa1-\\u2027\\u202a-\\uffff]',
+            SHEET_SIMPLE_CHAR_TOKEN = '[\\w.\xa1-\u2027\u202a-\uffff]',
 
             // RE pattern snippet for any character of a complex sheet name (enclosed in apostrophes).
-            SHEET_COMPLEX_CHAR_TOKEN = '[^\\x00-\\x1f\\x80-\\x9f\\[\\]\'*?:/\\\\]',
+            SHEET_COMPLEX_CHAR_TOKEN = '[^\x00-\x1f\x80-\x9f\\[\\]\'*?:/\\\\]',
+
+            // RE pattern snippet for a characters of a simple external file name (without apostrophes).
+            FILE_SIMPLE_CHAR_TOKEN = '[\\w.\\\\\xa1-\u2027\u202a-\uffff]',
 
             // RE pattern snippet for the leading character of a name or identifier.
-            NAME_LEADING_CHAR_TOKEN = '[a-z_\\xa1-\\u2027\\u202a-\\uffff]',
+            NAME_LEADING_CHAR_TOKEN = '[a-z_\xa1-\u2027\u202a-\uffff]',
 
             // RE pattern snippet for other characters of a name or identifier.
-            NAME_INNER_CHAR_TOKEN = '[\\w.\\xa1-\\u2027\\u202a-\\uffff]',
+            NAME_INNER_CHAR_TOKEN = '[\\w.\xa1-\u2027\u202a-\uffff]',
 
             // RE pattern for a simple sheet name.
             SHEET_NAME_SIMPLE_PATTERN = SHEET_LEADING_CHAR_TOKEN + SHEET_SIMPLE_CHAR_TOKEN + '*',
@@ -149,9 +156,9 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
             // RE pattern for a complex sheet name (with embedded double-apostrophes).
             SHEET_NAME_COMPLEX_PATTERN = SHEET_COMPLEX_CHAR_TOKEN + '+(?:(?:\'\')+' + SHEET_COMPLEX_CHAR_TOKEN + '+)*',
 
-            // RE pattern for a number literal: With or without decimal separator, digits preceding
-            // and/or following the separator, optional scientific prefix. No leading sign character.
-            NUMBER_PATTERN = '(\\d*' + _.escapeRegExp(config.DEC) + '\\d+|\\d+' + _.escapeRegExp(config.DEC) + '?)([eE][-+]?\\d+)?',
+            // RE pattern for a simple external file name (enclosed in brackets, without apostrophes).
+            // - Group 1: The name of the external file.
+            FILE_NAME_SIMPLE_PATTERN = '(?:\\[(' + FILE_SIMPLE_CHAR_TOKEN + '+)\\])',
 
             // RE pattern for a #REF! error code
             REF_ERROR_PATTERN = _.escapeRegExp(config.REF_ERROR),
@@ -234,12 +241,6 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
         // create the map of all regular expressions
         config.RE = {
 
-            // Number literals: With or without decimal separator, digits preceding
-            // and/or following the separator, optional scientific prefix. No
-            // leading sign character, will be parsed as operator, otherwise a formula
-            // such as =1-1 becomes an invalid sequence of two numbers.
-            NUMBER_LIT: new RegExp('^' + NUMBER_PATTERN),
-
             // String literals: Enclosed in double quotes, embedded double quotes
             // are represented by a sequence of two double quote characters.
             STRING_LIT: /^("[^"]*")+/,
@@ -255,9 +256,6 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
             // Opening parenthesis of a matrix literal.
             MATRIX_OPEN: /^{/,
 
-            // Numbers in matrix literals: always with leading sign character (plus and minus).
-            MATRIX_NUMBER_LIT: new RegExp('^[-+]?' + NUMBER_PATTERN),
-
             // The column separator in a matrix literal.
             MATRIX_COL_SEPARATOR: new RegExp('^' + _.escapeRegExp(config.COL)),
 
@@ -423,8 +421,13 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
             // A single unsupported character.
             INVALID_CHAR: new RegExp('^' + INVALID_CHAR_PATTERN),
 
-            // Matches a simple sheet name that does not need to be enclosed in apostrophes.
-            SIMPLE_SHEET_NAME: new RegExp('^' + SHEET_NAME_SIMPLE_PATTERN + '$', 'i')
+            // Matches a simple sheet name that does not need to be enclosed in apostrophes (full string match).
+            UNENCODED_SHEET_NAME_COMPLETE: new RegExp('^' + SHEET_NAME_SIMPLE_PATTERN + '$', 'i'),
+
+            // Matches a simple sheet name or a sheet range that does not need to be enclosed in apostrophes
+            // (full string match). Supports an optional leading reference to an external file (simple name),
+            // enclosed in brackets. Examples: 'Sheet1', 'Sheet1:Sheet2', '[file1.xlsx]Sheet1', '[path\to\file1.xlsx]Sheet1:Sheet2'
+            UNENCODED_EXT_SHEET_COMPLETE: new RegExp('^' + FILE_NAME_SIMPLE_PATTERN + '?' + SHEET_NAME_SIMPLE_PATTERN + '(:' + SHEET_NAME_SIMPLE_PATTERN + ')?$', 'i')
         };
 
         TokenUtils.info('Tokenizer.createConfig(): grammar=' + grammar + ' config=', config);
@@ -460,7 +463,10 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
             self = this,
 
             // the document model
-            model = app.getModel(),
+            docModel = app.getModel(),
+
+            // the number formatter of the document
+            numberFormatter = docModel.getNumberFormatter(),
 
             // the formula grammar configuration
             config = app.getOrCreateSingleton('tokenizer:config' + grammar, function () {
@@ -489,7 +495,7 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
             if (_.isNumber(value)) {
                 return !_.isFinite(value) ? config.getErrorCodeText(ErrorCodes.NUM) :
                     (Math.abs(value) < TokenUtils.MIN_VALUE) ? '0' :
-                    app.convertNumberToString(value, SheetUtils.MAX_LENGTH_STANDARD_EDIT, config.SEP);
+                    numberFormatter.formatStandardNumber(value, SheetUtils.MAX_LENGTH_STANDARD_EDIT).replace(app.getDecimalSeparator(), config.SEP);
             }
 
             // strings
@@ -549,11 +555,11 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
                 sheetIndex = null;
 
             if (_.isString(simpleSheet)) {
-                sheetIndex = model.getSheetIndex(simpleSheet);
+                sheetIndex = docModel.getSheetIndex(simpleSheet);
                 sheetRef = { sheet: (sheetIndex >= 0) ? sheetIndex : simpleSheet, abs: true };
             } else if (_.isString(complexSheet)) {
                 complexSheet = complexSheet.replace(/''/g, '\'');
-                sheetIndex = model.getSheetIndex(complexSheet);
+                sheetIndex = docModel.getSheetIndex(complexSheet);
                 sheetRef = { sheet: (sheetIndex >= 0) ? sheetIndex : complexSheet, abs: true };
             } else if (_.isString(refError)) {
                 sheetRef = { sheet: -1, abs: true };
@@ -601,11 +607,13 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
          *  A sheet reference structure. Can be null, in that case the method
          *  does nothing and returns false.
          *
-         * @param {Number} to
-         *  The new index of the inserted or moved sheet.
+         * @param {Number|Null} to
+         *  The new index of the inserted or moved sheet; or null for deleted
+         *  sheets.
          *
-         * @param {Number} from
-         *  The old index of the deleted or moved sheet.
+         * @param {Number|Null} from
+         *  The old index of the deleted or moved sheet; or null for inserted
+         *  sheets.
          *
          * @returns {Boolean}
          *  Whether the sheet reference structure has been modified.
@@ -668,7 +676,7 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
                 return false;
             }
 
-            // TODO: always for Excel formulas, but only relative sheet references for ODF
+            // TODO: also for absolute sheet references in ODF?
             if (sheetRef.sheet === oldSheet) {
                 sheetRef.sheet = newSheet;
                 return true;
@@ -689,15 +697,15 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
 
             // convert first sheet name
             if (!sheet1Ref) { return ''; }
-            sheet1Name = _.isNumber(sheet1Ref.sheet) ? model.getSheetName(sheet1Ref.sheet) : sheet1Ref.sheet;
+            sheet1Name = _.isNumber(sheet1Ref.sheet) ? docModel.getSheetName(sheet1Ref.sheet) : sheet1Ref.sheet;
             if (!_.isString(sheet1Name)) { return config.REF_ERROR + '!'; }
-            simpleNames = config.RE.SIMPLE_SHEET_NAME.test(sheet1Name);
+            simpleNames = config.RE.UNENCODED_SHEET_NAME_COMPLETE.test(sheet1Name);
 
             // convert second sheet name
             if (sheet2Ref && !_.isEqual(sheet1Ref, sheet2Ref)) {
-                sheet2Name = _.isNumber(sheet2Ref.sheet) ? model.getSheetName(sheet2Ref.sheet) : sheet2Ref.sheet;
+                sheet2Name = _.isNumber(sheet2Ref.sheet) ? docModel.getSheetName(sheet2Ref.sheet) : sheet2Ref.sheet;
                 if (!_.isString(sheet2Name)) { return config.REF_ERROR + '!'; }
-                simpleNames = simpleNames && config.RE.SIMPLE_SHEET_NAME.test(sheet2Name);
+                simpleNames = simpleNames && config.RE.UNENCODED_SHEET_NAME_COMPLETE.test(sheet2Name);
                 sheet1Name += ':' + sheet2Name;
             }
 
@@ -739,7 +747,7 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
 
             var address = [_.isNumber(col) ? col : SheetUtils.parseColName(col), _.isNumber(row) ? row : SheetUtils.parseRowName(row)];
 
-            return model.isValidAddress(address) ? {
+            return docModel.isValidAddress(address) ? {
                 address: address,
                 absCol: _.isBoolean(absCol) ? absCol : (absCol === '$'),
                 absRow: _.isBoolean(absRow) ? absRow : (absRow === '$')
@@ -793,17 +801,17 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
                 return generateColName(cellRef) + generateRowName(cellRef);
             }
 
-            if (!model.isValidRange(range)) {
+            if (!docModel.isValidRange(range)) {
                 text += config.REF_ERROR;
             }
 
             // generate row interval (preferred over column interval for entire sheet range)
-            else if (cell1Ref.absCol && cell2Ref && cell2Ref.absCol && model.isRowRange(range)) {
+            else if (cell1Ref.absCol && cell2Ref && cell2Ref.absCol && docModel.isRowRange(range)) {
                 text += generateRowName(cell1Ref) + ':' + generateRowName(cell2Ref);
             }
 
             // generate column interval
-            else if (cell1Ref.absRow && cell2Ref && cell2Ref.absRow && model.isColRange(range)) {
+            else if (cell1Ref.absRow && cell2Ref && cell2Ref.absRow && docModel.isColRange(range)) {
                 text += generateColName(cell1Ref) + ':' + generateColName(cell2Ref);
             }
 
@@ -1069,11 +1077,11 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
 
                 if (result.cell1Ref) {
                     relocateCellRef(result.cell1Ref, refAddress, targetAddress, wrapReferences);
-                    if (!model.isValidAddress(result.cell1Ref.address)) { result.cell1Ref = null; }
+                    if (!docModel.isValidAddress(result.cell1Ref.address)) { result.cell1Ref = null; }
                 }
                 if (result.cell1Ref && result.cell2Ref) {
                     relocateCellRef(result.cell2Ref, refAddress, targetAddress, wrapReferences);
-                    if (!model.isValidAddress(result.cell2Ref.address)) { result.cell1Ref = result.cell2Ref = null; }
+                    if (!docModel.isValidAddress(result.cell2Ref.address)) { result.cell1Ref = result.cell2Ref = null; }
                 }
 
                 return result;
@@ -1193,7 +1201,7 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
                 var // the original range
                     oldRange = this.getRange(),
                     // the transformed range address
-                    newRange = model.transformRange(oldRange, interval, insert, columns);
+                    newRange = docModel.transformRange(oldRange, interval, insert, columns);
 
                 // nothing to do, if the range does not change
                 if (_.isEqual(oldRange, newRange)) { return false; }
@@ -1224,7 +1232,7 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
                 if (cell2Ref) { relocateCellRef(cell2Ref, refAddress, targetAddress, wrapReferences); }
 
                 // check validity of the new references
-                if (!model.isValidAddress(cell1Ref.address) || (cell2Ref && !model.isValidAddress(cell2Ref.address))) {
+                if (!docModel.isValidAddress(cell1Ref.address) || (cell2Ref && !docModel.isValidAddress(cell2Ref.address))) {
                     cell1Ref = cell2Ref = null;
                 }
 
@@ -1243,7 +1251,11 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
             this.transformSheet = function (to, from) {
                 var changed1 = transformSheetRef(sheet1Ref, to, from),
                     changed2 = transformSheetRef(sheet2Ref, to, from);
-                return changed1 || changed2;
+                if (changed1 || changed2) {
+                    this.updateText();
+                    return true;
+                }
+                return false;
             };
 
             /**
@@ -1333,6 +1345,16 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
                 return value;
             };
 
+            /**
+             * Returns whether this token represents a built-in sheet function.
+             *
+             * @returns {Boolean}
+             *  Whether this token represents a built-in sheet function.
+             */
+            this.isSheetFunction = function () {
+                return isFunc && !sheetRef && config.isSheetFunction(this.getText());
+            };
+
             /**
              * Returns whether the token contains a sheet reference structure,
              * regardless if it is valid.
@@ -1392,12 +1414,12 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
 
                 // sheet reference exists (e.g.: Sheet2!name): get name from specified sheet
                 if (sheetRef) {
-                    var refSheetModel = model.getSheetModel(this.getSheet());
+                    var refSheetModel = docModel.getSheetModel(this.getSheet());
                     return refSheetModel ? refSheetModel.getNameCollection().getTokenArray(value) : null;
                 }
 
                 // no sheet reference: try sheet names (if token array in sheet context), then global names
-                return (sheetModel && sheetModel.getNameCollection().getTokenArray(value)) || model.getNameCollection().getTokenArray(value);
+                return (sheetModel && sheetModel.getNameCollection().getTokenArray(value)) || docModel.getNameCollection().getTokenArray(value);
             };
 
             /**
@@ -1408,7 +1430,11 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
              *  Whether the token has been changed.
              */
             this.transformSheet = function (to, from) {
-                return transformSheetRef(sheetRef, to, from);
+                if (transformSheetRef(sheetRef, to, from)) {
+                    this.updateText();
+                    return true;
+                }
+                return false;
             };
 
             /**
@@ -1476,10 +1502,10 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
          * @returns {String}
          *  The encoded sheet name ready to be used in a formula expression.
          */
-        this.encodeSheetName = function (sheetName) {
+        this.encodeExtendedSheetRef = function (sheetName) {
 
             var // whether encoding is not needed
-                simpleName = config.RE.SIMPLE_SHEET_NAME.test(sheetName);
+                simpleName = config.RE.UNENCODED_EXT_SHEET_COMPLETE.test(sheetName);
 
             // enclose complex names in apostrophes
             return simpleName ? sheetName : ('\'' + sheetName.replace(/'/g, '\'\'') + '\'');
@@ -1977,10 +2003,12 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
 
                     // literals are valid in regular formula structure, and in matrix literals
 
-                    // number literal (with leading sign character in matrix literals)
-                    if ((matches = (matrix ? config.RE.MATRIX_NUMBER_LIT : config.RE.NUMBER_LIT).exec(formula))) {
-                        value = app.convertStringToNumber(matches[0], config.DEC);
-                        tokens.push(this.createLiteralToken(value, matches[0]));
+                    // number literal (with leading sign character in matrix literals), using
+                    // the parse method of the number formatter (returns a different result object)
+                    if ((matches = numberFormatter.parseNumberFromString(formula, { sep: config.DEC, sign: matrix }))) {
+                        tokens.push(this.createLiteralToken(matches.number, matches.text));
+                        // simulate the result array of the RegExp.exec() method for the 'finally' block
+                        matches = [matches.text];
                         continue;
                     }
 
@@ -2091,7 +2119,7 @@ define('io.ox/office/spreadsheet/model/formula/tokenizer',
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = sheetModel = self = model = config = null;
+            app = sheetModel = self = docModel = numberFormatter = config = null;
         });
 
     } // class Tokenizer
