@@ -1,22 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/mergecollection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/spreadsheet/utils/sheetutils'
-    ], function (Utils, ModelObject, SheetUtils) {
+define('io.ox/office/spreadsheet/model/mergecollection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/model/modelobject',
+    'io.ox/office/spreadsheet/utils/sheetutils'
+], function (Utils, ModelObject, SheetUtils) {
 
     'use strict';
 
@@ -70,7 +70,7 @@ define('io.ox/office/spreadsheet/model/mergecollection',
             self = this,
 
             // the spreadsheet document model
-            model = app.getModel(),
+            docModel = app.getModel(),
 
             // all merged ranges
             mergedRanges = [];
@@ -158,7 +158,30 @@ define('io.ox/office/spreadsheet/model/mergecollection',
             triggerEvent('insert:merged', insertedRanges, { implicit: true });
         }
 
-        // methods ------------------------------------------------------------
+        // protected methods --------------------------------------------------
+
+        /**
+         * Creates and returns a cloned instance of this collection for the
+         * specified sheet.
+         *
+         * @internal
+         *  Used by the class SheetModel during clone construction. DO NOT CALL
+         *  from external code!
+         *
+         * @param {SheetModel} targetModel
+         *  The model instance of the new cloned sheet that will own the clone
+         *  returned by this method.
+         *
+         * @returns {MergeCollection}
+         *  A clone of this collection, initialized for ownership by the passed
+         *  sheet model.
+         */
+        this.clone = function (targetModel) {
+            // construct a new collection, pass all own ranges as hidden parameter
+            return new MergeCollection(app, targetModel, mergedRanges);
+        };
+
+        // public methods -----------------------------------------------------
 
         /**
          * Returns, whether the cell at the specified address is the upper left
@@ -417,7 +440,7 @@ define('io.ox/office/spreadsheet/model/mergecollection',
             }
 
             // return the fully expanded range
-            return range;
+            return _.copy(range, true);
         };
 
         /**
@@ -599,7 +622,7 @@ define('io.ox/office/spreadsheet/model/mergecollection',
          */
         this.transformRange = function (mergedRange, operationInterval, insert, columns) {
             // transform the passed range without expanding the end of the range while inserting
-            mergedRange = model.transformRange(mergedRange, operationInterval, insert, columns);
+            mergedRange = docModel.transformRange(mergedRange, operationInterval, insert, columns);
             // a transformed merged range covering a single cell is considered deleted
             return (mergedRange && (SheetUtils.getCellCount(mergedRange) > 1)) ? mergedRange : null;
         };
@@ -609,17 +632,16 @@ define('io.ox/office/spreadsheet/model/mergecollection',
         // update merged ranges after inserting/deleting columns or rows in the sheet
         sheetModel.registerTransformationHandler(transformationHandler);
 
-        // constructor called from BaseObject.clone()
-        this.registerCloneConstructor(function (newSheetModel) {
-            // pass own entries as hidden argument to the constructor
-            return new MergeCollection(app, newSheetModel, mergedRanges);
-        });
-
-        // clone merged ranges passed as hidden argument to the c'tor
+        // clone merged ranges passed as hidden argument by the clone() method
         if (_.isArray(arguments[MergeCollection.length])) {
             mergedRanges = _.copy(arguments[MergeCollection.length], true);
         }
 
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            app = sheetModel = self = docModel = null;
+        });
+
     } // class MergeCollection
 
     // exports ================================================================
