@@ -1,21 +1,21 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/namecollection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/spreadsheet/model/formula/tokenarray'
-    ], function (Utils, ModelObject, TokenArray) {
+define('io.ox/office/spreadsheet/model/namecollection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/model/modelobject',
+    'io.ox/office/spreadsheet/model/formula/tokenarray'
+], function (Utils, ModelObject, TokenArray) {
 
     'use strict';
 
@@ -27,7 +27,7 @@ define('io.ox/office/spreadsheet/model/namecollection',
      *
      * @constructor
      *
-     * @extends AttributedModel
+     * @extends ModelObject
      *
      * @param {SpreadsheetApplication} app
      *  The application that contains this defined name.
@@ -48,7 +48,32 @@ define('io.ox/office/spreadsheet/model/namecollection',
 
         ModelObject.call(this, app);
 
-        // methods ------------------------------------------------------------
+        // protected methods --------------------------------------------------
+
+        /**
+         * Creates and returns a cloned instance of this name for the specified
+         * sheet.
+         *
+         * @internal
+         *  Used by the class NameCollection during clone construction. DO NOT
+         *  CALL from external code!
+         *
+         * @param {SheetModel} targetModel
+         *  The model instance of the new cloned sheet that will own the clone
+         *  returned by this method.
+         *
+         * @returns {NameModel}
+         *  A clone of this name model, initialized for ownership by the passed
+         *  sheet model.
+         */
+        this.clone = function (targetModel) {
+            // construct a new name model and relocate sheet references
+            var nameModel = new NameModel(app, targetModel, tokenArray.getFormula());
+            nameModel.getTokenArray().relocateSheet(sheetModel.getIndex(), targetModel.getIndex());
+            return nameModel;
+        };
+
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the token array representing the formula definition of this
@@ -80,20 +105,6 @@ define('io.ox/office/spreadsheet/model/namecollection',
         // parse formula definition
         this.setDefinition(formula);
 
-        // constructor called from BaseObject.clone()
-        this.registerCloneConstructor(function (newSheetModel, newSheet) {
-            var cloneData = sheetModel ? { oldSheet: sheetModel.getIndex(), newSheet: newSheet } : null;
-            return new NameModel(app, newSheetModel, tokenArray.getFormula(), cloneData);
-        });
-
-        // adjust formula according to sheet indexes passed as hidden argument to the c'tor
-        (function (args) {
-            var cloneData = args[NameModel.length];
-            if (_.isObject(cloneData)) {
-                tokenArray.relocateSheet(cloneData.oldSheet, cloneData.newSheet);
-            }
-        }(arguments));
-
         // destroy all class members on destruction
         this.registerDestructor(function () {
             tokenArray.destroy();
@@ -139,7 +150,30 @@ define('io.ox/office/spreadsheet/model/namecollection',
 
         ModelObject.call(this, app);
 
-        // methods ------------------------------------------------------------
+        // protected methods --------------------------------------------------
+
+        /**
+         * Creates and returns a cloned instance of this name collection for
+         * the specified sheet.
+         *
+         * @internal
+         *  Used by the class SheetModel during clone construction. DO NOT CALL
+         *  from external code!
+         *
+         * @param {SheetModel} targetModel
+         *  The model instance of the new cloned sheet that will own the clone
+         *  returned by this method.
+         *
+         * @returns {NameCollection}
+         *  A clone of this name collection, initialized for ownership by the
+         *  passed sheet model.
+         */
+        this.clone = function (targetModel) {
+            // construct a new collection, pass all own names as hidden parameter
+            return new NameCollection(app, targetModel, entries);
+        };
+
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the token array representing the formula definition of the
@@ -235,19 +269,12 @@ define('io.ox/office/spreadsheet/model/namecollection',
 
         // initialization -----------------------------------------------------
 
-        // constructor called from BaseObject.clone()
-        this.registerCloneConstructor(function (newSheetModel, newSheet) {
-            // pass own entries as hidden argument to the constructor
-            return new NameCollection(app, newSheetModel, { entries: entries, newSheet: newSheet });
-        });
-
-        // clone collection entries passed as hidden argument to the c'tor
-        (function (args) {
-            var cloneData = args[NameCollection.length];
-            if (_.isObject(cloneData)) {
-                entries = ModelObject.cloneMap(cloneData.entries, sheetModel, cloneData.newSheet);
-            }
-        }(arguments));
+        // clone collection entries passed as hidden argument by the clone() method
+        if (_.isObject(arguments[NameCollection.length])) {
+            _.each(arguments[NameCollection.length], function (sourceEntry, key) {
+                entries[key] = sourceEntry.clone(sheetModel);
+            });
+        }
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
