@@ -1,23 +1,23 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/numberformatter',
-    ['io.ox/core/date',
-     'io.ox/office/tk/utils',
-     'io.ox/office/tk/io',
-     'io.ox/office/tk/object/baseobject',
-     'io.ox/office/spreadsheet/utils/sheetutils'
-    ], function (CoreDate, Utils, IO, BaseObject, SheetUtils) {
+define('io.ox/office/spreadsheet/model/numberformatter', [
+    'io.ox/core/date',
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/io',
+    'io.ox/office/tk/object/baseobject',
+    'io.ox/office/spreadsheet/utils/sheetutils'
+], function (CoreDate, Utils, IO, BaseObject, SheetUtils) {
 
     'use strict';
 
@@ -64,13 +64,13 @@ define('io.ox/office/spreadsheet/model/numberformatter',
         this.isAmPm = false;
 
         /**
-         * Scientific numbers set expo on true,
+         * Scientific numbers set exponent on true,
          * so mantissa and exponent will be calculated before formating
          */
         this.expo = false;
 
         /**
-         * if the format is a fraction or a signifant number,
+         * if the format is a fraction or a significant number,
          * its integer part must not be rounded
          */
         this.notInteger = false;
@@ -284,12 +284,18 @@ define('io.ox/office/spreadsheet/model/numberformatter',
                     continue;
                 }
 
-                // white space
+                // whitespace
                 if ((matches = /^_./.exec(formatCode))) {
                     pushLiteralToken(' ');
                     continue;
                 }
 
+                // repeated fill character
+                if ((matches = /^\*./.exec(formatCode))) {
+                    // bug 36439: not supported by CalcEngine, ignore silently
+                    continue;
+                }
+
                 // character literal, preceded by backspace
                 if ((matches = /^\\(.)/.exec(formatCode))) {
                     pushLiteralToken(matches[1]);
@@ -302,7 +308,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
                     continue;
                 }
 
-				// text
+                // text
                 if ((matches = /^@/.exec(formatCode))) {
                     pushLiteralToken('');
                     self.onlyText = true;
@@ -349,22 +355,32 @@ define('io.ox/office/spreadsheet/model/numberformatter',
             // all built-in number format codes, mapped by identifier
             builtInFormats = {},
 
+            // all built-in number format codes, mapped by format code
+            invBuiltInFormats = {},
+
+            // the standard format code
+            standardCode = 'General',
+
             // all format codes for in-place edit mode
             editCodesDatabase = { date: 'DD/MM/YYYY', time: 'hh:mm:ss' },
 
             // default currency symbol
             currencySymbol = '$',
 
-			// '.' separator between integer and significant part
+            // localized decimal separator
             decimalSymbol = null,
 
-			// ',' 1000-values-separator
+            // localized group separator
             groupSymbol = null,
 
-            // lazy inited helper unformating the date
+            // the localized Boolean literals
+            falseLiteral = null,
+            trueLiteral = null,
+
+            // lazy initialized helper parsing the date
             unformatDate = null,
 
-            // lazy inited helper unformating the time
+            // lazy initialized helper parsing the time
             unformatTime = null,
 
             // null date (corresponding to cell value zero) TODO: use null date from document settings
@@ -683,7 +699,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
 
         }
 
-        // methods ------------------------------------------------------------
+        // public methods -----------------------------------------------------
 
         /**
          * Returns all available number format codes for the specified format
@@ -726,19 +742,95 @@ define('io.ox/office/spreadsheet/model/numberformatter',
             return (_.isArray(categoryCodes) && _.isObject(categoryCodes[0])) ? categoryCodes[0].value : null;
         };
 
+        /**
+         * Returns the standard format code.
+         *
+         * @returns {String}
+         *  The standard format code.
+         */
+        this.getStandardCode = function () {
+            return standardCode;
+        };
+
         /**
          * Returns the predefined number format code for the specified format
          * code identifier.
          *
-         * @param {Number} id
+         * @param {Number} formatId
          *  The format code identifier.
          *
          * @returns {String|Null}
          *  The number format code for the specified identifier if existing,
          *  otherwise null.
          */
-        this.getBuiltInFormatCode = function (id) {
-            return (id in builtInFormats) ? builtInFormats[id] : null;
+        this.getBuiltInFormatCode = function (formatId) {
+            return (formatId in builtInFormats) ? builtInFormats[formatId] : null;
+        };
+
+        /**
+         * Returns the predefined number format code for the specified format
+         * code identifier.
+         *
+         * @param {String} formatCode
+         *  The number format code to be converted to a built-in identifier.
+         *
+         * @returns {Number|Null}
+         *  The built-in identifier of the passed format code if available,
+         *  otherwise null.
+         */
+        this.getBuiltInFormatId = function (formatCode) {
+            return (formatCode in invBuiltInFormats) ? invBuiltInFormats[formatCode] : null;
+        };
+
+        /**
+         * Returns the format code of the passed value of a 'numberFormat'
+         * formatting attribute.
+         *
+         * @param {Object} numberFormat
+         *  The value of a 'numberFormat' formatting attribute. Must contain at
+         *  least one of the properties 'id' (numeric identifier of a built-in
+         *  format code), and 'code' (explicit format code, as string). If an
+         *  identifier and an explicit format code exists, the format code will
+         *  be ignored, and the built-in format code of the identifier will be
+         *  returned.
+         *
+         * @returns {String}
+         *  The effective format code of the passed attribute value.
+         */
+        this.resolveFormatCode = function (numberFormat) {
+
+            var // the built-in identifier
+                id = Utils.getIntegerOption(numberFormat, 'id'),
+                // explicit format code
+                code = Utils.getStringOption(numberFormat, 'code');
+
+            // prefer built-in identifier over format code
+            if (_.isNumber(id) && (id in builtInFormats)) { return builtInFormats[id]; }
+
+            // return existing explicit format code
+            if (_.isString(code) && (code.length > 0)) { return code; }
+
+            Utils.warn('NumberFormatter.resolveFormatCode(): invalid number format: ' + JSON.stringify(numberFormat));
+            return standardCode;
+        };
+
+        /**
+         * Converts the passed format code to a value that can be used as
+         * 'numberFormat' formatting attribute. Tries to resolve the identifier
+         * of a built-in number format if possible.
+         *
+         * @param {String} formatCode
+         *  A format code.
+         *
+         * @returns {Object}
+         *  A number format attribute value. The object will contain the
+         *  property 'id', if the format code represents a built-in number
+         *  format; otherwise the property 'code' with the passed format code.
+         */
+        this.createNumberFormat = function (formatCode) {
+            // replace format codes of built-in number formats with identifier
+            var id = this.getBuiltInFormatId(formatCode);
+            return _.isNumber(id) ? { id: id } : { code: formatCode };
         };
 
         /**
@@ -816,6 +908,139 @@ define('io.ox/office/spreadsheet/model/numberformatter',
             return (date.getTime() - nullDate) / MSEC_PER_DAY;
         };
 
+        /**
+         * Tries to parse a floating-point number from the leading part of the
+         * passed string.
+         *
+         * @param {String} text
+         *  The text containing a floating-point number.
+         *
+         * @param {Object} [options]
+         *  Optional parameters:
+         *  @param {String} [options.sep]
+         *      If specified, a custom decimal separator character. By default,
+         *      the decimal separator of the current UI language will be used.
+         *  @param {Boolean} [options.sign=false]
+         *      If set to true, the passed string may contain a leading sign
+         *      character ('-' or '+').
+         *  @param {Boolean} [options.complete=false]
+         *      If set to true, the entire passed string must represent a valid
+         *      floating-point number. No remaining 'garbage' text will be
+         *      accepted.
+         *
+         * @returns {Object|Null}
+         *  A result descriptor, if the passed string starts with a valid
+         *  floating-point number (according to the passed options); otherwise
+         *  null. The result descriptor contains the following properties:
+         *  - {Number} number
+         *      The parsed floating-point number.
+         *  - {String} text
+         *      The original text from the passed string that has been parsed
+         *      to the floating-point number.
+         *  - {String} sign
+         *      The sign character. Will be the empty string, if no sign exists
+         *      in the passed text, or if the option 'sign' has not been set
+         *      (see above).
+         *  - {String} remaining
+         *      The remaining characters from the passed string following the
+         *      parsed floating-point number. Will be the empty string, if the
+         *      option 'complete' has been set (see above).
+         */
+        this.parseNumberFromString = (function () {
+
+            // TODO: parse other formats: date/time, percent, currency, etc.
+
+            // returns the regular expression used to match a floating-point number
+            // Group 1: the leading sign (may be an empty string)
+            // Group 2: the floating-point number (decimal or scientific), without sign
+            var getNumberRegExp = _.memoize(function (sep) {
+                sep = _.escapeRegExp(sep);
+                return new RegExp('^([-+]?)((?:\\d*' + sep + '\\d+|\\d+' + sep + '?)(?:E[-+]?\\d+)?)', 'i');
+            });
+
+            // the actual implementation of the parseNumberFromString() method
+            return function parseNumberFromString(text, options) {
+
+                var // the decimal separator used to parse the number
+                    sep = Utils.getStringOption(options, 'sep', decimalSymbol),
+                    // whether to accept a leading sign
+                    sign = Utils.getBooleanOption(options, 'sign', false),
+                    // whether to reject trailing garbage
+                    complete = Utils.getBooleanOption(options, 'complete', false),
+                    // try to parse the passed string for a leading floating-point number
+                    matches = getNumberRegExp(sep).exec(text),
+                    // the parsed floating-point number
+                    number = null;
+
+                // reject leading sign unless specified in the passed options
+                if (!matches || (!sign && matches[1].length > 0)) { return null; }
+
+                // reject trailing garbage if specified in the passed options
+                if (complete && (matches[0].length < text.length)) { return null; }
+
+                // parse the folating-point number
+                number = parseFloat(matches[2].replace(decimalSymbol, '.'));
+                if (!_.isFinite(number)) { return null; }
+                if (matches[1] === '-') { number = -number; }
+
+                // build the result descriptor
+                return { number: number, text: matches[0], sign: matches[1], remaining: text.substr(matches[0].length) };
+            };
+        }());
+
+        /**
+         * Converts the passed text to a floating-point number, if possible.
+         *
+         * @param {String} text
+         *  The string to be converted to a floating-point number.
+         *
+         * @returns {Number}
+         *  The floating-point number represented by the passed string; or NaN,
+         *  if the string cannot be parsed to a number.
+         */
+        this.convertStringToNumber = function (text) {
+            var parseResult = this.parseNumberFromString(text, { sign: true, complete: true });
+            return _.isObject(parseResult) ? parseResult.number : Number.NaN;
+        };
+
+        /**
+         * Tries to convert the passed text to a number, Boolean, or error code
+         * literal.
+         *
+         * @param {String} text
+         *  The string to be converted to another data type.
+         *
+         * @param {Object} [options]
+         *  Optional parameters:
+         *  @param {Boolean} [options.keepApos=false]
+         *      If set to true, a leading apostrophe in the passed text (used
+         *      to protect a string from conversion to other data types) will
+         *      be kept in the result. by default, the result will be a string
+         *      without the leading aposthrophe.
+         *
+         * @returns {Number|Boolean|String|ErrorCode}
+         *  The resulting converted value (the original string, if the passed
+         *  text cannot be converted to anyother data type).
+         */
+        this.parseString = function (text, options) {
+
+            // try to parse Boolean literals
+            if (text.toUpperCase() === falseLiteral) { return false; }
+            if (text.toUpperCase() === trueLiteral) { return true; }
+
+            // try to parse error code literals
+            if (app.isLocalizedErrorCode(text)) {
+                return app.convertStringToErrorCode(text);
+            }
+
+            // try to convert passed text to a floating-point number
+            var number = this.convertStringToNumber(text);
+            if (_.isFinite(number)) { return number; }
+
+            // everything else is a regular string
+            return Utils.getBooleanOption(options, 'keepApos', false) ? text : text.replace(/^'/, '');
+        };
+
         /**
          * Returns the display string of the passed number formatted with the
          * 'General' number format, fitting into the specified text width.
@@ -849,21 +1074,10 @@ define('io.ox/office/spreadsheet/model/numberformatter',
          *  width restriction (but still with the restriction to a maximum
          *  number of text characters specified in the 'maxLength' parameter).
          *
-         * @returns {Object|Null}
-         *  A result object containing the formatted number and other data, in
-         *  the following properties:
-         *  - {String} text
-         *      The resulting display text of the number, formatted with the
-         *      'General' number format.
-         *  - {Number} digits
-         *      The number of significant decimal places in the integral and
-         *      fractional part in the string representation of the rounded
-         *      number.
-         *  - {Number} [width]
-         *      The width of the display text, in pixels. Will be omitted, if
-         *      no character attributes have been passed (see parameter
-         *      'attributes' above).
-         *  If the passed maximum pixel width is too small to take the passed
+         * @returns {String|Null}
+         *  The resulting display text of the number, formatted with the
+         *  'General' number format. If a passed maximum pixel width has been
+         *  passed to this method, and if it is too small to take the passed
          *  number in either decimal or scientific notation, the null value
          *  will be returned instead.
          */
@@ -941,7 +1155,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
                 return (result.abs.length <= maxLength) && (getTextWidth(result.text) <= maxWidth);
             }
 
-            // set the the validator callback function, depending on pixel mode
+            // set the validator callback function, depending on pixel mode
             validateFunc = useMaxWidth ? validateLengthAndWidth : validateLength;
 
             // find the best decimal representation
@@ -970,7 +1184,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
 
             // prefer decimal notation, if it shows at least as much significant digits as the scientific notation
             finalResult = (decResult && (!scientResult || (decResult.digits >= scientResult.digits))) ? decResult : scientResult;
-            return finalResult ? _.extend({ text: finalResult.text, digits: finalResult.digits }, hasAttributes ? { width: getTextWidth(finalResult.text) } : null) : null;
+            return finalResult ? finalResult.text : null;
         };
 
         /**
@@ -998,10 +1212,9 @@ define('io.ox/office/spreadsheet/model/numberformatter',
                 result = '',
                 // the passed value, as Date object
                 date = null,
-				// default exponent is 0, so original value cant be changed, if not explicit wanted
+                // default exponent is 0, so original value cant be changed, if not explicit wanted
                 exp = 0;
 
-
             // if resulting text is shorter, fills with leading zeros
             function fillZeros(int, length) {
                 return (ZEROS + int).substr(-Math.max(String(int).length, length));
@@ -1012,7 +1225,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
                 return (ZEROS + int).substr(-length);
             }
 
-			// len 3,4 & 5 are localized names
+            // len 3,4 & 5 are localized names
             function getMonth(month, len) {
                 switch (len) {
                     case 3: return CoreDate.locale.monthsShort[month].replace('.', '');
@@ -1022,10 +1235,10 @@ define('io.ox/office/spreadsheet/model/numberformatter',
                 }
             }
 
-			// len 3 & 4 are localized names
+            // len 3 & 4 are localized names
             function getWeekDay(day, date, len) {
                 switch (len) {
-                    case 3: return CoreDate.locale.dayssShort[day];
+                    case 3: return CoreDate.locale.daysShort[day];
                     case 4: return CoreDate.locale.days[day];
                     default: {
                         return fillZeros(date, len);
@@ -1033,7 +1246,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
                 }
             }
 
-			//format of the integer part of the number
+            //format of the integer part of the number
             function formatNumber(token) {
 
                 //standard group seperation! if the integer-format has a ',' inside every thee numbers comes the groupsymbol
@@ -1052,25 +1265,25 @@ define('io.ox/office/spreadsheet/model/numberformatter',
 
                 if(parsedSection.dateTime){
                     return token.text;
-                }else {
-                    var result = {number: value, abs: String(Math.abs(value))};
-
-                    if(parsedSection.notInteger){
-                        result.text = String(parseInt(value));
-                    }else{
-                        result.text = String(Math.round(value));
-                    }
-
-                    handleGroupSeparator(result, token.text);
-
-                    result.text = setPlaceHolders(result.text, token.text.replace(/\,/g, ''));
-
-                    return result.text;
                 }
+
+                var result = {number: value, abs: String(Math.abs(value))};
+
+                if(parsedSection.notInteger){
+                    result.text = String(parseInt(value));
+                }else{
+                    result.text = String(Math.round(value));
+                }
+
+                handleGroupSeparator(result, token.text);
+
+                result.text = setPlaceHolders(result.text, token.text.replace(/\,/g, ''));
+
+                return result.text;
             }
 
-			//format of the fraction part numerator & denominator
-			//at the moment we only support untill 3signs for the denominator
+            //format of the fraction part numerator & denominator
+            //at the moment we only support untill 3signs for the denominator
             function formatFraction(token) {
                 var parts = token.text.split('/');
 
@@ -1119,7 +1332,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
                 return decimalSymbol + frac;
             }
 
-			//format the significant part of the number incl. decimal-symbol
+            //format the significant part of the number incl. decimal-symbol
             function formatSign(token) {
                 var format = token.text.substring(1, token.text.length);
                 var digits = format.length;
@@ -1194,7 +1407,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
             _.each(parsedSection.tokens, function (token) {
                 switch (token.type) {
                 case 'general':
-                    result += self.formatStandardNumber(value, SheetUtils.MAX_LENGTH_STANDARD_CELL).text;
+                    result += self.formatStandardNumber(value, SheetUtils.MAX_LENGTH_STANDARD_CELL);
                     break;
                 case 'lit':
                     result += token.text;
@@ -1258,7 +1471,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
 
             // strings: use plain unformatted string for editing (bug 34421: add an aposthrophe if necessary)
             if (_.isString(value)) {
-                return ((/^['=]/).test(value) || app.isLocalizedErrorCode(value) || _.isFinite(app.convertStringToNumber(value))) ? ('\'' + value) : value;
+                return ((/^['=]/).test(value) || app.isLocalizedErrorCode(value) || _.isFinite(this.convertStringToNumber(value))) ? ('\'' + value) : value;
             }
 
             // Booleans: use plain Boolean literal for editing
@@ -1282,7 +1495,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
                 // percent: multiply by 100, add percent sign without whitespace
                 case 'percent':
                     value *= 100; // may become infinite
-                    result = (isFinite(value) ? this.formatStandardNumber(value, SheetUtils.MAX_LENGTH_STANDARD_EDIT).text : '') + '%';
+                    result = (isFinite(value) ? this.formatStandardNumber(value, SheetUtils.MAX_LENGTH_STANDARD_EDIT) : '') + '%';
                     break;
 
                 // automatically show date and/or time, according to the number
@@ -1308,7 +1521,7 @@ define('io.ox/office/spreadsheet/model/numberformatter',
                 }
 
                 // use 'General' number format for all other format codes
-                return _.isString(result) ? result : this.formatStandardNumber(value, SheetUtils.MAX_LENGTH_STANDARD_EDIT).text;
+                return _.isString(result) ? result : this.formatStandardNumber(value, SheetUtils.MAX_LENGTH_STANDARD_EDIT);
             }
 
             // empty cells
@@ -1363,29 +1576,16 @@ define('io.ox/office/spreadsheet/model/numberformatter',
             return value;
         };
 
-		/**
-		 * returns the display String for the cell, by assigned value & numberformat in the attributes
-		 */
+        /**
+         * returns the display String for the cell, by assigned value & numberformat in the attributes
+         */
         this.formatCellContent = function(value, attrs) {
-            var formatCode = null;
 
-            var cell = attrs.cell;
-            if(cell && cell.numberFormat){
-                formatCode = cell.numberFormat.code;
-                if(!formatCode || !formatCode.length){
-                    formatCode = self.getBuiltInFormatCode(cell.numberFormat.id);
-                }
-            }
-            if(!formatCode){
-                formatCode = '';
-            }
+            var cell = attrs.cell,
+                formatCode = (cell && cell.numberFormat) ? this.resolveFormatCode(cell.numberFormat) : standardCode;
 
             var res = self.formatValue(value, formatCode);
-            if(res === null){
-                return String(value);
-            }else{
-                return res;
-            }
+            return _.isNull(res) ? String(value) : res;
         };
 
         // initialization -----------------------------------------------------
@@ -1399,6 +1599,8 @@ define('io.ox/office/spreadsheet/model/numberformatter',
         app.onInitLocale(function () {
             decimalSymbol = app.getDecimalSeparator();
             groupSymbol = app.getGroupSeparator();
+            falseLiteral = app.getBooleanLiteral(false);
+            trueLiteral = app.getBooleanLiteral(true);
         });
 
         // get localized predefined format codes
@@ -1406,6 +1608,8 @@ define('io.ox/office/spreadsheet/model/numberformatter',
             _.extend(categoryCodesDatabase, data.categories);
             _.extend(editCodesDatabase, data.edit);
             _.extend(builtInFormats, data.builtInFormats);
+            invBuiltInFormats = Utils.mapProperties(_.invert(builtInFormats), function (id) { return parseInt(id, 10); });
+            standardCode = self.getCategoryDefaultCode('standard') || standardCode;
             currencySymbol = Utils.getStringOption(data, 'currencySymbol', currencySymbol, true);
             initUnformatData();
         });
