@@ -1,27 +1,27 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
- * @author Marko Benigar <marko.benigar@open-xchange.com>
+ * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/model/validationcollection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/editframework/model/attributedmodel',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/model/formula/tokenarray'
-    ], function (Utils, ModelObject, AttributedModel, SheetUtils, TokenArray) {
+define('io.ox/office/spreadsheet/model/validationcollection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/model/modelobject',
+    'io.ox/office/editframework/model/attributedmodel',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/model/formula/tokenarray'
+], function (Utils, ModelObject, AttributedModel, SheetUtils, TokenArray) {
 
     'use strict';
 
-    // class EntryModel =======================================================
+    // class ValidationModel ==================================================
 
     /**
      * Stores data validation settings for a single range in a specific sheet.
@@ -37,19 +37,19 @@ define('io.ox/office/spreadsheet/model/validationcollection',
      *  The sheet model instance containing this data validation range.
      *
      * @param {Array} ranges
-     *  The logical addresses of all cell ranges that contain the data
-     *  validation settings.
+     *  The addresses of all cell ranges that contain the data validation
+     *  settings.
      *
      * @param {Object} [attrs]
      *  The data validation attribute set.
      */
-    var EntryModel = AttributedModel.extend({ constructor: function (app, sheetModel, ranges, attrs) {
+    var ValidationModel = AttributedModel.extend({ constructor: function (app, sheetModel, ranges, attrs) {
 
         var // self reference
             self = this,
 
             // the spreadsheet document model
-            model = app.getModel(),
+            docModel = app.getModel(),
 
             // token array representing the validation attribute 'value1'
             tokenArray1 = new TokenArray(app, sheetModel),
@@ -59,7 +59,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
 
         // base constructor ---------------------------------------------------
 
-        AttributedModel.call(this, app, attrs, { additionalFamilies: 'validation' });
+        AttributedModel.call(this, app, attrs, { families: 'validation' });
 
         // private methods ----------------------------------------------------
 
@@ -84,7 +84,32 @@ define('io.ox/office/spreadsheet/model/validationcollection',
             }
         }
 
-        // methods ------------------------------------------------------------
+        // protected methods --------------------------------------------------
+
+        /**
+         * Creates and returns a cloned instance of this validation model for
+         * the specified sheet.
+         *
+         * @internal
+         *  Used by the class ValidationCollection during clone construction.
+         *  DO NOT CALL from external code!
+         *
+         * @param {SheetModel} targetModel
+         *  The model instance of the new cloned sheet that will own the clone
+         *  returned by this method.
+         *
+         * @returns {ValidationModel}
+         *  A clone of this validation model, initialized for ownership by the
+         *  passed sheet model.
+         */
+        this.clone = function (targetModel) {
+            // construct a new model instance and relocate the sheet references
+            var valModel = new ValidationModel(app, targetModel, ranges, this.getExplicitAttributes(true));
+            valModel.relocateSheet(sheetModel.getIndex(), targetModel.getIndex());
+            return valModel;
+        };
+
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the addresses of all cell ranges of this collection entry.
@@ -101,7 +126,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          * ranges of this collection entry.
          *
          * @param {Number[]} address
-         *  The logical address of the cell to be checked.
+         *  The address of the cell to be checked.
          *
          * @returns {Boolean}
          *  Whether the specified cell is contained in this collection entry.
@@ -116,7 +141,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          * ranges of this collection entry.
          *
          * @returns {Number[]}
-         *  The logical address of the reference cell of this collection entry.
+         *  The address of the reference cell of this collection entry.
          */
         this.getRefAddress = function () {
             return SheetUtils.getBoundingRange(ranges).start;
@@ -159,7 +184,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
                 var refAddress = self.getRefAddress(),
                     sourceRanges = tokenArray1.resolveRangeList({ refAddress: refAddress, targetAddress: targetAddress });
                 if (!_.isArray(sourceRanges)) { return $.Deferred().reject(); }
-                sourceQuery = model.queryCellContents([sourceRanges], { maxCount: 1000 });
+                sourceQuery = docModel.queryCellContents([sourceRanges], { hidden: true, maxCount: 1000 });
                 return sourceQuery.then(function (resultContents) {
                     return _.pluck(resultContents[0], 'display');
                 });
@@ -194,7 +219,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          *  The addresses of the new cell ranges. The old ranges of this
          *  collection entry will be replaced completely.
          *
-         * @returns {EntryModel}
+         * @returns {ValidationModel}
          *  A reference to this instance.
          */
         this.setRanges = function (newRanges) {
@@ -227,7 +252,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          *  the validation collection.
          */
         this.transformRanges = function (interval, insert, columns) {
-            ranges = model.transformRanges(ranges, interval, insert, columns, 'expandEnd');
+            ranges = docModel.transformRanges(ranges, interval, insert, columns, 'expandEnd');
             return ranges.length > 0;
         };
 
@@ -242,6 +267,20 @@ define('io.ox/office/spreadsheet/model/validationcollection',
             return true;
         };
 
+        /**
+         * Changes all formula expressions stored in the validation attributes
+         * 'value1' and 'value2' with sheet references containing the specified
+         * old sheet index to the new sheet index.
+         *
+         * @returns {ValidationModel}
+         *  A reference to this instance.
+         */
+        this.relocateSheet = function (oldSheet, newSheet) {
+            tokenArray1.relocateSheet(oldSheet, newSheet);
+            tokenArray2.relocateSheet(oldSheet, newSheet);
+            return this;
+        };
+
         // initialization -----------------------------------------------------
 
         // unify the range addresses
@@ -263,15 +302,9 @@ define('io.ox/office/spreadsheet/model/validationcollection',
             self.on('change:attributes', changeAttributesHandler);
         });
 
-        // constructor called from BaseObject.clone()
-        this.registerCloneConstructor(function (newSheetModel, newSheet) {
-            // pass own entries as hidden argument to the constructor
-            return new EntryModel(app, newSheetModel, ranges, this.getExplicitAttributes(), { oldSheet: sheetModel.getIndex(), newSheet: newSheet });
-        });
-
         // adjust formulas according to sheet indexes passed as hidden argument to the c'tor
         (function (args) {
-            var cloneData = args[EntryModel.length];
+            var cloneData = args[ValidationModel.length];
             if (_.isObject(cloneData)) {
                 tokenArray1.relocateSheet(cloneData.oldSheet, cloneData.newSheet);
                 tokenArray2.relocateSheet(cloneData.oldSheet, cloneData.newSheet);
@@ -282,11 +315,11 @@ define('io.ox/office/spreadsheet/model/validationcollection',
         this.registerDestructor(function () {
             tokenArray1.destroy();
             tokenArray2.destroy();
-            model = tokenArray1 = tokenArray2 = null;
-            app = sheetModel = ranges = null;
+            app = sheetModel = null;
+            self = docModel = tokenArray1 = tokenArray2 = null;
         });
 
-    }}); // class EntryModel
+    }}); // class ValidationModel
 
     // class ValidationCollection =============================================
 
@@ -308,7 +341,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
     function ValidationCollection(app, sheetModel) {
 
         var // the spreadsheet document model
-            model = app.getModel(),
+            docModel = app.getModel(),
 
             // all collection entries with validation settings
             entries = [];
@@ -323,9 +356,9 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          * Returns the collection entry that covers the specified cell.
          *
          * @param {Number[]} address
-         *  The logical address of a cell.
+         *  The address of a cell.
          *
-         * @returns {EntryModel|Null}
+         * @returns {ValidationModel|Null}
          *  The collection entry for the specified cell. If the cell is not
          *  validated at all, returns null.
          */
@@ -344,7 +377,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          *  the method returns the value false, the respective entry will be
          *  removed from the collection.
          *
-         * @param {EntryModel|Null} excludeEntry
+         * @param {ValidationModel|Null} excludeEntry
          *  A collection entry that will be skipped (its method will not be
          *  invoked).
          *
@@ -355,7 +388,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
             var args = _.toArray(arguments).slice(invokeEntryMethod.length);
             Utils.iterateArray(entries, function (entry, index) {
                 if ((entry !== excludeEntry) && (entry[methodName].apply(entry, args) === false)) {
-                    entries[index].destroy();
+                    entry.destroy();
                     entries.splice(index, 1);
                 }
             }, { reverse: true }); // reverse to be able to delete in-place
@@ -378,20 +411,43 @@ define('io.ox/office/spreadsheet/model/validationcollection',
             invokeEntryMethod('transformValues', null, sheet, interval, insert, columns);
         }
 
-        // methods ------------------------------------------------------------
+        // protected methods --------------------------------------------------
+
+        /**
+         * Creates and returns a cloned instance of this collection for the
+         * specified sheet.
+         *
+         * @internal
+         *  Used by the class SheetModel during clone construction. DO NOT CALL
+         *  from external code!
+         *
+         * @param {SheetModel} targetModel
+         *  The model instance of the new cloned sheet that will own the clone
+         *  returned by this method.
+         *
+         * @returns {ValidationCollection}
+         *  A clone of this collection, initialized for ownership by the passed
+         *  sheet model.
+         */
+        this.clone = function (targetModel) {
+            // construct a new collection, pass all own entries as hidden parameter
+            return new ValidationCollection(app, targetModel, entries);
+        };
+
+        // public methods -----------------------------------------------------
 
         /**
          * Returns the validation settings for the specified cell.
          *
          * @param {Number[]} address
-         *  The logical address of a cell.
+         *  The address of a cell.
          *
          * @returns {Object|Null}
          *  The validation settings of the specified cell, in the following
          *  properties:
          *  - {Array} ranges
-         *      The logical addresses of all related cell ranges containing the
-         *      same validation settings.
+         *      The addresses of all related cell ranges containing the same
+         *      validation settings.
          *  - {Object} attributes
          *      All validation attributes.
          *  If the cell is not validated at all, returns null.
@@ -409,7 +465,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          * 'source' at the specified cell address.
          *
          * @param {Number[]} address
-         *  The logical address of a cell.
+         *  The address of a cell.
          *
          * @returns {jQuery.Promise}
          *  The promise of a Deferred object that will be resolved with an
@@ -447,7 +503,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
         this.insertValidation = function (ranges, attrs, index) {
 
             var // the new collection entry
-                entry = new EntryModel(app, sheetModel, ranges, { validation: attrs });
+                entry = new ValidationModel(app, sheetModel, ranges, { validation: attrs });
 
             // reduce ranges of existing entries (this may result in removing entries completely)
             invokeEntryMethod('reduceRanges', null, entry.getRanges());
@@ -457,8 +513,6 @@ define('io.ox/office/spreadsheet/model/validationcollection',
             entries.splice(index, 0, entry);
 
             // TODO: trigger change events
-
-            // no known error case
             return true;
         };
 
@@ -471,11 +525,11 @@ define('io.ox/office/spreadsheet/model/validationcollection',
          *
          * @param {Array|Null} ranges
          *  The new addresses of the cell ranges for the collection entry. If
-         *  set to null, the original ranges will not be modified. Otherwise,
-         *  the
+         *  set to null, the original ranges will not be modified.
          *
-         * @param {Object} attrs
-         *  The validation attributes.
+         * @param {Object|Null} attrs
+         *  The new validation attributes. If set to null, the current
+         *  attributes will not be modified.
          *
          * @returns {Boolean}
          *  Whether the validation settings have been changed successfully.
@@ -483,9 +537,7 @@ define('io.ox/office/spreadsheet/model/validationcollection',
         this.changeValidation = function (index, ranges, attrs) {
 
             var // the collection entry to be modified
-                entry = entries[index],
-                // the new ranges for the entry
-                ranges = Utils.getArrayOption(attrs, 'ranges', null);
+                entry = entries[index];
 
             // validate passed collection index
             if (!_.isObject(entry)) { return false; }
@@ -500,8 +552,10 @@ define('io.ox/office/spreadsheet/model/validationcollection',
                 entry.setRanges(ranges);
             }
 
-            // change all other validation properties
-            entry.setAttributes({ validation: attrs });
+            // change all other validation attributes
+            if (_.isObject(attrs)) {
+                entry.setAttributes({ validation: attrs });
+            }
 
             return true;
         };
@@ -536,27 +590,18 @@ define('io.ox/office/spreadsheet/model/validationcollection',
         sheetModel.registerTransformationHandler(transformRanges);
 
         // update source formulas after inserting/deleting columns or rows in any sheet
-        model.registerTransformationHandler(transformValues);
+        docModel.registerTransformationHandler(transformValues);
 
-        // constructor called from BaseObject.clone()
-        this.registerCloneConstructor(function (newSheetModel, newSheet) {
-            // pass own entries as hidden argument to the constructor
-            return new ValidationCollection(app, newSheetModel, { entries: entries, newSheet: newSheet });
-        });
-
-        // clone collection entries passed as hidden argument to the c'tor
-        (function (args) {
-            var cloneData = args[ValidationCollection.length];
-            if (_.isObject(cloneData)) {
-                entries = ModelObject.cloneArray(cloneData.entries, sheetModel, cloneData.newSheet);
-            }
-        }(arguments));
+        // clone collection entries passed as hidden argument by the clone() method
+        if (_.isArray(arguments[ValidationCollection.length])) {
+            entries = _.invoke(arguments[ValidationCollection.length], 'clone', sheetModel);
+        }
 
         // destroy all class members
         this.registerDestructor(function () {
-            model.unregisterTransformationHandler(transformValues);
+            docModel.unregisterTransformationHandler(transformValues);
             _.invoke(entries, 'destroy');
-            model = entries = null;
+            app = sheetModel = docModel = entries = null;
         });
 
     } // class ValidationCollection
