@@ -1,20 +1,20 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/utils/paneutils',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/spreadsheet/utils/sheetutils'
-    ], function (Utils, SheetUtils) {
+define('io.ox/office/spreadsheet/utils/paneutils', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/spreadsheet/utils/sheetutils'
+], function (Utils, SheetUtils) {
 
     'use strict';
 
@@ -51,6 +51,13 @@ define('io.ox/office/spreadsheet/utils/paneutils',
 
     // constants --------------------------------------------------------------
 
+    /**
+     * The minimum size of grid/header panes, in pixels.
+     *
+     * @constant
+     */
+    PaneUtils.MIN_PANE_SIZE = 10;
+
     /**
      * Default settings for mouse/touch tracking in header and grid panes.
      *
