@@ -1,20 +1,20 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/utils/sheetutils',
-    ['io.ox/office/tk/utils',
-     'gettext!io.ox/office/spreadsheet'
-    ], function (Utils, gt) {
+define('io.ox/office/spreadsheet/utils/sheetutils', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/logger'
+], function (Utils, Logger) {
 
     'use strict';
 
@@ -40,7 +40,18 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         RE_CELL_NAME_A1 = new RegExp('^(' + COL_PATTERN + ')(' + ROW_PATTERN + ')$', 'i'),
 
         // regular expression object matching a range name
-        RE_RANGE_NAME_A1 = new RegExp('^(' + COL_PATTERN + ROW_PATTERN + '):(' + COL_PATTERN + ROW_PATTERN + ')$', 'i');
+        RE_RANGE_NAME_A1 = new RegExp('^(' + COL_PATTERN + ROW_PATTERN + '):(' + COL_PATTERN + ROW_PATTERN + ')$', 'i'),
+
+        // maps native error codes to integers (used e.g. for comparison in filters)
+        ERROR_CODE_NUMBERS = {
+            '#NULL!': 1,
+            '#DIV/0!': 2,
+            '#VALUE!': 3,
+            '#REF!': 4,
+            '#NAME?': 5,
+            '#NUM!': 6,
+            '#N/A': 7
+        };
 
     // class ErrorCode ========================================================
 
@@ -58,8 +69,16 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      * @property {String} code
      *  The native string representation of the error code, as passed to the
      *  constructor.
+     *
+     * @property {Number} num
+     *  The internal numeric index of built-in error codes, used for example
+     *  when comparing error codes in filters and filtering sheet functions.
+     *  Will be set to NaN, if the passed error code is not a built-in code.
      */
-    function ErrorCode(code) { this.code = code; }
+    function ErrorCode(code) {
+        this.code = code;
+        this.num = ERROR_CODE_NUMBERS[code] || Number.NaN;
+    }
 
     // prototype methods ------------------------------------------------------
 
@@ -79,12 +98,19 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         return (value instanceof ErrorCode) && (value.code === this.code);
     };
 
-    // string representation for debug logging
+    /**
+     * returns the string representation of this error code for debug logging.
+     */
     ErrorCode.prototype.toString = function () { return this.code; };
 
     // static class SheetUtils ================================================
 
-    var SheetUtils = {};
+    /**
+     * The static class SheetUtils is a console logger bound to the URL hash
+     * flag 'spreadsheet:log-models', that logs detailed information about the
+     * document model instances and collections.
+     */
+    var SheetUtils = new Logger({ enable: 'spreadsheet:log-models', prefix: 'MODEL' });
 
     // constants --------------------------------------------------------------
 
@@ -104,6 +130,14 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.MAX_ROW_HEIGHT = 20000;
 
+    /**
+     * The maximum number of cells whose content values can be queried from the
+     * document with a single request.
+     *
+     * @constant
+     */
+    SheetUtils.MAX_QUERY_CELL_COUNT = 10000;
+
     /**
      * The maximum number of cells to be filled with one range operation, or
      * auto-fill operation.
@@ -156,6 +190,13 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.MAX_SELECTION_CELL_COUNT = 100000;
 
+    /**
+     * The maximum number of search results returned by a single request.
+     *
+     * @constant
+     */
+    SheetUtils.MAX_SEARCH_RESULT_COUNT = 100;
+
     /**
      * Maximum number of characters for the absolute part of a number formatted
      * with the 'General' number format (in cells).
@@ -172,6 +213,15 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.MAX_LENGTH_STANDARD_EDIT = 21;
 
+    /**
+     * Maximum number of characters for the absolute part of a number formatted
+     * with the 'General' number format (in formula interpretation, e.g. when
+     * converting numbers to strings for functions expecting strings).
+     *
+     * @constant
+     */
+    SheetUtils.MAX_LENGTH_STANDARD_FORMULA = 20;
+
     /**
      * Minimum effective size of a cell, in pixels. Also used as maximum width
      * of cell border lines, to prevent that the left/right borders or the top/
@@ -202,7 +252,6 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
      */
     SheetUtils.MULTI_SELECTION = !Modernizr.touch;
 
-
     /**
      * Predefined native error codes, as used in operations.
      *
@@ -445,6 +494,21 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         return SheetUtils.rangeContainsCell(boundRange, range.start) && SheetUtils.rangeContainsCell(boundRange, range.end);
     };
 
+    /**
+     * Returns the number of cells covered by the passed cell ranges. Cells
+     * covered by multiple ranges will be counted several times (no implicit
+     * unification).
+     *
+     * @param {Object|Array} ranges
+     *  The address of a single cell range, or an array with range addresses.
+     *
+     * @returns {Number}
+     *  The number of cells covered by the passed ranges.
+     */
+    SheetUtils.getCellCountInRanges = function (ranges) {
+        return Utils.getSum(_.getArray(ranges), SheetUtils.getCellCount);
+    };
+
     /**
      * Returns whether any of the cell ranges in the passed array contains the
      * specified column index.
@@ -551,6 +615,32 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
 
     // range intersections and bounding range ---------------------------------
 
+    /**
+     * Returns a range address consisting of the first row in the passed range.
+     *
+     * @param {Object} range
+     *  The original range address.
+     *
+     * @returns {Object}
+     *  A range address consisting of the first row in the passed range.
+     */
+    SheetUtils.getHeaderRange = function (range) {
+        return { start: _.clone(range.start), end: [range.end[0], range.start[1]] };
+    };
+
+    /**
+     * Returns a range address consisting of the last row in the passed range.
+     *
+     * @param {Object} range
+     *  The original range address.
+     *
+     * @returns {Object}
+     *  A range address consisting of the last row in the passed range.
+     */
+    SheetUtils.getFooterRange = function (range) {
+        return { start: [range.start[0], range.end[1]], end: _.clone(range.end) };
+    };
+
     /**
      * Returns the unique ranges of the passed range addresses. Does NOT join
      * overlapping or adjacent ranges.
@@ -937,6 +1027,70 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         return intervals;
     };
 
+    /**
+     * Returns the difference of the passed index intervals (all indexes
+     * contained in the first interval list, that are not contained in the
+     * second interval list).
+     *
+     * @param {Object|Array} intervals1
+     *  A single index interval, or an array with index intervals.
+     *
+     * @param {Object|Array} intervals2
+     *  A single index interval, or an array with index intervals.
+     *
+     * @returns {Array}
+     *  All index intervals that are contained in the parameter 'intervals1'
+     *  but not in 'intervals2'. May be an empty array, if the intervals in
+     *  'intervals2' completely cover all intervals in 'intervals1'.
+     */
+    SheetUtils.getRemainingIntervals = function (intervals1, intervals2) {
+
+        var // the source intervals to be reduced while iterating
+            sourceIntervals = null,
+            // the resulting intervals
+            resultIntervals = _.getArray(intervals1);
+
+        // reduce the intervals from 'intervals1' with each interval from 'intervals2'
+        Utils.iterateArray(_.getArray(intervals2), function (interval2) {
+
+            // initialize the arrays for this iteration
+            sourceIntervals = resultIntervals;
+            resultIntervals = [];
+
+            // process each interval in the source interval list
+            _.each(sourceIntervals, function (interval1) {
+
+                // check if the intervals cover each other at all
+                if (!SheetUtils.intervalOverlapsInterval(interval1, interval2)) {
+                    resultIntervals.push(interval1);
+                    return;
+                }
+
+                // do nothing if interval2 covers interval1 completely (delete interval1 from the result)
+                if (SheetUtils.intervalContainsInterval(interval2, interval1)) {
+                    return;
+                }
+
+                // if interval1 starts before interval2, extract the leading part of interval1
+                if (interval1.first < interval2.first) {
+                    resultIntervals.push({ first: interval1.first, last: interval2.first - 1 });
+                }
+
+                // if interval1 ends after interval2, extract the trailing part of interval1
+                if (interval1.last > interval2.last) {
+                    resultIntervals.push({ first: interval2.last + 1, last: interval1.last });
+                }
+            });
+
+            // early exit the loop, if all source intervals have been deleted already
+            return (resultIntervals.length === 0) ? Utils.BREAK : undefined;
+        });
+
+        return SheetUtils.getUnifiedIntervals(resultIntervals);
+    };
+
+    // interval/range conversion ----------------------------------------------
+
     /**
      * Returns the column interval covered by the passed cell range.
      *
@@ -1718,26 +1872,6 @@ define('io.ox/office/spreadsheet/utils/sheetutils',
         return _.map(_.getArray(ranges), SheetUtils.getRangeName).join(separator);
     };
 
-    /**
-     * Generates a localized sheet name with the specified index.
-     *
-     * @param {Number} sheet
-     *  The zero-based sheet index to be inserted into the sheet name. The
-     *  number inserted into the sheet name will be increased by one.
-     *
-     * @returns {String}
-     *  The generated sheet name (e.g. 'Sheet1', 'Sheet2', etc.).
-     */
-    SheetUtils.generateSheetName = function (sheet) {
-        return (
-            //#. Default sheet names in spreadsheet documents. Should be equal to the sheet
-            //#. names used in existing spreadsheet applications (Excel, OpenOffice Calc, ...).
-            //#. %1$d is the numeric index in the sheet name (e.g. "Sheet1", "Sheet2", etc.)
-            //#. No white-space between the "Sheet" label and sheet index!
-            //#, c-format
-            gt('Sheet%1$d', _.noI18n(sheet + 1)));
-    };
-
     // parse UI strings -------------------------------------------------------
 
     /**
