@@ -1,24 +1,24 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
 define('io.ox/office/spreadsheet/view/chartcreator',
     ['io.ox/office/tk/utils',
+     'io.ox/office/spreadsheet/utils/operations',
      'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/model/operations',
      'io.ox/office/spreadsheet/model/formula/tokenarray',
      'io.ox/office/drawinglayer/view/chartstyleutil',
      'gettext!io.ox/office/spreadsheet'
-    ], function (Utils, SheetUtils, Operations, TokenArray, ChartStyleUtil, gt) {
+    ], function (Utils, Operations, SheetUtils, TokenArray, ChartStyleUtil, gt) {
 
     'use strict';
 
@@ -30,7 +30,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
 
     var DEFAULT_TEXTCOLOR = {type: 'auto'};
     var DEFAULT_LINECOLOR = {transformations: [{type: 'lumMod', value: 15000}, {type: 'lumOff',value: 85000}],type: 'scheme', value: 'text1'};
-    var STANDARD_SHAPE = {type: 'solid', color: DEFAULT_LINECOLOR, width: 1};
+    var STANDARD_SHAPE = {type: 'solid', color: DEFAULT_LINECOLOR, width: 36};
     var NONE_SHAPE = {type: 'none'};
     var STANDARD_CHAR = {color: DEFAULT_TEXTCOLOR};
     var HEAD_CHAR = {color: DEFAULT_TEXTCOLOR, fontSize: 16};
@@ -68,37 +68,31 @@ define('io.ox/office/spreadsheet/view/chartcreator',
         if (!from) {
             throw 'no from assigned';
         }
-        var range = {};
-        range.start = from;
-        if (to) {
-            range.end = to;
-        } else {
-            range.end = range.start;
-        }
-        var tokenArray = new TokenArray(app, null, { silent: true });
-        tokenArray.appendRange(range, { abs: true, sheet: sheet });
+        var range = { start: from, end: to || from };
+        var tokenArray = new TokenArray(app, null, { trigger: 'never' });
+        tokenArray.appendRange(range, { sheet: sheet, abs: true });
         var formula = tokenArray.getFormula();
         tokenArray.destroy();
 
         return formula;
     }
 
-    function addDataSeries(app, generator, sheet, chartPos, series, keyFrom, keyTo, title, valueFrom, valueTo, bubbleFrom, bubbleTo, bubbleArray) {
+    function addDataSeries(app, generator, sourceSheet, position, series, keyFrom, keyTo, title, valueFrom, valueTo, bubbleFrom, bubbleTo, bubbleArray) {
         var insert = {
-            values: getFormula(app, sheet, valueFrom, valueTo)
+            values: getFormula(app, sourceSheet, valueFrom, valueTo)
         };
         if (title) {
-            insert.title = getFormula(app, sheet, title);
+            insert.title = getFormula(app, sourceSheet, title);
         }
         if (keyFrom) {
-            insert.names = getFormula(app, sheet, keyFrom, keyTo);
+            insert.names = getFormula(app, sourceSheet, keyFrom, keyTo);
         }
         if (bubbleFrom) {
-            insert.bubbles = getFormula(app, sheet, bubbleFrom, bubbleTo);
+            insert.bubbles = getFormula(app, sourceSheet, bubbleFrom, bubbleTo);
         } else if (bubbleArray) {
             insert.bubbles = bubbleArray;
         }
-        var options = {
+        var properties = {
             series: series,
             attrs: {
                 series: insert,
@@ -106,18 +100,19 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                 fill: {type: 'solid'},
             }
         };
-        generator.generateDrawingOperation(Operations.INSERT_CHART_DATASERIES, chartPos[0], chartPos[1], options);
+        generator.generateDrawingOperation(Operations.INSERT_CHART_DATASERIES, position, properties);
     }
 
-    function generateSeriesOperations(app, range, sheet, generator, chartPos, cells, axis, chart) {
+    function generateSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, axis, chart) {
+
         if (_.isUndefined(chart.varyColors)) {
             chart.varyColors = true;
         }
 
-        generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, chartPos[0], chartPos[1], {attrs: { chart: chart}});
+        generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, position, { attrs: { chart: chart } });
 
-        var start = range.start;
-        var end = range.end;
+        var start = sourceRange.start;
+        var end = sourceRange.end;
 
         var useAxis;
         if (axis !== 0 && axis !== 1) {
@@ -140,9 +135,9 @@ define('io.ox/office/spreadsheet/view/chartcreator',
 
         var seriesCount;
         if (chart.type && chart.type.indexOf('bubble') === 0) {
-            seriesCount = generateBubbleSeriesOperations(app, range, sheet, generator, chartPos, cells, useAxis);
+            seriesCount = generateBubbleSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, useAxis);
         } else {
-            seriesCount =  generateStandardSeriesOperations(app, range, sheet, generator, chartPos, cells, useAxis);
+            seriesCount = generateStandardSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, useAxis);
         }
         if ((seriesCount === 1 && !chart.varyColors) || (chart.type === 'scatter2d' || chart.type === 'line2d')) {
             var send = {};
@@ -151,7 +146,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
             }
             //CURVED is a workaround, filter has not all the information it needs!
             send.curved = chart.curved;
-            generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, chartPos[0], chartPos[1], {attrs: { chart: send}});
+            generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, position, { attrs: { chart: send } });
         }
         return seriesCount;
     }
@@ -175,9 +170,10 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      *                         second is a Boolean for the cell right of the first cell
      *  @param activeAxis [1 or 0] column or row for the direction of any series data
      */
-    function generateStandardSeriesOperations(app, range, sheet, generator, chartPos, cells, activeAxis) {
-        var start = range.start;
-        var end = range.end;
+    function generateStandardSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, activeAxis) {
+
+        var start = sourceRange.start;
+        var end = sourceRange.end;
 
         var max = end[activeAxis] - start[activeAxis];
         var otherAxis = (activeAxis - 1) * -1;
@@ -212,7 +208,7 @@ define('io.ox/office/spreadsheet/view/chartcreator',
             }
             var valueTo = makeAddress(activeAxis, otherAxis, activeIndex, end[otherAxis]);
 
-            addDataSeries(app, generator, sheet, chartPos, i, keyFrom, keyTo, title, valueFrom, valueTo);
+            addDataSeries(app, generator, sourceSheet, position, i, keyFrom, keyTo, title, valueFrom, valueTo);
         }
         return max;
     }
@@ -229,9 +225,10 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      *                         second is a Boolean for the cell right of the first cell
      *  @param activeAxis [1 or 0] column or row for the direction of any series data
      */
-    function generateBubbleSeriesOperations(app, range, sheet, generator, chartPos, cells, activeAxis) {
-        var start = range.start;
-        var end = range.end;
+    function generateBubbleSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, activeAxis) {
+
+        var start = sourceRange.start;
+        var end = sourceRange.end;
 
         var max = end[activeAxis] - start[activeAxis];
         if (max === 0) {
@@ -288,33 +285,36 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                 bubbleTo = makeAddress(activeAxis, otherAxis, valueTo[activeAxis] + 1, valueTo[otherAxis]);
             }
 
-            addDataSeries(app, generator, sheet, chartPos, size, keyFrom, keyTo, title, valueFrom, valueTo, bubbleFrom, bubbleTo, bubbleArray);
+            addDataSeries(app, generator, sourceSheet, position, size, keyFrom, keyTo, title, valueFrom, valueTo, bubbleFrom, bubbleTo, bubbleArray);
             size++;
         }
         return size;
     }
 
+    function getContentForRange(app, sourceRange, sourceSheet, def, follow, forceColLabel, forceRowLabel) {
 
-    function getContentForRange(app, range, sheet, def, follow) {
-        var view = app.getView();
+        var sheetModel = app.getModel().getSheetModel(sourceSheet);
 
-        var start = range.start;
-        var end = range.end;
-        var width = end[0] - start[0];
-        var height = end[1] - start[1];
+        var start = sourceRange.start;
+        var cols = SheetUtils.getColCount(sourceRange);
+        var rows = SheetUtils.getRowCount(sourceRange);
 
-        if (width > 50 || height > 50) {
-            view.yell({ type: 'warning', message: gt('It is not possible to create a chart out of more than 50 input cells.') });
+        if (cols > 50 || rows > 50) {
+            app.getView().yell({ type: 'warning', message: gt('It is not possible to create a chart out of more than 50 input cells.') });
             if (def) {
                 def.reject();
             }
         } else {
-            var bottAddress = [start[0], start[1] + 1];
-            var rightAddress = [start[0] + 1, start[1]];
+            var bottEntry = sheetModel.getRowCollection().getNextVisibleEntry(start[1] + 1);
+            var rightEntry = sheetModel.getColCollection().getNextVisibleEntry(start[0] + 1);
+
+            var bottAddress = [start[0], bottEntry ? bottEntry.index : start[1]];
+            var rightAddress = [rightEntry ? rightEntry.index : start[0], start[1]];
+
             var rangesArray = [[
-                    { sheet: sheet, start: bottAddress, end: bottAddress },
-                    { sheet: sheet, start: rightAddress, end: rightAddress },
-                    { sheet: sheet, start: start, end: start }
+                    { sheet: sourceSheet, start: bottAddress, end: bottAddress },
+                    { sheet: sourceSheet, start: rightAddress, end: rightAddress },
+                    { sheet: sourceSheet, start: start, end: start }
                 ]];
 
             app.getModel().queryCellContents(rangesArray, { attributes: true }).done(function (res) {
@@ -332,9 +332,15 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                     return (/^(date|time|datetime)$/).test(cellContents.format.cat);
                 }
 
-                if (width === 0 || height === 0) {
-                    var firstString = !_.isNumber(startCell.result);
-                    if (width === 0) {
+                function cellIsLabel(cell) {
+                    if (!startCell) { return false; }
+                    if (!startCell.display.length) { return true; }
+                    return isDate(cell) || (cell ? !_.isNumber(cell.result) : false);
+                }
+
+                if (cols === 1 || rows === 1) {
+                    var firstString = startCell ? !_.isNumber(startCell.result) : false;
+                    if (cols === 1) {
                         axis = 0;
                         if (firstString) {
                             right = true;
@@ -346,8 +352,15 @@ define('io.ox/office/spreadsheet/view/chartcreator',
                         }
                     }
                 } else {
-                    bottom = (startCell.display.length === 0) || isDate(bottomCell) || !_.isNumber(bottomCell.result);
-                    right = (startCell.display.length === 0) || isDate(rightCell) || !_.isNumber(rightCell.result);
+                    bottom = cellIsLabel(bottomCell);
+                    right = cellIsLabel(rightCell);
+                }
+
+                if (!_.isUndefined(forceColLabel)) {
+                    bottom = forceColLabel;
+                }
+                if (!_.isUndefined(forceRowLabel)) {
+                    right = forceRowLabel;
                 }
                 follow([bottom, right], axis);
             });
@@ -361,19 +374,17 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      * and changes the Selection to a Multiselection by title, names and values
      */
     function ChartCreator(app, chart) {
-        var view = app.getView();
-        var gridPane = view.getActiveGridPane();
+        var docView = app.getView();
+        var gridPane = docView.getActiveGridPane();
 
         var def = $.Deferred();
 
-        var view = app.getView();
-        var drawingCollection = view.getDrawingCollection();
-        var sheet = view.getActiveSheet();
-        var selection = view.getSelection();
+        var drawingCollection = docView.getDrawingCollection();
+        var selection = docView.getSelection();
 
         var range = selection.ranges[selection.activeRange];
 
-        getContentForRange(app, range, sheet, def, function (cells, axis) {
+        getContentForRange(app, range, docView.getActiveSheet(), def, function (cells, axis) {
 
             var rect = gridPane.getVisibleRectangle();
 
@@ -386,20 +397,19 @@ define('io.ox/office/spreadsheet/view/chartcreator',
 
             var attrs = drawingCollection.getAttributesForRectangle(clip);
 
-            view.insertDrawing('chart', attrs, function (generator, sheet, position) {
-                var start = [sheet, position[0]];
+            docView.insertDrawing('chart', attrs, function (generator, sheet, position) {
 
-                var seriesCount = generateSeriesOperations(app, range, sheet, generator, start, cells, axis, chart);
+                var seriesCount = generateSeriesOperations(app, range, sheet, generator, position, cells, axis, chart);
 
                 var legendPos;
                 if (seriesCount === 1) {
                     //TODO: title
                     legendPos = 'off';
-                    generator.generateOperation(Operations.SET_CHART_TITLE_ATTRIBUTES, {start: start, axis: 'main', attrs: { character: HEAD_CHAR}});
+                    generator.generateDrawingOperation(Operations.SET_CHART_TITLE_ATTRIBUTES, position, { axis: 'main', attrs: { character: HEAD_CHAR } });
                 } else {
                     legendPos = 'bottom';
                 }
-                generator.generateOperation(Operations.SET_CHART_LEGEND_ATTRIBUTES, {start: start, attrs: { legend: { pos: legendPos}, character: STANDARD_CHAR}});
+                generator.generateDrawingOperation(Operations.SET_CHART_LEGEND_ATTRIBUTES, position, { attrs: { legend: { pos: legendPos}, character: STANDARD_CHAR } });
 
                 var defaults;
                 switch (chart.type) {
@@ -418,13 +428,13 @@ define('io.ox/office/spreadsheet/view/chartcreator',
 
                 if (!chart.chartStyleId) {
                     chart.chartStyleId = 2;
-                    generator.generateOperation(Operations.SET_DRAWING_ATTRIBUTES, {start: start, attrs:{chart:{chartStyleId:2},fill:{color:{type:'scheme',value:'light1',fallbackValue:'#ffffff'}}}});
+                    generator.generateDrawingOperation(Operations.SET_DRAWING_ATTRIBUTES, position, { attrs: { chart: { chartStyleId: 2 }, fill: { color: { type: 'scheme', value: 'light1', fallbackValue: 'FFFFFF' } } } });
                 }
 
-                generator.generateOperation(Operations.SET_CHART_AXIS_ATTRIBUTES, {axis: 'x', start: start, attrs: defaults.xAxis});
-                generator.generateOperation(Operations.SET_CHART_AXIS_ATTRIBUTES, {axis: 'y', start: start, attrs: defaults.yAxis});
-                generator.generateOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, {axis: 'x', start: start, attrs: defaults.xGrid});
-                generator.generateOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, {axis: 'y', start: start, attrs: defaults.yGrid});
+                generator.generateDrawingOperation(Operations.SET_CHART_AXIS_ATTRIBUTES, position, { axis: 'x', attrs: defaults.xAxis });
+                generator.generateDrawingOperation(Operations.SET_CHART_AXIS_ATTRIBUTES, position, { axis: 'y', attrs: defaults.yAxis });
+                generator.generateDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, position, { axis: 'x', attrs: defaults.xGrid });
+                generator.generateDrawingOperation(Operations.SET_CHART_GRIDLINE_ATTRIBUTES, position, { axis: 'y', attrs: defaults.yGrid });
             });
             def.resolve();
         });
@@ -449,9 +459,11 @@ define('io.ox/office/spreadsheet/view/chartcreator',
      * @param chartModel is used for the position and id in the document and as fallback for chart-type-checks
      * @param axis series direction
      * @param chartAttrs is used for changing other attributes of the chart and for the type info (bubble or not)
+     * @param forceColLabel is set, it overwrites the internal logic for detecting first column as label
+     * @param forceRowLabel is set, it overwrites the internal logic for detecting first column as label (headline)
      */
-    ChartCreator.generateOperations = function (app, range, sheet, chartModel, axis, chartAttrs) {
-        var pos = chartModel.getPosition();
+    ChartCreator.generateOperations = function (app, sourceRange, sourceSheet, chartModel, axis, chartAttrs, forceColLabel, forceRowLabel) {
+
         var chart;
         if (chartAttrs) {
             chart = chartAttrs;
@@ -465,25 +477,23 @@ define('io.ox/office/spreadsheet/view/chartcreator',
             };
         }
 
-
-        getContentForRange(app, range, sheet, null, function (cells, overwrittenAxis) {
+        getContentForRange(app, sourceRange, sourceSheet, null, function (cells, overwrittenAxis) {
 
             //only overwritten if source range height is 1 oder width is 1
             if (overwrittenAxis === 0 || overwrittenAxis === 1) {
                 axis = overwrittenAxis;
             }
-            var model = app.getModel();
-            var generator = model.createOperationsGenerator();
+            var docModel = app.getModel();
+            var generator = chartModel.getSheetModel().createOperationsGenerator();
+            var position = chartModel.getSheetPosition();
 
-            var count = chartModel.getSeriesCount();
-            _.times(count, function () {
-                generator.generateOperation(Operations.DELETE_CHART_DATASERIES, {start: pos, series: 0});
+            _.times(chartModel.getSeriesCount(), function () {
+                generator.generateDrawingOperation(Operations.DELETE_CHART_DATASERIES, position, { series: 0 });
             });
 
-
-            generateSeriesOperations(app, range, sheet, generator, pos, cells, axis, chart);
-            model.applyOperations(generator.getOperations());
-        });
+            generateSeriesOperations(app, sourceRange, sourceSheet, generator, position, cells, axis, chart);
+            docModel.applyOperations(generator.getOperations());
+        }, forceColLabel, forceRowLabel);
 
     };
 
