@@ -1,21 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/view/cornerpane',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/object/triggerobject',
-     'io.ox/office/spreadsheet/utils/sheetutils'
-    ], function (Utils, TriggerObject, SheetUtils) {
+define('io.ox/office/spreadsheet/view/cornerpane', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/spreadsheet/utils/sheetutils'
+], function (Utils, TriggerObject, TimerMixin, SheetUtils) {
 
     'use strict';
 
@@ -41,6 +42,7 @@ define('io.ox/office/spreadsheet/view/cornerpane',
      * @constructor
      *
      * @extends TriggerObject
+     * @extends TimerMixin
      *
      * @param {SpreadsheetApplication} app
      *  The application that contains this corner pane.
@@ -50,8 +52,8 @@ define('io.ox/office/spreadsheet/view/cornerpane',
         var // self reference
             self = this,
 
-            // the spreadsheet model and view
-            view = null,
+            // the spreadsheet view
+            docView = null,
 
             // the container node of this corner pane
             rootNode = $('<div class="corner-pane">'),
@@ -59,9 +61,10 @@ define('io.ox/office/spreadsheet/view/cornerpane',
             // the cell DOM node used to calculate the size
             cellNode = $('<div class="cell noI18n">');
 
-        // base constructor ---------------------------------------------------
+        // base constructors --------------------------------------------------
 
         TriggerObject.call(this);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
@@ -75,7 +78,7 @@ define('io.ox/office/spreadsheet/view/cornerpane',
             }
 
             // return focus to active grid pane, after the event has been processed
-            _.defer(function () { view.grabFocus(); });
+            self.executeDelayed(function () { docView.grabFocus(); });
         }
 
         // methods ------------------------------------------------------------
@@ -107,12 +110,12 @@ define('io.ox/office/spreadsheet/view/cornerpane',
             // prepare the cell node text and font size
             cellNode
                 .text(SheetUtils.getRowName(maxRow))
-                .css({ fontSize: view.getHeaderFontSize() + 'pt', lineHeight: 'normal' })
+                .css({ fontSize: docView.getHeaderFontSize() + 'pt', lineHeight: 'normal' })
                 .appendTo(rootNode);
 
             // copy size of cell node to root node, and remove the cell node
-            nodeSize = Utils.getExactNodeSize(cellNode);
-            rootNode.css({ width: Math.ceil(nodeSize.width) + 2, height: Math.ceil(nodeSize.height) + 2 });
+            nodeSize = Utils.getCeilNodeSize(cellNode);
+            rootNode.css({ width: nodeSize.width + 2, height: nodeSize.height + 2 });
             cellNode.remove();
 
             return this;
@@ -123,11 +126,16 @@ define('io.ox/office/spreadsheet/view/cornerpane',
         // initialize class members
         app.onInit(function () {
             // the spreadsheet view
-            view = app.getView();
+            docView = app.getView();
             // immediately react on mousedown (do not wait for click)
             rootNode.on('mousedown touchstart', clickHandler);
         });
 
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            app = self = docView = rootNode = cellNode = null;
+        });
+
     } // class CornerPane
 
     // exports ================================================================
