@@ -1,27 +1,28 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/view/headerpane',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/object/triggerobject',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/utils/paneutils',
-     'io.ox/office/spreadsheet/view/controls',
-     'io.ox/office/spreadsheet/view/render/renderutils',
-     'io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
-     'gettext!io.ox/office/spreadsheet'
-    ], function (Utils, Forms, TriggerObject, SheetUtils, PaneUtils, Controls, RenderUtils, HeaderTrackingMixin, gt) {
+define('io.ox/office/spreadsheet/view/headerpane', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/utils/paneutils',
+    'io.ox/office/spreadsheet/view/popup/headercontextmenu',
+    'io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
+    'io.ox/office/spreadsheet/view/render/renderutils',
+    'gettext!io.ox/office/spreadsheet'
+], function (Utils, Forms, TriggerObject, TimerMixin, SheetUtils, PaneUtils, HeaderContextMenu, HeaderTrackingMixin, RenderUtils, gt) {
 
     'use strict';
 
@@ -40,100 +41,6 @@ define('io.ox/office/spreadsheet/view/headerpane',
         ROW_RESIZER_VISIBLE = gt('Drag to change row height'),
         ROW_RESIZER_HIDDEN = gt('Drag to show hidden row');
 
-    // class HeaderContextMenu ================================================
-
-    /**
-     * A context menu for header panes. Provides menu actions to manipulate
-     * entire columns/rows in the active sheet.
-     *
-     * @constructor
-     *
-     * @extends Controls.ContextMenu
-     */
-    var HeaderContextMenu = Controls.ContextMenu.extend({ constructor: function (app, headerPane, columns) {
-
-        var // self reference
-            self = this,
-
-            // the document model and view
-            model = app.getModel(),
-            view = app.getView();
-
-        // base constructor ---------------------------------------------------
-
-        Controls.ContextMenu.call(this, app, headerPane.getNode(), {
-            prepareShowHandler: prepareShowContextMenu,
-            mouseDown: true, // open context menu on 'mousedown' instead of 'mouseup'
-            enableKey: 'sheet/unlocked' // do not show context menu at all in locked sheets
-        });
-
-        // private methods ----------------------------------------------------
-
-        /**
-         * Inserts column-specific menu entries into the context menu.
-         */
-        function initColumnMenu() {
-            self.addGroup('column/insert', new Controls.Button({ label: gt('Insert columns') }))
-                .addGroup('column/delete', new Controls.Button({ label: gt('Delete columns') }))
-                .addSeparator()
-                .addGroup('column/width/optimal', new Controls.Button({ label: gt('Set optimal column width') }))
-                .addGroup('column/hide', new Controls.Button({ label: gt('Hide columns') }))
-                .addGroup('column/show', new Controls.Button({ label: gt('Show hidden columns') }));
-        }
-
-        /**
-         * Inserts row-specific menu entries into the context menu.
-         */
-        function initRowMenu() {
-            self.addGroup('row/insert', new Controls.Button({ label: gt('Insert rows') }))
-                .addGroup('row/delete', new Controls.Button({ label: gt('Delete rows') }))
-                .addSeparator()
-                .addGroup('row/height/optimal', new Controls.Button({ label: gt('Set optimal row height') }))
-                .addGroup('row/hide', new Controls.Button({ label: gt('Hide rows') }))
-                .addGroup('row/show', new Controls.Button({ label: gt('Show hidden rows') }));
-        }
-
-        /**
-         * Selects the column/row that has been clicked with the right mouse
-         * button, before showing the context menu.
-         *
-         * @param {jQuery.Event} event
-         *  The browser event object of the 'mousedown' event.
-         */
-        function prepareShowContextMenu(event) {
-
-            var // the index of the column/row to be selected
-                index = Utils.getElementAttributeAsInteger($(event.target).closest('.cell'), 'data-index', -1);
-
-            // functor to check whether the column/row index is contained in a range address
-            function rangeContainsIndex(range) {
-                return model.isFullRange(range, columns) && SheetUtils.rangeContainsIndex(range, index, columns);
-            }
-
-            // Do nothing if no cell node has been clicked, or while in cell edit mode,
-            // or when the event is of type 'contextmenu' (otherwise, the context menu
-            // will be reopened after moving the mouse with pressed right mouse button).
-            if ((index < 0) || view.isCellEditMode() || (event.type === 'contextmenu')) { return Utils.BREAK; }
-
-            // select the clicked column/row if it is not contained in the selection
-            if (!_.any(view.getSelectedRanges(), rangeContainsIndex)) {
-                headerPane.trigger('select:start', index, 'select');
-                headerPane.trigger('select:end', index, true);
-            }
-        }
-
-        // initialization -----------------------------------------------------
-
-        // lazy initialization of the  menu items according to the orientation of the header pane
-        this.one('popup:beforeshow', columns ? initColumnMenu : initRowMenu);
-
-        // destroy all class members
-        this.registerDestructor(function () {
-            app = headerPane = self = model = view = null;
-        });
-
-    }}); // class HeaderContextMenu
-
     // class HeaderPane =======================================================
 
     /**
@@ -143,12 +50,6 @@ define('io.ox/office/spreadsheet/view/headerpane',
      * header panes, and top and bottom row header panes).
      *
      * Triggers the following events:
-     * - 'change:scrollsize'
-     *      After the size of the scrollable area in this header pane has been
-     *      changed. Event handlers receive the new scroll area size.
-     * - 'change:scrollpos'
-     *      After the absolute scroll position of this header pane has been
-     *      changed. Event handlers receive the new scroll position.
      * - 'change:interval'
      *      After the column/row interval shown by this header pane has been
      *      changed. Event handlers receive the following parameters:
@@ -180,8 +81,15 @@ define('io.ox/office/spreadsheet/view/headerpane',
      *              contents while scrolling through the sheet).
      * - 'hide:interval'
      *      After this header pane has been hidden, and the old column/row
-     *      interval has been reset to nothing. Listeners will reset or update
-     *      additional settings according to the hidden header pane.
+     *      interval has been reset to nothing. Event listeners are expected to
+     *      reset or update additional settings according to the hidden header
+     *      pane.
+     * - 'change:scrollsize'
+     *      After the size of the scrollable area in this header pane has been
+     *      changed. Event handlers receive the new scroll area size.
+     * - 'change:scrollpos'
+     *      After the absolute scroll position of this header pane has been
+     *      changed. Event handlers receive the new scroll position.
      * - 'select:start'
      *      After selection tracking has been started. Event handlers receive
      *      the following parameters:
@@ -225,6 +133,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
      * @constructor
      *
      * @extends TriggerObject
+     * @extends TimerMixin
      *
      * @param {SpreadsheetApplication} app
      *  The application that contains this header pane.
@@ -239,8 +148,8 @@ define('io.ox/office/spreadsheet/view/headerpane',
             self = this,
 
             // the spreadsheet model and view
-            model = null,
-            view = null,
+            docModel = null,
+            docView = null,
 
             // whether this header pane contains columns
             columns = PaneUtils.isColumnSide(paneSide),
@@ -287,6 +196,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
         // base constructor ---------------------------------------------------
 
         TriggerObject.call(this);
+        TimerMixin.call(this);
         HeaderTrackingMixin.call(this, app, paneSide, contentNode);
 
         // private methods ----------------------------------------------------
@@ -298,13 +208,13 @@ define('io.ox/office/spreadsheet/view/headerpane',
         function updateFocusDisplay() {
 
             var // which header pane is currently focused (while selection tracking is active)
-                focusPaneSide = view.getSheetViewAttribute('activePaneSide'),
+                focusPaneSide = docView.getSheetViewAttribute('activePaneSide'),
                 // which grid pane is currently focused
-                focusPanePos = view.getSheetViewAttribute('activePane'),
+                focusPanePos = docView.getSheetViewAttribute('activePane'),
                 // whether this grid pane appears focused
                 focused = false;
 
-            if (view.hasFrozenSplit()) {
+            if (docView.hasFrozenSplit()) {
                 // always highlight all panes in frozen split mode
                 focused = true;
             } else if (_.isString(focusPaneSide)) {
@@ -343,7 +253,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
         function renderSelection() {
 
             var // the selected ranges
-                ranges = view.getSelectedRanges(),
+                ranges = docView.getSelectedRanges(),
                 // the merged ranges of selected columns/rows
                 selectionIntervals = SheetUtils.getIntervals(ranges, columns),
                 // current index into the intervals array
@@ -366,7 +276,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
 
             // on touch devices, update the resize handle for entire columns/rows
             if (Modernizr.touch) {
-                if ((ranges.length === 1) && model.isFullRange(ranges[0], columns)) {
+                if ((ranges.length === 1) && docModel.isFullRange(ranges[0], columns)) {
                     contentNode.find('>.touchresizer').show().attr('data-index', ranges[0].end[columns ? 0 : 1]);
                     updateTouchResizerPosition();
                 } else {
@@ -437,7 +347,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
             }
 
             // set font size and line height (column headers only, at cell level for row headers) at content node
-            contentNode.css('font-size', view.getHeaderFontSize() + 'pt');
+            contentNode.css('font-size', docView.getHeaderFontSize() + 'pt');
             if (columns) { contentNode.css('line-height', (rootNode.height() - 1) + 'px' ); }
 
             // generate HTML mark-up text for all header cells
@@ -510,7 +420,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
             var // the column/row collection
                 collection = self.getColRowCollection(),
                 // number of used columns/rows in the sheet
-                usedCount = view.getCellCollection().getUsedCount(columns),
+                usedCount = docView.getCellCollection().getUsedCount(columns),
                 // the total size of the used area in the sheet, in pixels
                 usedSize = Math.max(0, collection.getEntry(usedCount).offset - hiddenSize),
                 // the absolute size of the scrollable area
@@ -556,6 +466,39 @@ define('io.ox/office/spreadsheet/view/headerpane',
             self.trigger('change:scrollpos', scrollPos);
         }
 
+        /**
+         * Returns the complete column/row interval that can be displayed in
+         * this header pane. If the active sheet does not contain a frozen
+         * column/row split, this interval will contain all columns/rows of the
+         * sheet, otherwise the interval will contain the available columns or
+         * rows according to the position of this header pane, and the current
+         * split settings.
+         *
+         * @returns {Object}
+         *  The complete column/row interval available in this header pane, in
+         *  the zero-based index properties 'first' and 'last'.
+         */
+        function getAvailableInterval() {
+
+            var // the model of the active sheet
+                sheetModel = docView.getSheetModel(),
+                // the frozen column/row interval
+                frozenInterval = null;
+
+            // get frozen interval from sheet model
+            if (sheetModel.hasFrozenSplit()) {
+                frozenInterval = columns ? sheetModel.getSplitColInterval() : sheetModel.getSplitRowInterval();
+            }
+
+            // return entire column/row interval of the sheet in unfrozen views
+            if (!frozenInterval) {
+                return self.getColRowCollection().getFullInterval();
+            }
+
+            // build the available interval according to the own position
+            return leading ? frozenInterval : { first: frozenInterval.last + 1, last: docModel.getMaxIndex(columns) };
+        }
+
         /**
          * Recalculates the current row/column interval and its position and
          * size in the sheet, and renders all visible header cells and the
@@ -576,7 +519,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
             var // the column/row collection
                 collection = self.getColRowCollection(),
                 // the current scroll anchor from the sheet attributes
-                scrollAnchor = view.getSheetViewAttribute(anchorName),
+                scrollAnchor = docView.getSheetViewAttribute(anchorName),
                 // the current effective scroll position
                 newScrollPos = getValidScrollPos(collection.convertScrollAnchorToPixel(scrollAnchor) - hiddenSize),
                 // the absolute position of the first visible pixel
@@ -595,7 +538,9 @@ define('io.ox/office/spreadsheet/view/headerpane',
                 ((intervalEndOffset < collection.getTotalSize()) && (intervalEndOffset - visibleEndOffset < paneSize * ADDITIONAL_SIZE_RATIO / 3))
             ) {
 
-                var // the old (current) column/row interval
+                var // the available interval
+                    availableInterval = getAvailableInterval(),
+                    // the old (current) column/row interval
                     oldInterval = interval,
                     // the data of the new first column/row
                     firstEntryData = collection.getEntryByOffset(visibleOffset - paneSize * ADDITIONAL_SIZE_RATIO, { pixel: true }),
@@ -608,10 +553,17 @@ define('io.ox/office/spreadsheet/view/headerpane',
 
                 // calculate the new visible column/row interval
                 interval = {
-                    first: (firstEntryData.offset === 0) ? 0 : firstEntryData.index,
-                    last: (lastEntryData.offset + lastEntryData.size >= collection.getTotalSize()) ? collection.getMaxIndex() : lastEntryData.index
+                    first: Math.max(availableInterval.first, (firstEntryData.offset === 0) ? 0 : firstEntryData.index),
+                    last: Math.min(availableInterval.last, (lastEntryData.offset + lastEntryData.size >= collection.getTotalSize()) ? collection.getMaxIndex() : lastEntryData.index)
                 };
 
+                // sanity check
+                if (interval.last < interval.first) {
+                    Utils.warn('HeaderPane.updateAndRenderInterval(): invalid interval: side=' + paneSide + ' interval=' + SheetUtils.getIntervalName(interval, columns));
+                    interval = null;
+                    return;
+                }
+
                 // start/end position of the interval (add all leading/trailing hidden columns/rows in the sheet)
                 position = { offset: firstEntryData.offset, size: lastEntryData.offset + lastEntryData.size - firstEntryData.offset };
 
@@ -628,7 +580,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
 
                         // update bounding interval in the rendering cache of the active sheet
                         // (use pane side position as identifier of the interval in the cache)
-                        view.getRenderCache().updateBoundingInterval(paneSide, columns, _.clone(interval), refresh);
+                        docView.getRenderCache().updateBoundingInterval(paneSide, columns, _.clone(interval), refresh);
 
                         // notify listeners
                         self.trigger('change:interval', _.clone(interval), refresh, options);
@@ -654,7 +606,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
 
             // clear bounding interval in the rendering cache of the active sheet
             // (use pane side position as identifier of the interval in the cache)
-            view.getRenderCache().updateBoundingInterval(paneSide, columns, null);
+            docView.getRenderCache().updateBoundingInterval(paneSide, columns, null);
 
             // notify listeners
             self.trigger('hide:interval');
@@ -684,8 +636,8 @@ define('io.ox/office/spreadsheet/view/headerpane',
             }
 
             // zoom or scroll position: update visible column/row interval
-            if ('zoom' in attributes) {
-                // zooming: refresh cell positions and sizes
+            if (Utils.hasProperty(attributes, /^(zoom$|split)/)) {
+                // zooming and splitting: refresh cell positions and sizes
                 updateAndRenderInterval({ geometry: true });
             } else if (anchorName in attributes) {
                 // scrolling: do not debounce updating the view (otherwise, cells are jumpy)
@@ -731,7 +683,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
          *  header pane; otherwise the row collection of the active sheet.
          */
         this.getColRowCollection = function () {
-            return columns ? view.getColCollection() : view.getRowCollection();
+            return columns ? docView.getColCollection() : docView.getRowCollection();
         };
 
         /**
@@ -776,8 +728,8 @@ define('io.ox/office/spreadsheet/view/headerpane',
 
                 // position and size
                 rootNode.css(columns ?
-                    { left: settings.offset, width: paneSize, height: view.getHeaderHeight() } :
-                    { top: settings.offset, width: view.getHeaderWidth(), height: paneSize }
+                    { left: settings.offset, width: paneSize, height: docView.getHeaderHeight() } :
+                    { top: settings.offset, width: docView.getHeaderWidth(), height: paneSize }
                 );
 
                 // recalculate the interval and header cells
@@ -786,7 +738,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
                 // recalculate the scroll size (must be done deferred after the grid
                 // panes have been initialized, otherwise scroll position in trailing
                 // panes may be wrong when enabling a split view)
-                _.defer(function () {
+                this.executeDelayed(function () {
                     updateScrollAreaSize(scrollPos);
                     updateScrollPosition(scrollPos);
                 });
@@ -831,27 +783,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
          *  this header pane is currently hidden.
          */
         this.getAvailableInterval = function () {
-
-            // always return null for invisible header panes
-            if (!this.isVisible()) { return null; }
-
-            var // the model of the active sheet
-                sheetModel = view.getSheetModel(),
-                // the frozen column/row interval
-                frozenInterval = null;
-
-            // get frozen interval from sheet model
-            if (sheetModel.hasFrozenSplit()) {
-                frozenInterval = columns ? sheetModel.getSplitColInterval() : sheetModel.getSplitRowInterval();
-            }
-
-            // return entire column/row interval of the sheet in unfrozen views
-            if (!frozenInterval) {
-                return { first: 0, last: model.getMaxIndex(columns) };
-            }
-
-            // build the available interval according to the own position
-            return leading ? frozenInterval : { first: frozenInterval.last + 1, last: model.getMaxIndex(columns) };
+            return this.isVisible() ? getAvailableInterval() : null;
         };
 
         /**
@@ -925,7 +857,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
             var // the column/row collection of the active sheet
                 collection = this.getColRowCollection(),
                 // the current scroll anchor
-                scrollAnchor = view.getSheetViewAttribute(anchorName),
+                scrollAnchor = docView.getSheetViewAttribute(anchorName),
                 // the layout data of the next column/row
                 nextEntry = null;
 
@@ -952,7 +884,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
          * @returns {HeaderPane}
          *  A reference to this instance.
          */
-        this.scrollTo = app.createDebouncedMethod(function (pos) {
+        this.scrollTo = this.createDebouncedMethod(function (pos) {
 
             var // the column/row collection of the active sheet
                 collection = this.getColRowCollection(),
@@ -969,11 +901,11 @@ define('io.ox/office/spreadsheet/view/headerpane',
 
             // update scroll anchor view properties, this will trigger change
             // handler from the view that repositions the content node
-            view.setSheetViewAttribute(anchorName, collection.getScrollAnchorByOffset(newScrollPos + hiddenSize, { pixel: true }));
+            docView.setSheetViewAttribute(anchorName, collection.getScrollAnchorByOffset(newScrollPos + hiddenSize, { pixel: true }));
 
             return this;
 
-        }, function () { updateScrollAreaSize(scrollPos); }, { context: this, delay: 500 });
+        }, function () { updateScrollAreaSize(scrollPos); }, { delay: 500 });
 
         /**
          * Changes the scroll position of this header pane relative to the
@@ -1072,21 +1004,21 @@ define('io.ox/office/spreadsheet/view/headerpane',
         app.onInit(function () {
 
             // the spreadsheet model and view
-            model = app.getModel();
-            view = app.getView();
+            docModel = app.getModel();
+            docView = app.getView();
 
             // create the context menu
             contextMenu = new HeaderContextMenu(app, self, columns);
 
             // hide the interval initially when the sheet changes
-            view.on('change:activesheet', hideInterval);
+            docView.on('change:activesheet', hideInterval);
 
             // update scroll area size according to the size of the used area in the sheet
-            view.on('change:usedarea', function () { updateScrollAreaSize(scrollPos); });
+            docView.on('change:usedarea', function () { updateScrollAreaSize(scrollPos); });
 
             // listen to view events (only if this pane is visible)
-            registerEventHandler(view, 'change:sheet:viewattributes', changeSheetViewAttributesHandler);
-            registerEventHandler(view, 'cache:operation', renderCacheOperationHandler);
+            registerEventHandler(docView, 'change:sheet:viewattributes', changeSheetViewAttributesHandler);
+            registerEventHandler(docView, 'cache:operation', renderCacheOperationHandler);
 
             // update the position of the touch resizer node while tracking
             if (Modernizr.touch) {
@@ -1100,7 +1032,7 @@ define('io.ox/office/spreadsheet/view/headerpane',
         // destroy all class members on destruction
         this.registerDestructor(function () {
             contextMenu.destroy();
-            app = self = model = view = contextMenu = null;
+            app = self = docModel = docView = contextMenu = null;
         });
 
     } // class HeaderPane
