@@ -1,30 +1,31 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/popup/tooltip',
-     'io.ox/office/tk/popup/listmenu',
-     'io.ox/office/editframework/utils/color',
-     'io.ox/office/editframework/view/hyperlinkutil',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/utils/paneutils',
-     'io.ox/office/spreadsheet/model/cellcollection',
-     'io.ox/office/spreadsheet/model/formula/tokenarray',
-     'gettext!io.ox/office/spreadsheet'
-    ], function (Utils, KeyCodes, Forms, ToolTip, ListMenu, Color, HyperlinkUtil, SheetUtils, PaneUtils, CellCollection, TokenArray, gt) {
+define('io.ox/office/spreadsheet/view/mixin/celleditmixin', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/popup/tooltip',
+    'io.ox/office/tk/popup/listmenu',
+    'io.ox/office/editframework/utils/color',
+    'io.ox/office/editframework/utils/hyperlinkutils',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/utils/paneutils',
+    'io.ox/office/spreadsheet/model/cellcollection',
+    'io.ox/office/spreadsheet/model/formula/tokenarray',
+    'io.ox/office/spreadsheet/view/popup/signaturetooltip',
+    'gettext!io.ox/office/spreadsheet'
+], function (Utils, KeyCodes, Forms, ToolTip, ListMenu, Color, HyperlinkUtils, SheetUtils, PaneUtils, CellCollection, TokenArray, SignatureToolTip, gt) {
 
     'use strict';
 
@@ -46,7 +47,7 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             self = this,
 
             // the spreadsheet model, and other model objects
-            model = null,
+            docModel = null,
             undoManager = null,
             documentStyles = null,
             fontCollection = null,
@@ -59,12 +60,12 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             // the container element for the text underlay node and the text area control
             textAreaContainer = $('<div>', { 'data-focus-role': 'textarea' }).addClass('textarea-container').append(textUnderlayNode, textAreaNode),
 
-            // tooltip attached to the text area (for function/parameter help)
-            textAreaToolTip = new ToolTip({ anchor: textAreaNode, classes: 'textarea-tooltip' }),
+            // tooltip attached to the text area for function/parameter help
+            signatureToolTip = new SignatureToolTip(app, textAreaNode),
             // drop-down list attached to the text area (for function names)
             textAreaListMenu = new ListMenu({ anchor: textAreaNode, autoFocus: false, focusableNodes: textAreaNode }),
             // tooltip attached to the text area drop-down list (for the selected function)
-            textAreaListItemToolTip = new ToolTip({ anchor: getTextAreaListButtonNode, border: textAreaListMenu.getNode(), position: 'right left', align: 'center' }),
+            textAreaListItemToolTip = new ToolTip({ anchor: textAreaListMenu.getNode(), anchorBorder: 'right left', anchorAlign: 'center', anchorAlignBox: getTextAreaListButtonNode }),
 
             // whether the in-place cell edit mode is active
             editActive = false,
@@ -99,8 +100,8 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             formulaString = null,
             // selection offset into the formula string (without leading equality sign)
             formulaOffset = 0,
-            // whether an array formula is currently edited
-            arrayFormula = false,
+            // whether a matrix formula is currently edited
+            matrixFormula = false,
             // whether to show auto-completion list for a function or name
             autoComplete = false,
             // token info for function auto-completion
@@ -274,6 +275,13 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
                 // get position and size of the visible area in the grid pane
                 visiblePosition = editGridPane.getVisibleRectangle();
 
+                if (Utils.CHROME_ON_ANDROID) {
+                    // on small android devices softkeyboard can overlay complete gridpane,
+                    // then the automatic height calculation has zero height,
+                    // after closing the softkeyboard it still keeps this height and that looks really wrong
+                    visiblePosition.height = Math.max(visiblePosition.height, charAttributes.lineHeight);
+                }
+
                 // calculate and set the new size of the text area
                 if (updateSize) {
 
@@ -337,115 +345,9 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             }
 
             // return the debounced method updateTextAreaPosition()
-            return app.createDebouncedMethod(directCallback, deferredCallback, { delay: 250, maxDelay: 500 });
+            return self.createDebouncedMethod(directCallback, deferredCallback, { delay: 250, maxDelay: 500 });
         }());
 
-        /**
-         * Updates and shows or hides the tooltip attached to the text area,
-         * containing the name, parameter list, and description of the current
-         * function.
-         */
-        function updateFunctionSignatureToolTip() {
-
-            var // the cursor position in the text area (exclude the equality sign)
-                textPos = textAreaNode[0].selectionStart - formulaOffset,
-                // function data for the cursor position
-                functionInfo = editTokenArray.getFunctionAtPosition(textPos),
-                // the help descriptor of the function, containing the description text
-                functionHelp = functionInfo ? app.getFunctionHelp(functionInfo.name) : null,
-                // current list separator
-                SEP = app.getListSeparator();
-
-            // do nothing if cursor is not inside a function, or no help is available for the function
-            if (!functionHelp) {
-                textAreaToolTip.hide();
-                return;
-            }
-
-            // function name node to be inserted to the popup
-            var functionNameNode = $('<span>')
-                .addClass('function-name')
-                .toggleClass('param-active', functionInfo.paramIndex < 0)
-                .text(functionInfo.name.toUpperCase());
-
-            var varargIndex = functionHelp.varargs,
-                optargIndex = functionHelp.optargs,
-                isVararg = _.isNumber(varargIndex),
-                isOptarg = _.isNumber(optargIndex),
-                paramsArray = _.clone(functionHelp.params),
-                isPair = isVararg && _.isString(paramsArray[varargIndex + 1]),
-                varargArray = [],
-                paramsNode = $('<span>'),
-                bracketStartIndex = 0;
-
-            // prepare parameters array plus generate custom parameters if we have an infinite parameter
-            if (isVararg) {
-                // handle special case with infinite parameter pairs like parameters from the function 'SUMIFS'
-                var varargName = paramsArray[varargIndex].trim(),
-                    secondVarargName = isPair ? paramsArray[varargIndex + 1].trim() : '',
-                    varargParamCount = 1,
-                    sliceCount = isPair ? 2 : 1;
-                // calculate number of variable parameters to generate
-                if (functionInfo.paramCount > paramsArray.length) {
-                    varargParamCount = isPair ? (functionInfo.paramCount - varargIndex) / 2 : functionInfo.paramCount - varargIndex;
-                }
-                // generate variable parameters
-                var i = 0;
-                while (i <= varargParamCount) {
-                    varargArray.push(varargName + (i + 1));
-                    if (isPair) varargArray.push(secondVarargName + (i + 1));
-                    i++;
-                }
-                // join initial parameter array with generated variable parameters array
-                paramsArray = paramsArray.slice(0, paramsArray.length - sliceCount).concat(varargArray);
-                bracketStartIndex = isPair ? varargIndex + 1 : varargIndex;
-            }
-
-            if (isOptarg) { bracketStartIndex = optargIndex - 1; }
-
-            // build mark-up and highlight active parameter (draw brackets, add highlight class, add delimiters)
-            _.each(paramsArray, function (elem, index, list) {
-                var elemMarkup = elem,
-                    paramNode = $('<span>');
-                // add brackets where necessary
-                if (index > bracketStartIndex) {
-                    if (isPair) {
-                        elemMarkup = (index % 2) ? ('[' + elem) : (elem + ']');
-                    } else {
-                        elemMarkup = (isVararg || isOptarg) ? ('[' + elem + ']') : elem;
-                    }
-                }
-                paramNode.html(elemMarkup);
-                if (index === functionInfo.paramIndex) paramNode.addClass('param-active');
-                paramsNode.append(paramNode);
-                if (index !== (list.length - 1)) paramsNode.append(SEP + ' ');
-            });
-
-            // add parentheses around the parameters and ellipsis in the end if we have infinite parameters
-            paramsNode.prepend('( ');
-            if (isVararg) paramsNode.append(SEP + ' \u2026');
-            paramsNode.append(' )');
-            textAreaToolTip.clearContents().appendContentNodes(functionNameNode, paramsNode);
-
-            // check if we have parameter help data, get it and build its mark-up
-            if (_.isArray(functionHelp.paramshelp)) {
-                // show right help in case of variable parameters
-                var paramHelpIndex = 0;
-                if (isVararg && (functionInfo.paramIndex > varargIndex)) {
-                    paramHelpIndex = varargIndex + (isPair ? ((functionInfo.paramIndex - varargIndex) % 2) : 0);
-                } else {
-                    paramHelpIndex = functionInfo.paramIndex;
-                }
-                var activeParamHelp = functionHelp.paramshelp[paramHelpIndex] || '';
-                if (activeParamHelp.length > 0) {
-                    textAreaToolTip.appendContentNodes($('<div>').addClass('param-help').text(activeParamHelp));
-                }
-            }
-
-            // and finally, show the tooltip
-            textAreaToolTip.show();
-        }
-
         /**
          * Returns the selected list item node in the drop-down menu element
          * for the text area.
@@ -484,7 +386,7 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
          * auto-completion pop-up menu.
          */
         function hideTextAreaPopups() {
-            textAreaToolTip.hide();
+            signatureToolTip.hide();
             textAreaListMenu.hide();
         }
 
@@ -499,7 +401,9 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
                 // token info for function auto-completion
                 tokenInfo = null,
                 // the text to be used for function auto-completion
-                autoText = null;
+                autoText = null,
+                // function data for the cursor position
+                functionInfo = null;
 
             // show pop-ups only in formula mode, do not show them while dragging
             // highlight ranges, or selecting new ranges for the formula
@@ -548,16 +452,18 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
 
                 // show the menu, if it contains at least one entry
                 if (textAreaListMenu.getItemNodes().length > 0) {
-                    textAreaToolTip.hide();
+                    signatureToolTip.hide();
                     textAreaListMenu.show().selectItem(0, { scroll: true });
                     updateFunctionDescriptionToolTip();
                     return;
                 }
             }
 
-            // try to show the signature (name and parameters) of the current function
-            textAreaListMenu.hide();
-            updateFunctionSignatureToolTip();
+            // try to show a tooltip for the current function in a formula
+            hideTextAreaPopups();
+            if ((functionInfo = editTokenArray.getFunctionAtPosition(textPos))) {
+                signatureToolTip.updateSignature(functionInfo.name, functionInfo.paramIndex);
+            }
         }
 
         /**
@@ -573,7 +479,7 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             autoCloseData = [];
 
             // defer execution to let the browser process pending key events
-            _.defer(function () {
+            self.executeDelayed(function () {
                 // fail-safe check that cell edit mode is still active
                 if (editActive) { updateTextAreaPopups(); }
             });
@@ -662,7 +568,7 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             }
 
             // return the debounced method updateEditTokenArray()
-            return app.createDebouncedMethod(directCallback, deferredCallback, { delay: 100, maxDelay: 300 });
+            return self.createDebouncedMethod(directCallback, deferredCallback, { delay: 100, maxDelay: 300 });
         }());
 
         /**
@@ -884,9 +790,9 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
                 return false;
             }
 
-            // ENTER with CTRL or META key: fill value to all cells (no SHIFT), or insert array formula (SHIFT)
+            // ENTER with CTRL or META key: fill value to all cells (no SHIFT), or insert matrix formula (SHIFT)
             if (KeyCodes.matchKeyCode(event, 'ENTER', { shift: null, ctrlOrMeta: true })) {
-                self.leaveCellEditMode(event.shiftKey ? 'array' : 'fill', { validate: true });
+                self.leaveCellEditMode(event.shiftKey ? 'matrix' : 'fill', { validate: true });
                 // do not let key event bubble up (no change of selection)
                 return false;
             }
@@ -988,7 +894,7 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
 
             // Mobile devices may add auto-correction text which modifies the selection after a text change
             // event has been processed. Recalculate the autoComplete flag after the browser has set the final selection.
-            _.defer(function () {
+            self.executeDelayed(function () {
                 var textSelection = Forms.getInputSelection(textAreaNode);
                 autoComplete = textSelection.start === textSelection.end;
             });
@@ -1003,7 +909,7 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             if (typeAhead && !_.isString(formulaString)) { //if user writes(adds) new letter -> auto suggest new text
 
                 var address = self.getActiveCell(),
-                    boundRange = { start: [address[0], Math.max(0, address[1] - 256)], end: [address[0], Math.min(model.getMaxRow(), address[1] + 256)] };
+                    boundRange = { start: [address[0], Math.max(0, address[1] - 256)], end: [address[0], Math.min(docModel.getMaxRow(), address[1] + 256)] };
 
                 self.getCellCollection().iterateCellsInLine(address, 'up down', function (cellData) {
                     if (CellCollection.isText(cellData) && (cellData.result.length > 0)) {
@@ -1106,12 +1012,12 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
                 // text area lost focus: hide pop-up nodes (unless the auto-complete menu
                 // has been clicked, in this case, move focus back to text area immediately)
                 case 'blur':
-                    _.defer(function () {
+                    self.executeDelayed(function () {
                         if (editActive && textAreaListMenu.hasFocus()) {
                             ignoreFocusEvent = true;
                             textAreaNode.focus();
                             // IE sends focus event deferred
-                            _.defer(function () { ignoreFocusEvent = false; });
+                            self.executeDelayed(function () { ignoreFocusEvent = false; });
                         } else {
                             textAreaContainer.removeClass(Forms.FOCUSED_CLASS);
                             hideTextAreaPopups();
@@ -1201,7 +1107,7 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             }
 
             // build the new formula term to be inserted into the text
-            tokenArray = new TokenArray(app, self.getSheetModel(), { silent: true, grammar: 'ui' });
+            tokenArray = new TokenArray(app, self.getSheetModel(), { trigger: 'never', grammar: 'ui' });
             tokenArray.appendRangeList(selection.ranges);
             references = tokenArray.getFormula();
             tokenArray.destroy();
@@ -1412,12 +1318,12 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             // are visible, nor the editable cell. Root cause is the soft-keyboard.
             // However, scroll back to 0,0 and everything is fine. (Delay of 1000 necessary for ios)
             if (Modernizr.touch) {
-                app.executeDelayed(function () { window.scrollTo(0,0); }, {delay: 1000});
+                this.executeDelayed(function () { window.scrollTo(0, 0); }, 1000);
             }
 
             // give user some info and quit if cell is locked
             if (this.isSheetLocked() && !editCellData.attributes.cell.unlocked) {
-                this.trigger('celledit:reject', 'protected');
+                this.trigger('celledit:reject', 'cells:locked');
                 return this;
             }
 
@@ -1425,8 +1331,8 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             editActive = true;
             setQuickEditMode(Utils.getBooleanOption(options, 'quick', false));
 
-            // TODO: array formula support
-            arrayFormula = false;
+            // TODO: matrix formula support
+            matrixFormula = false;
 
             // the token array stores the parsed formula for range highlighting
             editTokenArray = new TokenArray(app, this.getSheetModel(), { grammar: 'ui' });
@@ -1452,12 +1358,6 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             editGridPane.initializeCellEditMode();
             editGridPane.on('change:scrollpos', gridPaneScrollHandler);
 
-            // IE9 does not trigger 'input' events when deleting characters or
-            // pasting text, use a timer interval as a workaround
-            if (Utils.IE9) {
-                textAreaNode.data('updateTimer', app.repeatDelayed(textAreaInputHandler, { delay: 250 }));
-            }
-
             originalText = this.getTextForCellEditMode(editCellData);
             textUnderlayNode.empty();
 
@@ -1499,13 +1399,6 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             editTokenArray.destroy();
             editTokenArray = null;
 
-            // IE9 does not trigger 'input' events when deleting characters or pasting text,
-            // remove the timer interval that has been started as a workaround
-            if (textAreaNode.data('updateTimer')) {
-                textAreaNode.data('updateTimer').abort();
-                textAreaNode.data('updateTimer', null);
-            }
-
             // back to cell selection mode
             quickEditMode = null;
             formulaString = null;
@@ -1535,10 +1428,10 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
          *      the selection.
          *  - 'fill': The current value will be filled to all cells of the
          *      selection.
-         *  - 'array': The current value will be inserted as array formula into
-         *      the selected cell range.
+         *  - 'matrix': The current value will be inserted as matrix formula
+         *      into the selected cell range.
          *  - 'auto': Preserves the current formula type of the cell: either
-         *      'cell' or 'array'.
+         *      'cell' or 'matrix'.
          *
          * @param {Object} [options]
          *  Optional parameters:
@@ -1559,9 +1452,9 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
                 // whether to validate the formula
                 validate = Utils.getBooleanOption(options, 'validate', false),
                 // the trimmed URL extracted from the value
-                url = HyperlinkUtil.checkForHyperlink(editText),
+                url = HyperlinkUtils.checkForHyperlink(editText),
                 // additional properties for the operation
-                operationOptions = { parse: true };
+                properties = { parse: true };
 
             function triggerReject(cause) {
                 self.trigger('celledit:reject', cause);
@@ -1577,21 +1470,21 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             // nothing to do, if in-place edit mode is not active
             if (!editActive) { return true; }
 
-            // check validity of commit mode (e.g. disallow 'cell' in array formulas)
+            // check validity of commit mode (e.g. disallow 'cell' in matrix formulas)
             switch (commitMode) {
             case 'cell':
             case 'fill':
-                if (arrayFormula) {
-                    // TODO: define behavior (update entire array automatically?)
-                    triggerReject('array');
+                if (matrixFormula) {
+                    // TODO: define behavior (update entire matrix automatically?)
+                    triggerReject('formula:matrix');
                     return false;
                 }
                 break;
-            case 'array':
-                // TODO: check that selection matches range of array formula
+            case 'matrix':
+                // TODO: check that selection matches range of matrix formula
                 break;
             case 'auto':
-                commitMode = arrayFormula ? 'array' : 'cell';
+                commitMode = matrixFormula ? 'matrix' : 'cell';
                 break;
             default:
                 Utils.error('CellEditMixin.leaveCellEditMode(): unknown commit mode: ' + commitMode);
@@ -1599,7 +1492,7 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             }
 
             // do nothing else, if the document does not have edit rights anymore
-            if (!model.getEditMode() || !commitMode) {
+            if (!docModel.getEditMode() || !commitMode) {
                 this.cancelCellEditMode();
                 return true;
             }
@@ -1615,12 +1508,13 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
                 updateEditTokenArray({ direct: true });
                 if (_.isString(formulaString)) {
 
-                    var result = editTokenArray.interpretFormula(editAddress);
+                    var // TODO: get value/matrix result depending on commit mode
+                        result = editTokenArray.interpretFormula('val', editSheet, editAddress);
 
                     if (validate) {
                         switch (result.type) {
                         case 'error':
-                            triggerReject('invalid');
+                            triggerReject('formula:invalid');
                             return false;
                         case 'warn':
                             if (result.value === 'circular') {
@@ -1631,15 +1525,15 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
                     }
 
                     // insert the formula result value into the operation
-                    operationOptions.result = null;
+                    properties.result = null;
                     if (result.type === 'result') {
                         // convert error code objects to string representation, add leading apostrophe to strings if necessary
                         if (SheetUtils.isErrorCode(result.value)) {
-                            operationOptions.result = result.value.code;
+                            properties.result = result.value.code;
                         } else if (_.isString(result.value)) {
-                            operationOptions.result = result.value.replace(/^(['#])/, '\'$1');
+                            properties.result = result.value.replace(/^(['#])/, '\'$1');
                         } else {
-                            operationOptions.result = result.value;
+                            properties.result = result.value;
                         }
                     }
                 }
@@ -1658,28 +1552,31 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
                 addAttribute('character', 'url', url);
                 addAttribute('character', 'underline', true);
                 addAttribute('character', 'color', { type: 'scheme', value: 'hyperlink' });
+            } else if (!editText) {
+                //delete explicit the url, if text is cleared
+                addAttribute('character', 'url', null);
             }
 
             // do not pass empty attributes object
             if (_.isEmpty(editAttributes)) { editAttributes = null; }
 
             // send current value to document model after returning to selection mode
-            switch (model.getEditMode() ? commitMode : 'discard') {
+            switch (docModel.getEditMode() ? commitMode : 'discard') {
             case 'cell':
                 // single cell mode: do not send operations if text and formatting did not change
                 if ((originalText !== editText) || _.isObject(editAttributes)) {
-                    this.setCellContents(editText, editAttributes, operationOptions);
+                    this.setCellContents(editText, editAttributes, properties);
                     lastCommitData.ranges = [{ start: editAddress, end: editAddress }];
                 }
                 break;
             case 'fill':
                 // fill mode: always fill all cells in the selection
-                this.fillCellRanges(editText, editAttributes, operationOptions);
+                this.fillCellRanges(editText, editAttributes, properties);
                 lastCommitData.ranges = this.getSelectedRanges();
                 break;
-            case 'array':
-                // TODO: array formulas
-                Utils.warn('CellEditMixin.leaveCellEditMode(): array formulas not implemented');
+            case 'matrix':
+                // TODO: matrix formulas
+                Utils.warn('CellEditMixin.leaveCellEditMode(): matrix formulas not implemented');
                 lastCommitData.ranges = this.getSelectedRanges();
                 break;
             }
@@ -1897,14 +1794,14 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
         app.onInit(function () {
 
             // resolve reference to document model objects
-            model = app.getModel();
-            undoManager = model.getUndoManager();
-            documentStyles = model.getDocumentStyles();
-            fontCollection = model.getFontCollection();
-            numberFormatter = model.getNumberFormatter();
+            docModel = app.getModel();
+            undoManager = docModel.getUndoManager();
+            documentStyles = docModel.getDocumentStyles();
+            fontCollection = docModel.getFontCollection();
+            numberFormatter = docModel.getNumberFormatter();
 
             // cancel cell edit mode after the application loses edit rights
-            model.on('change:editmode', editModeHandler);
+            docModel.on('change:editmode', editModeHandler);
 
             // update zoom settings during edit mode
             self.on('change:sheet:viewattributes', changeSheetViewAttributesHandler);
@@ -1935,13 +1832,13 @@ define('io.ox/office/spreadsheet/view/mixin/celleditmixin',
             registerTokenArrayListeners('off');
 
             textAreaContainer.remove();
-            textAreaToolTip.destroy();
+            signatureToolTip.destroy();
             textAreaListMenu.destroy();
             textAreaListItemToolTip.destroy();
 
-            app = self = model = undoManager = documentStyles = fontCollection = numberFormatter = null;
+            app = self = docModel = undoManager = documentStyles = fontCollection = numberFormatter = null;
             textAreaContainer = textAreaNode = textUnderlayNode = null;
-            textAreaToolTip = textAreaListMenu = textAreaListItemToolTip = null;
+            signatureToolTip = textAreaListMenu = textAreaListItemToolTip = null;
             undoStack = highlightTokenArrays = highlightOptions = null;
         });
 
