@@ -1,31 +1,28 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/editframework/view/editdialogs',
-     'io.ox/office/drawinglayer/view/imageutil',
-     'gettext!io.ox/office/spreadsheet'
-    ], function (Utils, KeyCodes, Forms, EditDialogs, ImageUtil, gt) {
+define('io.ox/office/spreadsheet/view/mixin/dialogsmixin', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/editframework/view/editdialogs',
+    'io.ox/office/drawinglayer/view/imageutil',
+    'gettext!io.ox/office/spreadsheet'
+], function (Utils, KeyCodes, Forms, EditDialogs, ImageUtil, gt) {
 
     'use strict';
 
-    var // maximum size of an image in any direction, in 1/100 mm
-        MAX_IMAGE_SIZE = 21000;
-
     // mix-in class SpreadsheetDialogsMixin ===================================
 
     /**
@@ -132,7 +129,7 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
          */
         this.showCopySheetDialog = function () {
             // the callback function will be called in the context of this view instance
-            return showSheetNameDialog(gt('Copy Sheet'), this.copySheet, { okLabel: gt('Copy'), initialName: app.getModel().generateUnusedSheetName() });
+            return showSheetNameDialog(gt('Copy Sheet'), this.copySheet, { okLabel: gt('Copy'), initialName: this.generateUnusedSheetName() });
         };
 
         /**
@@ -151,6 +148,21 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
             return showSheetNameDialog(gt('Rename Sheet'), this.setSheetName, { okLabel: gt('Rename') });
         };
 
+        /**
+         * Shows a dialog that give the user the posibillity to decide wheather the
+         * current selection should expand or not.
+         *
+         * @returns {jQuery.Promise}
+         *  The Promise of a Deferred object representing the dialog. Will be
+         *  resolved with the expanding of the selection; or rejected without
+         *  expanding. After that, both options executes the sort.
+         */
+        this.showExpandSelectionForSortingDialog = function () {
+            var dialog = new EditDialogs.ModalQueryDialog(gt('Expand the selection?'), { width: 300, title: gt('Sorting') });
+
+            return dialog.show();
+        };
+
         /**
          * Shows the 'Insert Image' dialog and inserts the resulting image into
          * the active sheet of the document.
@@ -167,55 +179,15 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
                 return ImageUtil.showInsertImageDialog(app);
             }
 
-            // leave cell edit mode, show the dialog, handle the dialog result
-            return leaveCellEditMode().then(showInsertImageDialog).then(function (imageHolder) {
-
-                var // create the image DOM node, wait for it to load
-                    createImagePromise = app.createImageNode(ImageUtil.getFileUrl(app, imageHolder.url));
-
-                // on success, create the drawing object in the document
-                createImagePromise.done(function (imgNode) {
-
-                    var // current width of the image, in 1/100 mm
-                        width = Utils.convertLengthToHmm(imgNode[0].width, 'px'),
-                        // current height of the image, in 1/100 mm
-                        height = Utils.convertLengthToHmm(imgNode[0].height, 'px'),
-                        // active cell as target position for the image
-                        activeCell = self.getActiveCell();
-
-                    // restrict size to fixed limit
-                    if (width > height) {
-                        if (width > MAX_IMAGE_SIZE) {
-                            height = Math.round(height * MAX_IMAGE_SIZE / width);
-                            width = MAX_IMAGE_SIZE;
-                        }
-                    } else {
-                        if (height > MAX_IMAGE_SIZE) {
-                            width = Math.round(width * MAX_IMAGE_SIZE / height);
-                            height = MAX_IMAGE_SIZE;
-                        }
-                    }
-
-                    self.insertDrawing('image', { drawing: {
-                        anchorType: 'oneCell',
-                        startCol: activeCell[0],
-                        startRow: activeCell[1],
-                        width: width,
-                        height: height,
-                        name: imageHolder.name,
-                        imageUrl: imageHolder.url
-                    }});
-
-                    app.destroyImageNodes(imgNode);
-                });
-
-                // on failure, show a warning message
-                createImagePromise.fail(function () {
+            // inserts the image into the document
+            function insertImage(imageDescriptor) {
+                return self.insertImage(imageDescriptor).fail(function () {
                     app.rejectEditAttempt('image');
                 });
+            }
 
-                return createImagePromise;
-            });
+            // leave cell edit mode, show the dialog, handle the dialog result
+            return leaveCellEditMode().then(showInsertImageDialog).then(insertImage);
         };
 
         /**
@@ -231,9 +203,9 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
             // leave cell edit mode, show the dialog
             return leaveCellEditMode().then(function () {
 
-                var model = app.getModel(),
+                var docModel = app.getModel(),
 
-                    names = model.getSheetNames(),
+                    names = docModel.getSheetNames(),
                     currentSheet = self.getActiveSheet(),
                     currentChosen = null,
 
@@ -337,7 +309,7 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
                 self.closeDialogOnReadOnlyMode(dialog);
 
                 _.each(names, function (name, index) {
-                    var sheetModel = model.getSheetModel(index);
+                    var sheetModel = docModel.getSheetModel(index);
 
                     if (!sheetModel.getMergedAttributes().sheet.visible) {
                         return;
@@ -406,9 +378,9 @@ define('io.ox/office/spreadsheet/view/mixin/dialogsmixin',
                 checkScrollOutside();
 
                 return promise.done(function () {
-                    model.getUndoManager().enterUndoGroup(function () {
+                    docModel.getUndoManager().enterUndoGroup(function () {
                         _.each(collectOps, function (op) {
-                            model.moveSheet(op.from, op.to);
+                            docModel.moveSheet(op.from, op.to);
                         });
                     });
                 });
