@@ -1,24 +1,23 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/drawinglayer/utils/drawingutils',
-     'io.ox/office/drawinglayer/view/drawingframe',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/utils/paneutils'
-    ], function (Utils, DrawingUtils, DrawingFrame, SheetUtils, PaneUtils) {
+define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/drawinglayer/view/drawingframe',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/utils/paneutils'
+], function (Utils, DrawingFrame, SheetUtils, PaneUtils) {
 
     'use strict';
 
@@ -34,20 +33,17 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
      * @param {SpreadsheetApplication} app
      *  The application that contains this grid pane.
      *
-     * @param {String} panePos
-     *  The position of this grid pane.
-     *
      * @param {jQuery} layerRootNode
      *  The root node of all layer nodes contained in the grid pane.
      */
-    function GridTrackingMixin(app, panePos, layerRootNode) {
+    function GridTrackingMixin(app, layerRootNode) {
 
-        var // self reference
+        var // self reference (the GridPane instance)
             self = this,
 
             // the spreadsheet model and view
-            model = null,
-            view = null,
+            docModel = null,
+            docView = null,
 
             // the type of the current tracking, and the last finished tracking
             trackingType = null,
@@ -107,8 +103,8 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
             // add the column and row collection entry
             options = Utils.extendOptions(options, { pixel: true });
-            result.colEntry = view.getColCollection().getEntryByOffset(result.left, options);
-            result.rowEntry = view.getRowCollection().getEntryByOffset(result.top, options);
+            result.colEntry = docView.getColCollection().getEntryByOffset(result.left, options);
+            result.rowEntry = docView.getRowCollection().getEntryByOffset(result.top, options);
 
             // add the cell address
             result.address = [result.colEntry.index, result.rowEntry.index];
@@ -198,7 +194,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 var // the array index in cell addresses
                     index = columns ? 0 : 1,
                     // the maximum column/row index
-                    maxIndex = model.getMaxIndex(columns),
+                    maxIndex = docModel.getMaxIndex(columns),
                     // the column/row descriptor
                     entryData = columns ? trackingData.colEntry : trackingData.rowEntry,
                     // whether the leading border is tracked
@@ -236,7 +232,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 }
             } else {
                 // border mode: move entire range according to distance between tracked cell and anchor cell
-                range = model.getBoundedMovedRange(origRange, address[0] - anchorCell[0], address[1] - anchorCell[1]);
+                range = docModel.getBoundedMovedRange(origRange, address[0] - anchorCell[0], address[1] - anchorCell[1]);
             }
 
             return range;
@@ -307,7 +303,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
             function finalizeTracking(event) {
 
                 var // current selection
-                    selection = view.getSelection(),
+                    selection = docView.getSelection(),
                     // array index of the range that contains a tapped cell
                     rangeIndex = 0;
 
@@ -316,8 +312,8 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
                 if (event.duration <= 750) {
                     // short-tap: select a single cell, or start in-place edit mode
-                    if (!view.hasDrawingSelection() && (_.isEqual(selection.activeCell, startAddress) || SheetUtils.rangesContainCell(selection.ranges, startAddress))) {
-                        view.enterCellEditMode();
+                    if (!docView.hasDrawingSelection() && (_.isEqual(selection.activeCell, startAddress) || SheetUtils.rangesContainCell(selection.ranges, startAddress))) {
+                        docView.enterCellEditMode();
                     } else {
                         self.trigger('select:start', startAddress, 'select');
                         self.trigger('select:end', startAddress, true);
@@ -387,7 +383,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
                 // extract range address
                 rangeIndex = Utils.getElementAttributeAsInteger(rangeNode, 'data-index', -1);
-                origRange = view.getSelectedRanges()[rangeIndex];
+                origRange = docView.getSelectedRanges()[rangeIndex];
 
                 // initialize additional data needed while tracking
                 trackerPos = trackerNode.attr('data-pos');
@@ -454,7 +450,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                     // the root node of the highlighted range that will be tracked
                     rangeNode = trackerNode.closest('.range'),
                     // the highlighted ranges
-                    highlightRanges = view.getSheetViewAttribute('highlightRanges'),
+                    highlightRanges = docView.getSheetViewAttribute('highlightRanges'),
                     // the array index of the tracked range
                     highlightIndex = Utils.getElementAttributeAsInteger(rangeNode, 'data-index', -1);
 
@@ -473,7 +469,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 anchorCell = getAnchorCell(event, origRange, trackerPos, resize);
 
                 // render tracked range with special highlighting
-                view.setSheetViewAttribute('highlightIndex', highlightIndex);
+                docView.setSheetViewAttribute('highlightIndex', highlightIndex);
             }
 
             // updates the highlighted range according to the passed tracking event
@@ -486,13 +482,13 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 // trigger change events which will cause refreshing all grid panes
                 if (!_.isEqual(currRange, newRange)) {
                     currRange = newRange;
-                    view.modifyReferenceToken(origRange.id, currRange);
+                    docView.modifyReferenceToken(origRange.id, currRange);
                 }
             }
 
             // finalizes the highlighted range
             function finalizeTracking() {
-                view.setSheetViewAttribute('highlightIndex', null);
+                docView.setSheetViewAttribute('highlightIndex', null);
             }
 
             // return the actual highlightTrackingHandler() method
@@ -516,7 +512,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                     break;
                 case 'tracking:cancel':
                     finalizeTracking(event);
-                    view.modifyReferenceToken(origRange.id, origRange);
+                    docView.modifyReferenceToken(origRange.id, origRange);
                     break;
                 }
             };
@@ -552,19 +548,19 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
             function initializeTracking() {
 
                 // cancel custom selection when starting auto-fill
-                view.leaveCustomSelectionMode();
+                docView.cancelCustomSelectionMode();
 
                 // do not expand auto-fill range into leading/trailing hidden columns/rows
-                visibleRange = view.getVisibleRange(model.getSheetRange());
+                visibleRange = docView.getSheetModel().shrinkToVisibleRange(docModel.getSheetRange());
                 if (!visibleRange) { $.cancelTracking(); return; }
 
                 // store address and position of the original selection range
-                origRange = view.getSelectedRanges()[0];
-                origRectangle = view.getRangeRectangle(origRange);
+                origRange = docView.getSelectedRanges()[0];
+                origRectangle = docView.getRangeRectangle(origRange);
                 startPos = { left: origRectangle.left + origRectangle.width, top: origRectangle.top + origRectangle.height };
 
                 // special behavior for column/row ranges
-                fixedColumns = model.isColRange(origRange) ? true : model.isRowRange(origRange) ? false : null;
+                fixedColumns = docModel.isColRange(origRange) ? true : docModel.isRowRange(origRange) ? false : null;
 
                 // restrict number of columns/rows according to selection mode
                 var maxCols = 0, maxRows = 0;
@@ -585,7 +581,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
                 // render selected range with modified auto-fill style
                 autoFillData = { border: 'bottom', count: 0 };
-                view.setSheetViewAttribute('autoFillData', autoFillData);
+                docView.setSheetViewAttribute('autoFillData', autoFillData);
             }
 
             // updates the auto fill area according to the passed tracking event
@@ -594,7 +590,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 var // current tracking position
                     data = getTrackingData(event, { outerHidden: true }),
                     // whether to delete parts of the selected range
-                    deleteMode = DrawingUtils.rectangleContainsPosition(origRectangle, data),
+                    deleteMode = Utils.rectangleContainsPixel(origRectangle, data),
                     // the adjusted tracking position to determine the direction
                     left = (deleteMode || (data.left >= startPos.left)) ? data.left : Math.min(data.left + origRectangle.width, startPos.left),
                     top = (deleteMode || (data.top >= startPos.top)) ? data.top : Math.min(data.top + origRectangle.height, startPos.top),
@@ -623,7 +619,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                     autoFillData.border = columns ? 'right' : 'bottom';
                     autoFillData.count = Math.min(range.end[addrIndex] - origRange.end[addrIndex], columns ? maxTrailingCols : maxTrailingRows);
                 }
-                view.setSheetViewAttribute('autoFillData', autoFillData);
+                docView.setSheetViewAttribute('autoFillData', autoFillData);
 
                 // set correct mouse pointer according to direction
                 event.cursor = (_.isBoolean(fixedColumns) || (autoFillData.count === 0)) ? null : columns ? 'ew-resize' : 'ns-resize';
@@ -631,7 +627,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
             // finalizes the selection tracking
             function finalizeTracking() {
-                view.setSheetViewAttribute('autoFillData', null);
+                docView.setSheetViewAttribute('autoFillData', null);
             }
 
             // return the actual autoFillTrackingHandler() method
@@ -650,7 +646,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                     break;
                 case 'tracking:end':
                     finalizeTracking(event);
-                    view.autoFill(autoFillData.border, autoFillData.count);
+                    docView.autoFill(autoFillData.border, autoFillData.count);
                     break;
                 case 'tracking:cancel':
                     finalizeTracking(event);
@@ -707,10 +703,8 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
             function initializeTracking(event) {
 
                 var // the column/row collections of the active sheet
-                    colCollection = view.getColCollection(),
-                    rowCollection = view.getRowCollection(),
-                    // the drawing collection of the active sheet
-                    drawingCollection = view.getDrawingCollection(),
+                    colCollection = docView.getColCollection(),
+                    rowCollection = docView.getRowCollection(),
                     // the grid pane layer rectangle
                     layerRectangle = self.getLayerRectangle(),
                     // whether to toggle the selection of the clicked drawing frame
@@ -743,12 +737,12 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 selected = DrawingFrame.isSelected(drawingFrame);
                 if (toggle || !selected) {
                     drawingPos = [$(drawingFrame).index()];
-                    view.selectDrawing(drawingPos, { toggle: toggle });
-                    view.scrollToSelectedDrawing(drawingPos);
+                    docView.selectDrawing(drawingPos, { toggle: toggle });
+                    docView.scrollToDrawingFrame(drawingPos);
                 }
 
                 // check edit mode and sheet protection after selecting the drawing
-                if (!model.getEditMode() || view.isSheetLocked()) {
+                if (!docModel.getEditMode() || docView.isSheetLocked()) {
                     $.cancelTracking();
                     return;
                 }
@@ -768,13 +762,13 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 }
 
                 // receiving the current selection
-                selection = view.getSelection();
+                selection = docView.getSelection();
 
                 isMultiSelection = (selection.drawings.length > 1);
 
                 // Handling multi selection as special case for performance reasons
                 if (isMultiSelection) {
-                    drawings = view.getSelectedDrawings();
+                    drawings = docView.getSelectedDrawings();
 
                     // resetting all drawing collectors
                     allDrawingFrames = [];
@@ -785,7 +779,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
 
                     _.each(drawings, function (position) {
 
-                        var localDrawingFrame = self.getDrawingFrame(drawingCollection.findModel(position).getUid()).first(),
+                        var localDrawingFrame = self.getDrawingFrame(position),
                             localMoveBox = localDrawingFrame.find('.tracker');
 
                         // collecting drawings for multi selection handling
@@ -838,7 +832,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                     activeStartWidth = drawingFrame.width();
                     activeStartHeight = drawingFrame.height();
                     // receiving explicit attributes of the drawing
-                    drawingAttributes = DrawingFrame.getModel(drawingFrame).getExplicitAttributes().drawing;
+                    drawingAttributes = DrawingFrame.getModel(drawingFrame).getMergedAttributes().drawing;
                     activeStartWidthHmm = drawingAttributes.width;
                     activeStartHeightHmm = drawingAttributes.height;
                     // using hmm to calculate ratio instead of px because of increased precision
@@ -858,7 +852,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                     _.each(allDrawingFrames, function (oneDrawingFrame, index) {
                         if (doResize) {
                             // receiving explicit attributes of the drawing
-                            oneDrawingAttributes = DrawingFrame.getModel(oneDrawingFrame).getExplicitAttributes().drawing;
+                            oneDrawingAttributes = DrawingFrame.getModel(oneDrawingFrame).getMergedAttributes().drawing;
                             // taking care of the distance to the border and of the geometry of each drawing
                             widthRatio = Utils.round(activeStartWidthHmm / oneDrawingAttributes.width, 0.01);
                             heightRatio = Utils.round(activeStartHeightHmm / oneDrawingAttributes.height, 0.01);
@@ -1151,16 +1145,16 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 }
 
                 // modifying attributes
-                attributes = view.getDrawingCollection().getAttributesForRectangle({ left: newLeft, top: newTop, width: newWidth, height: newHeight});
+                attributes = docView.getDrawingCollection().getAttributesForRectangle({ left: newLeft, top: newTop, width: newWidth, height: newHeight});
 
                 // setting moveBox back into the drawing frame
                 localMoveBox.css({ left: 0, top: 0 });
 
                 // generating drawing attributes operation(s)
                 if (isMultiSelection) {
-                    view.setDrawingAttributes(attributes, [allDrawingPositions[index]]);
+                    docView.setDrawingAttributes(attributes, [allDrawingPositions[index]]);
                 } else {
-                    view.setDrawingAttributes(attributes);
+                    docView.setDrawingAttributes(attributes);
                 }
             }
 
@@ -1242,13 +1236,13 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
                 PaneUtils.processTrackingCycle(event, trackingHandler, function () {
                     lastTrackingType = trackingType;
                     trackingType = null;
-                    view.grabFocus();
+                    docView.grabFocus();
                 });
 
                 // Workaround for Firefox, it sends 'focusout' events after the old
                 // selection DOM node that has been clicked is removed (the
                 // 'focusout' event causes canceling the tracking sometimes).
-                _.defer(function () { self.grabFocus(); });
+                self.executeDelayed(function () { self.grabFocus(); });
             }
 
             // ignore inline pop-up menus, and other nodes that handle clicks internally
@@ -1284,7 +1278,7 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
          * Handles double clicks and starts the in-place edit mode.
          */
         function doubleClickHandler() {
-            view.enterCellEditMode();
+            docView.enterCellEditMode();
         }
 
         // initialization -----------------------------------------------------
@@ -1293,23 +1287,21 @@ define('io.ox/office/spreadsheet/view/mixin/gridtrackingmixin',
         app.onInit(function () {
 
             // the spreadsheet model and view instances
-            model = app.getModel();
-            view = app.getView();
+            docModel = app.getModel();
+            docView = app.getView();
 
             // leave in-place edit mode or cancel tracking, if document goes into read-only mode
-            model.on('change:editmode', editModeHandler);
+            self.listenTo(docModel, 'change:editmode', editModeHandler);
 
             // tracking for cell selection with mouse/touch
-            layerRootNode.on({
-                'tracking:start': trackingStartHandler,
-                dblclick: doubleClickHandler,
-                'dragenter dragexit dragover dragleave': false
-            });
+            self.listenToWhenVisible(layerRootNode, 'tracking:start', trackingStartHandler);
+            self.listenToWhenVisible(layerRootNode, 'dblclick', doubleClickHandler);
+            layerRootNode.on('dragenter dragexit dragover dragleave', false);
         });
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = layerRootNode = self = model = view = null;
+            app = layerRootNode = self = docModel = docView = null;
         });
 
     } // class GridTrackingMixin
