@@ -1,22 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/utils/paneutils'
-    ], function (Utils, SheetUtils, PaneUtils) {
+define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/utils/paneutils'
+], function (Utils, SheetUtils, PaneUtils) {
 
     'use strict';
 
@@ -47,8 +47,8 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
             columns = PaneUtils.isColumnSide(paneSide),
 
             // the spreadsheet model and view
-            model = null,
-            view = null,
+            docModel = null,
+            docView = null,
 
             // the type of the current tracking
             trackingType = null,
@@ -137,8 +137,8 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
             function initializeTracking(event) {
                 currentIndex = getTrackingStartEntryData(event).index;
                 self.scrollToEntry(currentIndex);
-                view.setSheetViewAttributes({
-                    activePane: PaneUtils.getNextPanePos(view.getSheetViewAttribute('activePane'), paneSide),
+                docView.setSheetViewAttributes({
+                    activePane: PaneUtils.getNextPanePos(docView.getSheetViewAttribute('activePane'), paneSide),
                     activePaneSide: paneSide
                 });
                 self.trigger('select:start', currentIndex, PaneUtils.getSelectionMode(event));
@@ -156,7 +156,7 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
             // finalizes selection tracking
             function finalizeTracking(event, success) {
                 self.scrollToEntry(currentIndex);
-                view.setSheetViewAttribute('activePaneSide', null);
+                docView.setSheetViewAttribute('activePaneSide', null);
                 self.trigger('select:end', currentIndex, success);
             }
 
@@ -247,7 +247,7 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
             function applyResize() {
                 if (currentSize !== entryData.size) {
                     // pass tracked column/row as target index, will cause to modify all columns/rows selected completely
-                    view[columns ? 'setColumnWidth' : 'setRowHeight'](view.convertPixelToHmm(currentSize), { custom: true, target: entryData.index });
+                    docView[columns ? 'setColumnWidth' : 'setRowHeight'](docView.convertPixelToHmm(currentSize), { custom: true, target: entryData.index });
                 }
             }
 
@@ -297,7 +297,7 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
                 // handle all tracking events for this tracking cycle
                 PaneUtils.processTrackingCycle(event, trackingHandler, function () {
                     trackingType = null;
-                    view.grabFocus();
+                    docView.grabFocus();
                 });
             }
 
@@ -318,13 +318,13 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
                 index = self.getColRowCollection().getEntry(Utils.getElementAttributeAsInteger(event.target, 'data-index', -1)).index;
 
             if (columns) {
-                view.setOptimalColumnWidth(index);
+                docView.setOptimalColumnWidth(index);
             } else {
-                view.setOptimalRowHeight(index);
+                docView.setOptimalRowHeight(index);
             }
 
             // return focus to active grid pane, after the event has been processed
-            _.defer(function () { view.grabFocus(); });
+            self.executeDelayed(function () { docView.grabFocus(); });
         }
 
         // initialization -----------------------------------------------------
@@ -333,14 +333,14 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
         app.onInit(function () {
 
             // the spreadsheet model and view instances
-            model = app.getModel();
-            view = app.getView();
+            docModel = app.getModel();
+            docView = app.getView();
 
             // enable/disable operations that modify the document according to edit mode
-            model.on('change:editmode', editModeHandler);
+            docModel.on('change:editmode', editModeHandler);
 
             // listen to view events
-            view.on('change:selection', changeSelectionHandler);
+            docView.on('change:selection', changeSelectionHandler);
 
             // tracking for column/row selection with mouse/touch
             self.getNode().on('tracking:start', trackingStartHandler);
@@ -351,7 +351,7 @@ define('io.ox/office/spreadsheet/view/mixin/headertrackingmixin',
 
         // destroy all class members on destruction
         this.registerDestructor(function () {
-            app = contentNode = self = model = view = null;
+            app = contentNode = self = docModel = docView = null;
         });
 
     } // class HeaderTrackingMixin
