@@ -1,18 +1,20 @@
 /**
- * All content on this website (including text, images, source code and any
- * other original works), unless otherwise noted, is licensed under a Creative
- * Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2013 Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Stefan Eckert <stefan.eckert@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/view/mixin/hyperlinkmixin',
-    ['io.ox/office/editframework/view/hyperlinkutil'
-    ], function (HyperlinkUtil) {
+define('io.ox/office/spreadsheet/view/mixin/hyperlinkmixin', [
+    'io.ox/office/editframework/utils/hyperlinkutils',
+    'io.ox/office/editframework/view/hyperlinkutil'
+], function (HyperlinkUtils, HyperlinkUtil) {
 
     'use strict';
 
@@ -66,7 +68,7 @@ define('io.ox/office/spreadsheet/view/mixin/hyperlinkmixin',
             }
 
             if (!popupNode.children().length) {
-                //FIXME: this is a workaround for the internet explorer, it seems, the IE deletes the children,w hen you hide an element
+                //FIXME: this is a workaround for IE, it seems, the IE deletes the children,w hen you hide an element
                 popupNode = HyperlinkUtil.createPopupNode(app).css({ marginTop: 3, minWidth: 130 });
             }
 
@@ -74,7 +76,7 @@ define('io.ox/office/spreadsheet/view/mixin/hyperlinkmixin',
             var cellData = self.getCellCollection().getCellEntry(address);
 
             var url = cellData.attributes.character.url;
-            if (url && url.length > 0 && HyperlinkUtil.isLegalUrl(url)) {
+            if (url && url.length > 0 && HyperlinkUtils.hasSupportedProtocol(url)) {
                 HyperlinkUtil.updatePopup(url, popupNode);
 
                 var parentnode = self.getActiveGridPane().getNode().find('.active-cell');
@@ -102,7 +104,7 @@ define('io.ox/office/spreadsheet/view/mixin/hyperlinkmixin',
                 url = item.attributes.character.url;
 
             if (!url || !url.length) {
-                url = HyperlinkUtil.checkForHyperlink(display);
+                url = HyperlinkUtils.checkForHyperlink(display);
             }
 
             var def = HyperlinkUtil.showDialog(display, url, app);
