@@ -1,26 +1,27 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/view/render/rendercache',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/object/baseobject',
-     'io.ox/office/tk/object/triggerobject',
-     'io.ox/office/editframework/utils/color',
-     'io.ox/office/spreadsheet/utils/sheetutils',
-     'io.ox/office/spreadsheet/utils/paneutils',
-     'io.ox/office/spreadsheet/model/cellcollection',
-     'io.ox/office/spreadsheet/view/render/renderutils'
-    ], function (Utils, BaseObject, TriggerObject, Color, SheetUtils, PaneUtils, CellCollection, RenderUtils) {
+define('io.ox/office/spreadsheet/view/render/rendercache', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/object/baseobject',
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/tk/object/timermixin',
+    'io.ox/office/editframework/utils/color',
+    'io.ox/office/spreadsheet/utils/sheetutils',
+    'io.ox/office/spreadsheet/utils/paneutils',
+    'io.ox/office/spreadsheet/model/cellcollection',
+    'io.ox/office/spreadsheet/view/render/renderutils'
+], function (Utils, BaseObject, TriggerObject, TimerMixin, Color, SheetUtils, PaneUtils, CellCollection, RenderUtils) {
 
     'use strict';
 
@@ -88,8 +89,8 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
     var ColRowCache = BaseObject.extend({ constructor: function (app, sheetModel, columns) {
 
         var // the spreadsheet model and its containers
-            model = app.getModel(),
-            documentStyles = model.getDocumentStyles(),
+            docModel = app.getModel(),
+            documentStyles = docModel.getDocumentStyles(),
 
             // the column/row collection represented by this cache
             collection = columns ? sheetModel.getColCollection() : sheetModel.getRowCollection(),
@@ -239,7 +240,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
 
             // transform the current bounding intervals
             _.each(intervalMap, function (boundIntervals, id) {
-                intervalMap[id] = model.transformIntervals(boundIntervals, interval, insert, columns, 'split');
+                intervalMap[id] = docModel.transformIntervals(boundIntervals, interval, insert, columns, 'split');
             });
 
             // update the cached entries
@@ -274,7 +275,8 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
          *      Color.getColorDetails() for details about this object.
          *  - {Object} borders
          *      The styles of the borders of all cells in the column/row, as
-         *      returned by the method RenderUtils.getBorderStyles().
+         *      returned by the method RenderUtils.getBorderStyle(), mapped by
+         *      border identifier ('t', 'b', 'l', or 'r').
          *  If the entry is not visible, the value null will be returned
          *  instead.
          */
@@ -369,7 +371,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
         // destroy all class members on destruction
         this.registerDestructor(function () {
             app = sheetModel = collection = null;
-            model = documentStyles = null;
+            docModel = documentStyles = null;
             intervalMap = intervals = entries = entryMap = null;
         });
 
@@ -412,6 +414,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
      * @constructor
      *
      * @extends TriggerObject
+     * @extends TimerMixin
      *
      * @param {SpreadsheetApplication} app
      *  The application that contains this cache instance.
@@ -425,8 +428,8 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
             self = this,
 
             // the spreadsheet model and its containers
-            model = app.getModel(),
-            documentStyles = model.getDocumentStyles(),
+            docModel = app.getModel(),
+            documentStyles = docModel.getDocumentStyles(),
 
             // the collections of the sheet
             cellCollection = sheetModel.getCellCollection(),
@@ -462,9 +465,10 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
             // whether the cell geometry have been invalidated
             dirtyGeometry = false;
 
-        // base constructor ---------------------------------------------------
+        // base constructors --------------------------------------------------
 
         TriggerObject.call(this);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
@@ -752,26 +756,23 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
             // direct callback: registers dirty cell ranges in the 'dirtyRanges' array
             function registerCellRanges(ranges, options) {
 
+                var refreshCol = Utils.getIntegerOption(options, 'refreshCol'),
+                    immediate = Utils.getBooleanOption(options, 'immediate', false);
+
                 // update array of cached ranges
-                var refreshCol = Utils.getIntegerOption(options, 'refreshCol');
                 ranges = _.getArray(ranges);
-                RenderUtils.log('RenderCache.registerCellRanges(): ranges=' + SheetUtils.getRangesName(ranges) + ' refreshCol=' + refreshCol);
+                RenderUtils.log('RenderCache.registerCellRanges(): ranges=' + SheetUtils.getRangesName(ranges) + ' refreshCol=' + refreshCol + ' immediate=' + immediate);
                 dirtyRanges = dirtyRanges.concat(ranges);
                 if (_.isNumber(refreshCol)) {
                     dirtyCol = _.isNumber(dirtyCol) ? Math.min(dirtyCol, refreshCol) : refreshCol;
                 }
 
                 // invoke deferred callback immediately if specified
-                if (Utils.getBooleanOption(options, 'immediate', false)) {
-                    updateCellRanges();
-                }
+                if (immediate) { updateCellRangesDeferred(); }
             }
 
-            // deferred callback: updates the dirty cell ranges
-            function updateCellRanges() {
-
-                // early exit, if nothing left to be updated
-                if ((dirtyRanges.length === 0) && !_.isNumber(dirtyCol)) { return; }
+            // implementation of updating the dirty cell ranges
+            var updateCellRanges = RenderUtils.profileMethod('RenderCache.updateCellRanges()', function () {
 
                 var // the ranges to be updated (dirty ranges, restricted to the bounding ranges)
                     updateRanges = [],
@@ -848,7 +849,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
                 RenderUtils.log('dirtyRanges=' + SheetUtils.getRangesName(updateRanges));
 
                 // shrink ranges to visible areas (remove leading/trailing hidden columns/rows)
-                updateRanges = sheetModel.getVisibleRanges(updateRanges);
+                updateRanges = sheetModel.shrinkToVisibleRanges(updateRanges);
 
                 // find merged ranges covering the ranges to be updated (reference address
                 // of the merged range may be located outside the update ranges)
@@ -877,7 +878,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
                 });
 
                 // update the cache entries of all cells inside the updated ranges
-                RenderUtils.takeTime('update ' + Utils.getSum(updateRanges, SheetUtils.getCellCount) + ' cells in cache', function () {
+                RenderUtils.takeTime('update ' + SheetUtils.getCellCountInRanges(updateRanges) + ' cells in cache', function () {
                     cellCollection.iterateCellsInRanges(updateRanges, function (cellData) {
                         updateCellEntry(cellData);
                     });
@@ -969,10 +970,23 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
                 _.each(updateRanges, function (updateRange) {
                     cellCollection.fetchCellRange(updateRange, { visible: true, merged: true });
                 });
+            });
+
+            // bug 36014: defer cache update if model currently applies more operations (e.g. Undo)
+            var waitForOpsAndUpdateCellRanges = app.createDebouncedActionsMethod(self, updateCellRanges);
+
+            // deferred callback: updates the dirty cell ranges, unless operations are still running
+            function updateCellRangesDeferred() {
+                // bug 37011: Check whether something is left to be updated, prevent extra run after updating cells
+                // immediately (see option 'immediate' above). Otherwise, cells will be rendered before changed
+                // column/row sizes have been applied which leaves text DOM nodes at wrong position.
+                if ((dirtyRanges.length > 0) || _.isNumber(dirtyCol)) {
+                    waitForOpsAndUpdateCellRanges();
+                }
             }
 
             // the debounced updateCellRanges() method to be returned from the local scope
-            return app.createDebouncedMethod(registerCellRanges, RenderUtils.profileMethod('RenderCache.updateCellRanges()', updateCellRanges));
+            return self.createDebouncedMethod(registerCellRanges, updateCellRangesDeferred);
         }());
 
         /**
@@ -1008,12 +1022,18 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
 
             var // the first dirty column/row index collected from multiple calls
                 col = null,
-                row = null,
-                // whether the deferred part is waiting for operations
-                waiting = false;
+                row = null;
+
+            // bug 36014: defer cache update if model currently applies more operations (e.g. Undo)
+            var debouncedTrigger = app.createDebouncedActionsMethod(self, function () {
+                RenderUtils.takeTime('trigger "cache:operation" operation: col=' + col + ' row=' + row, function () {
+                    self.trigger('cache:operation', col, row);
+                    col = row = null;
+                });
+            });
 
             // the actual triggerCacheOperationEvent() method returned from the local scope
-            function triggerCacheOperationEvent(interval, columns) {
+            return function triggerCacheOperationEvent(interval, columns) {
 
                 // update column/row index in the refresh descriptor
                 if (columns) {
@@ -1022,21 +1042,9 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
                     row = _.isNumber(row) ? Math.min(row, interval.first) : interval.first;
                 }
 
-                // nothing to do, if the method is already waiting for the operation promise
-                if (waiting) { return; }
-
-                // wait for the operation promise (operations may currently be applied asynchronously)
-                waiting = true;
-                self.listenTo(model.getOperationsPromise(), 'always', function () {
-                    RenderUtils.takeTime('trigger "cache:operation" operation: col=' + col + ' row=' + row, function () {
-                        waiting = false;
-                        self.trigger('cache:operation', col, row);
-                        col = row = null;
-                    });
-                });
-            }
-
-            return triggerCacheOperationEvent;
+                // bug 36014: defer cache update if model currently applies more operations (e.g. Undo)
+                debouncedTrigger();
+            };
         }());
 
         /**
@@ -1048,7 +1056,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
             // always transform the dirty ranges that will cause debounced update
             // of the cell cache (may contain addresses located inside the operation
             // range although the column/row cache did not change)
-            dirtyRanges = model.transformRanges(dirtyRanges, interval, insert, columns, 'split');
+            dirtyRanges = docModel.transformRanges(dirtyRanges, interval, insert, columns, 'split');
 
             // transform the bounding intervals in the column/row cache; do nothing else,
             // if the inserted/deleted interval is located after the bounding ranges
@@ -1080,7 +1088,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
                     // use start address of an existing merged range; otherwise transform cell address directly
                     cellData.address = cellData.mergedRange ?
                         _.clone(cellData.mergedRange.start) :
-                        model.transformAddress(cellData.address, interval, insert, columns);
+                        docModel.transformAddress(cellData.address, interval, insert, columns);
 
                     // insert updated cell data into the new map, if the cell is still visible (forget
                     // deleted cells, calculate new position and size of the cell, skip invisible cells)
@@ -1113,7 +1121,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
             dirtyGeometry = true;
 
             // update all cache entries in the changed interval (e.g. for showing hidden columns/rows)
-            updateCellRanges(model.makeFullRange(interval, columns));
+            updateCellRanges(docModel.makeFullRange(interval, columns));
 
             // notify listeners
             triggerCacheOperationEvent(interval, columns);
@@ -1198,7 +1206,8 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
          *      details about this object.
          *  - {Object} borders
          *      The styles of all cell borders, as returned by the method
-         *      RenderUtils.getBorderStyles().
+         *      RenderUtils.getBorderStyle(), mapped by border identifier ('t',
+         *      'b', 'l', or 'r').
          *  If no cell descriptor exists of the specified cell; null will be
          *  returned instead.
          */
@@ -1307,7 +1316,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
                         mapKey = SheetUtils.getCellKey(mergedRange.start);
 
                     // shrink to visible area, insert valid shrunken merged ranges (existing, and larger than 1x1) into the map
-                    mergedRange = sheetModel.getVisibleRange(mergedRange);
+                    mergedRange = sheetModel.shrinkToVisibleRange(mergedRange);
                     if (mergedRange && ((mergedRange.start[0] < mergedRange.end[0]) || (mergedRange.start[1] < mergedRange.end[1]))) {
                         shrunkenMergedRanges[mapKey] = mergedRange;
                     }
@@ -1419,7 +1428,8 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
          *  - {Object} entry.borders
          *      Border styles for the cell (borders of merged ranges will be
          *      distributed to the respective matrix elements), as returned by
-         *      the method RenderUtils.getBorderStyles().
+         *      the method RenderUtils.getBorderStyle(), mapped by border
+         *      identifier ('t', 'b', 'l', or 'r').
          *  If there are no visible borders at all in the entire rendering
          *  cache, this method returns null instead.
          */
@@ -1429,7 +1439,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
             if (!colCache.hasBorderStyles() && !rowCache.hasBorderStyles() && (borderEntries === 0)) { return null; }
 
             // shrink to visible area (needed for oversized merged ranges to find the correct start address)
-            range = sheetModel.getVisibleRange(range);
+            range = sheetModel.shrinkToVisibleRange(range);
             if (!range) { return null; }
 
             var // resulting cell matrix, mapped by cell address key
@@ -1520,7 +1530,11 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
                     }
                 } else {
                     var entry = matrix[key(cellData.address[0], cellData.address[1])];
-                    entry.borders = cellData.borders ? _.clone(cellData.borders) : {};
+                    // bug 36014: check that the matrix entry exists; merged range may start in
+                    // undefined area (e.g. undo of deleted columns with merged ranges)
+                    if (entry) {
+                        entry.borders = cellData.borders ? _.clone(cellData.borders) : {};
+                    }
                 }
             });
 
@@ -1616,7 +1630,7 @@ define('io.ox/office/spreadsheet/view/render/rendercache',
             colCache.destroy();
             rowCache.destroy();
             sheetModel.unregisterTransformationHandler(transformationHandler);
-            app = self = model = documentStyles = null;
+            app = self = docModel = documentStyles = null;
             sheetModel = cellCollection = colCollection = rowCollection = mergeCollection = null;
             colCache = rowCache = cellEntries = null;
             visibleMergedRanges = shrunkenMergedRanges = mergedColIntervals = null;
