@@ -1,32 +1,28 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/spreadsheet/view/statuspane',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/baseframework/view/toolpane',
-     'io.ox/office/baseframework/view/toolbar',
-     'io.ox/office/spreadsheet/view/controls',
-     'gettext!io.ox/office/spreadsheet'
-    ], function (Utils, Forms, ToolPane, ToolBar, Controls, gt) {
+define('io.ox/office/spreadsheet/view/statuspane', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/baseframework/view/toolpane',
+    'io.ox/office/baseframework/view/toolbar',
+    'io.ox/office/spreadsheet/view/labels',
+    'io.ox/office/spreadsheet/view/controls',
+    'gettext!io.ox/office/spreadsheet'
+], function (Utils, Forms, ToolPane, ToolBar, Labels, Controls, gt) {
 
     'use strict';
 
-    var // class name shortcuts
-        Button = Controls.Button,
-        DynamicLabel = Controls.DynamicLabel,
-        ComponentMenuButton = Controls.ComponentMenuButton;
-
     // class StatusPane =======================================================
 
     /**
@@ -42,22 +38,22 @@ define('io.ox/office/spreadsheet/view/statuspane',
         var // self reference
             self = this,
 
-            // get application model and view
-            view = app.getView(),
+            // application view instance
+            docView = app.getView(),
 
             // the tool bar for all sheet controls
             sheetToolBar = null,
 
-            // the tool bar for additional infos
-            infoToolBar = null,
-
             // the sheet selector controls
             sheetGroup = null,
             largeSheetList = null,
             smallSheetList = null,
+            moreMenuGroup = null,
 
-            // label for raw formula-view
-            formulaText = null;
+            // label control for the formula in the active cell
+            formulaLabel = null,
+            // old visible boolean for hiding this bar when soft keyboard is open on Android devices
+            visibleOverwrite = null;
 
         // base constructor ---------------------------------------------------
 
@@ -79,30 +75,31 @@ define('io.ox/office/spreadsheet/view/statuspane',
                 // the node of the sheet tool bar
                 sheetNode = sheetToolBar.getNode(),
                 // the width of the center node
-                avaSheetWidth = sheetNode.closest('.float-wrapper').width(),
+                avaSheetWidth = Utils.getFloorNodeSize(sheetNode.closest('.float-wrapper')).width,
 
                 // max width of the info node in percent
                 minSheetWidth = 240;
 
-            function getCurrentWidth(node) {
-                return Math.ceil(node.outerWidth());
+            function getToolBarWidth() {
+                return Utils.getCeilNodeSize(sheetNode).width;
             }
 
             // calculate and set remaining space for the specified sheet selector
             function setRemainingWidth(group) {
-                var remainingWidth = group.getNode().outerWidth() + avaSheetWidth - getCurrentWidth(sheetNode);
+                var remainingWidth = group.getNode().outerWidth() + avaSheetWidth - getToolBarWidth();
                 group.setWidth(remainingWidth);
             }
 
+            // hide the sheet list selectors, show the sheet tab selector with automatic width
             largeSheetList.hide();
             smallSheetList.hide();
             sheetGroup.show();
             sheetGroup.setFullWidth();
 
-            // hide the sheet list selectors, show the sheet tab selector with automatic width
-            sheetNode.css({ width: 'auto', paddingLeft: view.getHeaderWidth() + 'px' });
+            // add leading padding with the width of the row header panes
+            sheetNode.css({ width: 'auto', paddingLeft: docView.getHeaderWidth() + 'px' });
 
-            if (avaSheetWidth < getCurrentWidth(sheetNode)) {
+            if (avaSheetWidth < getToolBarWidth()) {
                 sheetNode.css({ paddingLeft: '' });
 
                 if (avaSheetWidth < 380) {
@@ -111,10 +108,10 @@ define('io.ox/office/spreadsheet/view/statuspane',
                         // set fix min width for sheets
                         avaSheetWidth = minSheetWidth;
                         // set remaining width (incl. 20px padding) to formula
-                        formulaText.setWidth(paneWidth - minSheetWidth - 20);
+                        formulaLabel.setWidth(paneWidth - minSheetWidth - 20);
                     } else {
                         // reset width of formula
-                        formulaText.setWidth('auto');
+                        formulaLabel.setWidth('auto');
                     }
                     // show large sheet drop-down list for small devices
                     sheetGroup.hide();
@@ -130,7 +127,7 @@ define('io.ox/office/spreadsheet/view/statuspane',
                 }
             } else {
                 // reset width of formula
-                formulaText.setWidth('auto');
+                formulaLabel.setWidth('auto');
             }
 
             // reset sheet component to its current width
@@ -139,6 +136,16 @@ define('io.ox/office/spreadsheet/view/statuspane',
 
         // initialization -----------------------------------------------------
 
+        // create the 'More' drop-down menu with all standard actions for sheets
+        moreMenuGroup = new Controls.CompoundButton(app, { label: Labels.MORE_LABEL, tooltip: gt('More sheet operations') })
+            .addGroup('sheet/rename/dialog', new Controls.Button({ label: Labels.RENAME_SHEET_LABEL }))
+            .addGroup('sheet/copy/dialog', new Controls.Button({ label: Labels.COPY_SHEET_LABEL }))
+            .addGroup('sheet/delete', new Controls.Button({ label: Labels.DELETE_SHEET_LABEL }))
+            .addGroup('sheet/visible', new Controls.Button({ label: Labels.HIDE_SHEET_LABEL, value: false }))
+            .addSeparator()
+            .addGroup('sheet/reorder/dialog', new Controls.Button({ label: Labels.REORDER_SHEETS_LABEL }))
+            .addGroup('sheet/showall', new Controls.Button({ label: Labels.SHOW_SHEETS_LABEL }));
+
         this.addViewComponent(sheetToolBar = new ToolBar(app)
             .addGroup('view/sheet/active', largeSheetList = new Controls.ActiveSheetList(app, { showNames: true }))
             .addGap()
@@ -146,22 +153,14 @@ define('io.ox/office/spreadsheet/view/statuspane',
             .addGap()
             .addGroup('view/sheet/active', sheetGroup = new Controls.ActiveSheetGroup(app))
             .addGap()
-            .addGroup('sheet/insert', new Button({ icon: 'fa-plus', tooltip: gt('Insert sheet') }))
-            .addGroup(null, new ComponentMenuButton(app, { label: Controls.MORE_LABEL, tooltip: gt('More sheet operations'), position: 'top' })
-                .addGroup('sheet/rename/dialog', new Button({ label: gt('Rename sheet') }))
-                .addGroup('sheet/copy/dialog', new Button({ label: gt('Copy sheet') }))
-                .addGroup('sheet/delete', new Button({ label: gt('Delete sheet') }))
-                .addGroup('sheet/visible', new Button({ label: gt('Hide sheet'), value: false }))
-                .addSeparator()
-                .addGroup('sheet/reorder/dialog', new Button({ label: gt('Reorder sheets') }))
-                .addGroup('sheet/showall', new Button({ label: gt('Show all hidden sheets') }))
-            )
+            .addGroup('sheet/insert', new Controls.Button({ icon: 'fa-plus', tooltip: gt('Insert sheet') }))
+            .addGroup('view/sheet/more', moreMenuGroup)
         );
 
-        this.addViewComponent(infoToolBar = new ToolBar(app)
+        this.addViewComponent(new ToolBar(app)
             .addGroup('view/status', new Controls.DynamicLabel(), { visibleKey: 'view/status' })
-            .addGroup('view/subtotals/type', new Controls.SubtotalList(app, { position: 'top' }), { visibleKey: 'view/subtotals/type' })
-            .addGroup('view/formula', formulaText = new DynamicLabel({ classes: 'formula-text', setToolTip: true }), { visibleKey: 'view/formula'} ),
+            .addGroup('view/subtotals/type', new Controls.SubtotalList(app), { visibleKey: 'view/subtotals/type' })
+            .addGroup('view/formula', formulaLabel = new Controls.DynamicLabel({ classes: 'formula-text', setToolTip: true }), { visibleKey: 'view/formula'} ),
         { targetArea: 'trailing' });
 
         // refresh sheet tool bar after sheet tabs have been inserted, deleted, or changed
@@ -171,9 +170,30 @@ define('io.ox/office/spreadsheet/view/statuspane',
         // always hide the status pane when an internal application error occurs
         this.listenTo(app, 'docs:state:error', function () { self.hide(); });
 
+        if (_.browser.Android) {
+            //".window-container" is oriented in screen with "top left right bottom" set to zero,
+            //when the soft keyboard on Android devices pops up, the status bar stays over the keyboard
+            //this makes no sense and looks ugly, so we detect the open keyboard and hide the status pane
+            app.registerWindowResizeHandler(function () {
+                var softKeyboard = Utils.isSoftKeyboardOpen();
+                if (softKeyboard) {
+                    if (visibleOverwrite === null && self.isVisible()) {
+                        visibleOverwrite = true;
+                        self.hide();
+                    }
+                } else {
+                    if (visibleOverwrite !== null) {
+                        self.toggle(visibleOverwrite);
+                    }
+                    visibleOverwrite = null;
+                }
+            } );
+        }
+
         // destroy class members on destruction
         this.registerDestructor(function () {
-            app = self = view = sheetToolBar = sheetGroup = largeSheetList = smallSheetList = null;
+            app = self = docView = null;
+            sheetToolBar = sheetGroup = largeSheetList = smallSheetList = moreMenuGroup = formulaLabel = null;
         });
 
     } // class StatusPane
