@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  * @author Malte Timmermann <malte.timmermann@open-xchange.com>
@@ -57,7 +57,10 @@ define('io.ox/office/text/app/application',
             operationFilter: operationFilter,
             importFailedHandler: importFailedHandler,
             sendActionsDelay: 1000,
-            localStorageApp: true
+            localStorageApp: true,
+            requiredStorageVersion: 2,
+            supportedStorageVersion: 2,
+            supportsOnlineSync: true
         });
 
         // private methods ----------------------------------------------------
@@ -152,7 +155,7 @@ define('io.ox/office/text/app/application',
         // initialization -----------------------------------------------------
 
         // destroy all class members
-        this.on('docs:destroy', function () {
+        this.registerDestructor(function () {
             launchOptions = appOptions = null;
         });
 
