@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -17,14 +17,16 @@ define('io.ox/office/text/app/controller',
      'io.ox/office/editframework/utils/mixedborder',
      'io.ox/office/editframework/utils/lineheight',
      'io.ox/office/editframework/app/editcontroller',
+     'io.ox/office/drawinglayer/view/drawinglabels',
      'io.ox/office/drawinglayer/view/imageutil',
      'io.ox/office/text/utils/config',
      'io.ox/office/text/hyperlink',
+     'io.ox/office/text/position',
      'io.ox/office/text/format/listcollection',
      'io.ox/office/text/format/paragraphstyles',
      'io.ox/office/text/format/tablestyles',
      'io.ox/office/text/format/drawingstyles'
-    ], function (Utils, Border, MixedBorder, LineHeight, EditController, Image, Config, Hyperlink, ListCollection, ParagraphStyles, TableStyles, DrawingStyles) {
+    ], function (Utils, Border, MixedBorder, LineHeight, EditController, DrawingLabels, Image, Config, Hyperlink, Position, ListCollection, ParagraphStyles, TableStyles, DrawingStyles) {
 
     'use strict';
 
@@ -82,6 +84,13 @@ define('io.ox/office/text/app/controller',
                     enable: function () { return model.isTextSelected(); }
                 },
 
+                'document/editable/text/notextframe': {
+                    parent: 'document/editable/text',
+                    enable: function () {
+                        return !model.getSelection().isAdditionalTextframeSelection();
+                    }
+                },
+
                 'document/selectall': {
                     // enabled in read-only mode
                     set: function () { model.selectAll(); },
@@ -90,6 +99,19 @@ define('io.ox/office/text/app/controller',
                     preserveFocus: true // do not return focus to document
                 },
 
+                'document/selectDrawing': {
+                    set: function (options) {
+                        model.getSelection().selectNextDrawing({backwards: Utils.getBooleanOption(options, 'backwards', false)});
+                    },
+                    shortcut: { keyCode: 'F4', shift: true }
+                },
+                'document/setCursorIntoTextframe': {
+                    set: function () {
+                        model.getSelection().setSelectionIntoTextframe();
+                    },
+                    shortcut: { keyCode: 'F2' }
+                },
+
                 // document contents
 
                 'document/search/highlight': {
@@ -348,10 +370,15 @@ define('io.ox/office/text/app/controller',
                     set: function () { return model.insertTab(); }
                 },
                 'character/insert/pagebreak': {
-                    parent: 'document/editable/text',
+                    parent: 'document/editable/text/notextframe',
                     set: function () { return model.insertManualPageBreak(); }
                 },
 
+                // reduced odf text frame functionality
+                'insert/textframe/supported': {
+                    enable: function () { return !model.isReducedOdfTextframeFunctionality(); }
+                },
+
                 // tables
 
                 'table/group': {
@@ -362,7 +389,7 @@ define('io.ox/office/text/app/controller',
                     enable: function () { return Config.MAX_TABLE_CELLS > 0; }
                 },
                 'table/insert': {
-                    parent: ['document/editable/text', 'table/insert/available'],
+                    parent: ['document/editable/text', 'table/insert/available', 'insert/textframe/supported'],
                     set: function (size) { model.insertTable(size); },
                     preserveFocus: true // do not return focus to document
                 },
@@ -434,25 +461,114 @@ define('io.ox/office/text/app/controller',
                     parent: 'document/editable',
                     enable: function () { return model.isDrawingSelected(); }
                 },
-                'drawing/delete': {
-                    parent: 'document/editable/drawing',
-                    set: function () { return model.deleteSelected(); }
+                'document/editable/anydrawing': {
+                    parent: 'document/editable',
+                    enable: function () { return model.getSelection().isAnyDrawingSelection(); }
+                },
+                'document/editable/onlysupporteddrawings': {
+                    parent: 'document/editable',
+                    enable: function () { return model.getSelection().areOnlySupportedDrawingsSelected(); }
+                },
+                'drawing/shapes/support/fill': {
+                    parent: 'document/editable/onlysupporteddrawings',
+                    enable: function () { return (/^(shape|group)$/).test(model.getSelection().getSelectedDrawingType()) || (model.getSelection().getSelectionType() === 'text'); }
+                },
+                'drawing/shapes/support/line': {
+                    parent: 'document/editable/onlysupporteddrawings',
+                    enable: function () { return (/^(shape|group|image)$/).test(model.getSelection().getSelectedDrawingType()) || (model.getSelection().getSelectionType() === 'text'); }
                 },
 
                 'drawing/attributes': {
-                    parent: 'document/editable/drawing',
+                    parent: 'document/editable/anydrawing',
                     get: function () { return model.getAttributes('drawing').drawing || {}; }
                 },
+                'drawing/group/attributes': {
+                    parent: 'document/editable/anydrawing',
+                    get: function () { return model.getAttributes('drawing', {groupOnly: true}).drawing || {}; }
+                },
+
+                'drawing/delete': {
+                    parent: 'document/editable/anydrawing',
+                    set: function () {
+                        if (model.isDrawingSelected()) {
+                            return model.deleteSelected();
+
+                        } else if (model.getSelection().isAdditionalTextframeSelection()) {
+                            var start = Position.getOxoPosition(model.getNode(), model.getSelection().getSelectedTextFrameDrawing(), 0),
+                                end = Position.increaseLastIndex(start);
+
+                            model.getSelection().setTextSelection(start, end);
+                            return model.deleteSelected();
+                        }
+                    }
+                },
+                'drawing/anchorTo' : {
+                    parent: 'drawing/group/attributes',
+                    enable: function () { return !model.getSelection().isDrawingInTextFrameSelection() || model.getSelection().isDrawingInODFTextFrameSelection(); },
+                    get: function (attributes) { return (attributes.inline) ? 'inline' : attributes.anchorVertBase; },
+                    set: function (value) { model.anchorDrawingTo(value); }
+                },
                 'drawing/position': {
-                    parent: 'drawing/attributes',
+                    parent: 'drawing/group/attributes',
+                    enable: function (attributes) { return (!attributes.inline); },
                     get: function (attributes) { return DrawingStyles.getPositionFromAttributes(attributes); },
-                    set: function (position) { model.setAttributes('drawing', { drawing: DrawingStyles.getAttributesFromPosition(position, model.getAttributes('drawing').drawing) }, { ctSupportsDrawingPosition: model.getChangeTrack().ctSupportsDrawingPosition() }); }
+                    set: function (position) {
+                        model.setAttributes('drawing', {
+                            drawing: DrawingStyles.getAttributesFromPosition(position, model.getAttributes('drawing').drawing)
+                        }, {
+                            ctSupportsDrawingPosition: model.getChangeTrack().ctSupportsDrawingPosition()
+                        });
+                    }
+                },
+
+                'drawing/type/label': {
+                    parent: 'document/editable/drawing',
+                    get: function () { return DrawingLabels.getDrawingTypeLabel(model.getSelection().getClosestSelectedDrawingType()); }
+                },
+
+                // drawing fill formatting
+
+                'drawing/fill/attributes': {
+                    parent: 'document/editable/anydrawing',
+                    get: function () { return model.getAttributes('drawing').fill || {}; }
+                },
+
+                'drawing/fill/color' : {
+                    parent: 'drawing/fill/attributes',
+                    get: function (attributes) { return attributes.color; },
+                    set: function (color) { model.setDrawingFillColor(color); }
+                },
+
+                // drawing line formatting
+
+                'drawing/line/attributes': {
+                    parent: 'document/editable/anydrawing',
+                    get: function () { return model.getAttributes('drawing').line || {}; }
+                },
+
+                'drawing/line/style' : {
+                    parent: 'drawing/line/attributes',
+                    get: function (attributes) { return model.detectBorderStyle(attributes); },
+                    set: function (border) { model.setDrawingBorder(border); }
+                },
+
+                'drawing/line/color' : {
+                    parent: 'drawing/line/attributes',
+                    get: function (attributes) { return (attributes.type !== 'none') ? attributes.color : 'auto'; },
+                    set: function (color) { model.setDrawingBorderColor(color); }
+                },
+
+                // text frames
+
+                'textframe/insert' : {
+                    parent: 'document/editable/text/notextframe',
+                    set: function () { model.insertTextFrame(); }
                 },
 
                 // images
 
                 'image/insert/dialog': {
-                    parent: 'document/editable/text',
+                    parent: ['document/editable/text', 'insert/textframe/supported'],
                     set: function () { return model.showInsertImageDialog(); }
                 },
 
@@ -525,7 +641,6 @@ define('io.ox/office/text/app/controller',
                 },
 
                 // debug
-
                 'format/spellchecking' : {
                     parent: 'character/attributes',
                     enable: function () { return model.hasSelectedRange() && model.hasEnclosingParagraph(); },
@@ -536,6 +651,9 @@ define('io.ox/office/text/app/controller',
                     enable: function () { return model.hasSelectedRange(); },
                     set: function (mail) { model.accessRightsExport(mail); }
                 },
+                'debug/insertDrawingGroup' : {
+                    set: function () { model.insertDrawingGroup(); }
+                },
                 'debug/uselocalstorage' : {
                     get: function () { return app.getUserSettingsValue('debugUseLocalStorage', Config.USE_LOCAL_STORAGE); },
                     set: function (state) { app.setUserSettingsValue('debugUseLocalStorage', state); }
@@ -553,20 +671,6 @@ define('io.ox/office/text/app/controller',
                     enable: function () { return app.getView().getDebugOperationsContent() !== null; },
                     set: function () { model.replayOperations(app.getView().getDebugOperationsContent()); }
                 },
-                'debug/operations' : {
-                    parent: 'debug/enabled'
-                },
-                'debug/getpagehtmlcode' : {
-                    set: function () { model.getFullModelDescription().done(function (docString) { app.getView().setPageHtmlCode(docString); }); }
-                },
-                'debug/setpagehtmlcode' : {
-                    parent: 'document/editable',
-                    enable: function () { return app.getView().getPageHtmlCode() !== null; },
-                    set: function () { model.setFullModelNode(app.getView().getPageHtmlCode()); }
-                },
-                'debug/pagehtmlcode' : {
-                    parent: 'debug/enabled'
-                },
                 'debug/pagebreaks/toggle' : {
                     parent: 'debug/enabled',
                     enable: function () { return !model.isDraftMode() && !Utils.SMALL_DEVICE; },
@@ -578,7 +682,20 @@ define('io.ox/office/text/app/controller',
                     enable: function () { return !Utils.SMALL_DEVICE; },
                     get: function () { return model.isDraftMode(); },
                     set: function (state) { model.toggleDraftMode(state); }
-                }
+                },
+
+                // android-entry for selection which cant be deleted by softkeyboard
+                'document/editable/selection': {
+                    parent: 'document/editable',
+                    enable: function () {
+                        var selection = model.getSelection();
+                        return !_.isEqual(selection.getStartPosition(), selection.getEndPosition());
+                    }
+                },
+                'selection/delete': {
+                    parent: 'document/editable/selection',
+                    set: function () { return model.deleteSelected(); }
+                },
 
             };
 
@@ -660,8 +777,8 @@ define('io.ox/office/text/app/controller',
             var originalUpdate = self.update;
 
             self.update = function() {
-                if (app.getModel().isAndroidTyping()) {
-                    app.executeDelayed(self.update, {delay: 500});
+                if (model.isAndroidTyping()) {
+                    self.executeDelayed(self.update, 1000);
                 } else {
                     originalUpdate();
                 }
