@@ -1,21 +1,21 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2011
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Mario Schroeder <mario.schroeder@open-xchange.com>
  */
 
-define('io.ox/office/text/clipboardmixin',
-        ['io.ox/office/text/operations',
-         'io.ox/office/text/dom',
-         'io.ox/office/tk/utils'
-        ], function (Operations, DOM, Utils) {
+define('io.ox/office/text/clipboardmixin', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/text/utils/operations',
+    'io.ox/office/text/dom'
+], function (Utils, Operations, DOM) {
 
     'use strict';
 
@@ -126,7 +126,7 @@ define('io.ox/office/text/clipboardmixin',
                     if (child.nodeType === 3 && acceptTextNodes && !skipTextNodes) {
                         // handle non-whitespace characters and non-breaking spaces only
                         // except for our own text portion spans
-                        if (DOM.isTextSpan(child.parentNode) || (/\S|\u00a0/.test(child.nodeValue))) {
+                        if (DOM.isTextSpan(child.parentNode) || (/\S|\xa0/.test(child.nodeValue))) {
                             splitted = child.nodeValue.match(/[^\t]+|\t/g) || '';
                             for (var j = 0; j < splitted.length; j++) {
                                 if (splitted[j] === '\t') {
@@ -137,7 +137,7 @@ define('io.ox/office/text/clipboardmixin',
                                     // text
                                     // replace '\r' and '\n' with space to fix pastes from aoo,
                                     // replace invisible control characters and reduce space sequences to a single spaces
-                                    text = splitted[j].replace(/[\r\n]/g, ' ').replace(/[\u0000-\u001F]/g, '').replace(/\u0020+/g, ' ');
+                                    text = splitted[j].replace(/[\r\n]/g, ' ').replace(/[\x00-\x1f]/g, '').replace(/ +/g, ' ');
                                     insertLength += text.length;
                                     result.push({operation: Operations.TEXT_INSERT, data: text, depth: depth});
                                 }
@@ -782,7 +782,7 @@ define('io.ox/office/text/clipboardmixin',
             switch (char.charCodeAt(0)) {
             case 61472:
             case 32:
-                return { levelText: '\u0020', fontFamily: 'Times New Roman' };  // Space
+                return { levelText: ' ', fontFamily: 'Times New Roman' };  // Space
 
             case 61473:
             case 33:
@@ -950,11 +950,11 @@ define('io.ox/office/text/clipboardmixin',
 
             case 61546:
             case 106:
-                return { levelText: '\u0026', fontFamily: 'Times New Roman' };  // Ampersand (italic, lower-case)
+                return { levelText: '&', fontFamily: 'Times New Roman' };  // Ampersand (italic, lower-case)
 
             case 61547:
             case 107:
-                return { levelText: '\u0026', fontFamily: 'Times New Roman' };  // Ampersand (italic)
+                return { levelText: '&', fontFamily: 'Times New Roman' };  // Ampersand (italic)
 
             case 61548:
             case 108:
@@ -1118,7 +1118,7 @@ define('io.ox/office/text/clipboardmixin',
 
             case 61598:
             case 158:
-                return { levelText: '\u00b7', fontFamily: 'Times New Roman' };  // Middle dot
+                return { levelText: '\xb7', fontFamily: 'Times New Roman' };  // Middle dot
 
             case 61599:
             case 159:
@@ -1302,11 +1302,11 @@ define('io.ox/office/text/clipboardmixin',
             switch (char.charCodeAt(0)) {
             case 32:
             case 61472:
-                return { levelText: '\u0020', fontFamily: 'Times New Roman' };  // Space
+                return { levelText: ' ', fontFamily: 'Times New Roman' };  // Space
 
             case 33:
             case 61473:
-                return { levelText: '\u0021', fontFamily: 'Times New Roman' };  // Exclamation mark
+                return { levelText: '!', fontFamily: 'Times New Roman' };  // Exclamation mark
 
             case 34:
             case 61474:
@@ -1314,7 +1314,7 @@ define('io.ox/office/text/clipboardmixin',
 
             case 35:
             case 61475:
-                return { levelText: '\u0023', fontFamily: 'Times New Roman' };  // Number sign
+                return { levelText: '#', fontFamily: 'Times New Roman' };  // Number sign
 
             case 36:
             case 61476:
@@ -1322,11 +1322,11 @@ define('io.ox/office/text/clipboardmixin',
 
             case 37:
             case 61477:
-                return { levelText: '\u0025', fontFamily: 'Times New Roman' };  // Percent sign
+                return { levelText: '%', fontFamily: 'Times New Roman' };  // Percent sign
 
             case 38:
             case 61478:
-                return { levelText: '\u0026', fontFamily: 'Times New Roman' };  // Ampersand
+                return { levelText: '&', fontFamily: 'Times New Roman' };  // Ampersand
 
             case 39:
             case 61479:
@@ -1334,11 +1334,11 @@ define('io.ox/office/text/clipboardmixin',
 
             case 40:
             case 61480:
-                return { levelText: '\u0028', fontFamily: 'Times New Roman' };  // Left Parenthesis
+                return { levelText: '(', fontFamily: 'Times New Roman' };  // Left Parenthesis
 
             case 41:
             case 61481:
-                return { levelText: '\u0029', fontFamily: 'Times New Roman' };  // Right Parenthesis
+                return { levelText: ')', fontFamily: 'Times New Roman' };  // Right Parenthesis
 
             case 42:
             case 61482:
@@ -1346,11 +1346,11 @@ define('io.ox/office/text/clipboardmixin',
 
             case 43:
             case 61483:
-                return { levelText: '\u002b', fontFamily: 'Times New Roman' };  // Plus sign
+                return { levelText: '+', fontFamily: 'Times New Roman' };  // Plus sign
 
             case 44:
             case 61484:
-                return { levelText: '\u002c', fontFamily: 'Times New Roman' };  // Comma
+                return { levelText: ',', fontFamily: 'Times New Roman' };  // Comma
 
             case 45:
             case 61485:
@@ -1358,75 +1358,75 @@ define('io.ox/office/text/clipboardmixin',
 
             case 46:
             case 61486:
-                return { levelText: '\u002e', fontFamily: 'Times New Roman' };  // Full stop
+                return { levelText: '.', fontFamily: 'Times New Roman' };  // Full stop
 
             case 47:
             case 61487:
-                return { levelText: '\u002f', fontFamily: 'Times New Roman' };  // Solidus
+                return { levelText: '/', fontFamily: 'Times New Roman' };  // Solidus
 
             case 48:
             case 61488:
-                return { levelText: '\u0030', fontFamily: 'Times New Roman' };  // Digit zero
+                return { levelText: '0', fontFamily: 'Times New Roman' };  // Digit zero
 
             case 49:
             case 61489:
-                return { levelText: '\u0031', fontFamily: 'Times New Roman' };  // Digit one
+                return { levelText: '1', fontFamily: 'Times New Roman' };  // Digit one
 
             case 50:
             case 61490:
-                return { levelText: '\u0032', fontFamily: 'Times New Roman' };  // Digit two
+                return { levelText: '2', fontFamily: 'Times New Roman' };  // Digit two
 
             case 51:
             case 61491:
-                return { levelText: '\u0033', fontFamily: 'Times New Roman' };  // Digit three
+                return { levelText: '3', fontFamily: 'Times New Roman' };  // Digit three
 
             case 52:
             case 61492:
-                return { levelText: '\u0034', fontFamily: 'Times New Roman' };  // Digit four
+                return { levelText: '4', fontFamily: 'Times New Roman' };  // Digit four
 
             case 53:
             case 61493:
-                return { levelText: '\u0035', fontFamily: 'Times New Roman' };  // Digit five
+                return { levelText: '5', fontFamily: 'Times New Roman' };  // Digit five
 
             case 54:
             case 61494:
-                return { levelText: '\u0036', fontFamily: 'Times New Roman' };  // Digit six
+                return { levelText: '6', fontFamily: 'Times New Roman' };  // Digit six
 
             case 55:
             case 61495:
-                return { levelText: '\u0037', fontFamily: 'Times New Roman' };  // Digit seven
+                return { levelText: '7', fontFamily: 'Times New Roman' };  // Digit seven
 
             case 56:
             case 61496:
-                return { levelText: '\u0038', fontFamily: 'Times New Roman' };  // Digit eight
+                return { levelText: '8', fontFamily: 'Times New Roman' };  // Digit eight
 
             case 57:
             case 61497:
-                return { levelText: '\u0039', fontFamily: 'Times New Roman' };  // Digit nine
+                return { levelText: '9', fontFamily: 'Times New Roman' };  // Digit nine
 
             case 58:
             case 61498:
-                return { levelText: '\u003a', fontFamily: 'Times New Roman' };  // Colon
+                return { levelText: ':', fontFamily: 'Times New Roman' };  // Colon
 
             case 59:
             case 61499:
-                return { levelText: '\u003b', fontFamily: 'Times New Roman' };  // Semicolon
+                return { levelText: ';', fontFamily: 'Times New Roman' };  // Semicolon
 
             case 60:
             case 61500:
-                return { levelText: '\u003c', fontFamily: 'Times New Roman' };  // Less-than sign
+                return { levelText: '<', fontFamily: 'Times New Roman' };  // Less-than sign
 
             case 61:
             case 61501:
-                return { levelText: '\u003d', fontFamily: 'Times New Roman' };  // Equals sign
+                return { levelText: '=', fontFamily: 'Times New Roman' };  // Equals sign
 
             case 62:
             case 61502:
-                return { levelText: '\u003e', fontFamily: 'Times New Roman' };  // Greater-than sign
+                return { levelText: '>', fontFamily: 'Times New Roman' };  // Greater-than sign
 
             case 63:
             case 61503:
-                return { levelText: '\u003f', fontFamily: 'Times New Roman' };  // Question mark
+                return { levelText: '?', fontFamily: 'Times New Roman' };  // Question mark
 
             case 64:
             case 61504:
@@ -1538,7 +1538,7 @@ define('io.ox/office/text/clipboardmixin',
 
             case 91:
             case 61531:
-                return { levelText: '\u005b', fontFamily: 'Times New Roman' };  // Left square bracket
+                return { levelText: '[', fontFamily: 'Times New Roman' };  // Left square bracket
 
             case 92:
             case 61532:
@@ -1546,7 +1546,7 @@ define('io.ox/office/text/clipboardmixin',
 
             case 93:
             case 61533:
-                return { levelText: '\u005d', fontFamily: 'Times New Roman' };  // Right square bracket
+                return { levelText: ']', fontFamily: 'Times New Roman' };  // Right square bracket
 
             case 94:
             case 61534:
@@ -1554,11 +1554,11 @@ define('io.ox/office/text/clipboardmixin',
 
             case 95:
             case 61535:
-                return { levelText: '\u005f', fontFamily: 'Times New Roman' };  // Low line
+                return { levelText: '_', fontFamily: 'Times New Roman' };  // Low line
 
             case 96:
             case 61536:
-                return { levelText: '\u00af', fontFamily: 'Times New Roman' };  // Macron
+                return { levelText: '\xaf', fontFamily: 'Times New Roman' };  // Macron
 
             case 97:
             case 61537:
@@ -1666,15 +1666,15 @@ define('io.ox/office/text/clipboardmixin',
 
             case 123:
             case 61563:
-                return { levelText: '\u007b', fontFamily: 'Times New Roman' };  // Left curly bracket
+                return { levelText: '{', fontFamily: 'Times New Roman' };  // Left curly bracket
 
             case 124:
             case 61564:
-                return { levelText: '\u007c', fontFamily: 'Times New Roman' };  // Vertical line
+                return { levelText: '|', fontFamily: 'Times New Roman' };  // Vertical line
 
             case 125:
             case 61565:
-                return { levelText: '\u007d', fontFamily: 'Times New Roman' };  // Right curly bracket
+                return { levelText: '}', fontFamily: 'Times New Roman' };  // Right curly bracket
 
             case 126:
             case 61566:
@@ -1746,11 +1746,11 @@ define('io.ox/office/text/clipboardmixin',
 
             case 176:
             case 61616:
-                return { levelText: '\u00b0', fontFamily: 'Times New Roman' };  // Degree sign
+                return { levelText: '\xb0', fontFamily: 'Times New Roman' };  // Degree sign
 
             case 177:
             case 61617:
-                return { levelText: '\u00b1', fontFamily: 'Times New Roman' };  // Plus-minus sign
+                return { levelText: '\xb1', fontFamily: 'Times New Roman' };  // Plus-minus sign
 
             case 178:
             case 61618:
@@ -1762,7 +1762,7 @@ define('io.ox/office/text/clipboardmixin',
 
             case 180:
             case 61620:
-                return { levelText: '\u00d7', fontFamily: 'Times New Roman' };  // Multiplication sign
+                return { levelText: '\xd7', fontFamily: 'Times New Roman' };  // Multiplication sign
 
             case 181:
             case 61621:
@@ -1778,7 +1778,7 @@ define('io.ox/office/text/clipboardmixin',
 
             case 184:
             case 61624:
-                return { levelText: '\u00f7', fontFamily: 'Times New Roman' };  // Division sign
+                return { levelText: '\xf7', fontFamily: 'Times New Roman' };  // Division sign
 
             case 185:
             case 61625:
@@ -1794,11 +1794,11 @@ define('io.ox/office/text/clipboardmixin',
 
             case 188:
             case 61628:
-                return { levelText: '\u2026', fontFamily: 'Times New Roman' };  // Horizontal ellipsis
+                return { levelText: Utils.ELLIPSIS_CHAR, fontFamily: 'Times New Roman' };  // Horizontal ellipsis
 
             case 189:
             case 61629:
-                return { levelText: '\u007c', fontFamily: 'Times New Roman' };  // Pipe symbol
+                return { levelText: '|', fontFamily: 'Times New Roman' };  // Pipe symbol
 
             case 190:
             case 61630:
@@ -1882,11 +1882,11 @@ define('io.ox/office/text/clipboardmixin',
 
             case 210:
             case 61650:
-                return { levelText: '\u00ae', fontFamily: 'Times New Roman' };  // Registered sign
+                return { levelText: '\xae', fontFamily: 'Times New Roman' };  // Registered sign
 
             case 211:
             case 61651:
-                return { levelText: '\u00a9', fontFamily: 'Times New Roman' };  // Copyright sign
+                return { levelText: '\xa9', fontFamily: 'Times New Roman' };  // Copyright sign
 
             case 212:
             case 61652:
@@ -1906,7 +1906,7 @@ define('io.ox/office/text/clipboardmixin',
 
             case 216:
             case 61656:
-                return { levelText: '\u00ac', fontFamily: 'Times New Roman' };  // Not sign
+                return { levelText: '\xac', fontFamily: 'Times New Roman' };  // Not sign
 
             case 217:
             case 61657:
@@ -1946,11 +1946,11 @@ define('io.ox/office/text/clipboardmixin',
 
             case 226:
             case 61666:
-                return { levelText: '\u00ae', fontFamily: 'Arial' };  // Registered sign
+                return { levelText: '\xae', fontFamily: 'Arial' };  // Registered sign
 
             case 227:
             case 61667:
-                return { levelText: '\u00a9', fontFamily: 'Arial' };  // Copyright sign
+                return { levelText: '\xa9', fontFamily: 'Arial' };  // Copyright sign
 
             case 228:
             case 61668:
