@@ -1,22 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Oliver Specht <oliver.specht@open-xchange.com>
  */
 
-define('io.ox/office/text/format/listcollection',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/baseframework/model/modelobject',
-     'io.ox/office/text/operations',
-     'gettext!io.ox/office/text'
-    ], function (Utils, ModelObject, Operations, gt) {
+define('io.ox/office/text/format/listcollection', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/baseframework/model/modelobject',
+    'io.ox/office/text/utils/operations',
+    'gettext!io.ox/office/text'
+], function (Utils, ModelObject, Operations, gt) {
 
     'use strict';
 
