@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -128,7 +128,7 @@ define('io.ox/office/text/format/paragraphstyles',
             pageBreakBefore: {
                 def: false,
                 format: function (element, value) {
-                    if (value) {
+                    if (value && !DOM.isNodeInsideTextFrame(element)) {
                         if (element.closest('tr').length > 0) {
                             element.parents('table').last().addClass('manual-page-break');
                         }
@@ -139,7 +139,7 @@ define('io.ox/office/text/format/paragraphstyles',
             pageBreakAfter: {
                 def: false,
                 format: function (element, value) {
-                    if (value) {
+                    if (value && !DOM.isNodeInsideTextFrame(element)) {
                         if (element.closest('tr').length > 0) {
                             element.parents('table').last().addClass('manual-pb-after');
                         }
@@ -155,7 +155,9 @@ define('io.ox/office/text/format/paragraphstyles',
         },
 
         // maps fill character attribute values to fill characters
-        TAB_FILL_CHARS = { dot: '.', hyphen: '-', underscore: '_' };
+        TAB_FILL_CHARS = { dot: '.', hyphen: '-', underscore: '_' },
+
+        NBSP = '\xa0';
 
     // private static functions ===============================================
 
@@ -170,6 +172,19 @@ define('io.ox/office/text/format/paragraphstyles',
                  'listStyleId', 'listLevel', 'indentLeft', 'indentRight', 'indentFirstLine']);
     }
 
+    /**
+     * Utility check if attributes have properties pageBreakBefore or pageBreakAfter
+     * set to true.
+     *
+     * @param {Object} attributes
+     *
+     * @returns {Boolean}
+     *
+     */
+    function isSetPageBreakAttribute(attributes) {
+        return attributes.pageBreakBefore === true || attributes.pageBreakAfter === true;
+    }
+
     /**
      * Fills the passed text span with a sufficient number of the specified
      * fill character.
@@ -186,14 +201,16 @@ define('io.ox/office/text/format/paragraphstyles',
      */
     function insertTabFillChar(spanNode, fillChar, width) {
 
-        var // 5 fill characters, used to calculate average character width
-            checkString = Utils.repeatString(fillChar, 5),
+        var // multiplier for a better average calulation
+            multiplier = 15,
+            // multiplier fill characters, used to calculate average character width
+            checkString = Utils.repeatString(fillChar, multiplier),
             // average character width, in 1/100 mm
-            charWidth = Utils.convertLengthToHmm(spanNode.contents().remove().end().text(checkString).width(), 'px') / 5,
+            charWidth = Utils.convertLengthToHmm(spanNode.contents().remove().end().text(checkString).width(), 'px') / multiplier,
             // number of characters needed to fill the specified width
             charCount = Math.floor(width / charWidth),
             // the fill character, repeated by the calculated number
-            fillString = (charCount > 0) ? Utils.repeatString(fillChar, charCount) : '\u00a0',
+            fillString = (charCount > 0) ? Utils.repeatString(fillChar, charCount) : NBSP,
             // a shortened string, if element is too wide
             shortFillString = null;
 
@@ -213,7 +230,7 @@ define('io.ox/office/text/format/paragraphstyles',
         // with empty spans
         if (_.device('ios') && _.device('safari')) {
             if (!spanNode[0].nextSibling) {
-                spanNode.after(document.createTextNode('\u00a0'));
+                spanNode.after(document.createTextNode(NBSP));
             }
         }
     }
@@ -244,7 +261,7 @@ define('io.ox/office/text/format/paragraphstyles',
         // base constructor ---------------------------------------------------
 
         StyleCollection.call(this, app, documentStyles, 'paragraph', {
-            additionalFamilies: ['character', 'changes'],
+            families: 'character changes',
             parentResolvers: PARENT_RESOLVERS,
             formatHandler: updateParagraphFormatting,
             previewHandler: setParagraphPreviewFormatting
@@ -297,7 +314,9 @@ define('io.ox/office/text/format/paragraphstyles',
                 // active paragraph tab stops
                 activeParaTabStops = null,
                 // zoom factor in floating point notation
-                zoomFactor = view.getZoomFactor() / 100;
+                zoomFactor = view.getZoomFactor() / 100,
+                // if paraTabStops has left and right oriented tabs, mixedTabs is set by the right-tab. if it is set it overwrites behaviro of text-aligment and para-margin
+                mixedTab = getMixedTab(paraTabStops);
 
             if (paragraph.length) {
                 // Calculate top position and height of first paragraph child. Our paragraph
@@ -306,6 +325,13 @@ define('io.ox/office/text/format/paragraphstyles',
                 topFirst = firstNode.length ? Utils.convertLengthToHmm(firstNode.position().top, 'px') : 0;
                 heightFirst = firstNode.length ? Utils.convertLengthToHmm(firstNode.height(), 'px') : 0;
 
+
+                //reset all tab sizes, for a much cleaner new calculating
+                //(for example changing text-alignment from justify to left)
+                paragraph.children(DOM.TAB_NODE_SELECTOR).each(function () {
+                   $(this).css('width', '0px');
+                });
+
                 paragraph.children(DOM.TAB_NODE_SELECTOR).each(function () {
 
                     var // the current tab node of the paragraph
@@ -371,7 +397,7 @@ define('io.ox/office/text/format/paragraphstyles',
                         indentFirstLineTabStopValue = Math.abs(indentFirstLine);
                         // Fix for 29265 and 30847: Removing empty text node in span with '.contents().remove().end()'.
                         // Otherwise there are two text nodes in span after '.text('abc')' in IE.
-                        tabSpan.contents().remove().end().text('\u00a0');
+                        tabSpan.contents().remove().end().text(NBSP);
                         heightSpanHMM = Utils.convertLengthToHmm(tabSpan.height(), 'px');
                         topSpanHMM = Utils.convertLengthToHmm(tabSpan.position().top, 'px');
                         // checkout if the first line indent is active for this special tab stop
@@ -387,7 +413,7 @@ define('io.ox/office/text/format/paragraphstyles',
                             }) + 1;
                             // create a copy of the paragraph tab stops and add the first indent tab stop at
                             // the correct position to the active paragraph tab stop array
-                            indentFirstLineTabStop = {value: 'left', pos: indentFirstLineTabStopValue, fillChar: '\u00a0', processed: false };
+                            indentFirstLineTabStop = {value: 'left', pos: indentFirstLineTabStopValue, fillChar: NBSP, processed: false };
                             activeParaTabStops = paraTabStops.slice();
                             activeParaTabStops.splice(indentFirstLineTabStopIndex, 0, indentFirstLineTabStop);
                             // reset width - the tab stop position must be calculated using all tab stop positions
@@ -431,7 +457,7 @@ define('io.ox/office/text/format/paragraphstyles',
 
                             // insert fill character
                             if (width > 1) {
-                                fillChar = _.isString(tabStop.fillChar) ? TAB_FILL_CHARS[tabStop.fillChar] : '\u00a0';
+                                fillChar = _.isString(tabStop.fillChar) ? TAB_FILL_CHARS[tabStop.fillChar] : NBSP;
                                 if (fillChar) {
                                     insertTabFillChar(tabSpan, fillChar, width);
                                 }
@@ -449,15 +475,18 @@ define('io.ox/office/text/format/paragraphstyles',
                             width = Math.max(0, defaultTabStop - (leftHMM % defaultTabStop));
                             width = (width <= 10) ? defaultTabStop : width; // no 0 tab size allowed, check for <= 10 to prevent rounding errors
                             // reset possible fill character
-                            insertTabFillChar(tabSpan, '\u00a0', width);
+                            insertTabFillChar(tabSpan, NBSP, width);
                         }
                         prevTabEndPos = { right: leftHMM + width, top: topHMM };
                     }
 
                     // always set a width to the tab div (even an ignored tab div needs to set the size to zero)
-                    tabNode.css('width', (width / 100) + 'mm');
+                    // small offset for FF & IE
+                    tabNode.css('width', (width / 100 - 0.01) + 'mm');
                 });
 
+                calculateMarginForMixedTab(documentStyles, paragraph, mergedAttributes, mixedTab);
+
                 // reset processed flag for tab stops again
                 paraTabStops.forEach(function (tab) {
                     tab.processed = false;
@@ -465,6 +494,42 @@ define('io.ox/office/text/format/paragraphstyles',
             }
         }
 
+        /**
+         * right tab can lay right above from the border, this cannot be triggered by MS-WORD interface,
+         * but it is a typical behavior for MS-"table of content"
+         * so we calculate if right tab is outside of the margin, and if true,
+         * we reduce the margin-right of the paragraph
+         */
+        function calculateMarginForMixedTab(documentStyles, paragraph, paraAttributes, mixedTab) {
+            paragraph.css('margin-right', null);
+
+            if (!mixedTab) {
+                return;
+            }
+
+            var pageAttrs = documentStyles.getDefaultAttributes('page').page;
+            var maxParaWidth = pageAttrs.width - pageAttrs.marginLeft - pageAttrs.marginRight;
+            var indentRight = paraAttributes.paragraph.indentRight;
+            var currentParaWidth = mixedTab.pos + indentRight;
+
+            if (currentParaWidth > maxParaWidth) {
+                var newIndentRight = indentRight - (currentParaWidth - maxParaWidth);
+                paragraph.css('margin-right', (newIndentRight / 100) + 'mm');
+            }
+        }
+
+        function getMixedTab(paraTabStops) {
+            var leftTab = _.find(paraTabStops, function(tab) {
+                return !tab.value || tab.value === 'left';
+            });
+            var rightTab = _.find(paraTabStops, function(tab) {
+                return tab.value === 'right';
+            });
+            if(leftTab && rightTab) {
+                return rightTab;
+            }
+        }
+
         /**
          * Calculates the width of a right aligned tab element
          * depending on the position and the remaining size of
@@ -532,6 +597,9 @@ define('io.ox/office/text/format/paragraphstyles',
                 leftMargin = 0,
                 rightMargin = 0,
 
+                leftMarginTmp = 0,
+                rightMarginTmp = 0,
+
                 prevParagraph = paragraph.prev(),
                 prevAttributes = (prevParagraph.length > 0) ? self.getElementAttributes(prevParagraph) : { paragraph: {} },
 
@@ -565,14 +633,23 @@ define('io.ox/office/text/format/paragraphstyles',
 
             // update borders
             var leftPadding = paragraphAttributes.borderLeft && paragraphAttributes.borderLeft.space ? paragraphAttributes.borderLeft.space : 0;
-            leftMargin += setBorder(paragraphAttributes.borderLeft, 'left', 0);
-            rightMargin += setBorder(paragraphAttributes.borderRight, 'right', 0);
+            leftMarginTmp = setBorder(paragraphAttributes.borderLeft, 'left', 0);
+            rightMarginTmp = setBorder(paragraphAttributes.borderRight, 'right', 0);
+
+            // Fix for BUG #36298:
+            // only add the border-margins (left/right) if the paragraph isn't in a textframe
+            // or if they were positive. Otherwise the left/right border will be hidden.
+            if (!DOM.isNodeInsideTextFrame(paragraph) || (leftMarginTmp > 0 && rightMarginTmp > 0)) {
+                leftMargin += leftMarginTmp;
+                rightMargin += rightMarginTmp;
+            }
+
 
             var topMargin = paragraphAttributes.marginTop;
             var bottomMargin = paragraphAttributes.marginBottom;
 
             // top border is not set if previous paragraph uses the same border settings
-            if (isMergeBorders(paragraphAttributes, prevAttributes.paragraph)) {
+            if (isMergeBorders(paragraphAttributes, prevAttributes.paragraph) && !isSetPageBreakAttribute(paragraphAttributes)) { // #36293
                 setBorder({ style: 'none' }, 'top', topMargin);
                 prevParagraph.css({
                     paddingBottom: this.getCssBorder(paragraphAttributes.borderInside.space + bottomMargin),
@@ -585,7 +662,7 @@ define('io.ox/office/text/format/paragraphstyles',
             }
 
             // bottom border is replaced by inner border if next paragraph uses the same border settings
-            if (isMergeBorders(paragraphAttributes, nextAttributes.paragraph)) {
+            if (isMergeBorders(paragraphAttributes, nextAttributes.paragraph) && !isSetPageBreakAttribute(nextAttributes.paragraph)) { // #36293
                 setBorder(paragraphAttributes.borderInside, 'bottom');
                 setBorder(paragraphAttributes.borderInside, 'top', 0, nextParagraph);  // Task 30473: Updating merged borders
                 prevParagraph.css('padding-bottom', this.getCssBorder(bottomMargin));
@@ -668,7 +745,8 @@ define('io.ox/office/text/format/paragraphstyles',
             });
 
             // taking care of implicit paragraph nodes behind tables after loading the document
-            if ((DOM.isImplicitParagraphNode(paragraph)) && (DOM.isFinalParagraphBehindTable(paragraph))) {
+            // (also handling final empty paragraphs behind tables inside table cell (36537)).
+            if (DOM.isIncreasableParagraph(paragraph) && (!app.isImportFinished() || !app.getModel().selectionInNode(paragraph))) {
                 paragraph.css('height', 0);
             }
 
