@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
@@ -33,10 +33,14 @@ define('io.ox/office/text/format/tablestyles',
             width: {
                 def: 'auto',
                 format: function (element, width) {
+                    var parentWidth = 0;
                     if (width === 'auto') {
                         element.css('width', '100%');
                     } else {
-                        element.css('width', Utils.convertHmmToCssLength(width, 'px', 1));
+                        // adapt size of table to its parent (36132,36163)
+                        width = Utils.convertHmmToLength(width, 'px', 1);
+                        parentWidth = $(element).parent().width();
+                        (parentWidth < width) ? element.css('width', '100%') : element.css('width', width + 'px');
                     }
                 }
             },
@@ -149,10 +153,13 @@ define('io.ox/office/text/format/tablestyles',
      */
     function TableStyles(app, documentStyles) {
 
+        var // self reference
+            self = this;
+
         // base constructor ---------------------------------------------------
 
         StyleCollection.call(this, app, documentStyles, 'table', {
-            additionalFamilies: ['cell', 'paragraph', 'character', 'changes'],
+            families: 'cell paragraph character changes',
             styleAttributesResolver: resolveTableStyleAttributes,
             elementAttributesResolver: resolveTableElementAttributes,
             formatHandler: updateTableFormatting,
@@ -530,10 +537,10 @@ define('io.ox/office/text/format/tablestyles',
             // update rows, cells, ... , but only if required
             if (async) {
                 // process the table asynchronously
-                def = app.iterateArraySliced(rowNodes, updateRowFormatting);
+                def = self.iterateArraySliced(rowNodes, updateRowFormatting);
             } else {
                 // update all rows at once (but asynchronous, so that for example a new column is visible immediately)
-                def = app.executeDelayed(function () {
+                def = self.executeDelayed(function () {
                     rowNodes.each(function () { updateRowFormatting(this); });
                 });
             }
