@@ -1,26 +1,27 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Carsten Driesner <carsten.driesner@open-xchange.com>
  */
 
-define('io.ox/office/text/hyperlink',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/dialogs',
-     'io.ox/office/editframework/utils/attributeutils',
-     'io.ox/office/editframework/view/hyperlinkutil',
-     'io.ox/office/text/dom',
-     'io.ox/office/text/position',
-     'io.ox/office/text/operations'
-    ], function (Utils, KeyCodes, Dialogs, AttributeUtils, HyperlinkUtil, DOM, Position, Operations) {
+define('io.ox/office/text/hyperlink', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/dialogs',
+    'io.ox/office/editframework/utils/attributeutils',
+    'io.ox/office/editframework/utils/hyperlinkutils',
+    'io.ox/office/editframework/view/hyperlinkutil',
+    'io.ox/office/text/utils/operations',
+    'io.ox/office/text/dom',
+    'io.ox/office/text/position'
+], function (Utils, KeyCodes, Dialogs, AttributeUtils, HyperlinkUtils, HyperlinkUtil, Operations, DOM, Position) {
 
     'use strict';
 
@@ -68,7 +69,7 @@ define('io.ox/office/text/hyperlink',
      * of a hyperlink.
      */
     var Hyperlink = {
-        Separators: [ '!', '?', '.', ' ', '-', ':', ',', '\u00a0'],
+        Separators: [ '!', '?', '.', ' ', '-', ':', ',', '\xa0'],
         Protocols: [ 'http://', 'https://', 'ftp://', 'mailto:' ],
 
         // templates for web/ftp identifications
@@ -542,9 +543,14 @@ define('io.ox/office/text/hyperlink',
     Hyperlink.checkForHyperlinkText = function (editor, paragraph, position) {
         if (position !== null) {
             var pos = position[position.length - 1],
-                hyperlinkSelection = Hyperlink.findLeftText(editor, paragraph, pos, [' ', '\u00a0']);
+                hyperlinkSelection = Hyperlink.findLeftText(editor, paragraph, pos, [' ', '\xa0']),
+                url = null;
 
-            var url = hyperlinkSelection.url || HyperlinkUtil.checkForHyperlink(hyperlinkSelection.text);
+            if (hyperlinkSelection.url) {
+                url = hyperlinkSelection.url;
+            } else if(hyperlinkSelection.text) {
+                url = HyperlinkUtils.checkForHyperlink(hyperlinkSelection.text);
+            }
             if (url) {
                 var result = createResultFromHyperlinkSelection(position, hyperlinkSelection);
                 result.url = url;
@@ -769,8 +775,7 @@ define('io.ox/office/text/hyperlink',
                     width = hyperlinkPopup.width();
                     diffWidth = outerWidth - width;
                     if ((left + outerWidth) > parentWidth) {
-                        left -= ((left + outerWidth) - parentWidth);
-                        left = Math.max(Utils.convertCssLength(page.css('padding-left'), 'px'), left);
+                        left = Math.max(Utils.getElementCssLength(page, 'padding-left'), parentWidth - outerWidth);
                         if (!_.browser.IE) {
                             outerWidth = Math.min(outerWidth, parentWidth);
                             width = outerWidth - diffWidth;
@@ -782,7 +787,7 @@ define('io.ox/office/text/hyperlink',
                         // write with normal style
                         editor.addPreselectedAttributes(Hyperlink.CLEAR_ATTRIBUTES);
                     }
-                    if (HyperlinkUtil.isLegalUrl(result.url)) {
+                    if (HyperlinkUtils.hasSupportedProtocol(result.url)) {
                         if (hyperlinkPopup.css('display') === 'none') {
                             showPopup(app, hyperlinkPopup);
                         }
