@@ -1,12 +1,12 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
@@ -14,11 +14,11 @@
 define('io.ox/office/text/table',
     ['io.ox/office/tk/utils',
      'io.ox/office/editframework/utils/attributeutils',
+     'io.ox/office/text/utils/config',
+     'io.ox/office/text/utils/operations',
      'io.ox/office/text/dom',
-     'io.ox/office/text/position',
-     'io.ox/office/text/operations',
-     'io.ox/office/text/utils/config'
-    ], function (Utils, AttributeUtils, DOM, Position, Operations, Config) {
+     'io.ox/office/text/position'
+    ], function (Utils, AttributeUtils, Config, Operations, DOM, Position) {
 
     'use strict';
 
