@@ -1,23 +1,23 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
  */
 
 define('io.ox/office/text/tableResize',
     ['io.ox/office/tk/utils',
+     'io.ox/office/text/utils/operations',
      'io.ox/office/text/dom',
-     'io.ox/office/text/operations',
      'io.ox/office/text/position',
      'io.ox/office/text/table'
-    ], function (Utils, DOM, Operations, Position, Table) {
+    ], function (Utils, Operations, DOM, Position, Table) {
 
     'use strict';
 
@@ -122,7 +122,7 @@ define('io.ox/office/text/tableResize',
                 // the start scroll position
                 startScrollX = 0, startScrollY = 0,
                 //maximum height of page content in pixels (without margins)
-                pageMaxHeight = editor.getPageMaxHeight() - 5; // -5 px because of difference in rounding and offsetHeight, position top in IE, FF
+                pageMaxHeight = editor.getPageLayout().getDefPageActiveHeight({convertToPixel : true}) - 5; // -5 px because of difference in rounding and offsetHeight, position top in IE, FF
 
 
             /**
