@@ -1,20 +1,20 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define.async('io.ox/office/tk/config',
-    ['io.ox/core/gettext',
-     'settings!io.ox/office'
-    ], function (GetText, settings) {
+define.async('io.ox/office/tk/config', [
+    'io.ox/core/gettext',
+    'settings!io.ox/office'
+], function (GetText, settings) {
 
     'use strict';
 
@@ -151,13 +151,13 @@ define.async('io.ox/office/tk/config',
     var userPromise = GetText.language.done(function (locale) {
 
         var // language and country identifier from the locale
-            matches = /^([a-z]+)(_([A-Z]+))?/.exec(locale);
+            matches = /^([a-z]+)(?:_([A-Z]+))?/.exec(locale);
 
         // locale settings
         if (_.isArray(matches)) {
             Config.LOCALE = locale;
             Config.LANGUAGE = matches[1] || '';
-            Config.COUNTRY = matches[3] || '';
+            Config.COUNTRY = matches[2] || '';
         } else {
             window.console.error('Config: unrecognized locale "' + locale + '"');
         }
