@@ -1,23 +1,23 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/control/button',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/control/group',
-     'io.ox/office/tk/control/captionmixin',
-     'io.ox/office/tk/control/widthmixin'
-    ], function (Utils, Forms, Group, CaptionMixin, WidthMixin) {
+define('io.ox/office/tk/control/button', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/control/group',
+    'io.ox/office/tk/control/captionmixin',
+    'io.ox/office/tk/control/widthmixin'
+], function (Utils, Forms, Group, CaptionMixin, WidthMixin) {
 
     'use strict';
 
@@ -40,9 +40,12 @@ define('io.ox/office/tk/control/button',
      *  WidthMixin. Additionally, the following options are supported:
      *  @param {Any} [initOptions.value]
      *      A value, object, or function that will be used as fixed value for
-     *      the button when it has been clicked. Must not be null.
-     *      Ignored, if this button is a toggle button (see option 'toggle'
-     *      below).
+     *      the button when it has been clicked. Must not be null. A function
+     *      will be evaluated every time the button has been activated. If
+     *      another control group (instance of class Group) has been passed,
+     *      the current value of that group will be used and forwarded as
+     *      button value instead. Will be ignored, if this button is a toggle
+     *      button (see option 'toggle' below, value is always a Boolean).
      *  @param {String} [initOptions.dataValue]
      *      A string that will be inserted into the 'data-value' attribute of
      *      the button node. If omitted, the JSON string representation of the
@@ -61,7 +64,7 @@ define('io.ox/office/tk/control/button',
      *      if this option is omitted.
      *  @param {Object} [initOptions.dropDownVersion]
      *      If specified, the button will known, how it should act, if it will
-     *      shrinked in a dropdown-menu.
+     *      shrunken in a drop-down menu.
      *  @param {Object} [initOptions.smallerVersion]
      *      If specified, the button will known, how it should act, if there
      *      will be not enough free place to show normal view
@@ -77,12 +80,21 @@ define('io.ox/office/tk/control/button',
             // toggle button or push button
             toggle = Utils.getBooleanOption(initOptions, 'toggle', false),
 
+            // the fixed value to be sent when activating the button (not in toggle mode)
+            value = toggle ? null : Utils.getOption(initOptions, 'value'),
+
             // custom predicate callback for button highlighting
             highlightHandler = Utils.getFunctionOption(initOptions, 'highlight'),
 
+            // add caret
+            caret = Utils.getBooleanOption(initOptions, 'caret', false),
+
             // special options for a drop-down-version of this button
             dropDownVersion = Utils.getObjectOption(initOptions, 'dropDownVersion'),
 
+            //info if this button open a popup, important for aria
+            haspopup = Utils.getBooleanOption(initOptions, 'haspopup', false),
+
             // saves the (old) styles in case of switching to small version
             savedStyles = {},
 
@@ -103,9 +115,9 @@ define('io.ox/office/tk/control/button',
         function updateHandler(value) {
 
             if (_.isFunction(highlightHandler)) {
-                Forms.selectButtonNodes(buttonNode, highlightHandler.call(self, value));
+                Forms.checkButtonNodes(buttonNode, highlightHandler.call(self, value), { ambiguous: true, haspopup: haspopup });
             } else if (toggle) {
-                Forms.selectButtonNodes(buttonNode, value);
+                Forms.checkButtonNodes(buttonNode, value, { ambiguous: true, haspopup: haspopup });
             }
 
             // set value attributes for Selenium testing
@@ -116,7 +128,7 @@ define('io.ox/office/tk/control/button',
         }
 
         /**
-         * Modifies the button for the dropdown-view
+         * Modifies the button for the drop-down view
          *
          * @param {Object} dropDownVersion
          *  The following options are supported:
@@ -137,12 +149,12 @@ define('io.ox/office/tk/control/button',
          * Activates/Deactivates a smaller version of the Button
          *
          * @param {Boolean} value
-         *  decides wether the Button should be small or wide
+         *  decides whether the Button should be small or wide
          */
         function smallButton(value){
             // show the small version
             if (value === true) {
-                // set new css if exists
+                // set new CSS if exists
                 if (_.isObject(smallerVersion.css)) {
                     if (_.isEmpty(savedStyles.css) && !_.isEmpty(buttonNode.attr('style'))) {
                         savedStyles.css = buttonNode.attr('style');
@@ -160,8 +172,8 @@ define('io.ox/office/tk/control/button',
                     buttonNode.removeAttr('style');
                     buttonNode.attr('style',savedStyles.css);
                     savedStyles.css = null;
-                // if there were noch old styles, remove the style-attribute
-                } else if (_.has(smallerVersion, 'css')){
+                // if there were old styles, remove the style attribute
+                } else if (_.has(smallerVersion, 'css')) {
                     buttonNode.removeAttr('style');
                 }
 
@@ -199,12 +211,23 @@ define('io.ox/office/tk/control/button',
 
         // initialization -----------------------------------------------------
 
-        // set the fixed value of push buttons
+        // set the fixed value of push buttons (replace Group instances by a
+        // anonymous function that returns the current group value)
         if (!toggle) {
-            Forms.setButtonValue(buttonNode, Utils.getOption(initOptions, 'value'), initOptions);
+            if (value instanceof Group) {
+                value = (function (group) {
+                    return function () { return group.getValue(); };
+                }(value));
+            }
+            Forms.setButtonValue(buttonNode, value, initOptions);
         }
 
-        // creates the dropdown-version of this button, if specified
+        // show a caret
+        if (caret) {
+            buttonNode.addClass('caret-button').append(Forms.createCaretMarkup('down'));
+        }
+
+        // creates the drop-down version of this button, if specified
         if (dropDownVersion) {
             createDropDownVersion(dropDownVersion);
         }
@@ -215,9 +238,12 @@ define('io.ox/office/tk/control/button',
         // update handler for selected state
         this.registerUpdateHandler(updateHandler);
 
+        buttonNode.on(Forms.DEFAULT_CLICK_TYPE, Utils.preventFocusChange);
+        Utils.addToPreventFocusChange(buttonNode);
+
         // button click handler (convert ENTER and SPACE keys to click events)
         Forms.setButtonKeyHandler(buttonNode);
-        buttonNode.on('click', function (event) {
+        buttonNode.on(Forms.DEFAULT_CLICK_TYPE, function (event) {
             var value = toggle ? !self.getValue() : Forms.getButtonValue(buttonNode);
             self.triggerChange(value, { sourceEvent: event });
         });
@@ -225,7 +251,7 @@ define('io.ox/office/tk/control/button',
         // destroy all class members on destruction
         this.registerDestructor(function () {
             buttonNode.remove();
-            initOptions = self = buttonNode = highlightHandler = null;
+            initOptions = self = buttonNode = value = highlightHandler = null;
         });
 
     } // class Button
