@@ -1,20 +1,20 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/control/captionmixin',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms'
-    ], function (Utils, Forms) {
+define('io.ox/office/tk/control/captionmixin', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms'
+], function (Utils, Forms) {
 
     'use strict';
 
@@ -190,7 +190,7 @@ define('io.ox/office/tk/control/captionmixin',
                 }
 
                 // hide label
-                if (_.has(smallerVersion, 'hideLabel') && smallerVersion.hideLabel === true) {
+                if (_.has(smallerVersion, 'hideLabel') && smallerVersion.hideLabel === true) {
                     saveStyles.label = saveStyles.label || this.getLabel();
                     this.removeLabel();
                 }
@@ -211,7 +211,7 @@ define('io.ox/office/tk/control/captionmixin',
                 }
 
                 // show old label
-                if (_.has(saveStyles, 'hideLabel') && smallerVersion.hideLabel === true) {
+                if (_.has(saveStyles, 'hideLabel') && smallerVersion.hideLabel === true) {
                     this.setLabel(saveStyles.label);
                     delete saveStyles.label;
                 }
