@@ -1,21 +1,21 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/control/checkbox',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/control/button'
-    ], function (Utils, Forms, Button) {
+define('io.ox/office/tk/control/checkbox', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/control/button'
+], function (Utils, Forms, Button) {
 
     'use strict';
 
@@ -38,50 +38,54 @@ define('io.ox/office/tk/control/checkbox',
      *      If set to true, the icons to visualize the unchecked and checked
      *      state will contain rectangular boxes. Otherwise, unchecked state is
      *      simply empty space, and checked state is a simple check mark.
+     *  @param {Boolean} [options.ambiguous=false]
+     *      If set to true, the ambiguous state (any value but Booleans) will
+     *      be represented by a specific distinct icon. Otherwise, the icon for
+     *      the unchecked state will be used.
      */
     function CheckBox(initOptions) {
 
-        var // boxed icon mode
-            boxed = Utils.getBooleanOption(initOptions, 'boxed', false),
+        var // self reference
+            self = this,
 
-            // container for the check mark icon
-            checkNode = $('<span class="check-mark">' + createIconMarkup(false) + '</span>');
+            // options for the check icon
+            checkOptions = {
+                design: Utils.getBooleanOption(initOptions, 'boxed', false) ? 'boxed' : 'check',
+                ambiguous: Utils.getBooleanOption(initOptions, 'ambiguous', false)
+            };
 
         // base constructor ---------------------------------------------------
 
-        Button.call(this, Utils.extendOptions({attributes: { role: 'checkbox' }} /* just default aria role */ , initOptions, {
+        Button.call(this, Utils.extendOptions({
+            attributes: { role: 'checkbox' } // default ARIA role
+        }, initOptions, {
             toggle: true,
-            highlight: function () { return false; }, // never highlight the check box
+            highlight: _.constant(false), // never highlight the check box
         }));
 
         // private methods ----------------------------------------------------
 
-        /**
-         * Returns the HTML mark-up for the check mark.
-         */
-        function createIconMarkup(state) {
-            return Forms.createIconMarkup(state ? (boxed ? 'fa-check-square-o' : 'fa-check') : (boxed ? 'fa-square-o' : 'fa-none'));
-        }
-
         /**
          * The update handler for this button.
          */
-        function updateHandler(state, options, oldVal) {
-        	if (state !== oldVal) {
-        		checkNode[0].innerHTML = createIconMarkup(state);
-            	this.getButtonNode().attr({'aria-pressed': state, 'aria-checked': state});
-        	}
+        function updateHandler(state) {
+            Forms.checkButtonNodes(self.getButtonNode(), state, checkOptions);
         }
 
         // initialization -----------------------------------------------------
 
         // add special marker classes used to adjust formatting
         this.getNode().addClass('check-box');
-        this.getButtonNode().prepend(checkNode);
+        Forms.checkButtonNodes(this.getButtonNode(), false, checkOptions);
 
-        // register an update handler that updates the check icon
+        // register an update handler that updates the check mark
         this.registerUpdateHandler(updateHandler);
 
+        // destroy all class members on destruction
+        this.registerDestructor(function () {
+            initOptions = self = null;
+        });
+
     } // class CheckBox
 
     // exports ================================================================
