@@ -1,24 +1,24 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/control/combofield',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/control/textfield',
-     'io.ox/office/tk/control/menumixin',
-     'io.ox/office/tk/popup/listmenu'
-    ], function (Utils, KeyCodes, Forms, TextField, MenuMixin, ListMenu) {
+define('io.ox/office/tk/control/combofield', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/control/textfield',
+    'io.ox/office/tk/control/menumixin',
+    'io.ox/office/tk/popup/listmenu'
+], function (Utils, KeyCodes, Forms, TextField, MenuMixin, ListMenu) {
 
     'use strict';
 
@@ -59,7 +59,7 @@ define('io.ox/office/tk/control/combofield',
         // base constructors --------------------------------------------------
 
         TextField.call(this, Utils.extendOptions(initOptions, { attributes: { role: 'combobox', 'aria-autocomplete': 'inline' } }));
-        menu = new ListMenu(Utils.extendOptions(initOptions, { anchor: this.getNode(), itemDesign: 'list' }));
+        menu = new ListMenu(Utils.extendOptions({ expandWidth: true }, initOptions, { anchor: this.getNode(), itemDesign: 'list' }));
         MenuMixin.call(this, menu, { button: menuButton, ariaOwner: this.getInputNode() });
 
         // private methods ----------------------------------------------------
@@ -71,7 +71,7 @@ define('io.ox/office/tk/control/combofield',
             // not on touch devices to prevent flickering keyboard etc.
             if (!Modernizr.touch) {
                 // deferred to not confuse focus listeners due to mixed focus events triggered from mouse click and focus() call
-                _.defer(function () {
+                self.executeDelayed(function () {
                     self.getInputNode().focus();
                 });
             }
@@ -87,16 +87,9 @@ define('io.ox/office/tk/control/combofield',
         }
 
         /**
-         * Handles 'menu:beforeopen' events.
+         * Handles 'popup:show' events and moves the focus to the text field.
          */
-        function menuBeforeOpenHandler() {
-            menu.getSectionNodes().css({ minWidth: self.getNode().outerWidth() });
-        }
-
-        /**
-         * Handles 'menu:open' events and moves the focus to the text field.
-         */
-        function menuOpenHandler() {
+        function popupShowHandler() {
             returnFocusToTextField();
             scrollToListItem(menu.getSelectedItemNodes().first());
         }
@@ -119,12 +112,17 @@ define('io.ox/office/tk/control/combofield',
          * entry according to cursor keys.
          */
         function inputKeyDownHandler(event) {
-
             var // item node currently selected
                 buttonNode = menu.getSelectedItemNodes().first(),
                 // the <input> control
                 inputNode = self.getInputNode();
 
+            // open drop-down menu on specific key sequence
+            if (KeyCodes.matchKeyCode(event, 'SPACE', { alt: true, ctrl: true })) {
+                self.showMenu();
+                return;
+            }
+            
             // open drop-down menu on specific navigation keys
             switch (event.keyCode) {
             case KeyCodes.UP_ARROW:
@@ -252,21 +250,17 @@ define('io.ox/office/tk/control/combofield',
         // highlight list entry
         this.registerUpdateHandler(itemUpdateHandler);
 
+        // register event handlers
+        this.on('group:validate', groupValidateHandler);
+        this.getInputNode().on('keydown', inputKeyDownHandler);
+
         // handle click events in the drop-down list
-        menu.getNode().on('click', Forms.BUTTON_SELECTOR, function (event) {
+        menu.getNode().on(Forms.DEFAULT_CLICK_TYPE, Forms.BUTTON_SELECTOR, function (event) {
             self.triggerChange(Forms.getButtonValue(this), { sourceEvent: event });
         });
 
-        // register event handlers
-        this.on({
-            'menu:beforeopen': menuBeforeOpenHandler,
-            'menu:open': menuOpenHandler,
-            'group:validate': groupValidateHandler
-        });
-        this.getInputNode().on('keydown', inputKeyDownHandler);
-
-        // refresh list item highlighting after inserting new items
-        menu.on('create:item', function () { self.refresh(); });
+        // register pop-up event handlers
+        menu.on({ 'popup:show': popupShowHandler, 'create:item': function () { self.refresh(); } });
 
         // register input control for focus handling (to keep menu visible)
         menu.registerFocusableNodes(this.getInputNode());
