@@ -1,22 +1,23 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/control/group',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/object/triggerobject'
-    ], function (Utils, KeyCodes, Forms, TriggerObject) {
+define('io.ox/office/tk/control/group', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/object/triggerobject',
+    'io.ox/office/tk/object/timermixin'
+], function (Utils, KeyCodes, Forms, TriggerObject, TimerMixin) {
 
     'use strict';
 
@@ -123,6 +124,7 @@ define('io.ox/office/tk/control/group',
      * @constructor
      *
      * @extends TriggerObject
+     * @extends TimerMixin
      *
      * @param {Object} [initOptions]
      *  Optional parameters:
@@ -147,18 +149,16 @@ define('io.ox/office/tk/control/group',
             // the current value of this control group (must be undefined initially, NOT null)
             groupValue,
 
-            // the current value options of this control group
-            groupOptions = null,
-
             // update handler functions
             updateHandlers = [],
 
-            // special options for a drop-down-version of this group
+            // special options for a drop-down version of this group
             dropDownVersion = Utils.getOption(initOptions, 'dropDownVersion');
 
-        // base constructor ---------------------------------------------------
+        // base constructors --------------------------------------------------
 
         TriggerObject.call(this);
+        TimerMixin.call(this);
 
         // private methods ----------------------------------------------------
 
@@ -180,21 +180,36 @@ define('io.ox/office/tk/control/group',
             }
         }
 
+        /**
+         * Updates the current value of this group, and invokes the registered
+         * update handlers which will update the visual appearance of this
+         * group.
+         */
+        function updateValue(newValue, force) {
+
+            // do nothing, if the value does not change (unless forced)
+            if (!force && _.isEqual(newValue, groupValue)) { return; }
+
+            var // the old value of this group (for event handlers)
+                oldValue = groupValue;
+
+            // set new value, invoke the update handlers which refresh the appearance of the group
+            groupValue = newValue;
+            _.each(updateHandlers, function (updateHandler) {
+                updateHandler.call(self, newValue, oldValue);
+            });
+
+            // update group size
+            self.layout();
+        }
+
         /**
          * Returns an object with focus options for the passed source event,
          * intended to be inserted into a 'group:change' or 'group:cancel'
          * event triggered by this control group.
          *
          * @param {Object} [options]
-         *  Optional parameters:
-         *  @param {jQuery.Event} [options.sourceEvent]
-         *      The source event that has caused the 'group:change' or
-         *      'group:cancel' event. According to the type of this source
-         *      event, the returned options object will contain specific focus
-         *      options.
-         *  @param {Boolean} [options.preserveFocus]
-         *      If specified, overrides the 'preserveFocus' option that would
-         *      be created from the passed source event.
+         *  Optional parameters. See method Group.triggerChange() for details.
          *
          * @returns {Object}
          *  An options object with the following properties:
@@ -230,11 +245,16 @@ define('io.ox/office/tk/control/group',
 
                 // handle specific event types
                 switch (sourceEvent.type) {
+                case 'tap':
                 case 'click':
-                    // Forms.setButtonKeyHandler() triggers 'click' events with 'keyCode' field
+                        // Forms.setButtonKeyHandler() triggers 'click' events with 'keyCode' field
                     eventOptions.sourceType = ('keyCode' in sourceEvent) ? 'keyboard' : 'click';
                     // click events from keyboard: keep focus on button after SPACE or TAB key (see Forms.setButtonKeyHandler() method)
                     eventOptions.preserveFocus = (sourceEvent.keyCode === KeyCodes.SPACE) || (sourceEvent.keyCode === KeyCodes.TAB);
+
+                    //necessary on touchdevice that the focus goes back in the document so the keyboard stays open
+                    Utils.preventFocusChange(sourceEvent);
+
                     break;
                 case 'keydown':
                 case 'keypress':
@@ -264,12 +284,9 @@ define('io.ox/office/tk/control/group',
          * @param {Function} updateHandler
          *  The update handler function. Will be called in the context of this
          *  group. Receives the following parameters:
-         *  (1) {Any} value
-         *      The value passed to the method Group.setValue().
-         *  (2) {Object} [options]
-         *      The options map passed to the method Group.setValue(). Will be
-         *      undefined if no options have been passed.
-         *  (3) {Any} oldValue
+         *  (1) {Any} newValue
+         *      The new value passed to the method Group.setValue().
+         *  (2) {Any} oldValue
          *      The old value of the group.
          *
          * @returns {Group}
@@ -304,6 +321,10 @@ define('io.ox/office/tk/control/group',
 
         // methods ------------------------------------------------------------
 
+        this.clone = function (additionalOptions) {
+            return new this.constructor(Utils.extendOptions(initOptions, additionalOptions));
+        };
+
         /**
          * Returns the DOM container element for this group as jQuery object.
          */
@@ -499,39 +520,43 @@ define('io.ox/office/tk/control/group',
          * @param {Any} value
          *  The new value to be displayed in the controls of this group.
          *
-         * @param {Object} [options]
-         *  Optional parameters that will be passed as second parameter to all
-         *  registered update handlers.
-         *
          * @returns {Group}
          *  A reference to this group.
          */
-        this.setValue = function (value, options) {
-            var oldValue = groupValue;
-            groupValue = value;
-            groupOptions = options;
-            _.each(updateHandlers, function (updateHandler) {
-                updateHandler.call(this, value, options, oldValue);
-            }, this);
-            return this.layout();
+        this.setValue = function (value) {
+            updateValue(value);
+            return this;
         };
 
         /**
          * Debounced call of the Group.setValue() method with the current value
          * of this group. Can be used to refresh the appearance of the group
          * after changing its content.
+         *
+         * @returns {Group}
+         *  A reference to this group.
          */
-        this.refresh = _.debounce(function () {
-            // prevent calling method on destroyed object in debounced callback
-            if (self && !self.destroyed) {
-                self.setValue(groupValue, groupOptions);
-            }
+        this.refresh = this.createDebouncedMethod(Utils.NOOP, function () {
+            updateValue(groupValue, true);
         });
 
         /**
          * Triggers a 'group:change' event at this control group instance, and
          * inserts special focus options into the event object. If the control
-         * group is disabled, a 'group'cancel' event will be triggered instead.
+         * group is disabled, a 'group:cancel' event will be triggered instead.
+         *
+         * @param {Any} value
+         *  The new value for this group to be set and notified.
+         *
+         * @param {Object} [options]
+         *  Optional parameters:
+         *  @param {jQuery.Event} [options.sourceEvent]
+         *      The source event that has caused the new event. This source
+         *      event may influence the focus behavior, (e.g.: browser focus is
+         *      kept on a button, if the SPACE key has been used).
+         *  @param {Boolean} [options.preserveFocus]
+         *      If specified, forces to preserve the browser focus regardless
+         *      of the passed source event.
          *
          * @returns {Group}
          *  A reference to this group.
@@ -551,10 +576,8 @@ define('io.ox/office/tk/control/group',
          * special focus options into the event object, according to the passed
          * source event.
          *
-         * @param {jQuery.Event} [sourceEvent]
-         *  The source event that causes the 'group:cancel' event. According to
-         *  the type of this source event, the cancel event will contain
-         *  specific focus options.
+         * @param {Object} [options]
+         *  Optional parameters. See method Group.triggerChange() for details.
          *
          * @returns {Group}
          *  A reference to this group.
@@ -565,37 +588,35 @@ define('io.ox/office/tk/control/group',
         };
 
         /**
-		 * Needs to be overwritten by controls, if there were some special
+         * Needs to be overwritten by controls, if there were some special
          * small version options.
          */
-        this.activateSmallVersion = function(){ };
-        this.deactivateSmallVersion = function(){ };
+        this.activateSmallVersion = Utils.NOOP;
+        this.deactivateSmallVersion = Utils.NOOP;
 
         /**
-         * Decides, whether the group should be shown in the current state
+         * Shows or hides this control group according to the passed state and
+         * the drop-down configuration passed to the constructor.
          *
-         * @param {Boolean} [isShrinked]
-         *  With this option and the current shrinked-state, it will be
-         *  decided whether the dropdown-version should be visible or not.
+         * @param {Boolean} shrunkenMode
+         *  Whether the container is in shrunken mode. This control group will
+         *  only be updated, if the drop-down configuration passed to the
+         *  constructor contains a 'visible' option. The control will be shown,
+         *  if the flag 'shrunkenMode' equals the 'visible' option, otherwise
+         *  it will be hidden.
+         *
+         * @returns {Group}
+         *  A reference to this group.
          */
-        this.dropDownVisible = function(isShrinked){
-            if (_.isObject(dropDownVersion) && _.has(dropDownVersion, 'visible')){
-                if ((isShrinked === true && dropDownVersion.visible === true) || (isShrinked === false && dropDownVersion.visible === false)) {
-                    // ANZEIGEN
-                    this.show();
-                } else if((isShrinked === true && dropDownVersion.visible === false) || (isShrinked === false && dropDownVersion.visible === true)) {
-                    // AUSBLENDEN
-                    this.hide();
-                }
+        this.toggleDropDownMode = function (shrunkenMode) {
+            if (_.isObject(dropDownVersion) && _.isBoolean(dropDownVersion.visible)) {
+                this.toggle(shrunkenMode === dropDownVersion.visible);
             }
+            return this;
         };
 
-
         // initialization -----------------------------------------------------
 
-        // show/hide initially this group
-        this.dropDownVisible(false);
-
         // register for global focus handling
         registerFocusGroup(this);
 
@@ -605,6 +626,9 @@ define('io.ox/office/tk/control/group',
         // ARIA role
         groupNode.attr('role', Utils.getStringOption(initOptions, 'role', null, true));
 
+        // hide groups only shown in shrunken mode
+        this.toggleDropDownMode(false);
+
         // log focus and layout events
         Utils.FocusLogger.logEvent(this, 'group:focus group:blur');
         Utils.LayoutLogger.logEvent(this, 'group:resize', 'width', 'height');
