@@ -1,21 +1,21 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/control/menumixin',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms'
-    ], function (Utils, KeyCodes, Forms) {
+define('io.ox/office/tk/control/menumixin', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms'
+], function (Utils, KeyCodes, Forms) {
 
     'use strict';
 
@@ -36,18 +36,6 @@ define('io.ox/office/tk/control/menumixin',
      * the class Group or one of its derived classes. Expects the symbol 'this'
      * to be bound to an instance of Group.
      *
-     * Instances of this class trigger the following events:
-     * - 'menu:beforeopen': Before the drop-down menu will be opened.
-     * - 'menu:open': After the drop-down menu has been opened.
-     * - 'menu:beforeclose': Before the drop-down menu will be closed.
-     * - 'menu:close': After the drop-down menu has been closed.
-     * - 'menu:beforelayout': If the absolute position of the group or the
-     *      browser window size has changed, and the position and size of the
-     *      drop-down menu node needs to be adjusted.
-     * - 'menu:layout': If the absolute position of the group or the browser
-     *      window size has changed, and the position and size of the drop-down
-     *      menu node has been adjusted (in auto-layout mode).
-     *
      * @constructor
      *
      * @param {BaseMenu} menu
@@ -99,27 +87,16 @@ define('io.ox/office/tk/control/menumixin',
         // private methods ----------------------------------------------------
 
         /**
-         * Initializes the drop-down menu after it has been opened, and
-         * triggers a 'menu:open' event at the group.
+         * Initializes the drop-down menu after it has been shown.
          */
         function popupShowHandler() {
-            // add CSS marker class to the group, and notify own listeners
+            // add CSS marker class to the group
             groupNode.addClass(OPEN_CLASS);
-            ariaOwner.attr('aria-expanded', 'true');
-            self.trigger('menu:open');
+            ariaOwner.attr('aria-expanded', true);
         }
 
         /**
-         * Triggers a 'menu:close' event at the group.
-         */
-        function popupHideHandler() {
-            ariaOwner.attr('aria-expanded', 'false');
-            self.trigger('menu:close');
-        }
-
-        /**
-         * Deinitializes the drop-down menu before it will be closed, and
-         * triggers a 'menu:beforeclose' event at the group.
+         * Deinitializes the drop-down menu before it will be hidden.
          */
         function popupBeforeHideHandler() {
 
@@ -128,9 +105,9 @@ define('io.ox/office/tk/control/menumixin',
                 menuButton.focus();
             }
 
-            // remove CSS marker class from the group, and notify own listeners
-            self.trigger('menu:beforeclose');
+            // remove CSS marker class from the group
             groupNode.removeClass(OPEN_CLASS);
+            ariaOwner.attr('aria-expanded', false);
         }
 
         /**
@@ -324,16 +301,17 @@ define('io.ox/office/tk/control/menumixin',
         this.on('group:beforehide', function () { menu.hide(); });
         this.on('group:enable', function (event, state) { if (!state) { menu.hide(); } });
         groupNode.on('keydown', groupKeyDownHandler);
-        menuButton.on({ click: menuButtonClickHandler, keydown: menuButtonKeyDownHandler });
+
+        Utils.addToPreventFocusChange(menuButton);
+        Utils.addToPreventFocusChange(ariaOwner);
+
+        menuButton.on(Forms.DEFAULT_CLICK_TYPE, menuButtonClickHandler);
+        menuButton.on('keydown', menuButtonKeyDownHandler);
 
         // register event handlers for the drop-down menu
         menu.on({
-            'popup:beforeshow': function () { self.trigger('menu:beforeopen'); },
             'popup:show': popupShowHandler,
             'popup:beforehide': popupBeforeHideHandler,
-            'popup:hide': popupHideHandler,
-            'popup:beforelayout': function () { self.trigger('menu:beforelayout'); },
-            'popup:layout': function () { self.trigger('menu:layout'); }
         });
         menuNode.on('keydown keypress keyup', menuKeyHandler);
 
