@@ -1,25 +1,25 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/control/radiolist',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/control/group',
-     'io.ox/office/tk/control/captionmixin',
-     'io.ox/office/tk/control/widthmixin',
-     'io.ox/office/tk/control/menumixin',
-     'io.ox/office/tk/popup/listmenu'
-    ], function (Utils, Forms, Group, CaptionMixin, WidthMixin, MenuMixin, ListMenu) {
+define('io.ox/office/tk/control/radiolist', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/control/group',
+    'io.ox/office/tk/control/captionmixin',
+    'io.ox/office/tk/control/widthmixin',
+    'io.ox/office/tk/control/menumixin',
+    'io.ox/office/tk/popup/listmenu'
+], function (Utils, Forms, Group, CaptionMixin, WidthMixin, MenuMixin, ListMenu) {
 
     'use strict';
 
@@ -88,10 +88,10 @@ define('io.ox/office/tk/control/radiolist',
      *      labelStyle'. The keyword 'none' is equivalent to the empty string.
      *  @param {Object} [initOptions.dropDownVersion]
      *      If specified, the RadioList will known, how it should act, if it
-     *      will shrinked in a dropdown-menu.
+     *      will be shrunken in a drop-down menu.
      *  @param {Object} [initOptions.smallerVersion]
-     *      If specified, the RadioList will known, how it should act, if there
-     *      will be not enough free place to show normal view
+     *      If specified, the radio list will know how it should act, if there
+     *      is not enough free place to show normal view.
      */
     function RadioList(initOptions) {
 
@@ -128,13 +128,13 @@ define('io.ox/office/tk/control/radiolist',
             // the drop-down menu instance (must be created after Group base constructor!)
             menu = null,
 
-            // smaller version of radiolist (to fit in smaller resolutions)
+            // smaller version of the radio list (to fit in smaller resolutions)
             smallerVersion = Utils.getOption(initOptions, 'smallerVersion', false),
 
             // saves the (old) styles in case of switching to small version
             savedStyles = {},
 
-            // special options for a drop-down-version of this radiolist
+            // special options for a drop-down-version of this radio list
             dropDownVersion = Utils.getObjectOption(initOptions, 'dropDownVersion');
 
         // base constructors --------------------------------------------------
@@ -142,7 +142,7 @@ define('io.ox/office/tk/control/radiolist',
         Group.call(this, initOptions);
         CaptionMixin.call(this, captionButton);
         WidthMixin.call(this, captionButton, initOptions);
-        menu = new ListMenu(Utils.extendOptions(initOptions, { anchor: this.getNode() }));
+        menu = new ListMenu(Utils.extendOptions({ expandWidth: true }, initOptions, { anchor: this.getNode() }));
         MenuMixin.call(this, menu, Utils.extendOptions(initOptions, { button: menuButton }));
 
         // private methods ----------------------------------------------------
@@ -156,28 +156,6 @@ define('io.ox/office/tk/control/radiolist',
             }
         }
 
-        /**
-         * Sets the minimum width of the menu to the width of the group.
-         */
-        function popupBeforeShowHandler() {
-            menu.getSectionNodes().css({ minWidth: self.getNode().outerWidth() });
-        }
-
-        /**
-         * Scrolls to the selected list item if available.
-         */
-        function popupShowHandler() {
-            scrollToListItem(menu.getSelectedItemNodes().first());
-        }
-
-        /**
-         * Clean up data stored in button nodes, before the entire menu node
-         * will be cleared.
-         */
-        function popupBeforeClearHandler() {
-            menu.getItemNodes().data('options', null).off();
-        }
-
         /**
          * Activates an option button in this radio group.
          *
@@ -218,7 +196,7 @@ define('io.ox/office/tk/control/radiolist',
             } else {
                 isHighlighted = highlight && (selectedButtons.length > 0);
             }
-            Forms.selectButtonNodes(menuButton.add(captionButton), isHighlighted);
+            Forms.checkButtonNodes(menuButton.add(captionButton), isHighlighted, { ambiguous: true });
 
             // update the caption of the drop-down menu button
             if (updateCaptionMode.length > 0) {
@@ -242,12 +220,35 @@ define('io.ox/office/tk/control/radiolist',
         }
 
         /**
-		 * Activates/Deactivates a smaller version of the radiolist
-		 *
-		 * @param {Boolean} value
-         *  decides wether the radiolist should be small or wide
+         * Updates dynamic visibility (via callback functions) of the list
+         * items before the menu will be shown.
          */
-        function smallCaptionButton(value){
+        function popupBeforeShowHandler() {
+            itemUpdateHandler(self.getValue());
+        }
+
+        /**
+         * Scrolls to the selected list item if available.
+         */
+        function popupShowHandler() {
+            scrollToListItem(menu.getSelectedItemNodes().first());
+        }
+
+        /**
+         * Clean up data stored in button nodes, before the entire menu node
+         * will be cleared.
+         */
+        function popupBeforeClearHandler() {
+            menu.getItemNodes().data('options', null).off();
+        }
+
+        /**
+         * Activates or deactivates a smaller version of the radio list.
+         *
+         * @param {Boolean} value
+         *  Decides whether the radio list should be small or wide.
+         */
+        function smallCaptionButton(value) {
             // show the small version
             if (value === true) {
                 // set new css if exists
@@ -264,10 +265,10 @@ define('io.ox/office/tk/control/radiolist',
 
                 // set new/other icon
                 if (!_.isEmpty(smallerVersion.icon)) {
-                    var icon = captionButton.find('> .caret-node > i');
-                    savedStyles.icon = icon.attr('class');
-                    icon.removeAttr('class');
-                    icon.addClass(smallerVersion.icon);
+                    var caret_node = captionButton.find('> .caret-node');
+                    savedStyles.icon = caret_node.find('i')[0];
+                    caret_node.find('i').remove();
+                    caret_node.append(Forms.createIconMarkup(smallerVersion.icon));
                 }
 
                 // hide text of label (if set)
@@ -290,9 +291,9 @@ define('io.ox/office/tk/control/radiolist',
 
                 // re-set default icon
                 if (_.has(savedStyles, 'icon') && !_.isNull(savedStyles.icon)) {
-                    var icon = captionButton.find('> .caret-node > i');
-                    icon.removeAttr('class');
-                    icon.addClass(savedStyles.icon);
+                    var caret_node = captionButton.find('> .caret-node');
+                    caret_node.find('i').remove();
+                    caret_node.append(savedStyles.icon);
                     savedStyles.icon = null;
                 }
 
@@ -302,10 +303,10 @@ define('io.ox/office/tk/control/radiolist',
         }
 
         /**
-         * Modifies the captionbutton for the dropdown-view
+         * Modifies the caption button for the drop-down view.
          */
-        function createDropDownVersion(){
-            if (_.has(dropDownVersion, 'label') && captionButton.find('> .caption > .drop-down').length === 0) {
+        function createDropDownVersion() {
+            if (_.isObject(dropDownVersion) && _.isString(dropDownVersion.label) && captionButton.find('> .caption > .drop-down').length === 0) {
                 captionButton.find('> .caption').append(
                     Forms.createSpanMarkup(dropDownVersion.label, {
                         classes: 'drop-down'
@@ -430,8 +431,8 @@ define('io.ox/office/tk/control/radiolist',
         this.registerUpdateHandler(itemUpdateHandler);
 
         // handle click events in the list
-        menu.getNode().on('click', Forms.BUTTON_SELECTOR, function (event) {
-            var toggleClick = Forms.isSelectedNode(this) && !_.isNull(toggleValue) && !_.isUndefined(toggleValue),
+        menu.getNode().on(Forms.DEFAULT_CLICK_TYPE, Forms.BUTTON_SELECTOR, function (event) {
+            var toggleClick = Forms.isCheckedButtonNode(this) && !_.isNull(toggleValue) && !_.isUndefined(toggleValue),
                 value = toggleClick ? toggleValue : Forms.getButtonValue(this);
             self.triggerChange(value, { sourceEvent: event });
         });
