@@ -1,22 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/control/spinfield',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/control/textfield'
-    ], function (Utils, KeyCodes, Forms, TextField) {
+define('io.ox/office/tk/control/spinfield', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/control/textfield'
+], function (Utils, KeyCodes, Forms, TextField) {
 
     'use strict';
 
@@ -120,7 +120,7 @@ define('io.ox/office/tk/control/spinfield',
             // register tracking event listeners to change the current value
             spinButton.on({
                 'tracking:start tracking:repeat': function () { changeValue(increase ? smallStep : -smallStep); },
-                'tracking:end tracking:cancel': function () { _.defer(function () { self.getInputNode().focus(); }); }
+                'tracking:end tracking:cancel': function () { self.executeDelayed(function () { self.getInputNode().focus(); }); }
             });
 
             // enable/disable the button according to the current value
