@@ -1,24 +1,24 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/control/textfield',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/object/baseobject',
-     'io.ox/office/tk/control/group',
-     'io.ox/office/tk/control/widthmixin'
-    ], function (Utils, KeyCodes, Forms, BaseObject, Group, WidthMixin) {
+define('io.ox/office/tk/control/textfield', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/object/baseobject',
+    'io.ox/office/tk/control/group',
+    'io.ox/office/tk/control/widthmixin'
+], function (Utils, KeyCodes, Forms, BaseObject, Group, WidthMixin) {
 
     'use strict';
 
@@ -369,13 +369,13 @@ define('io.ox/office/tk/control/textfield',
         };
 
         /**
-         * Is applying a new validator 
+         * Is applying a new validator
          */
         this.setValidator = function(value) {
             validator = value;
             return this;
         };
-        
+
         /**
          * Converts the passed value to a text using the validator of this
          * control group.
