@@ -1,23 +1,23 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/dialogs',
-    ['io.ox/core/tk/dialogs',
-     'io.ox/office/tk/utils',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/keycodes',
-     'gettext!io.ox/office/main'
-    ], function (CoreDialogs, Utils, Forms, KeyCodes, gt) {
+define('io.ox/office/tk/dialogs', [
+    'io.ox/core/tk/dialogs',
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/keycodes',
+    'gettext!io.ox/office/main'
+], function (CoreDialogs, Utils, Forms, KeyCodes, gt) {
 
     'use strict';
 
@@ -350,25 +350,30 @@ define('io.ox/office/tk/dialogs',
             var // index of the active tab panel
                 activeIndex = _.indexOf(tabPanelIds, activeId),
                 // identifier of the new tab panel to be activated
-        		newTabId = null;
+                newTabId = null;
 
-        	switch (event.keyCode) {
-        	case KeyCodes.RIGHT_ARROW:
-    			newTabId = tabPanelIds[activeIndex + 1];
-        		break;
-        	case KeyCodes.LEFT_ARROW:
-        	    newTabId = tabPanelIds[activeIndex - 1];
-        		break;
-        	case KeyCodes.SPACE:
-        	    newTabId = $(event.target).attr('data-user-id');
-        		break;
-        	}
+            switch (event.keyCode) {
+            case KeyCodes.RIGHT_ARROW:
+                newTabId = tabPanelIds[activeIndex + 1];
+                break;
+            case KeyCodes.LEFT_ARROW:
+                newTabId = tabPanelIds[activeIndex - 1];
+                break;
+            case KeyCodes.SPACE:
+                newTabId = $(event.target).attr('data-user-id');
+                break;
+            }
 
-        	if (newTabId) {
-        		self.activateTabPanel(newTabId);
-        		focusActiveTabButton();
-        		return false;
-        	}
+            if (newTabId) {
+                // focus active tab button first to allow the active tab panel
+                // to finally focus a control inside the panel
+                focusActiveTabButton();
+                self.activateTabPanel(newTabId);
+                return false;
+            }
+
+            //we need to call preventDefault() because of focus travelling back to the document when this dialog has no input-fields
+            event.preventDefault();
         }
 
         // methods ------------------------------------------------------------
