@@ -1,21 +1,21 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/forms',
-    ['io.ox/office/tk/config',
-     'io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes'
-    ], function (Config, Utils, KeyCodes) {
+define('io.ox/office/tk/forms', [
+    'io.ox/office/tk/config',
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes'
+], function (Config, Utils, KeyCodes) {
 
     'use strict';
 
@@ -29,7 +29,26 @@ define('io.ox/office/tk/forms',
         ICON_SELECTOR = '>i',
 
         // CSS selector for text spans in control captions
-        SPAN_SELECTOR = '>span';
+        SPAN_SELECTOR = '>span',
+
+        // CSS class for buttons with ambiguous value
+        AMBIGUOUS_CLASS = 'ambiguous',
+
+        // CSS class name for check marks in check boxes
+        CHECK_MARK_CLASS = 'check-mark',
+
+        // attribute name for selected/checked buttons
+        DATA_CHECKED_ATTR = 'data-checked',
+
+        // CSS selector for selected buttons
+        DATA_CHECKED_SELECTOR = '[' + DATA_CHECKED_ATTR + '="true"]',
+
+        // icon sets for checked/unchecked icons (arrays with 'false', 'true', and 'ambiguous' icons)
+        CHECK_ICON_SETS = {
+            check: ['fa-none', 'fa-check', 'fa-minus'],
+            boxed: ['fa-square-o', 'fa-check-square-o', 'fa-minus-square-o'],
+            radio: ['fa-circle-o', 'fa-dot-circle-o', 'fa-circle-o']
+        };
 
     // private global functions ===============================================
 
@@ -142,13 +161,6 @@ define('io.ox/office/tk/forms',
      */
     Forms.SELECTED_SELECTOR = '.' + Forms.SELECTED_CLASS;
 
-    /**
-     * CSS class for buttons with ambiguous value.
-     *
-     * @constant
-     */
-    Forms.AMBIGUOUS_CLASS = 'ambiguous';
-
     /**
      * CSS class for DOM container elements that contain the browser focus.
      *
@@ -180,6 +192,27 @@ define('io.ox/office/tk/forms',
      */
     Forms.BUTTON_SELECTOR = '.' + Forms.BUTTON_CLASS;
 
+    /**
+     * CSS class name for option button elements in a button group.
+     *
+     * @constant
+     */
+    Forms.OPTION_BUTTON_CLASS = 'option-button';
+
+    /**
+     * CSS element selector for option button elements in a button group.
+     *
+     * @constant
+     */
+    Forms.OPTION_BUTTON_SELECTOR = Forms.BUTTON_SELECTOR + '.' + Forms.OPTION_BUTTON_CLASS;
+
+    /**
+     * CSS class for DOM container elements that contain the browser focus.
+     *
+     * @constant
+     */
+    Forms.DEFAULT_CLICK_TYPE = Utils.TOUCHDEVICE ? 'tap' : 'click';
+
     // generic helpers --------------------------------------------------------
 
     /**
@@ -514,6 +547,40 @@ define('io.ox/office/tk/forms',
         return $(nodes).find(Forms.FOCUSABLE_SELECTOR);
     };
 
+    /**
+     * Returns whether one of the descendant elements in the passed element
+     * contains the active (focused) element.
+     *
+     * @param {HTMLElement|jQuery} nodes
+     *  A single DOM element, or a jQuery collection, whose descendants will be
+     *  checked for the active (focused) element.
+     *
+     * @returns {Boolean}
+     *  Whether one of the descendants of the passed elements contains the
+     *  active (focused) element.
+     */
+    Forms.containsFocus = function (nodes) {
+        return $(nodes).find(window.document.activeElement).length > 0;
+    };
+
+    /**
+     * Returns whether one of the descendant elements in the passed element, or
+     * contains the active (focused) element; or the element itself is the
+     * active element.
+     *
+     * @param {HTMLElement|jQuery} nodes
+     *  A single DOM element, or a jQuery collection, that will be checked for
+     *  the active (focused) element.
+     *
+     * @returns {Boolean}
+     *  Whether one of the descendants in the passed elements contains the
+     *  active (focused) element, or one of the elements itself is the active
+     *  element.
+     */
+    Forms.hasOrContainsFocus = function (nodes) {
+        return ($(nodes).filter(window.document.activeElement).length > 0) || Forms.containsFocus(nodes);
+    };
+
     /**
      * Sets the browser focus to the first focusable control element contained
      * in the specified DOM element.
@@ -671,7 +738,7 @@ define('io.ox/office/tk/forms',
         $(rootNode).first().on('keydown', keyDownHandler);
     };
 
-    // control captions -------------------------------------------------------
+    // icons ------------------------------------------------------------------
 
     /**
      * Returns the complete CSS class names of the specified icon. For icons
@@ -739,6 +806,8 @@ define('io.ox/office/tk/forms',
         });
     };
 
+    // control captions -------------------------------------------------------
+
     /**
      * Creates the HTML mark-up of a <span> DOM element representing the text
      * label for a control.
@@ -806,7 +875,7 @@ define('io.ox/office/tk/forms',
             // custom picture width
             pictureWidth = Utils.getIntegerOption(options, 'pictureWidth', 48),
             // the element attributes
-            attributes = { src: source, 'class': classes ? classes : null, width: pictureWidth, height: pictureHeight, alt: altText };
+            attributes = { src: source, 'class': classes ? classes : '', width: pictureWidth, height: pictureHeight, alt: altText };
 
         return Forms.createElementMarkup('img', {
             attributes: attributes
@@ -1061,23 +1130,6 @@ define('io.ox/office/tk/forms',
         });
     };
 
-    // drop-down carets -------------------------------------------------------
-
-    /**
-     * Creates the HTML mark-up of a DOM element containing a drop-down caret
-     * symbol intended to be inserted into a button control element.
-     *
-     * @param {String} direction
-     *  The direction for the caret symbol. Can be one of 'up', 'down', 'left',
-     *  or 'right'.
-     *
-     * @returns {String}
-     *  The HTML mark-up of the caret node, as string.
-     */
-    Forms.createCaretMarkup = function (direction) {
-        return '<span class="caret-node">' + Forms.createIconMarkup('fa-caret-' + direction) + '</span>';
-    };
-
     // label elements ---------------------------------------------------------
 
     /**
@@ -1103,6 +1155,62 @@ define('io.ox/office/tk/forms',
 
     // button elements --------------------------------------------------------
 
+    /**
+     * Creates the HTML mark-up of a DOM element containing a drop-down caret
+     * symbol intended to be inserted into a button control element.
+     *
+     * @param {String} direction
+     *  The direction for the caret symbol. Can be one of 'up', 'down', 'left',
+     *  or 'right'.
+     *
+     * @returns {String}
+     *  The HTML mark-up of the caret node, as string.
+     */
+    Forms.createCaretMarkup = function (direction) {
+        return '<span class="caret-node">' + Forms.createIconMarkup('fa-caret-' + direction) + '</span>';
+    };
+
+    /**
+     * Creates the HTML mark-up for the check mark of a check box or option
+     * button of a radio group.
+     *
+     * @param {Boolean|Null} state
+     *  The state of the created check mark. The Boolean value true results in
+     *  a checked icon. The Boolean value false results in an unchecked icon.
+     *  The value null is treated as ambiguous state and results in a specific
+     *  icon, if the option 'ambiguous' is set. Otherwise, this will be the
+     *  unchecked icon.
+     *
+     * @param {String} design
+     *  Specifies the icon set that will be used to visualize the unchecked and
+     *  checked state. MUST be one of the following values:
+     *  - 'check': Unchecked state is empty space, and checked state is a
+     *      simple check mark.
+     *  - 'boxed': Unchecked state is an empty rectangle, and checked state is
+     *      a rectangle with a check mark.
+     *  - 'radio': Unchecked state is an empty circle, and checked state is a
+     *      circle with a dot inside.
+     *
+     * @param {Object} [options]
+     *  Optional parameters:
+     *  @param {Boolean} [options.ambiguous=false]
+     *      If set to true, the ambiguous state (any value but Booleans) will
+     *      be represented by a specific distinct icon. Otherwise, the icon for
+     *      the unchecked state will be used.
+     *
+     * @returns {String}
+     *  The HTML mark-up for the check mark of a check box or option button.
+     */
+    Forms.createCheckMarkMarkup = function (state, design, options) {
+
+        var // special icon for ambiguous state
+            ambiguous = Utils.getBooleanOption(options, 'ambiguous', false),
+            // the icon class name
+            index = (state === true) ? 1 : (ambiguous && _.isNull(state)) ? 2 : 0;
+
+        return '<span class="' + CHECK_MARK_CLASS + '">' + Forms.createIconMarkup(CHECK_ICON_SETS[design][index]) + '</span>';
+    };
+
     /**
      * Creates and returns the HTML mark-up for a button element.
      *
@@ -1111,6 +1219,9 @@ define('io.ox/office/tk/forms',
      *  Forms.createElementMarkup() and Forms.setControlCaption(). Additional
      *  HTML content specified with the option 'content' will be appended to
      *  the button caption. Additionally, the following options are supported:
+     *  @param {String} [options.nodeName='a']
+     *      The name of the HTML element that will represent the button. By
+     *      default, an <a> anchor element will be used.
      *  @param {Boolean} [options.focusable=true]
      *      If set to false, the 'tabindex' attribute of the button element
      *      will be set to the value -1 in order to remove the button from the
@@ -1121,18 +1232,22 @@ define('io.ox/office/tk/forms',
      */
     Forms.createButtonMarkup = function (options) {
 
-        var attributes = Utils.getObjectOption(options, 'attributes'),
+        var nodeName = Utils.getStringOption(options, 'nodeName', 'a'),
+            attributes = Utils.getObjectOption(options, 'attributes'),
             classes = Utils.getStringOption(attributes, 'class', ''),
             focusable = Utils.getBooleanOption(options, 'focusable', true),
             captionMarkup = Forms.createCaptionMarkup(options),
-            contentMarkup = Utils.getStringOption(options, 'content', '');
+            contentMarkup = Utils.getStringOption(options, 'content', ''),
+            ariaOwns = Utils.getStringOption(options, 'aria-owns');
 
         // add more element attributes, return the mark-up
         attributes = _.extend({ tabindex: focusable ? 1 : -1, role: 'button' }, attributes);
-        if (!focusable) { attributes = _.extend({ 'aria-hidden': 'true' }, attributes); }
+        if (!focusable) { attributes = _.extend({ 'aria-hidden': true }, attributes); }
         attributes['class'] = Forms.BUTTON_CLASS;
         if (classes.length > 0) { attributes['class'] += ' ' + classes; }
-        return Forms.createElementMarkup('a', Utils.extendOptions(options, { attributes: attributes, content: captionMarkup + contentMarkup }));
+        if (ariaOwns) {attributes.ariaOwns = ariaOwns; }
+
+        return Forms.createElementMarkup(nodeName, Utils.extendOptions(options, { attributes: attributes, content: captionMarkup + contentMarkup }));
     };
 
     /**
@@ -1191,25 +1306,6 @@ define('io.ox/office/tk/forms',
         }
     };
 
-    /**
-     * Selects or deselects the passed button elements, or sets them into the
-     * mixed state in case the value represented by the buttons is ambiguous.
-     *
-     * @param {HTMLElement|jQuery} buttonNodes
-     *  The button elements to be manipulated. If this object is a jQuery
-     *  collection, uses all nodes it contains.
-     *
-     * @param {Boolean|Null} state
-     *  If set to a Boolean value, selects or deselects all button elements
-     *  according to the value. If set to null, sets the button elements to the
-     *  mixed (ambiguous) state.
-     */
-    Forms.selectButtonNodes = function (buttonNodes, state) {
-        var selected = state === true;
-        Forms.selectNodes(buttonNodes, selected);
-        $(buttonNodes).toggleClass(Forms.AMBIGUOUS_CLASS, _.isNull(state)).attr({'aria-pressed': selected, 'aria-checked': selected, 'aria-selected': selected });
-    };
-
     /**
      * Filters the passed button elements by comparing to a specific value.
      *
@@ -1220,50 +1316,160 @@ define('io.ox/office/tk/forms',
      * @param {Any} value
      *  The button value to be filtered for.
      *
-     * @param {Function} [options.matcher=_.isEqual]
-     *  A comparison function that returns whether the actual button values
-     *  should be included in the result. Receives the passed value as first
-     *  parameter, and the value of the current button element as second
-     *  parameter. The function must return the Boolean value true in order to
-     *  include the respective button element in the result. If omitted, uses
-     *  _.isEqual() which compares arrays and objects deeply.
+     * @param {Object} [options]
+     *  Optional parameters:
+     *  @param {Function} [options.matcher=_.isEqual]
+     *      A comparison function that returns whether the actual button values
+     *      should be included in the result. Receives the passed value as
+     *      first parameter, and the value of the current button element as
+     *      second parameter. The function must return the Boolean value true
+     *      in order to include the respective button element in the result. If
+     *      omitted, uses _.isEqual() which compares arrays and objects deeply.
      *
      * @returns {jQuery}
      *  A jQuery collection with all button nodes whose value is equal to the
      *  passed value.
      */
-    Forms.filterButtonNodes = function (buttonNodes, value, matcher) {
-        if (!_.isFunction(matcher)) { matcher = _.isEqual; }
+    Forms.filterButtonNodes = function (buttonNodes, value, options) {
+        var matcher = Utils.getFunctionOption(options, 'matcher', _.isEqual);
         return $(buttonNodes).filter(function () {
             return matcher(value, Forms.getButtonValue(this)) === true;
         });
     };
 
     /**
-     * Filters the passed button elements by a specific value.
+     * Returns whether the specified button element is in selected/checked
+     * state.
+     *
+     * @param {HTMLElement|jQuery} buttonNode
+     *  The button element. If this object is a jQuery collection, uses the
+     *  first node it contains.
+     *
+     * @returns {Boolean}
+     *  Whether the button node is selected/checked.
+     */
+    Forms.isCheckedButtonNode = function (buttonNode) {
+        return $(buttonNode).first().is(Forms.BUTTON_SELECTOR + DATA_CHECKED_SELECTOR);
+    };
+
+    /**
+     * Filters the selected/checked button elements from the passed jQuery
+     * collection.
      *
      * @param {HTMLElement|jQuery} buttonNodes
-     *  The button elements to be filtered. If this object is a jQuery
+     *  The button elements to be filtered for.
+     *
+     * @returns {jQuery}
+     *  A jQuery collection with all selected/checked button elements.
+     */
+    Forms.filterCheckedButtonNodes = function (buttonNodes) {
+        return $(buttonNodes).filter(Forms.BUTTON_SELECTOR + DATA_CHECKED_SELECTOR);
+    };
+
+    /**
+     * Selects or deselects the passed button elements, or sets them into the
+     * mixed state in case the value represented by the buttons is ambiguous.
+     *
+     * @param {HTMLElement|jQuery} buttonNodes
+     *  The button elements to be manipulated. If this object is a jQuery
      *  collection, uses all nodes it contains.
      *
+     * @param {Boolean|Null} state
+     *  The new state of the button nodes. The Boolean value true results in a
+     *  selected/checked design. The Boolean value false results in an
+     *  unselected/unchecked design. The value null is treated as ambiguous
+     *  state and results in a specific icon, if the option 'ambiguous' is set.
+     *  Otherwise, this will be the unselected/unchecked design.
+     *
+     * @param {Object} [options]
+     *  Optional parameters:
+     *  @param {String} [options.design='select']
+     *      Specifies the design that will be used to visualize the selected
+     *      state. The following values are supported:
+     *      - 'select' (default): Selected state will be visualized by a
+     *          colored button background.
+     *      - 'check': Space for an icon will be added in front of the button
+     *          caption. Unselected state remains empty space, and selected
+     *          state is a simple check mark.
+     *      - 'boxed': Space for an icon will be added in front of the button
+     *          caption. Unselected state is an empty rectangle, and selected
+     *          state is a rectangle with a check mark.
+     *      - 'radio': Space for an icon will be added in front of the button
+     *          caption. Unselected state is an empty circle, and selected
+     *          state is a circle with a dot inside.
+     *  @param {Boolean} [options.ambiguous=false]
+     *      If set to true, the ambiguous state (any value but Booleans) will
+     *      be represented by a fill pattern (if design is 'select'), otherwise
+     *      by a specific distinct icon. Without this option, the design for
+     *      the unselected state will be used for the ambiguous state.
+     */
+    Forms.checkButtonNodes = function (buttonNodes, state, options) {
+
+        var // the design mode
+            design = Utils.getStringOption(options, 'design', 'select'),
+            // whether to use special formatting for the ambiguous state
+            ambiguous = Utils.getBooleanOption(options, 'ambiguous', false),
+
+            haspopup = Utils.getBooleanOption(options, 'haspopup', false),
+            // Boolean selected state (without ambiguous state)
+            selected = state === true;
+
+        // remove old check mark icons (if ever someone switches between icon design and selection design)
+        buttonNodes = $(buttonNodes);
+        buttonNodes.find('>.' + CHECK_MARK_CLASS).remove();
+
+        // update button contents/formatting according to design mode
+        if (design === 'select') {
+            Forms.selectNodes(buttonNodes, selected);
+            buttonNodes.toggleClass(AMBIGUOUS_CLASS, ambiguous && _.isNull(state)).attr('aria-selected', selected);
+        } else {
+            buttonNodes.prepend(Forms.createCheckMarkMarkup(state, design, { ambiguous: ambiguous }));
+        }
+
+        // update generic attributes
+        if (haspopup) {
+            buttonNodes.attr(DATA_CHECKED_ATTR, selected).attr({ 'aria-haspopup': true, 'aria-expanded': selected });
+        } else {
+            buttonNodes.attr(DATA_CHECKED_ATTR, selected).attr({ 'aria-pressed': selected, 'aria-checked': selected });
+        }
+    };
+
+    /**
+     * Selects or deselects the button elements with a specific value.
+     *
+     * @param {HTMLElement|jQuery} buttonNodes
+     *  The button elements to be selected or deselected. If this object is a
+     *  jQuery collection, uses all nodes it contains.
+     *
      * @param {Any} value
      *  The button value to be filtered for.
      *
-     * @param {Function} [options.matcher=_.isEqual]
-     *  A comparison function that returns whether the actual button values
-     *  should be selected, deselected, or skipped. Receives the passed value
-     *  as first parameter, and the value of the current button element as
-     *  second parameter. The function must return the Boolean value true to
-     *  select the respective button element, the Boolean value false to
-     *  deselect the button element, or any other value to leave the button
-     *  element unmodified. If omitted, uses _.isEqual() which compares arrays
-     *  and objects deeply, and does not skip any button element.
+     * @param {Object} [options]
+     *  Optional parameters:
+     *  @param {Function} [options.matcher=_.isEqual]
+     *      A comparison function that returns whether the actual button values
+     *      should be selected, deselected, or skipped. Receives the passed
+     *      value as first parameter, and the value of the current button
+     *      element as second parameter. The function must return the Boolean
+     *      value true to select the respective button element, the Boolean
+     *      value false to deselect the button element, or any other value to
+     *      leave the button element unmodified. If omitted, uses _.isEqual()
+     *      which compares arrays and objects deeply, and does not skip any
+     *      button element.
+     *  @param {String} [options.design='select']
+     *      Specifies the design that will be used to visualize the selected
+     *      state. See method Forms.checkButtonNodes() for details.
      */
-    Forms.selectMatchingButtonNodes = function (buttonNodes, value, matcher) {
-        if (!_.isFunction(matcher)) { matcher = _.isEqual; }
+    Forms.checkMatchingButtonNodes = function (buttonNodes, value, options) {
+
+        var // the matcher predicate function
+            matcher = Utils.getFunctionOption(options, 'matcher', _.isEqual);
+
         $(buttonNodes).each(function () {
             var state = matcher(value, Forms.getButtonValue(this));
-            if (_.isBoolean(state)) { Forms.selectButtonNodes(this, state); }
+            if (_.isBoolean(state)) {
+                Forms.checkButtonNodes(this, state, options);
+            }
         });
     };
 
