@@ -1,21 +1,21 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/io',
-    ['io.ox/core/http',
-     'io.ox/office/tk/config',
-     'io.ox/office/tk/utils'
-    ], function (CoreHTTP, Config, Utils) {
+define('io.ox/office/tk/io', [
+    'io.ox/core/http',
+    'io.ox/office/tk/config',
+    'io.ox/office/tk/utils'
+], function (CoreHTTP, Config, Utils) {
 
     'use strict';
 
@@ -30,6 +30,18 @@ define('io.ox/office/tk/io',
      */
     var IO = {};
 
+    // constants --------------------------------------------------------------
+
+    /**
+     * The name of the document filter server module.
+     */
+    IO.FILTER_MODULE_NAME = 'oxodocumentfilter';
+
+    /**
+     * The name of the document converter server module.
+     */
+    IO.CONVERTER_MODULE_NAME = 'oxodocumentconverter';
+
     // public methods ---------------------------------------------------------
 
     /**
@@ -72,16 +84,18 @@ define('io.ox/office/tk/io',
      * Sends a request to the server and returns the promise of a Deferred
      * object waiting for the response.
      *
-     * @param {Object} options
-     *  Additional options. The following options are supported:
+     * @param {String} module
+     *  The name of the server module that will receive the request.
+     *
+     * @param {Object} params
+     *  Parameters that will be inserted into the request URL (method GET), or
+     *  into the request body (method POST).
+     *
+     * @param {Object} [options]
+     *  Optional parameters:
      *  @param {String} [options.method='GET']
      *      The request method. Must be an upper-case string (for example,
      *      'GET', 'POST', etc.). Defaults to 'GET'.
-     *  @param {String} options.module
-     *      The name of the server module that will receive the request.
-     *  @param {Object} [options.params]
-     *      Parameters that will be inserted into the request URL (method GET),
-     *      or into the request body (method POST).
      *  @param {Function} [options.resultFilter]
      *      A function that will be called if the request returns successfully,
      *      and filters the resulting 'data' object returned by the request.
@@ -98,23 +112,21 @@ define('io.ox/office/tk/io',
      *  allows to abort the running request which rejects the promise. Calling
      *  this method has no effect, if the request is finished already.
      */
-    IO.sendRequest = function (options) {
+    IO.sendRequest = function (module, params, options) {
 
         var // extract the request method
             method = Utils.getStringOption(options, 'method', 'GET'),
             // extract the result filter callback
             resultFilter = Utils.getFunctionOption(options, 'resultFilter'),
+            // properties passed to the server request
+            requestProps = { module: module, params: params },
             // the Deferred object representing the core AJAX request
             ajaxRequest = null,
             // the Promise returned by this method
             promise = null;
 
-        // remove the internal options
-        delete options.method;
-        delete options.resultFilter;
-
         // send the AJAX request
-        ajaxRequest = CoreHTTP[method](options);
+        ajaxRequest = CoreHTTP[method](requestProps);
 
         // reject, if the response contains 'hasErrors:true'
         promise = ajaxRequest.then(function (response) {
@@ -282,18 +294,16 @@ define('io.ox/office/tk/io',
      *  the file 'main.js' inside this directory.
      */
     IO.prefetchModuleSource = _.memoize(function (modulePath) {
-        var // determine the module whose source shall be prefetched
+
+        var // determine the module for server-side initialization
             module = modulePath.substr(modulePath.lastIndexOf('/') + 1);
-        if (module) {
-            // send prefetch request to backend
-            IO.sendRequest({
-                module: 'oxodocumentfilter',
-                params: {
-                    action: 'prefetchofficeparts',
-                    part: module
-                }
-            });
+
+        // send prefetch request to server
+        if (module.length > 0) {
+            IO.sendRequest(IO.FILTER_MODULE_NAME, { action: 'prefetchofficeparts', part: module });
         }
+
+        // fetch source code for module
         _.delay(function () {
             Utils.log('IO.prefetchModuleSource(): fetching ' + modulePath);
             require([modulePath + '/main']);
