@@ -1,17 +1,19 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/logger', ['io.ox/office/tk/config'], function (Config) {
+define('io.ox/office/tk/logger', [
+    'io.ox/office/tk/config'
+], function (Config) {
 
     'use strict';
 
@@ -19,7 +21,26 @@ define('io.ox/office/tk/logger', ['io.ox/office/tk/config'], function (Config) {
         indent = '',
 
         // prevent collapsing space characters in browser console
-        INDENT_PARTICLE = '\xa0 ';
+        INDENT_PARTICLE = '\xa0 ',
+
+        // function returning the current system time stamp
+        now = (window.performance && window.performance.now) ? function () { return window.performance.now(); } : _.now;
+
+    // private global functions ===============================================
+
+    /**
+     * Increases the global indentation level for all logger messages.
+     */
+    function incIndent() {
+        indent += INDENT_PARTICLE;
+    }
+
+    /**
+     * Decreases the global indentation level for all logger messages.
+     */
+    function decIndent() {
+        indent = indent.substring(INDENT_PARTICLE.length);
+    }
 
     // class Logger ===========================================================
 
@@ -246,21 +267,49 @@ define('io.ox/office/tk/logger', ['io.ox/office/tk/config'], function (Config) {
          * @returns {Any}
          *  The return value of the callback function.
          */
-        this.takeTime = enabled ? (function (logger) {
+        this.takeTime = enabled ? function (message, callback, context) {
+            var t0 = 0;
+            try {
+                this.log('=>', message);
+                t0 = now();
+                return Logger.indent(callback, context);
+            } finally {
+                this.log('<=', (Math.round((now() - t0) * 1000) / 1000) + 'ms');
+            }
+        } : function (message, callback, context) {
+            return callback.call(context);
+        };
 
-            var now = (window.performance && window.performance.now) ? _.bind(window.performance.now, window.performance) : _.now;
-
-            return function (message, callback, context) {
-                var t0 = 0;
-                try {
-                    logger.log('=>', message);
-                    t0 = now();
-                    return Logger.indent(callback, context);
-                } finally {
-                    logger.log('<=', (Math.round((now() - t0) * 1000) / 1000) + 'ms');
-                }
-            };
-        })(this) : function (message, callback, context) {
+        /**
+         * Executes the passed asynchronous callback function and writes a
+         * message to the browser console containing the time it needs to
+         * resolve the promise returned by the callback. The callback function
+         * will be invoked ALWAYS, regardless whether logging is enabled or
+         * not.
+         *
+         * @param {String} message
+         *  The initial message printed to the browser console before invoking
+         *  the callback function. The execution time will be shown after the
+         *  promise returned by the callback function has been resolved or
+         *  rejected.
+         *
+         * @param {Function} callback
+         *  The callback function to be executed. MUST return a Deferred object
+         *  or a promise of a Deferred object.
+         *
+         * @param {Object} [context]
+         *  The context to be bound to the callback function.
+         *
+         * @returns {jQuery.Promise}
+         *  The promise returned by the callback function.
+         */
+        this.takeAsyncTime = enabled ? function (message, callback, context) {
+            this.log('=>', message);
+            var logger = this, t0 = now();
+            return Logger.indent(callback, context).always(function () {
+                logger.log('<=', (Math.round((now() - t0) * 1000) / 1000) + 'ms');
+            });
+        } : function (message, callback, context) {
             return callback.call(context);
         };
 
@@ -295,6 +344,38 @@ define('io.ox/office/tk/logger', ['io.ox/office/tk/config'], function (Config) {
             return method;
         };
 
+        /**
+         * Creates and returns a version of the passed asynchronous callback
+         * function that shows the time each of the promises it returns needs
+         * to be resolved or rejected, if this logger is enabled (wraps the
+         * callback function in a call to Logger.takeAsyncTime() internally).
+         *
+         * @param {String} message
+         *  The initial message printed to the browser console before invoking
+         *  the callback function. The execution time will be shown after the
+         *  promise returned by the method has been resolved or rejected.
+         *
+         * @param {Function} method
+         *  The callback function to be profiled. MUST return a Deferred object
+         *  or a promise of a Deferred object.
+         *
+         * @returns {Function}
+         *  The passed callback function wrapped into a call to the method
+         *  Logger.takeAsyncTime(). Forwards the current calling context to the
+         *  callback function, and returns its return value.
+         */
+        this.profileAsyncMethod = enabled ? function (message, method) {
+            var logger = this;
+            return function () {
+                var args = arguments;
+                return logger.takeAsyncTime(message, function () {
+                    return method.apply(this, args);
+                }, this);
+            };
+        } : function (message, method) {
+            return method;
+        };
+
     } // class Logger
 
     // static methods ---------------------------------------------------------
@@ -330,10 +411,13 @@ define('io.ox/office/tk/logger', ['io.ox/office/tk/config'], function (Config) {
      */
     Logger.indent = function (callback, context) {
         try {
-            indent += INDENT_PARTICLE;
-            return callback.call(context);
-        } finally {
-            indent = indent.substring(INDENT_PARTICLE.length);
+            incIndent();
+            var result = callback.call(context);
+            $.when(result).always(decIndent);
+            return result;
+        } catch (ex) {
+            decIndent();
+            throw ex;
         }
     };
 
@@ -358,7 +442,15 @@ define('io.ox/office/tk/logger', ['io.ox/office/tk/config'], function (Config) {
             var rowNode = $('<p>').addClass(methodName).appendTo(outputNode);
             if (Logger.LOG_TIMESTAMP) { rowNode.append($('<span>').text(args.shift())); }
             if (methodName in ICONS) { rowNode.append('<i class="fa ' + ICONS[methodName] + '"></i>'); }
-            _.each(args, function (msg) { rowNode.append($('<span>').text(msg)); });
+            _.each(args, function (msg) {
+                if (msg === null) {
+                    rowNode.append($('<span style="font-weight:bold">').text('null'));
+                } else if (msg === undefined) {
+                    rowNode.append($('<span style="font-weight:bold">').text('undefined'));
+                } else {
+                    rowNode.append($('<span>').text(msg));
+                }
+            });
             if (!locked) { outputNode.scrollTop(outputNode[0].scrollHeight); }
         }
 
