@@ -1,26 +1,29 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/popup/basemenu',
-    ['io.ox/office/tk/config',
-     'io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/popup/basepopup'
-    ], function (Config, Utils, KeyCodes, Forms, BasePopup) {
+define('io.ox/office/tk/popup/basemenu', [
+    'io.ox/office/tk/config',
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/popup/basepopup'
+], function (Config, Utils, KeyCodes, Forms, BasePopup) {
 
     'use strict';
 
+    var // whether pop-up nodes remain open when losing focus (for debugging)
+        STICKY_POPUPS = Config.DEBUG && Config.getUrlFlag('office:sticky-popups');
+
     // class BaseMenu =========================================================
 
     /**
@@ -126,14 +129,22 @@ define('io.ox/office/tk/popup/basemenu',
         function hideMenuOnFocusNode(node) {
             if (autoClose && !(node && isFocusableNode(node))) {
                 // do not close the menu, if the debug URL flag 'office:sticky-popups' is set (but still return true)
-                if (!Config.DEBUG || !Config.getUrlFlag('office:sticky-popups')) {
-                    self.hide();
-                }
+                if (!STICKY_POPUPS) { self.hide(); }
                 return true;
             }
             return false;
         }
 
+        /**
+         * Moves the browser focus to the content node, if automatic focus
+         * handling is enabled, and the virtual keyboard is not open.
+         */
+        function autoFocusContentNode() {
+            if (autoFocus && !Utils.isSoftKeyboardOpen()) {
+                self.getContentNode().focus();
+            }
+        }
+
         /**
          * Hides the menu node automatically when the browser focus leaves.
          */
@@ -164,10 +175,10 @@ define('io.ox/office/tk/popup/basemenu',
         function popupShowHandler() {
 
             // move focus into the menu node, for keyboard handling
-            if (autoFocus) { self.getContentNode().focus(); }
+            autoFocusContentNode();
 
             // listen to focus events, hide the menu when focus leaves the menu
-            Utils.on('change:focus', changeFocusHandler);
+            self.listenTo(Utils, 'change:focus', changeFocusHandler);
 
             // Add global click handler that closes the menu automatically when
             // clicking somewhere in the page without changing the browser focus
@@ -175,9 +186,9 @@ define('io.ox/office/tk/popup/basemenu',
             // deferred, otherwise it would close the context menu immediately
             // after it has been opened with a mouse click while the event bubbles
             // up to the document root.
-            _.defer(function () {
+            self.executeDelayed(function () {
                 if (self.isVisible()) {
-                    $(document).on(DOCUMENT_EVENTS, globalClickHandler);
+                    self.listenTo($(document), DOCUMENT_EVENTS, globalClickHandler);
                 }
             });
         }
@@ -187,19 +198,33 @@ define('io.ox/office/tk/popup/basemenu',
          */
         function popupBeforeHideHandler() {
             // unregister the focus handler
-            Utils.off('change:focus', changeFocusHandler);
+            self.stopListeningTo(Utils, 'change:focus', changeFocusHandler);
             // unregister the global click/touch handler
-            $(document).off(DOCUMENT_EVENTS, globalClickHandler);
+            self.stopListeningTo($(document), DOCUMENT_EVENTS, globalClickHandler);
+        }
+
+        /**
+         * Initializes the pop-up menu after it has left the busy mode.
+         */
+        function popupIdleHandler() {
+            // move focus into the menu node, for keyboard handling
+            autoFocusContentNode();
         }
 
         /**
          * Hides the pop-up menu, if the ESCAPE key has been pressed inside.
          */
         function keyDownHandler(event) {
+
             if (event.keyCode === KeyCodes.ESCAPE) {
                 self.hide();
                 return false;
             }
+
+            // we need to call preventDefault() because of focus traveling back to the document when this menu has no input fields
+            if (!$(event.target).is('input,textarea')) {
+                event.preventDefault();
+            }
         }
 
         // methods ------------------------------------------------------------
@@ -211,7 +236,7 @@ define('io.ox/office/tk/popup/basemenu',
          *  Whether this pop-up menu contains the browser focus.
          */
         this.hasFocus = function () {
-            return Utils.containsFocus(this.getNode());
+            return Forms.containsFocus(this.getNode());
         };
 
         /**
@@ -230,7 +255,7 @@ define('io.ox/office/tk/popup/basemenu',
         this.grabFocus = function (options) {
 
             // do nothing if a descendant of the content node is already focused
-            if (Utils.containsFocus(this.getContentNode())) { return this; }
+            if (Forms.containsFocus(this.getContentNode())) { return this; }
 
             var // all focusable control elements
                 focusNodes = Forms.findFocusableNodes(this.getNode()),
@@ -240,8 +265,10 @@ define('io.ox/office/tk/popup/basemenu',
                 bottom = Utils.getBooleanOption(options, 'bottom', false);
 
             // fall back to all focusable controls if no preferred controls are available
-            if (preferredNodes.length === 0) { preferredNodes = focusNodes; }
-            preferredNodes[bottom ? 'last' : 'first']().focus();
+            if (preferredNodes.length === 0) {
+                preferredNodes = (focusNodes.length > 0) ? focusNodes : this.getContentNode();
+            }
+            (bottom ? preferredNodes.last() : preferredNodes.first()).focus();
             return this;
         };
 
@@ -281,7 +308,7 @@ define('io.ox/office/tk/popup/basemenu',
         this.registerFocusableNodes(this.getNode());
 
         // register event handlers
-        this.on({ 'popup:show': popupShowHandler, 'popup:beforehide': popupBeforeHideHandler });
+        this.on({ 'popup:show': popupShowHandler, 'popup:beforehide': popupBeforeHideHandler, 'popup:idle': popupIdleHandler });
         this.getNode().on('keydown', keyDownHandler);
 
         // Bug 35239: Windows/Chrome only: When clicking a button in a pop-up menu while
