@@ -1,22 +1,22 @@
 /**
- * All content on this website (including text, images, source
- * code and any other original works), unless otherwise noted,
- * is licensed under a Creative Commons License.
+ * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
+ * LICENSE. This work is protected by copyright and/or other applicable
+ * law. Any use of the work other than as authorized under this license
+ * or copyright law is prohibited.
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * Copyright (C) Open-Xchange Inc., 2006-2012
- * Mail: info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
 
-define('io.ox/office/tk/popup/listmenu',
-    ['io.ox/office/tk/utils',
-     'io.ox/office/tk/keycodes',
-     'io.ox/office/tk/forms',
-     'io.ox/office/tk/popup/basemenu'
-    ], function (Utils, KeyCodes, Forms, BaseMenu) {
+define('io.ox/office/tk/popup/listmenu', [
+    'io.ox/office/tk/utils',
+    'io.ox/office/tk/keycodes',
+    'io.ox/office/tk/forms',
+    'io.ox/office/tk/popup/basemenu'
+], function (Utils, KeyCodes, Forms, BaseMenu) {
 
     'use strict';
 
@@ -116,72 +116,108 @@ define('io.ox/office/tk/popup/listmenu',
 
         // base constructor ---------------------------------------------------
 
-        BaseMenu.call(this, Utils.extendOptions({ preferFocusFilter: Forms.SELECTED_SELECTOR }, initOptions));
+        BaseMenu.call(this, Utils.extendOptions({
+            scrollPadding: 5,
+            preferFocusFilter: Forms.SELECTED_SELECTOR
+        }, initOptions, {
+            prepareLayoutHandler: prepareLayoutHandler
+        }));
 
         // private methods ----------------------------------------------------
 
         /**
-         * Sorts all list items in the specified section node.
+         * Prepares layouting this list in grid design. Calculates the maximum
+         * width of the pop-up menu according to the widths of all existing
+         * button elements, and distributes the available space equally to the
+         * button nodes.
          */
-        function sortSectionNode(sectionNode) {
+        function prepareLayoutHandler() {
 
-            var // all existing item buttons in the section, as plain array (for sorting)
-                buttonNodes = sectionNode.find(Forms.BUTTON_SELECTOR).get(),
-                // table cell nodes (grid design)
-                cellNodes = null;
+            if (design !== 'grid') { return; }
 
-            // sort by fixed index, default to label text of the button
-            buttonNodes = _.sortBy(buttonNodes, function (btnNode) {
-                var index = $(btnNode).data('index');
-                return _.isNull(index) ? $(btnNode).text().toLowerCase() : index;
+            var // all section nodes
+                sectionNodes = self.getSectionNodes(),
+                // the total width needed to show all sections with evenly sized columns
+                totalWidth = 0;
+
+            // remove explicit widths
+            self.getItemNodes().css('width', '');
+
+            // calculate required width of all sections
+            sectionNodes.each(function () {
+
+                var // all button nodes in the current section
+                    buttonNodes = $(this).find(Forms.BUTTON_SELECTOR),
+                    // maximum width of the button elements, used as grid column width
+                    //maxWidth = _.max(buttonNodes.get(), function (node) { return Utils.getCeilNodeSize(node).width; });
+                    maxWidth = _.max(_.map(buttonNodes.get(), function (node) { return Utils.getCeilNodeSize(node).width; }));
+
+                // update total width with the width requited by the current section
+                totalWidth = Math.max(totalWidth, maxWidth * $(this).data('grid-columns'));
             });
 
-            // insert button nodes depending on design mode
-            switch (design) {
-            case 'list':
-                sectionNode.append.apply(sectionNode, buttonNodes);
-                break;
-            case 'grid':
-                cellNodes = sectionNode.find('>table>tbody>tr>td');
-                _.each(buttonNodes, function (btnNode, index) {
-                    cellNodes.eq(index).append(btnNode);
-                });
-                break;
-            }
+            // set button width in all sections
+            sectionNodes.each(function () {
+                $(this).find(Forms.BUTTON_SELECTOR).css('width', Math.floor(totalWidth / $(this).data('grid-columns')));
+            });
         }
 
         /**
          * Sorts all list items in the specified section node debounced.
          */
-        var sortSectionNodeDebounced = (function () {
+        var sortSectionNode = (function () {
 
             var // all section nodes to be sorted
                 sectionNodes = $();
 
-            // sorts all registered sections debounced
-            var sortAllSectionNodes = _.debounce(function () {
+            // direct callback: register a section node
+            function registerSectionNode(sectionNode) {
+                // jQuery keeps the collection unique
+                sectionNodes = sectionNodes.add(sectionNode);
+            }
 
-                // do not run debounced code in a destroyed instance
-                if (!self || self.destroyed) { return; }
+            // deferred callback: sorts all registered sections debounced
+            function sortAllSectionNodes() {
 
                 // protect the focused list item
                 self.guardFocusedListItem(function () {
+
                     // process all registered section nodes
-                    sectionNodes.each(function () { sortSectionNode($(this)); });
+                    sectionNodes.each(function () {
+
+                        var // the current section node
+                            sectionNode = $(this),
+                            // all existing item buttons in the section, as plain array (for sorting)
+                            buttonNodes = sectionNode.find(Forms.BUTTON_SELECTOR).get(),
+                            // cell nodes in grid design
+                            cellNodes = null;
+
+                        // sort by fixed index, default to label text of the button
+                        buttonNodes = _.sortBy(buttonNodes, function (btnNode) {
+                            var index = $(btnNode).data('index');
+                            return _.isNull(index) ? $(btnNode).text().toLowerCase() : index;
+                        });
+
+                        // insert button nodes depending on design mode
+                        switch (design) {
+                        case 'list':
+                            sectionNode.append.apply(sectionNode, buttonNodes);
+                            break;
+                        case 'grid':
+                            cellNodes = sectionNode.find('>.grid-row>.grid-cell');
+                            _.each(buttonNodes, function (btnNode, index) {
+                                cellNodes.eq(index).append(btnNode);
+                            });
+                            break;
+                        }
+                    });
+
                     // reset the cache of dirty section nodes
                     sectionNodes = $();
                 });
-            }, 10);
-
-            // the actual sortSectionNodeDebounced() method to be returned from the local scope
-            function sortSectionNodeDebounced(sectionNode) {
-                // jQuery keeps the collection unique
-                sectionNodes = sectionNodes.add(sectionNode);
-                // sort all registered sections debounced
-                sortAllSectionNodes();
             }
 
-            return sortSectionNodeDebounced;
+            return self.createDebouncedMethod(registerSectionNode, sortAllSectionNodes);
         }());
 
         /**
@@ -275,6 +311,11 @@ define('io.ox/office/tk/popup/listmenu',
                 // add the number of grid columns as jQuery data attribute
                 sectionNode.data('grid-columns', Utils.getIntegerOption(options, 'gridColumns', defColumns, 1));
 
+                // add ARIA role for grid design
+                if (design === 'grid') {
+                    sectionNode.attr({ role: 'grid', 'aria-label': Utils.getStringOption(initOptions, 'tooltip', null, true) });
+                }
+
                 // insert the new section into the drop-down menu, notify listeners
                 this.appendContentNodes(sectionNode);
                 this.trigger('create:section', sectionNode, sectionId, options);
@@ -355,25 +396,19 @@ define('io.ox/office/tk/popup/listmenu',
                 sectionNode.append(buttonNode);
                 break;
             case 'grid':
-                // create a new table element for the button if required
-                var tableNode = sectionNode.find('>table');
-                if (tableNode.length === 0) {
-                    tableNode = $('<table>').attr({ role: 'grid', 'aria-label': Utils.getStringOption(initOptions, 'tooltip', null, true) }).appendTo(sectionNode);
+                // create a new row element for the button if required
+                var rowNode = sectionNode.find('>.grid-row').last();
+                if ((rowNode.length === 0) || (rowNode.children().length === sectionNode.data('grid-columns'))) {
+                    rowNode = $('<div class="grid-row" role="row">').appendTo(sectionNode);
                 }
-                // create a new table row element for the button if required
-                var rowNode = tableNode.find('>tbody>tr').last();
-                if ((rowNode.length === 0) || (rowNode.children().last().children().length > 0)) {
-                    // IE9 does not allow to write to the 'innerHTML' property for <TR> nodes :-O
-                    rowNode = $('<tr role="row">' + Utils.repeatString('<td></td>', sectionNode.data('grid-columns')) + '</tr>').appendTo(tableNode);
-                }
-                // insert the button into the first empty cell
-                var cellNode = rowNode.find('>td:empty').first();
-                cellNode.attr({ role: 'gridcell', 'aria-label': buttonNode.attr('aria-label') || '' }).append(buttonNode);
+                // append the button to the row (wrap into a helper node for ARIA)
+                var cellNode = $('<div class="grid-cell" role="gridcell">').attr('aria-label', buttonNode.attr('aria-label') || '').append(buttonNode);
+                rowNode.append(cellNode);
                 break;
             }
 
             // sort the button nodes in the section
-            if (sorted) { sortSectionNodeDebounced(sectionNode); }
+            if (sorted) { sortSectionNode(sectionNode); }
 
             // notify listeners
             this.trigger('create:item', buttonNode, value, options);
@@ -406,7 +441,7 @@ define('io.ox/office/tk/popup/listmenu',
             // remove button nodes from DOM
             buttonNodes.remove();
 
-            // update tables in grid design mode, hide empty sections
+            // update nodes in grid design mode, hide empty sections
             sectionNodes.each(function () {
 
                 var // current section node
@@ -415,21 +450,15 @@ define('io.ox/office/tk/popup/listmenu',
                 // all existing buttons in the section
                 buttonNodes = sectionNode.find(Forms.BUTTON_SELECTOR);
 
-                // reinsert buttons in grid design mode, update or remove table
+                // reinsert buttons in grid design mode
                 if (design === 'grid') {
-                    var tableNode = sectionNode.find('>table');
-                    if (buttonNodes.length === 0) {
-                        tableNode.remove();
-                    } else {
-                        // redistribute all buttons
-                        var cellNodes = tableNode.find('>tbody>tr>td');
-                        buttonNodes.each(function (index) { cellNodes.eq(index).append(this); });
-                        // delete last empty row
-                        var rowNode = cellNodes.last().parent();
-                        if (rowNode.find(Forms.BUTTON_SELECTOR).length === 0) {
-                            rowNode.remove();
-                        }
-                    }
+                    // redistribute all buttons without moving the cell nodes
+                    var rowNodes = sectionNode.find('>.grid-row'),
+                        cellNodes = rowNodes.find('>.grid-cell');
+                    buttonNodes.each(function (index) { cellNodes.eq(index).append(this); });
+                    // remove trailing empty cell nodes and empty trailing row nodes
+                    cellNodes.filter(':empty').remove();
+                    rowNodes.filter(':empty').remove();
                 }
 
                 // hide empty sections
@@ -463,7 +492,7 @@ define('io.ox/office/tk/popup/listmenu',
          *  value, as jQuery collection.
          */
         this.findItemNodes = function (value) {
-            return Forms.filterButtonNodes(this.getItemNodes(), value, itemMatcher);
+            return Forms.filterButtonNodes(this.getItemNodes(), value, { matcher: itemMatcher });
         };
 
         /**
@@ -475,7 +504,7 @@ define('io.ox/office/tk/popup/listmenu',
          *  collection.
          */
         this.getSelectedItemNodes = function () {
-            return Forms.filterSelectedNodes(this.getItemNodes());
+            return Forms.filterCheckedButtonNodes(this.getItemNodes());
         };
 
         /**
@@ -520,8 +549,8 @@ define('io.ox/office/tk/popup/listmenu',
          */
         this.selectItemNode = function (buttonNode, options) {
             buttonNode = $(buttonNode);
-            Forms.selectButtonNodes(this.getItemNodes(), false);
-            Forms.selectButtonNodes(buttonNode, true);
+            Forms.checkButtonNodes(this.getItemNodes(), false);
+            Forms.checkButtonNodes(buttonNode, true);
             if ((buttonNode.length > 0) && Utils.getBooleanOption(options, 'scroll', false)) {
                 this.scrollToChildNode(buttonNode);
             }
@@ -542,7 +571,7 @@ define('io.ox/office/tk/popup/listmenu',
          *  specified value, as jQuery collection.
          */
         this.selectMatchingItemNodes = function (value) {
-            Forms.selectMatchingButtonNodes(this.getItemNodes(), value, itemMatcher);
+            Forms.checkMatchingButtonNodes(this.getItemNodes(), value, { matcher: itemMatcher });
             return this.getSelectedItemNodes();
         };
 
@@ -613,22 +642,22 @@ define('io.ox/office/tk/popup/listmenu',
                 return $();
             }
 
-            // returns the item node in the previous or next table row in 'grid' design
+            // returns the item node in the previous or next row in 'grid' design
             function getItemNodeInNextRow(diff) {
 
-                var // the table cell node containing the focused button
-                    cellNode = $(Utils.findFarthest(self.getNode(), buttonNode, 'td')),
-                    // the table row node containing the focused button
+                var // the cell node containing the focused button
+                    cellNode = buttonNode.parent(),
+                    // the row node containing the focused button
                     rowNode = cellNode.parent(),
                     // all button elements in the focused row
                     rowButtonNodes = rowNode.children().children(),
-                    // all row nodes in all tables
-                    rowNodes = self.getSectionNodes().find('>table>tbody>tr'),
+                    // all row nodes in all sections
+                    rowNodes = self.getSectionNodes().find('>.grid-row'),
                     // the column index of the cell in the row
                     colIndex = cellNode.index(),
-                    // the row index of the cell in the table
+                    // the global row index of the cell
                     rowIndex = rowNodes.index(rowNode),
-                    // the column index of the cell in the table
+                    // the relative column index of the cell
                     colRatio = (colIndex >= 0) ? (colIndex / rowButtonNodes.length) : 0;
 
                 if ((rowNodes.length === 0) || (rowIndex < 0)) {
@@ -682,29 +711,6 @@ define('io.ox/office/tk/popup/listmenu',
             return $();
         };
 
-        /**
-         * Scrolls the passed list item node into the visible area of the menu.
-         *
-         * @param {HTMLElement|jQuery} buttonNode
-         *  The list item DOM element that will be made visible by scrolling
-         *  the menu root node. If this object is a jQuery collection, uses the
-         *  first node it contains.
-         *
-         * @param {Object} [options]
-         *  Optional parameters. Supports all options that are supported by the
-         *  method Utils.scrollToChildNode(). The option 'padding' is defaulted
-         *  to the value 5.
-         *
-         * @returns {ListMenu}
-         *  A reference to this instance.
-         */
-        this.scrollToChildNode = (function () {
-            var baseMethod = _.bind(self.scrollToChildNode, self);
-            return function (buttonNode, options) {
-                return baseMethod(buttonNode, Utils.extendOptions({ padding: 5 }, options));
-            };
-        }());
-
         // initialization -----------------------------------------------------
 
         // add CSS marker for the item design
@@ -716,14 +722,6 @@ define('io.ox/office/tk/popup/listmenu',
         // convert ENTER, SPACE, and TAB keys in the menu to click events
         Forms.setButtonKeyHandler(this.getNode(), { tab: true });
 
-        // special handling for tables in 'grid' design
-        if (design === 'grid') {
-            this.on({
-                'popup:beforeshow popup:beforelayout': function () { self.getSectionNodes().find('>table').css('width', 'auto'); },
-                'popup:show popup:layout': function () { self.getSectionNodes().find('>table').css('width', ''); }
-            });
-        }
-
         // destroy all class members on destruction
         this.registerDestructor(function () {
             initOptions = self = itemMatcher = null;
