@@ -19,6 +19,20 @@ module.exports = function (grunt) {
                     cwd: 'lib/canvasjs/',
                     dest: 'build/apps/io.ox/office/drawinglayer/',
                     filter: 'isFile'
+                },
+                {
+                    expand: true,
+                    src: ['lib/pdf.js'],
+                    cwd: 'lib/pdfjs/',
+                    dest: 'build/apps/io.ox/office/baseframework/',
+                    filter: 'isFile'
+                },
+                {
+                    expand: true,
+                    src: ['lib/pdftextlayerbuilder.js'],
+                    cwd: 'lib/pdfjs/',
+                    dest: 'build/apps/io.ox/office/baseframework/',
+                    filter: 'isFile'
                 }
             ]
         },
