@@ -17,7 +17,7 @@ module.exports = function(config) {
       '../node_modules/underscore/underscore.js',
       '../bower_components/jquery/dist/jquery.js',
       'spec/lib/**/*.js',
-      'spec/test-main.js',
+      'spec/test-all.js',
       {pattern: 'apps/**/*.js', included: false},
       {pattern: 'spec/**/*_spec.js', included: false}
     ],
