@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2014 Open-Xchange Inc., Tarrytown, NY, USA. info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Daniel Rentz <daniel.rentz@open-xchange.com>
  */
@@ -80,13 +80,6 @@ define(['io.ox/office/tk/forms'], function (Forms) {
                 });
             });
 
-            describe('constant "AMBIGUOUS_CLASS"', function () {
-                it('should exist', function () {
-                    expect(Forms.AMBIGUOUS_CLASS).to.be.a('string');
-                    expect(Forms.AMBIGUOUS_CLASS).to.not.be.empty;
-                });
-            });
-
             describe('constant "FOCUSED_CLASS"', function () {
                 it('should exist', function () {
                     expect(Forms.FOCUSED_CLASS).to.be.a('string');
