@@ -3,12 +3,14 @@ define('io.ox/core/gettext', function () {
     'use strict';
 
     function gt() {
-        return function (text) {
+        var gt = function (text) {
             var args = arguments;
             return text.replace(/%(\d+)\$[ds]/g, function (match, index) {
                 return args[index];
             });
         };
+        gt.pgettext = function (ctxt, text) { return gt(text); };
+        return gt;
     }
 
     gt.language = $.Deferred().resolve('en_US');
