@@ -6,7 +6,7 @@
  *
  * http://creativecommons.org/licenses/by-nc-sa/2.5/
  *
- * © 2011 Open-Xchange Inc., Tarrytown, NY, USA. info@open-xchange.com
+ * © 2016 OX Software GmbH
  *
  * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
  */
@@ -15,6 +15,9 @@
 
     'use strict';
 
+    // TODO: HACK: how to get a real window.ox?
+    window.ox = { base: '' };
+
     // shortcut
     var slice = Array.prototype.slice,
         // deserialize
